@@ -1,9 +1,11 @@
 import fileinput
+import glob
+import logging
 import os
 import shutil
 import sys
-
 import yaml
+import sphinx.util
 
 
 top_level = \
@@ -11,6 +13,9 @@ top_level = \
         os.path.dirname(
             os.path.abspath(__file__)))
 
+pybind_rgw_mod = __import__('rgw', globals(), locals(), [], 0)
+sys.modules['pybind_rgw_mod'] = pybind_rgw_mod
+
 
 def parse_ceph_release():
     with open(os.path.join(top_level, 'src/ceph_release')) as f:
@@ -44,6 +49,20 @@ pygments_style = 'sphinx'
 
 # HTML output options
 html_theme = 'ceph'
+html_theme_options = {
+    'logo_only': True,
+    'display_version': False,
+    'prev_next_buttons_location': 'bottom',
+    'style_external_links': False,
+    'vcs_pageview_mode': '',
+    'style_nav_header_background': '#eee',
+    # Toc options
+    'collapse_navigation': True,
+    'sticky_navigation': True,
+    'navigation_depth': 4,
+    'includehidden': True,
+    'titles_only': False
+}
 html_theme_path = ['_themes']
 html_title = "Ceph Documentation"
 html_logo = 'logo.png'
@@ -104,11 +123,16 @@ extensions = [
     'breathe',
     'ceph_commands',
     'ceph_releases',
-    'sphinxcontrib.openapi'
+    'ceph_confval',
+    'sphinxcontrib.openapi',
+    'sphinxcontrib.seqdiag',
     ]
 
 ditaa = shutil.which("ditaa")
 if ditaa is not None:
+    # in case we don't have binfmt_misc enabled or jar is not registered
+    ditaa_args = ['-jar', ditaa]
+    ditaa = 'java'
     extensions += ['sphinxcontrib.ditaa']
 else:
     extensions += ['plantweb.directive']
@@ -143,17 +167,9 @@ breathe_domain_by_extension = {'py': 'py',
 breathe_doxygen_config_options = {
     'EXPAND_ONLY_PREDEF': 'YES',
     'MACRO_EXPANSION': 'YES',
-    'PREDEFINED': 'CEPH_RADOS_API= ',
-    'WARN_IF_UNDOCUMENTED': 'NO',
+    'PREDEFINED': 'CEPH_RADOS_API= '
 }
 
-# edit_on_github options
-# the docs are rendered with github links pointing to master. the javascript
-# snippet in _static/ceph.js rewrites the edit links when a page is loaded, to
-# point to the correct branch.
-edit_on_github_project = 'ceph/ceph'
-edit_on_github_branch = 'master'
-
 # graphviz options
 graphviz_output_format = 'svg'
 
@@ -214,10 +230,23 @@ for c in pybinds:
     if pybind not in sys.path:
         sys.path.insert(0, pybind)
 
+# openapi
+openapi_logger = sphinx.util.logging.getLogger('sphinxcontrib.openapi.openapi30')
+openapi_logger.setLevel(logging.WARNING)
+
+# seqdiag
+seqdiag_antialias = True
+seqdiag_html_image_format = 'SVG'
+
+# ceph_confval
+ceph_confval_imports = glob.glob(os.path.join(top_level,
+                                              'src/common/options',
+                                              '*.yaml.in'))
+ceph_confval_mgr_module_path = 'src/pybind/mgr'
+ceph_confval_mgr_python_path = 'src/pybind'
 
 # handles edit-on-github and old version warning display
 def setup(app):
-    app.add_js_file('js/ceph.js')
     if ditaa is None:
         # add "ditaa" as an alias of "diagram"
         from plantweb.directive import DiagramDirective
