@@ -1,5 +1,4 @@
 from io import BytesIO
-import six
 import logging
 from tasks.cephfs.cephfs_test_case import CephFSTestCase
 
@@ -51,16 +50,10 @@ class XFSTestsDev(CephFSTestCase):
 
     def get_admin_key(self):
         import configparser
-        from sys import version_info as sys_version_info
 
         cp = configparser.ConfigParser()
-        if sys_version_info.major > 2:
-            cp.read_string(self.fs.mon_manager.raw_cluster_cmd(
-                'auth', 'get-or-create', 'client.admin'))
-        # TODO: remove this part when we stop supporting Python 2
-        elif sys_version_info.major <= 2:
-            cp.read_string(six.text_type(self.fs.mon_manager.raw_cluster_cmd(
-                'auth', 'get-or-create', 'client.admin')))
+        cp.read_string(self.fs.mon_manager.raw_cluster_cmd(
+            'auth', 'get-or-create', 'client.admin'))
 
         return cp['client.admin']['key']
 
@@ -102,7 +95,7 @@ class XFSTestsDev(CephFSTestCase):
         # we keep fedora here so that right deps are installed when this test
         # is run locally by a dev.
         if distro in ('redhatenterpriseserver', 'redhatenterprise', 'fedora',
-                      'centos'):
+                      'centos', 'centosstream'):
             deps = """acl attr automake bc dbench dump e2fsprogs fio \
             gawk gcc indent libtool lvm2 make psmisc quota sed \
             xfsdump xfsprogs \
@@ -140,7 +133,6 @@ class XFSTestsDev(CephFSTestCase):
     def write_local_config(self):
         from os.path import join
         from textwrap import dedent
-        from teuthology.misc import sudo_write_file
 
         mon_sock = self.fs.mon_manager.get_msgrv1_mon_socks()[0]
         self.test_dev = mon_sock + ':/' + self.test_dirname
@@ -156,8 +148,8 @@ class XFSTestsDev(CephFSTestCase):
             ''').format(self.test_dev, self.test_dirs_mount_path, self.scratch_dev,
                         self.scratch_dirs_mount_path, self.get_admin_key())
 
-        sudo_write_file(self.mount_a.client_remote, join(
-            self.repo_path, 'local.config'), xfstests_config_contents)
+        self.mount_a.client_remote.write_file(join(self.repo_path, 'local.config'),
+                                              xfstests_config_contents, sudo=True)
 
     def tearDown(self):
         self.mount_a.client_remote.run(args=['sudo', 'userdel', '--force',
