@@ -44,7 +44,7 @@ function cleanup () {
 
 function get_sources () {
     for s in $SOURCES ; do
-        curl --progress-bar --output $s -L ${DL_PREFIX}$s
+        curl --progress-bar --output $s ${DL_PREFIX}$s
     done
 }
 
@@ -71,7 +71,7 @@ pushd $DESTDIR
 case $(distro_id) in
     centos|fedora|rhel|opensuse*|suse|sles)
         install gcc-c++ make libradospp-devel librados-devel;;
-    ubuntu|debian|devuan)
+    ubuntu|debian|devuan|softiron)
         install g++ make libradospp-dev librados-dev;;
     *)
         echo "$(distro_id) is unknown, $@ will have to be installed manually."
