@@ -13,7 +13,12 @@
 
 #include <sstream>
 #include <iomanip>
-#include <boost/uuid/sha1.hpp>
+#include <boost/version.hpp>
+#if BOOST_VERSION >= 106600
+#  include <boost/uuid/detail/sha1.hpp>
+#else
+#  include <boost/uuid/sha1.hpp>
+#endif
 
 namespace boost {
 namespace compute {
