@@ -1,9 +1,10 @@
 // Boost.Geometry (aka GGL, Generic Geometry Library)
 
 // Copyright (c) 2015 Barend Gehrels, Amsterdam, the Netherlands.
+// Copyright (c) 2017 Adam Wulkiewicz, Lodz, Poland.
 
-// This file was modified by Oracle on 2017.
-// Modifications copyright (c) 2017 Oracle and/or its affiliates.
+// This file was modified by Oracle on 2017, 2019.
+// Modifications copyright (c) 2017, 2019 Oracle and/or its affiliates.
 
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
@@ -18,12 +19,13 @@
 #include <map>
 #include <vector>
 
-#include <boost/geometry/algorithms/num_points.hpp>
 #include <boost/geometry/algorithms/detail/overlay/copy_segment_point.hpp>
 #include <boost/geometry/algorithms/detail/overlay/get_ring.hpp>
 #include <boost/geometry/algorithms/detail/direction_code.hpp>
 #include <boost/geometry/algorithms/detail/overlay/turn_info.hpp>
 
+#include <boost/geometry/util/condition.hpp>
+
 namespace boost { namespace geometry
 {
 
@@ -33,6 +35,9 @@ namespace detail { namespace overlay { namespace sort_by_side
 
 enum direction_type { dir_unknown = -1, dir_from = 0, dir_to = 1 };
 
+typedef signed_size_type rank_type;
+
+
 // Point-wrapper, adding some properties
 template <typename Point>
 struct ranked_point
@@ -47,9 +52,8 @@ struct ranked_point
         , operation(operation_none)
     {}
 
-    template <typename Op>
-    ranked_point(const Point& p, signed_size_type ti, int oi,
-                 direction_type d, Op op)
+    ranked_point(Point const& p, signed_size_type ti, int oi,
+                 direction_type d, operation_type op, segment_identifier const& si)
         : point(p)
         , rank(0)
         , zone(-1)
@@ -58,12 +62,12 @@ struct ranked_point
         , direction(d)
         , count_left(0)
         , count_right(0)
-        , operation(op.operation)
-        , seg_id(op.seg_id)
+        , operation(op)
+        , seg_id(si)
     {}
 
     Point point;
-    std::size_t rank;
+    rank_type rank;
     signed_size_type zone; // index of closed zone, in uu turn there would be 2 zones
     signed_size_type turn_index;
     int operation_index; // 0,1
@@ -121,27 +125,29 @@ template <typename Point, typename SideStrategy, typename LessOnSame, typename C
 struct less_by_side
 {
     less_by_side(const Point& p1, const Point& p2, SideStrategy const& strategy)
-        : m_p1(p1)
-        , m_p2(p2)
+        : m_origin(p1)
+        , m_turn_point(p2)
         , m_strategy(strategy)
     {}
 
     template <typename T>
     inline bool operator()(const T& first, const T& second) const
     {
+        typedef typename SideStrategy::cs_tag cs_tag;
+
         LessOnSame on_same;
         Compare compare;
 
-        int const side_first = m_strategy.apply(m_p1, m_p2, first.point);
-        int const side_second = m_strategy.apply(m_p1, m_p2, second.point);
+        int const side_first = m_strategy.apply(m_origin, m_turn_point, first.point);
+        int const side_second = m_strategy.apply(m_origin, m_turn_point, second.point);
 
         if (side_first == 0 && side_second == 0)
         {
             // Both collinear. They might point into different directions: <------*------>
             // If so, order the one going backwards as the very first.
 
-            int const first_code = direction_code(m_p1, m_p2, first.point);
-            int const second_code = direction_code(m_p1, m_p2, second.point);
+            int const first_code = direction_code<cs_tag>(m_origin, m_turn_point, first.point);
+            int const second_code = direction_code<cs_tag>(m_origin, m_turn_point, second.point);
 
             // Order by code, backwards first, then forward.
             return first_code != second_code
@@ -150,14 +156,14 @@ struct less_by_side
                 ;
         }
         else if (side_first == 0
-                && direction_code(m_p1, m_p2, first.point) == -1)
+                && direction_code<cs_tag>(m_origin, m_turn_point, first.point) == -1)
         {
             // First collinear and going backwards.
             // Order as the very first, so return always true
             return true;
         }
         else if (side_second == 0
-            && direction_code(m_p1, m_p2, second.point) == -1)
+            && direction_code<cs_tag>(m_origin, m_turn_point, second.point) == -1)
         {
             // Second is collinear and going backwards
             // Order as very last, so return always false
@@ -173,14 +179,21 @@ struct less_by_side
 
         // They are both left, both right, and/or both collinear (with each other and/or with p1,p2)
         // Check mutual side
-        int const side_second_wrt_first = m_strategy.apply(m_p2, first.point, second.point);
+        int const side_second_wrt_first = m_strategy.apply(m_turn_point, first.point, second.point);
 
         if (side_second_wrt_first == 0)
         {
             return on_same(first, second);
         }
 
-        int const side_first_wrt_second = -side_second_wrt_first;
+        int const side_first_wrt_second = m_strategy.apply(m_turn_point, second.point, first.point);
+        if (side_second_wrt_first != -side_first_wrt_second)
+        {
+            // (FP) accuracy error in side calculation, the sides are not opposite.
+            // In that case they can be handled as collinear.
+            // If not, then the sort-order might not be stable.
+            return on_same(first, second);
+        }
 
         // Both are on same side, and not collinear
         // Union: return true if second is right w.r.t. first, so -1,
@@ -190,7 +203,8 @@ struct less_by_side
     }
 
 private :
-    Point m_p1, m_p2;
+    Point const& m_origin;
+    Point const& m_turn_point;
     SideStrategy const& m_strategy;
 };
 
@@ -238,8 +252,37 @@ public :
         , m_strategy(strategy)
     {}
 
+    void add_segment_from(signed_size_type turn_index, int op_index,
+            Point const& point_from,
+            operation_type op, segment_identifier const& si,
+            bool is_origin)
+    {
+        m_ranked_points.push_back(rp(point_from, turn_index, op_index, dir_from, op, si));
+        if (is_origin)
+        {
+            m_origin = point_from;
+            m_origin_count++;
+        }
+    }
+
+    void add_segment_to(signed_size_type turn_index, int op_index,
+            Point const& point_to,
+            operation_type op, segment_identifier const& si)
+    {
+        m_ranked_points.push_back(rp(point_to, turn_index, op_index, dir_to, op, si));
+    }
+
+    void add_segment(signed_size_type turn_index, int op_index,
+            Point const& point_from, Point const& point_to,
+            operation_type op, segment_identifier const& si,
+            bool is_origin)
+    {
+        add_segment_from(turn_index, op_index, point_from, op, si, is_origin);
+        add_segment_to(turn_index, op_index, point_to, op, si);
+    }
+
     template <typename Operation, typename Geometry1, typename Geometry2>
-    Point add(Operation const& op, signed_size_type turn_index, signed_size_type op_index,
+    Point add(Operation const& op, signed_size_type turn_index, int op_index,
             Geometry1 const& geometry1,
             Geometry2 const& geometry2,
             bool is_origin)
@@ -248,19 +291,12 @@ public :
         geometry::copy_segment_points<Reverse1, Reverse2>(geometry1, geometry2,
                 op.seg_id, point1, point2, point3);
         Point const& point_to = op.fraction.is_one() ? point3 : point2;
-
-        m_ranked_points.push_back(rp(point1, turn_index, op_index, dir_from, op));
-        m_ranked_points.push_back(rp(point_to, turn_index, op_index, dir_to, op));
-        if (is_origin)
-        {
-            m_origin = point1;
-            m_origin_count++;
-        }
+        add_segment(turn_index, op_index, point1, point_to, op.operation, op.seg_id, is_origin);
         return point1;
     }
 
     template <typename Operation, typename Geometry1, typename Geometry2>
-    void add(Operation const& op, signed_size_type turn_index, signed_size_type op_index,
+    void add(Operation const& op, signed_size_type turn_index, int op_index,
             segment_identifier const& departure_seg_id,
             Geometry1 const& geometry1,
             Geometry2 const& geometry2,
@@ -277,7 +313,7 @@ public :
 
             if (is_origin)
             {
-                int const segment_distance = calculate_segment_distance(op, departure_seg_id, geometry1, geometry2);
+                signed_size_type const segment_distance = calculate_segment_distance(op, departure_seg_id, geometry1, geometry2);
                 if (m_origin_count == 0 ||
                         segment_distance < m_origin_segment_distance)
                 {
@@ -290,24 +326,24 @@ public :
     }
 
     template <typename Operation, typename Geometry1, typename Geometry2>
-    static int calculate_segment_distance(Operation const& op,
+    static signed_size_type calculate_segment_distance(Operation const& op,
             segment_identifier const& departure_seg_id,
             Geometry1 const& geometry1,
             Geometry2 const& geometry2)
     {
         if (op.seg_id.segment_index >= departure_seg_id.segment_index)
         {
+            // dep.seg_id=5, op.seg_id=7, distance=2, being segments 5,6
             return op.seg_id.segment_index - departure_seg_id.segment_index;
         }
         // Take wrap into account
-        // Suppose ring_count=10 (10 points, 9 segments), dep.seg_id=7, op.seg_id=2, then distance=10-9+2
-        // Generic function (is this used somewhere else too?)
-        ring_identifier const rid(op.seg_id.source_index, op.seg_id.multi_index, op.seg_id.ring_index);
-        int const segment_count
-                    (op.seg_id.source_index == 0
-                    ? geometry::num_points(detail::overlay::get_ring<typename geometry::tag<Geometry1>::type>::apply(rid, geometry1))
-                    : geometry::num_points(detail::overlay::get_ring<typename geometry::tag<Geometry2>::type>::apply(rid, geometry2)));
-        return ((segment_count - 1) - departure_seg_id.segment_index) + op.seg_id.segment_index;
+        // Suppose point_count=10 (10 points, 9 segments), dep.seg_id=7, op.seg_id=2,
+        // then distance=9-7+2=4, being segments 7,8,0,1
+        std::size_t const segment_count
+                    = op.seg_id.source_index == 0
+                    ? segment_count_on_ring(geometry1, op.seg_id)
+                    : segment_count_on_ring(geometry2, op.seg_id);
+        return segment_count - departure_seg_id.segment_index + op.seg_id.segment_index;
     }
 
     void apply(Point const& turn_point)
@@ -338,9 +374,11 @@ public :
         }
     }
 
-    template <signed_size_type segment_identifier::*Member, typename Map>
-    void find_open_generic(Map& handled, bool check)
+    void find_open_by_piece_index()
     {
+        // For buffers, use piece index
+        std::set<signed_size_type> handled;
+
         for (std::size_t i = 0; i < m_ranked_points.size(); i++)
         {
             const rp& ranked = m_ranked_points[i];
@@ -349,39 +387,47 @@ public :
                 continue;
             }
 
-            signed_size_type const& index = ranked.seg_id.*Member;
-            if (check && (index < 0 || index > 1))
+            signed_size_type const& index = ranked.seg_id.piece_index;
+            if (handled.count(index) > 0)
             {
-                // Should not occur
                 continue;
             }
-            if (! handled[index])
+            find_polygons_for_source<&segment_identifier::piece_index>(index, i);
+            handled.insert(index);
+        }
+    }
+
+    void find_open_by_source_index()
+    {
+        // Check for source index 0 and 1
+        bool handled[2] = {false, false};
+        for (std::size_t i = 0; i < m_ranked_points.size(); i++)
+        {
+            const rp& ranked = m_ranked_points[i];
+            if (ranked.direction != dir_from)
             {
-                find_polygons_for_source<Member>(index, i);
-                handled[index] = true;
+                continue;
             }
+
+            signed_size_type const& index = ranked.seg_id.source_index;
+            if (index < 0 || index > 1 || handled[index])
+            {
+                continue;
+            }
+            find_polygons_for_source<&segment_identifier::source_index>(index, i);
+            handled[index] = true;
         }
     }
 
     void find_open()
     {
-        if (OverlayType == overlay_buffer)
+        if (BOOST_GEOMETRY_CONDITION(OverlayType == overlay_buffer))
         {
-            // For buffers, use piece index
-            std::map<signed_size_type, bool> handled;
-            find_open_generic
-                <
-                    &segment_identifier::piece_index
-                >(handled, false);
+            find_open_by_piece_index();
         }
         else
         {
-            // For other operations, by source (there should only source 0,1)
-            bool handled[2] = {false, false};
-            find_open_generic
-                <
-                    &segment_identifier::source_index
-                >(handled, true);
+            find_open_by_source_index();
         }
     }
 
@@ -434,7 +480,7 @@ public :
     container_type m_ranked_points;
     Point m_origin;
     std::size_t m_origin_count;
-    int m_origin_segment_distance;
+    signed_size_type m_origin_segment_distance;
     SideStrategy m_strategy;
 
 private :
@@ -444,7 +490,7 @@ private :
     inline std::size_t open_count(Include const& include_functor) const
     {
         std::size_t result = 0;
-        std::size_t last_rank = 0;
+        rank_type last_rank = 0;
         for (std::size_t i = 0; i < m_ranked_points.size(); i++)
         {
             rp const& ranked_point = m_ranked_points[i];
@@ -478,7 +524,7 @@ private :
         return result;
     }
 
-    void assign_ranks(std::size_t min_rank, std::size_t max_rank, int side_index)
+    void assign_ranks(rank_type min_rank, rank_type max_rank, int side_index)
     {
         for (std::size_t i = 0; i < m_ranked_points.size(); i++)
         {
@@ -511,8 +557,8 @@ private :
     {
         bool in_polygon = true; // Because start_index is "from", arrives at the turn
         rp const& start_rp = m_ranked_points[start_index];
-        std::size_t last_from_rank = start_rp.rank;
-        std::size_t previous_rank = start_rp.rank;
+        rank_type last_from_rank = start_rp.rank;
+        rank_type previous_rank = start_rp.rank;
 
         for (std::size_t index = move<Member>(the_index, start_index);
              ;
@@ -551,9 +597,9 @@ private :
     {
         // Find a starting point (the first rank after an outgoing rank
         // with no polygons on the left side)
-        std::size_t start_rank = m_ranked_points.size() + 1;
+        rank_type start_rank = m_ranked_points.size() + 1;
         std::size_t start_index = 0;
-        std::size_t max_rank = 0;
+        rank_type max_rank = 0;
         for (std::size_t i = 0; i < m_ranked_points.size(); i++)
         {
             rp const& ranked_point = m_ranked_points[i];
@@ -573,10 +619,10 @@ private :
         }
 
         // Assign the zones
-        std::size_t const undefined_rank = max_rank + 1;
+        rank_type const undefined_rank = max_rank + 1;
         std::size_t zone_id = 0;
-        std::size_t last_rank = 0;
-        std::size_t rank_at_next_zone = undefined_rank;
+        rank_type last_rank = 0;
+        rank_type rank_at_next_zone = undefined_rank;
         std::size_t index = start_index;
         for (std::size_t i = 0; i < m_ranked_points.size(); i++)
         {
@@ -635,6 +681,23 @@ public :
 };
 
 
+//! Metafunction to define side_order (clockwise, ccw) by operation_type
+template <operation_type OpType>
+struct side_compare {};
+
+template <>
+struct side_compare<operation_union>
+{
+    typedef std::greater<int> type;
+};
+
+template <>
+struct side_compare<operation_intersection>
+{
+    typedef std::less<int> type;
+};
+
+
 }}} // namespace detail::overlay::sort_by_side
 #endif //DOXYGEN_NO_DETAIL
 
