@@ -21,6 +21,8 @@
 #include <boost/spirit/home/karma/detail/get_casetag.hpp>
 #include <boost/spirit/home/karma/detail/string_generate.hpp>
 #include <boost/config.hpp>
+#include <boost/proto/extends.hpp>
+#include <boost/proto/traits.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/mpl/if.hpp>
 #include <map>
@@ -411,9 +413,8 @@ namespace boost { namespace spirit { namespace karma
 
             symbols& sym;
 
-        private:
             // silence MSVC warning C4512: assignment operator could not be generated
-            adder& operator= (adder const&);
+            BOOST_DELETED_FUNCTION(adder& operator= (adder const&))
         };
 
         struct remover
@@ -444,9 +445,8 @@ namespace boost { namespace spirit { namespace karma
 
             symbols& sym;
 
-        private:
             // silence MSVC warning C4512: assignment operator could not be generated
-            remover& operator= (remover const&);
+            BOOST_DELETED_FUNCTION(remover& operator= (remover const&))
         };
 
         adder add;
@@ -670,9 +670,8 @@ namespace boost { namespace spirit { namespace karma
 
             symbols& sym;
 
-        private:
             // silence MSVC warning C4512: assignment operator could not be generated
-            adder& operator= (adder const&);
+            BOOST_DELETED_FUNCTION(adder& operator= (adder const&))
         };
 
         struct remover
@@ -703,9 +702,8 @@ namespace boost { namespace spirit { namespace karma
 
             symbols& sym;
 
-        private:
             // silence MSVC warning C4512: assignment operator could not be generated
-            remover& operator= (remover const&);
+            BOOST_DELETED_FUNCTION(remover& operator= (remover const&))
         };
 
         adder add;
