@@ -555,7 +555,7 @@ void test_spots(RealType)
     test_spots(0.0); // Test double.
     #ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
       test_spots(0.0L); // Test long double.
-      #if !BOOST_WORKAROUND(__BORLANDC__, BOOST_TESTED_AT(0x582))
+      #if !BOOST_WORKAROUND(BOOST_BORLANDC, BOOST_TESTED_AT(0x582))
         test_spots(boost::math::concepts::real_concept(0.)); // Test real concept.
       #endif
     #endif
