@@ -7,7 +7,7 @@ Distributed under the Boost Software License, Version 1.0.
 */
 #include <boost/config.hpp>
 #if !defined(BOOST_NO_CXX11_SMART_PTR)
-#include <boost/detail/lightweight_test.hpp>
+#include <boost/core/lightweight_test.hpp>
 #include <boost/smart_ptr/make_unique.hpp>
 
 class type {
