@@ -262,7 +262,9 @@ class Device(object):
             # retrieve device_id on FreeBSD. Still figuring out if/how the
             # python ioctl implementation does that on FreeBSD
             dev_id = ''
-        dev_id.replace(' ', '_')
+        dev_id = dev_id.replace(' ', '_')
+        while '__' in dev_id:
+            dev_id = dev_id.replace('__', '_')
         return dev_id
 
     def _set_lvm_membership(self):
@@ -474,15 +476,6 @@ class Device(object):
                 vg_free -= extent_size
             return [vg_free]
 
-    @property
-    def has_partitions(self):
-        '''
-        Boolean to determine if a given device has partitions.
-        '''
-        if self.sys_api.get('partitions'):
-            return True
-        return False
-
     def _check_generic_reject_reasons(self):
         reasons = [
             ('removable', 1, 'removable'),
@@ -521,8 +514,6 @@ class Device(object):
 
         if self.has_gpt_headers:
             rejected.append('Has GPT headers')
-        if self.has_partitions:
-            rejected.append('Has partitions')
         return rejected
 
     def _check_lvm_reject_reasons(self):
