@@ -9,7 +9,7 @@ import os
 import logging
 import json
 import time
-from ceph_volume import process, conf, __release__, terminal
+from ceph_volume import process, conf, terminal
 from ceph_volume.util import system, constants, str_to_int, disk
 
 logger = logging.getLogger(__name__)
@@ -183,6 +183,7 @@ def osd_id_available(osd_id):
     """
     if osd_id is None:
         return False
+
     bootstrap_keyring = '/var/lib/ceph/bootstrap-osd/%s.keyring' % conf.cluster
     stdout, stderr, returncode = process.call(
         [
@@ -202,7 +203,7 @@ def osd_id_available(osd_id):
     output = json.loads(''.join(stdout).strip())
     osds = output['nodes']
     osd = [osd for osd in osds if str(osd['id']) == str(osd_id)]
-    if osd and osd[0].get('status') == "destroyed":
+    if not osd or (osd and osd[0].get('status') == "destroyed"):
         return True
     return False
 
@@ -512,11 +513,8 @@ def osd_mkfs_filestore(osd_id, fsid, keyring):
     if get_osdspec_affinity():
         command.extend(['--osdspec-affinity', get_osdspec_affinity()])
 
-    if __release__ != 'luminous':
-        # goes through stdin
-        command.extend(['--keyfile', '-'])
-
     command.extend([
+        '--keyfile', '-',
         '--osd-data', path,
         '--osd-journal', journal,
         '--osd-uuid', fsid,
