@@ -103,11 +103,15 @@ public:
 
   /// Wait until the \c secs expires or \c complete() is called
   int wait_for(double secs) {
+    return wait_for(ceph::make_timespan(secs));
+  }
+
+  int wait_for(ceph::timespan secs) {
     std::unique_lock l{lock};
     if (done) {
       return rval;
     }
-    if (cond.wait_for(l, ceph::make_timespan(secs), [this] { return done; })) {
+    if (cond.wait_for(l, secs, [this] { return done; })) {
       return rval;
     } else {
       return ETIMEDOUT;
