@@ -24,11 +24,10 @@
 
 #include <cmath>
 
-#include "include/mempool.h"
 #include "include/encoding.h"
+#include "include/mempool.h"
 
-static const std::size_t bits_per_char = 0x08;    // 8 bits in 1 char(unsigned)
-static const unsigned char bit_mask[bits_per_char] = {
+static const unsigned char bit_mask[CHAR_BIT] = {
   0x01,  //00000001
   0x02,  //00000010
   0x04,  //00000100
@@ -39,17 +38,16 @@ static const unsigned char bit_mask[bits_per_char] = {
   0x80   //10000000
 };
 
-MEMPOOL_DECLARE_FACTORY(unsigned char, byte, bloom_filter);
-
 class bloom_filter
 {
 protected:
 
-  typedef unsigned int bloom_type;
-  typedef unsigned char cell_type;
+  using bloom_type = unsigned int;
+  using cell_type = unsigned char;
+  using table_type = mempool::bloom_filter::vector<cell_type>;
 
-  unsigned char*       bit_table_;   ///< pointer to bit map
   std::vector<bloom_type> salt_;     ///< vector of salts
+  table_type          bit_table_;    ///< bit map
   std::size_t         salt_count_;   ///< number of salts
   std::size_t         table_size_;   ///< bit table size in bytes
   std::size_t         insert_count_;  ///< insertion count
@@ -59,8 +57,7 @@ protected:
 public:
 
   bloom_filter()
-    : bit_table_(0),
-      salt_count_(0),
+    : salt_count_(0),
       table_size_(0),
       insert_count_(0),
       target_element_count_(0),
@@ -70,14 +67,14 @@ public:
   bloom_filter(const std::size_t& predicted_inserted_element_count,
 	       const double& false_positive_probability,
 	       const std::size_t& random_seed)
-    : bit_table_(0),
-      insert_count_(0),
+    : insert_count_(0),
       target_element_count_(predicted_inserted_element_count),
       random_seed_((random_seed) ? random_seed : 0xA5A5A5A5)
   {
     ceph_assert(false_positive_probability > 0.0);
-    find_optimal_parameters(predicted_inserted_element_count, false_positive_probability,
-			    &salt_count_, &table_size_);
+    std::tie(salt_count_, table_size_) =
+      find_optimal_parameters(predicted_inserted_element_count,
+			      false_positive_probability);
     init();
   }
 
@@ -85,8 +82,7 @@ public:
 	       std::size_t table_size,
 	       const std::size_t& random_seed,
 	       std::size_t target_element_count)
-    : bit_table_(0),
-      salt_count_(salt_count),
+    : salt_count_(salt_count),
       table_size_(table_size),
       insert_count_(0),
       target_element_count_(target_element_count),
@@ -97,16 +93,10 @@ public:
 
   void init() {
     generate_unique_salt();
-    if (table_size_) {
-      bit_table_ = mempool::bloom_filter::alloc_byte.allocate(table_size_);
-      std::fill_n(bit_table_, table_size_, 0x00);
-    } else {
-      bit_table_ = NULL;
-    }
+    bit_table_.resize(table_size_, static_cast<unsigned char>(0x00));
   }
 
   bloom_filter(const bloom_filter& filter)
-    : bit_table_(0)
   {
     this->operator=(filter);
   }
@@ -114,25 +104,18 @@ public:
   bloom_filter& operator = (const bloom_filter& filter)
   {
     if (this != &filter) {
-      if (bit_table_) {
-	mempool::bloom_filter::alloc_byte.deallocate(bit_table_, table_size_);
-      }
       salt_count_ = filter.salt_count_;
       table_size_ = filter.table_size_;
       insert_count_ = filter.insert_count_;
       target_element_count_ = filter.target_element_count_;
       random_seed_ = filter.random_seed_;
-      bit_table_ = mempool::bloom_filter::alloc_byte.allocate(table_size_);
-      std::copy(filter.bit_table_, filter.bit_table_ + table_size_, bit_table_);
+      bit_table_ = filter.bit_table_;
       salt_ = filter.salt_;
     }
     return *this;
   }
 
-  virtual ~bloom_filter()
-  {
-    mempool::bloom_filter::alloc_byte.deallocate(bit_table_, table_size_);
-  }
+  virtual ~bloom_filter() = default;
 
   inline bool operator!() const
   {
@@ -141,8 +124,8 @@ public:
 
   inline void clear()
   {
-    if (bit_table_)
-      std::fill_n(bit_table_, table_size_, 0x00);
+    std::fill(bit_table_.begin(), bit_table_.end(),
+	      static_cast<unsigned char>(0x00));
     insert_count_ = 0;
   }
 
@@ -156,12 +139,8 @@ public:
    * @param val integer value to insert
    */
   inline void insert(uint32_t val) {
-    ceph_assert(bit_table_);
-    std::size_t bit_index = 0;
-    std::size_t bit = 0;
-    for (std::size_t i = 0; i < salt_.size(); ++i)
-    {
-      compute_indices(hash_ap(val,salt_[i]),bit_index,bit);
+    for (auto salt : salt_) {
+      auto [bit_index, bit] = compute_indices(hash_ap(val, salt));
       bit_table_[bit_index >> 3] |= bit_mask[bit];
     }
     ++insert_count_;
@@ -169,12 +148,8 @@ public:
 
   inline void insert(const unsigned char* key_begin, const std::size_t& length)
   {
-    ceph_assert(bit_table_);
-    std::size_t bit_index = 0;
-    std::size_t bit = 0;
-    for (std::size_t i = 0; i < salt_.size(); ++i)
-    {
-      compute_indices(hash_ap(key_begin,length,salt_[i]),bit_index,bit);
+    for (auto salt : salt_) {
+      auto [bit_index, bit] = compute_indices(hash_ap(key_begin, length, salt));
       bit_table_[bit_index >> 3] |= bit_mask[bit];
     }
     ++insert_count_;
@@ -212,15 +187,12 @@ public:
    */
   inline virtual bool contains(uint32_t val) const
   {
-    if (!bit_table_)
+    if (table_size_ == 0) {
       return false;
-    std::size_t bit_index = 0;
-    std::size_t bit = 0;
-    for (std::size_t i = 0; i < salt_.size(); ++i)
-    {
-      compute_indices(hash_ap(val,salt_[i]),bit_index,bit);
-      if ((bit_table_[bit_index >> 3] & bit_mask[bit]) != bit_mask[bit])
-      {
+    }
+    for (auto salt : salt_) {
+      auto [bit_index, bit] = compute_indices(hash_ap(val, salt));
+      if ((bit_table_[bit_index >> 3] & bit_mask[bit]) != bit_mask[bit]) {
         return false;
       }
     }
@@ -229,15 +201,12 @@ public:
 
   inline virtual bool contains(const unsigned char* key_begin, const std::size_t length) const
   {
-    if (!bit_table_)
+    if (table_size_ == 0) {
       return false;
-    std::size_t bit_index = 0;
-    std::size_t bit = 0;
-    for (std::size_t i = 0; i < salt_.size(); ++i)
-    {
-      compute_indices(hash_ap(key_begin,length,salt_[i]),bit_index,bit);
-      if ((bit_table_[bit_index >> 3] & bit_mask[bit]) != bit_mask[bit])
-      {
+    }
+    for (auto salt : salt_) {
+      auto [bit_index, bit] = compute_indices(hash_ap(key_begin, length, salt));
+      if ((bit_table_[bit_index >> 3] & bit_mask[bit]) != bit_mask[bit]) {
         return false;
       }
     }
@@ -286,7 +255,7 @@ public:
 
   inline virtual std::size_t size() const
   {
-    return table_size_ * bits_per_char;
+    return table_size_ * CHAR_BIT;
   }
 
   inline std::size_t element_count() const
@@ -306,21 +275,7 @@ public:
    *    .75 = 200% target insertions
    *   1.0  = all bits set... infinite insertions
    */
-  inline double density() const
-  {
-    if (!bit_table_)
-      return 0.0;
-    size_t set = 0;
-    uint8_t *p = bit_table_;
-    size_t left = table_size_;
-    while (left-- > 0) {
-      uint8_t c = *p;
-      for (; c; ++set)
-	c &= c - 1;
-      ++p;
-    }
-    return (double)set / (double)(table_size_ << 3);
-  }
+  double density() const;
 
   virtual inline double approx_unique_element_count() const {
     // this is not a very good estimate; a better solution should have
@@ -342,15 +297,18 @@ public:
 
   inline const cell_type* table() const
   {
-    return bit_table_;
+    return bit_table_.data();
   }
 
 protected:
 
-  inline virtual void compute_indices(const bloom_type& hash, std::size_t& bit_index, std::size_t& bit) const
+  virtual std::pair<size_t /* bit_index */,
+		    size_t /* bit */>
+  compute_indices(const bloom_type& hash) const
   {
-    bit_index = hash % (table_size_ << 3);
-    bit = bit_index & 7;
+    size_t bit_index = hash % (table_size_ << 3);
+    size_t bit = bit_index & 7;
+    return {bit_index, bit};
   }
 
   void generate_unique_salt()
@@ -431,10 +389,10 @@ protected:
     }
   }
 
-  static void find_optimal_parameters(std::size_t target_insert_count,
-				      double target_fpp,
-				      std::size_t *salt_count,
-				      std::size_t *table_size)
+  static std::pair<std::size_t /* salt_count */,
+		   std::size_t /* table_size */>
+  find_optimal_parameters(std::size_t target_insert_count,
+			  double target_fpp)
   {
     /*
       Note:
@@ -462,10 +420,11 @@ protected:
       k += 1.0;
     }
 
-    *salt_count = static_cast<std::size_t>(min_k);
+    size_t salt_count = static_cast<std::size_t>(min_k);
     size_t t = static_cast<std::size_t>(min_m);
-    t += (((t & 7) != 0) ? (bits_per_char - (t & 7)) : 0);
-    *table_size = t >> 3;
+    t += (((t & 7) != 0) ? (CHAR_BIT - (t & 7)) : 0);
+    size_t table_size = t >> 3;
+    return {salt_count, table_size};
   }
 
   inline bloom_type hash_ap(uint32_t val, bloom_type hash) const
@@ -539,12 +498,12 @@ public:
 
   inline std::size_t size() const override
   {
-    return size_list.back() * bits_per_char;
+    return size_list.back() * CHAR_BIT;
   }
 
   inline bool compress(const double& target_ratio)
   {
-    if (!bit_table_)
+    if (bit_table_.empty())
       return false;
 
     if ((0.0 >= target_ratio) || (target_ratio >= 1.0))
@@ -560,21 +519,19 @@ public:
       return false;
     }
 
-    cell_type* tmp = mempool::bloom_filter::alloc_byte.allocate(new_table_size);
-    std::copy(bit_table_, bit_table_ + (new_table_size), tmp);
-    cell_type* itr = bit_table_ + (new_table_size);
-    cell_type* end = bit_table_ + (original_table_size);
-    cell_type* itr_tmp = tmp;
-    cell_type* itr_end = tmp + (new_table_size);
-    while (end != itr)
-    {
+    table_type tmp(new_table_size);
+    std::copy(bit_table_.begin(), bit_table_.begin() + new_table_size, tmp.begin());
+    auto itr = bit_table_.begin() + new_table_size;
+    auto end = bit_table_.begin() + original_table_size;
+    auto itr_tmp = tmp.begin();
+    auto itr_end = tmp.begin() + new_table_size;
+    while (end != itr) {
       *(itr_tmp++) |= (*itr++);
-      if (itr_tmp == itr_end)
-	itr_tmp = tmp;
+      if (itr_tmp == itr_end) {
+	itr_tmp = tmp.begin();
+      }
     }
-
-    mempool::bloom_filter::alloc_byte.deallocate(bit_table_, table_size_);
-    bit_table_ = tmp;
+    std::swap(bit_table_, tmp);
     size_list.push_back(new_table_size);
     table_size_ = new_table_size;
 
@@ -591,14 +548,16 @@ public:
 
 private:
 
-  inline void compute_indices(const bloom_type& hash, std::size_t& bit_index, std::size_t& bit) const override
+  std::pair<size_t /* bit_index */,
+	    size_t /* bit */>
+  compute_indices(const bloom_type& hash) const final
   {
-    bit_index = hash;
-    for (std::size_t i = 0; i < size_list.size(); ++i)
-    {
-      bit_index %= size_list[i] << 3;
+    size_t bit_index = hash;
+    for (auto size : size_list) {
+      bit_index %= size << 3;
     }
-    bit = bit_index & 7;
+    size_t bit = bit_index & 7;
+    return {bit_index, bit};
   }
 
   std::vector<std::size_t> size_list;
@@ -615,10 +574,10 @@ WRITE_CLASS_ENCODER(compressible_bloom_filter)
 
 /*
   Note 1:
-  If it can be guaranteed that bits_per_char will be of the form 2^n then
+  If it can be guaranteed that CHAR_BIT will be of the form 2^n then
   the following optimization can be used:
 
-  hash_table[bit_index >> n] |= bit_mask[bit_index & (bits_per_char - 1)];
+  bit_table_[bit_index >> n] |= bit_mask[bit_index & (CHAR_BIT - 1)];
 
   Note 2:
   For performance reasons where possible when allocating memory it should
