@@ -19,6 +19,15 @@
 #include "common/strtol.h"
 #include "json_spirit/json_spirit.h"
 
+using std::is_same_v;
+using std::ostringstream;
+using std::string;
+using std::stringstream;
+using std::string_view;
+using std::vector;
+
+using namespace std::literals;
+
 /**
  * Given a cmddesc like "foo baz name=bar,type=CephString",
  * return the prefix "foo baz".
@@ -85,7 +94,7 @@ std::string cmddesc_get_prenautilus_compat(const std::string &cmddesc)
       // Instruct legacy clients or mons to send --foo-bar string in place
       // of a 'true'/'false' value
       std::ostringstream oss;
-      oss << std::string("--") << desckv["name"];
+      oss << "--" << desckv["name"];
       std::string val = oss.str();
       std::replace(val.begin(), val.end(), '_', '-');
       desckv["type"] = "CephChoices";
@@ -131,13 +140,20 @@ dump_cmd_to_json(Formatter *f, uint64_t features, const string& cmd)
 
   stringstream ss(cmd);
   std::string word;
+  bool positional = true;
 
   while (std::getline(ss, word, ' ')) {
+    if (word == "--") {
+      positional = false;
+      continue;
+    }
+
     // if no , or =, must be a plain word to put out
     if (word.find_first_of(",=") == string::npos) {
       f->dump_string("arg", word);
       continue;
     }
+
     // accumulate descriptor keywords in desckv
     auto desckv = cmddesc_get_args(word);
     // name the individual desc object based on the name key
@@ -151,7 +167,7 @@ dump_cmd_to_json(Formatter *f, uint64_t features, const string& cmd)
         // Instruct legacy clients to send --foo-bar string in place
         // of a 'true'/'false' value
         std::ostringstream oss;
-        oss << std::string("--") << desckv["name"];
+        oss << "--" << desckv["name"];
         val = oss.str();
         std::replace(val.begin(), val.end(), '_', '-');
 
@@ -161,8 +177,20 @@ dump_cmd_to_json(Formatter *f, uint64_t features, const string& cmd)
     }
 
     // dump all the keys including name into the array
+    if (!positional) {
+      desckv["positional"] = "false";
+    }
     for (auto [key, value] : desckv) {
-      f->dump_string(key, value);
+      if (key == "positional") {
+	if (!HAVE_FEATURE(features, SERVER_QUINCY)) {
+	  continue;
+	}
+	f->dump_bool(key, value == "true" || value == "True");
+      } else if (key == "req" && HAVE_FEATURE(features, SERVER_QUINCY)) {
+	f->dump_bool(key, value == "true" || value == "True");
+      } else {
+	f->dump_string(key, value);
+      }
     }
     f->close_section(); // attribute object for individual desc
   }
@@ -282,7 +310,7 @@ void cmdmap_dump(const cmdmap_t &cmdmap, Formatter *f)
  * false, ss is valid */
 
 bool
-cmdmap_from_json(const vector<string>& cmd, cmdmap_t *mapp, stringstream &ss)
+cmdmap_from_json(const vector<string>& cmd, cmdmap_t *mapp, std::ostream& ss)
 {
   json_spirit::mValue v;
 
@@ -293,15 +321,14 @@ cmdmap_from_json(const vector<string>& cmd, cmdmap_t *mapp, stringstream &ss)
 
   try {
     if (!json_spirit::read(fullcmd, v))
-      throw runtime_error("unparseable JSON " + fullcmd);
+      throw std::runtime_error("unparseable JSON " + fullcmd);
     if (v.type() != json_spirit::obj_type)
-      throw(runtime_error("not JSON object " + fullcmd));
+      throw std::runtime_error("not JSON object " + fullcmd);
 
     // allocate new mObject (map) to return
     // make sure all contents are simple types (not arrays or objects)
     json_spirit::mObject o = v.get_obj();
-    for (map<string, json_spirit::mValue>::iterator it = o.begin();
-	 it != o.end(); ++it) {
+    for (auto it = o.begin(); it != o.end(); ++it) {
 
       // ok, marshal it into our string->cmd_vartype map, or throw an
       // exception if it's not a simple datatype.  This is kind of
@@ -312,7 +339,7 @@ cmdmap_from_json(const vector<string>& cmd, cmdmap_t *mapp, stringstream &ss)
 
       case json_spirit::obj_type:
       default:
-	throw(runtime_error("JSON array/object not allowed " + fullcmd));
+	throw std::runtime_error("JSON array/object not allowed " + fullcmd);
         break;
 
       case json_spirit::array_type:
@@ -329,7 +356,7 @@ cmdmap_from_json(const vector<string>& cmd, cmdmap_t *mapp, stringstream &ss)
 	    vector<string> outv;
 	    for (const auto& sv : spvals) {
 	      if (sv.type() != json_spirit::str_type) {
-		throw(runtime_error("Can't handle arrays of multiple types"));
+		throw std::runtime_error("Can't handle arrays of multiple types");
 	      }
 	      outv.push_back(sv.get_str());
 	    }
@@ -338,7 +365,7 @@ cmdmap_from_json(const vector<string>& cmd, cmdmap_t *mapp, stringstream &ss)
 	    vector<int64_t> outv;
 	    for (const auto& sv : spvals) {
 	      if (spvals.front().type() != json_spirit::int_type) {
-		throw(runtime_error("Can't handle arrays of multiple types"));
+		throw std::runtime_error("Can't handle arrays of multiple types");
 	      }
 	      outv.push_back(sv.get_int64());
 	    }
@@ -347,14 +374,14 @@ cmdmap_from_json(const vector<string>& cmd, cmdmap_t *mapp, stringstream &ss)
 	    vector<double> outv;
 	    for (const auto& sv : spvals) {
 	      if (spvals.front().type() != json_spirit::real_type) {
-		throw(runtime_error("Can't handle arrays of multiple types"));
+		throw std::runtime_error("Can't handle arrays of multiple types");
 	      }
 	      outv.push_back(sv.get_real());
 	    }
 	    (*mapp)[it->first] = std::move(outv);
 	  } else {
-	    throw(runtime_error("Can't handle arrays of types other than "
-				"int, string, or double"));
+	    throw std::runtime_error("Can't handle arrays of types other than "
+				     "int, string, or double");
 	  }
 	}
 	break;
@@ -376,7 +403,7 @@ cmdmap_from_json(const vector<string>& cmd, cmdmap_t *mapp, stringstream &ss)
       }
     }
     return true;
-  } catch (runtime_error &e) {
+  } catch (const std::runtime_error &e) {
     ss << e.what();
     return false;
   }
@@ -413,7 +440,7 @@ handle_bad_get(CephContext *cct, const string& k, const char *tname)
   lderr(cct) << errstr.str() << dendl;
 
   ostringstream oss;
-  oss << BackTrace(1);
+  oss << ClibBackTrace(1);
   lderr(cct) << oss.str() << dendl;
 
   if (status == 0)
@@ -502,7 +529,7 @@ bool arg_in_range(T value, const arg_desc_t& desc, std::ostream& os) {
   }
   auto min_max = get_str_list(string(range->second), "|");
   auto min = str_to_num<T>(min_max.front());
-  auto max = numeric_limits<T>::max();
+  auto max = std::numeric_limits<T>::max();
   if (min_max.size() > 1) {
     max = str_to_num<T>(min_max.back());
   }
@@ -520,7 +547,7 @@ bool validate_str_arg(std::string_view value,
 {
   if (type == "CephIPAddr") {
     entity_addr_t addr;
-    if (addr.parse(string(value).c_str())) {
+    if (addr.parse(value)) {
       return true;
     } else {
       os << "failed to parse addr '" << value << "', should be ip:[port]";
@@ -544,11 +571,35 @@ bool validate_str_arg(std::string_view value,
   }
 }
 
+bool validate_bool(CephContext *cct,
+		  const cmdmap_t& cmdmap,
+		  const arg_desc_t& desc,
+		  const std::string_view name,
+		  const std::string_view type,
+		  std::ostream& os)
+{
+  bool v;
+  try {
+    if (!cmd_getval(cmdmap, name, v)) {
+      if (auto req = desc.find("req");
+	  req != end(desc) && req->second == "false") {
+	return true;
+      } else {
+	os << "missing required parameter: '" << name << "'";
+	return false;
+      }
+    }
+    return true;
+  } catch (const bad_cmd_get& e) {
+    return false;
+  }
+}
+
 template<bool is_vector,
 	 typename T,
-	 typename Value = conditional_t<is_vector,
-					vector<T>,
-					T>>
+	 typename Value = std::conditional_t<is_vector,
+					     vector<T>,
+					     T>>
 bool validate_arg(CephContext* cct,
 		  const cmdmap_t& cmdmap,
 		  const arg_desc_t& desc,
@@ -558,7 +609,7 @@ bool validate_arg(CephContext* cct,
 {
   Value v;
   try {
-    if (!cmd_getval(cmdmap, string(name), v)) {
+    if (!cmd_getval(cmdmap, name, v)) {
       if constexpr (is_vector) {
 	  // an empty list is acceptable.
 	  return true;
@@ -623,6 +674,9 @@ bool validate_cmd(CephContext* cct,
       } else if (type == "CephFloat") {
 	return !validate_arg<false, double>(cct, cmdmap, arg_desc,
 					    name, type, os);
+      } else if (type == "CephBool") {
+	return !validate_bool(cct, cmdmap, arg_desc,
+			      name, type, os);
       } else {
 	return !validate_arg<false, string>(cct, cmdmap, arg_desc,
 					    name, type, os);
@@ -632,35 +686,56 @@ bool validate_cmd(CephContext* cct,
 }
 
 bool cmd_getval(const cmdmap_t& cmdmap,
-		const std::string& k, bool& val)
+		std::string_view k, bool& val)
 {
   /*
    * Specialized getval for booleans.  CephBool didn't exist before Nautilus,
    * so earlier clients are sent a CephChoices argdesc instead, and will
    * send us a "--foo-bar" value string for boolean arguments.
    */
-  if (cmdmap.count(k)) {
+  auto found = cmdmap.find(k);
+  if (found == cmdmap.end()) {
+    return false;
+  }
+  try {
+    val = boost::get<bool>(found->second);
+    return true;
+  } catch (boost::bad_get&) {
     try {
-      val = boost::get<bool>(cmdmap.find(k)->second);
-      return true;
-    } catch (boost::bad_get&) {
-      try {
-        std::string expected = "--" + k;
-        std::replace(expected.begin(), expected.end(), '_', '-');
+      std::string expected{"--"};
+      expected += k;
+      std::replace(expected.begin(), expected.end(), '_', '-');
 
-        std::string v_str = boost::get<std::string>(cmdmap.find(k)->second);
-        if (v_str == expected) {
-          val = true;
-          return true;
-        } else {
-          throw bad_cmd_get(k, cmdmap);
-        }
-      } catch (boost::bad_get&) {
-        throw bad_cmd_get(k, cmdmap);
+      std::string v_str = boost::get<std::string>(found->second);
+      if (v_str == expected) {
+	val = true;
+	return true;
+      } else {
+	throw bad_cmd_get(k, cmdmap);
       }
+    } catch (boost::bad_get&) {
+      throw bad_cmd_get(k, cmdmap);
     }
   }
-  return false;
+}
+
+bool cmd_getval_compat_cephbool(
+  const cmdmap_t& cmdmap,
+  const std::string& k, bool& val)
+{
+  try {
+    return cmd_getval(cmdmap, k, val);
+  } catch (bad_cmd_get& e) {
+    // try as legacy/compat CephChoices
+    std::string t;
+    if (!cmd_getval(cmdmap, k, t)) {
+      return false;
+    }
+    std::string expected = "--"s + k;
+    std::replace(expected.begin(), expected.end(), '_', '-');
+    val = (t == expected);
+    return true;
+  }
 }
 
 }
