@@ -37,7 +37,7 @@ SUBSYS(rados, 0, 5)
 SUBSYS(rbd, 0, 5)
 SUBSYS(rbd_mirror, 0, 5)
 SUBSYS(rbd_replay, 0, 5)
-SUBSYS(rbd_rwl, 0, 5)
+SUBSYS(rbd_pwl, 0, 5)
 SUBSYS(journaler, 0, 5)
 SUBSYS(objectcacher, 0, 5)
 SUBSYS(immutable_obj_cache, 0, 5)
@@ -60,6 +60,7 @@ SUBSYS(heartbeatmap, 1, 5)
 SUBSYS(perfcounter, 1, 5)
 SUBSYS(rgw, 1, 5)                 // log level for the Rados gateway
 SUBSYS(rgw_sync, 1, 5)
+SUBSYS(rgw_datacache, 1, 5)
 SUBSYS(civetweb, 1, 10)
 SUBSYS(javaclient, 1, 5)
 SUBSYS(asok, 1, 5)
@@ -80,3 +81,17 @@ SUBSYS(dpdk, 1, 5)
 SUBSYS(eventtrace, 1, 5)
 SUBSYS(prioritycache, 1, 5)
 SUBSYS(test, 0, 5)
+SUBSYS(cephfs_mirror, 0, 5)
+SUBSYS(cephsqlite, 0, 5)
+SUBSYS(seastore, 0, 5)       // logs above seastore tm
+SUBSYS(seastore_onode, 0, 5)
+SUBSYS(seastore_odata, 0, 5)
+SUBSYS(seastore_omap, 0, 5)
+SUBSYS(seastore_tm, 0, 5)    // logs below seastore tm
+SUBSYS(seastore_cleaner, 0, 5)
+SUBSYS(seastore_lba, 0, 5)
+SUBSYS(seastore_cache, 0, 5)
+SUBSYS(seastore_journal, 0, 5)
+SUBSYS(seastore_device, 0, 5)
+SUBSYS(alienstore, 0, 5)
+SUBSYS(mclock, 1, 5)
