@@ -4,17 +4,19 @@
 
 #include <memory>
 
-#include "admin_socket.h"
-
 namespace crimson::admin {
 
+class AdminSocketHook;
+
 class AssertAlwaysHook;
+class DumpMetricsHook;
+class DumpPGStateHistory;
+class DumpPerfCountersHook;
 class FlushPgStatsHook;
+class InjectDataErrorHook;
+class InjectMDataErrorHook;
 class OsdStatusHook;
 class SendBeaconHook;
-class DumpPGStateHistory;
-class SeastarMetricsHook;
-
 
 template<class Hook, class... Args>
 std::unique_ptr<AdminSocketHook> make_asok_hook(Args&&... args);
