@@ -31,14 +31,14 @@ namespace crimson {
       // cleaning process
       ServiceTracker()
       {
-	// emptry
-      }
-
-
-      void track_resp(const S& server_id, const NullData& ignore) {
 	// empty
       }
 
+      void track_resp(const S& server_id,
+		      const NullData& ignore,
+		      uint64_t request_cost) {
+	// empty
+      }
 
       /*
        * Returns the ReqParams for the given server.
