@@ -83,7 +83,6 @@
 DEFINE_CEPH_FEATURE( 0, 1, UID)
 DEFINE_CEPH_FEATURE( 1, 1, NOSRCADDR)        // 2.6.35 req
 DEFINE_CEPH_FEATURE_RETIRED( 2, 1, MONCLOCKCHECK, JEWEL, LUMINOUS)
-
 DEFINE_CEPH_FEATURE( 2, 3, SERVER_NAUTILUS)
 DEFINE_CEPH_FEATURE( 3, 1, FLOCK)            // 2.6.36
 DEFINE_CEPH_FEATURE( 4, 1, SUBSCRIBE2)       // 4.6 req
@@ -97,53 +96,49 @@ DEFINE_CEPH_FEATURE(11, 1, PGPOOL3)          // 3.9 req
 DEFINE_CEPH_FEATURE(12, 1, OSDREPLYMUX)
 DEFINE_CEPH_FEATURE(13, 1, OSDENC)           // 3.9 req
 DEFINE_CEPH_FEATURE_RETIRED(14, 1, OMAP, HAMMER, JEWEL)
-
 DEFINE_CEPH_FEATURE(14, 2, SERVER_KRAKEN)
 DEFINE_CEPH_FEATURE(15, 1, MONENC)
 DEFINE_CEPH_FEATURE_RETIRED(16, 1, QUERY_T, JEWEL, LUMINOUS)
-
 DEFINE_CEPH_FEATURE(16, 3, SERVER_OCTOPUS)
 DEFINE_CEPH_FEATURE(16, 3, OSD_REPOP_MLCOD)
 DEFINE_CEPH_FEATURE_RETIRED(17, 1, INDEP_PG_MAP, JEWEL, LUMINOUS)
-
 DEFINE_CEPH_FEATURE(17, 3, OS_PERF_STAT_NS)
 DEFINE_CEPH_FEATURE(18, 1, CRUSH_TUNABLES)   // 3.6
 DEFINE_CEPH_FEATURE_RETIRED(19, 1, CHUNKY_SCRUB, JEWEL, LUMINOUS)
 DEFINE_CEPH_FEATURE(19, 2, OSD_PGLOG_HARDLIMIT)
 DEFINE_CEPH_FEATURE_RETIRED(20, 1, MON_NULLROUTE, JEWEL, LUMINOUS)
-
+DEFINE_CEPH_FEATURE(20, 3, SERVER_PACIFIC)
 DEFINE_CEPH_FEATURE_RETIRED(21, 1, MON_GV, HAMMER, JEWEL)
-
 DEFINE_CEPH_FEATURE(21, 2, SERVER_LUMINOUS)  // 4.13
 DEFINE_CEPH_FEATURE(21, 2, RESEND_ON_SPLIT)  // overlap
 DEFINE_CEPH_FEATURE(21, 2, RADOS_BACKOFF)    // overlap
 DEFINE_CEPH_FEATURE(21, 2, OSDMAP_PG_UPMAP)  // overlap
 DEFINE_CEPH_FEATURE(21, 2, CRUSH_CHOOSE_ARGS) // overlap
 DEFINE_CEPH_FEATURE_RETIRED(22, 1, BACKFILL_RESERVATION, JEWEL, LUMINOUS)
-
+DEFINE_CEPH_FEATURE(22, 2, OSD_FIXED_COLLECTION_LIST)
 DEFINE_CEPH_FEATURE(23, 1, MSG_AUTH)         // 3.19 req (unless nocephx_require_signatures)
 DEFINE_CEPH_FEATURE_RETIRED(24, 1, RECOVERY_RESERVATION, JEWEL, LUMINOUS)
-
 DEFINE_CEPH_FEATURE(24, 2, RECOVERY_RESERVATION_2)
 DEFINE_CEPH_FEATURE(25, 1, CRUSH_TUNABLES2)  // 3.9
 DEFINE_CEPH_FEATURE(26, 1, CREATEPOOLID)
 DEFINE_CEPH_FEATURE(27, 1, REPLY_CREATE_INODE) // 3.9
 DEFINE_CEPH_FEATURE_RETIRED(28, 1, OSD_HBMSGS, HAMMER, JEWEL)
-
 DEFINE_CEPH_FEATURE(28, 2, SERVER_MIMIC)
 DEFINE_CEPH_FEATURE(29, 1, MDSENC)           // 4.7
 DEFINE_CEPH_FEATURE(30, 1, OSDHASHPSPOOL)    // 3.9
-DEFINE_CEPH_FEATURE_DEPRECATED(31, 1, MON_SINGLE_PAXOS, NAUTILUS)
+DEFINE_CEPH_FEATURE_RETIRED(31, 1, MON_SINGLE_PAXOS, NAUTILUS, PACIFIC)
+// available for R
 DEFINE_CEPH_FEATURE_RETIRED(32, 1, OSD_SNAPMAPPER, JEWEL, LUMINOUS)
-
+DEFINE_CEPH_FEATURE(32, 3, STRETCH_MODE)
 DEFINE_CEPH_FEATURE_RETIRED(33, 1, MON_SCRUB, JEWEL, LUMINOUS)
-
+DEFINE_CEPH_FEATURE(33, 3, SERVER_QUINCY)
 DEFINE_CEPH_FEATURE_RETIRED(34, 1, OSD_PACKED_RECOVERY, JEWEL, LUMINOUS)
-
+// available
 DEFINE_CEPH_FEATURE(35, 1, OSD_CACHEPOOL)    // 3.14
 DEFINE_CEPH_FEATURE(36, 1, CRUSH_V2)         // 3.14
 DEFINE_CEPH_FEATURE(37, 1, EXPORT_PEER)      // 3.14
 DEFINE_CEPH_FEATURE_RETIRED(38, 1, OSD_ERASURE_CODES, MIMIC, OCTOPUS)
+// available
 DEFINE_CEPH_FEATURE(39, 1, OSDMAP_ENC)       // 3.15
 DEFINE_CEPH_FEATURE(40, 1, MDS_INLINE_DATA)  // 3.19
 DEFINE_CEPH_FEATURE(41, 1, CRUSH_TUNABLES3)  // 3.15
@@ -151,25 +146,30 @@ DEFINE_CEPH_FEATURE(41, 1, OSD_PRIMARY_AFFINITY) // overlap
 DEFINE_CEPH_FEATURE(42, 1, MSGR_KEEPALIVE2)  // 4.3 (for consistency)
 DEFINE_CEPH_FEATURE(43, 1, OSD_POOLRESEND)   // 4.13
 DEFINE_CEPH_FEATURE_RETIRED(44, 1, ERASURE_CODE_PLUGINS_V2, MIMIC, OCTOPUS)
+// available
 DEFINE_CEPH_FEATURE_RETIRED(45, 1, OSD_SET_ALLOC_HINT, JEWEL, LUMINOUS)
-
+// available
 DEFINE_CEPH_FEATURE(46, 1, OSD_FADVISE_FLAGS)
 DEFINE_CEPH_FEATURE_RETIRED(46, 1, OSD_REPOP, JEWEL, LUMINOUS) // overlap
 DEFINE_CEPH_FEATURE_RETIRED(46, 1, OSD_OBJECT_DIGEST, JEWEL, LUMINOUS) // overlap
 DEFINE_CEPH_FEATURE_RETIRED(46, 1, OSD_TRANSACTION_MAY_LAYOUT, JEWEL, LUMINOUS) // overlap
-
 DEFINE_CEPH_FEATURE(47, 1, MDS_QUOTA)        // 4.17
 DEFINE_CEPH_FEATURE(48, 1, CRUSH_V4)         // 4.1
 DEFINE_CEPH_FEATURE_RETIRED(49, 1, OSD_MIN_SIZE_RECOVERY, JEWEL, LUMINOUS)
 DEFINE_CEPH_FEATURE_RETIRED(49, 1, OSD_PROXY_FEATURES, JEWEL, LUMINOUS) // overlap
-
+// available
 DEFINE_CEPH_FEATURE_RETIRED(50, 1, MON_METADATA, MIMIC, OCTOPUS)
+// available
 DEFINE_CEPH_FEATURE_RETIRED(51, 1, OSD_BITWISE_HOBJ_SORT, MIMIC, OCTOPUS)
+// available
 DEFINE_CEPH_FEATURE_RETIRED(52, 1, OSD_PROXY_WRITE_FEATURES, MIMIC, OCTOPUS)
+// available
 DEFINE_CEPH_FEATURE_RETIRED(53, 1, ERASURE_CODE_PLUGINS_V3, MIMIC, OCTOPUS)
+// available
 DEFINE_CEPH_FEATURE_RETIRED(54, 1, OSD_HITSET_GMT, MIMIC, OCTOPUS)
+// available
 DEFINE_CEPH_FEATURE_RETIRED(55, 1, HAMMER_0_94_4, MIMIC, OCTOPUS)
-
+// available
 DEFINE_CEPH_FEATURE(56, 1, NEW_OSDOP_ENCODING) // 4.13 (for pg_pool_t >= v25)
 DEFINE_CEPH_FEATURE(57, 1, MON_STATEFUL_SUB) // 4.13
 DEFINE_CEPH_FEATURE_RETIRED(57, 1, MON_ROUTE_OSDMAP, MIMIC, OCTOPUS) // overlap
@@ -214,7 +214,6 @@ DEFINE_CEPH_FEATURE_DEPRECATED(63, 1, RESERVED_BROKEN, LUMINOUS) // client-facin
 	 CEPH_FEATURE_OSDHASHPSPOOL |       \
 	 CEPH_FEATURE_NEW_OSDOP_ENCODING |        \
          CEPH_FEATURE_NEW_OSDOPREPLY_ENCODING | \
-	 DEPRECATED_CEPH_FEATURE_MON_SINGLE_PAXOS |    \
 	 CEPH_FEATURE_OSD_CACHEPOOL |	    \
 	 CEPH_FEATURE_CRUSH_V2 |	    \
 	 CEPH_FEATURE_EXPORT_PEER |	    \
@@ -245,7 +244,11 @@ DEFINE_CEPH_FEATURE_DEPRECATED(63, 1, RESERVED_BROKEN, LUMINOUS) // client-facin
 	 CEPH_FEATURE_CEPHX_V2 | \
 	 CEPH_FEATURE_OSD_PGLOG_HARDLIMIT | \
 	 CEPH_FEATUREMASK_SERVER_OCTOPUS | \
+	 CEPH_FEATUREMASK_STRETCH_MODE | \
 	 CEPH_FEATUREMASK_OSD_REPOP_MLCOD | \
+	 CEPH_FEATUREMASK_SERVER_PACIFIC | \
+	 CEPH_FEATURE_OSD_FIXED_COLLECTION_LIST | \
+	 CEPH_FEATUREMASK_SERVER_QUINCY | \
 	 0ULL)
 
 #define CEPH_FEATURES_SUPPORTED_DEFAULT  CEPH_FEATURES_ALL
