@@ -5,6 +5,7 @@
 #define CEPH_LIBRBD_CACHE_TYPES_H
 
 #include <list>
+#include <string>
 
 class Context;
 
@@ -13,10 +14,14 @@ namespace cache {
 
 enum ImageCacheType {
   IMAGE_CACHE_TYPE_RWL = 1,
+  IMAGE_CACHE_TYPE_SSD,
+  IMAGE_CACHE_TYPE_UNKNOWN
 };
 
 typedef std::list<Context *> Contexts;
 
+const std::string IMAGE_CACHE_STATE = ".librbd/image_cache_state";
+
 } // namespace cache
 } // namespace librbd
 
