@@ -7,20 +7,21 @@
 #include "librbd/Journal.h"
 #include "librbd/Types.h"
 #include "librbd/Utils.h"
-#include "librbd/cache/ImageCache.h"
+#include "librbd/asio/ContextWQ.h"
 #include "librbd/io/AioCompletion.h"
 #include "librbd/io/AsyncOperation.h"
 #include "librbd/io/ObjectDispatchInterface.h"
 #include "librbd/io/ObjectDispatchSpec.h"
-#include "librbd/io/ObjectDispatcher.h"
+#include "librbd/io/ObjectDispatcherInterface.h"
 #include "librbd/io/Utils.h"
 #include "librbd/journal/Types.h"
 #include "include/rados/librados.hpp"
+#include "common/errno.h"
 #include "common/perf_counters.h"
-#include "common/WorkQueue.h"
 #include "osdc/Striper.h"
 #include <algorithm>
 #include <functional>
+#include <map>
 
 #define dout_subsys ceph_subsys_rbd
 #undef dout_prefix
@@ -34,31 +35,101 @@ using librbd::util::get_image_ctx;
 
 namespace {
 
+template <typename I>
+struct C_AssembleSnapshotDeltas : public C_AioRequest {
+  I* image_ctx;
+  SnapshotDelta* snapshot_delta;
+
+  ceph::mutex lock = ceph::make_mutex(
+    "librbd::io::C_AssembleSnapshotDeltas::lock", false);
+  std::map<uint64_t, SnapshotDelta> object_snapshot_delta;
+
+  C_AssembleSnapshotDeltas(I* image_ctx, AioCompletion* aio_comp,
+                           SnapshotDelta* snapshot_delta)
+    : C_AioRequest(aio_comp),
+      image_ctx(image_ctx), snapshot_delta(snapshot_delta) {
+  }
+
+  SnapshotDelta* get_snapshot_delta(uint64_t object_no) {
+    std::unique_lock locker{lock};
+    return &object_snapshot_delta[object_no];
+  }
+
+  void finish(int r) override {
+    auto cct = image_ctx->cct;
+
+    if (r < 0) {
+      lderr(cct) << "C_AssembleSnapshotDeltas: list snaps failed: "
+                 << cpp_strerror(r) << dendl;
+      C_AioRequest::finish(r);
+      return;
+    }
+
+    std::unique_lock locker{lock};
+    *snapshot_delta = {};
+    for (auto& [object_no, object_snapshot_delta] : object_snapshot_delta) {
+      SnapshotDelta image_snapshot_delta;
+      object_to_image_intervals(object_no, object_snapshot_delta,
+                                &image_snapshot_delta, snapshot_delta);
+
+      ldout(cct, 20) << "object_no=" << object_no << ", "
+                     << "object_snapshot_delta="
+                     << object_snapshot_delta << ", "
+                     << "image_snapshot_delta=" << image_snapshot_delta
+                     << dendl;
+    }
+
+    ldout(cct, 20) << "snapshot_delta=" << *snapshot_delta << dendl;
+    C_AioRequest::finish(0);
+  }
+
+  void object_to_image_intervals(
+      uint64_t object_no, const SnapshotDelta& object_snapshot_delta,
+      SnapshotDelta* image_snapshot_delta,
+      SnapshotDelta* assembled_image_snapshot_delta) {
+    for (auto& [key, object_extents] : object_snapshot_delta) {
+      for (auto& object_extent : object_extents) {
+        Extents image_extents;
+        io::util::extent_to_file(image_ctx, object_no, object_extent.get_off(),
+                                 object_extent.get_len(), image_extents);
+
+        auto& intervals = (*image_snapshot_delta)[key];
+        auto& assembled_intervals = (*assembled_image_snapshot_delta)[key];
+        for (auto [image_offset, image_length] : image_extents) {
+          SparseExtent sparse_extent{object_extent.get_val().state,
+                                     image_length};
+          intervals.insert(image_offset, image_length, sparse_extent);
+          assembled_intervals.insert(image_offset, image_length,
+                                     sparse_extent);
+        }
+      }
+    }
+  }
+};
+
 template <typename I>
 struct C_RBD_Readahead : public Context {
   I *ictx;
   uint64_t object_no;
-  uint64_t offset;
-  uint64_t length;
-
-  bufferlist read_data;
-  io::ExtentMap extent_map;
+  io::ReadExtents extents;
 
   C_RBD_Readahead(I *ictx, uint64_t object_no, uint64_t offset, uint64_t length)
-    : ictx(ictx), object_no(object_no), offset(offset), length(length) {
+    : ictx(ictx), object_no(object_no), extents({{offset, length}}) {
     ictx->readahead.inc_pending();
   }
 
   void finish(int r) override {
+    ceph_assert(extents.size() == 1);
+    auto& extent = extents.front();
     ldout(ictx->cct, 20) << "C_RBD_Readahead on "
                          << data_object_name(ictx, object_no) << ": "
-                         << offset << "~" << length << dendl;
+                         << extent.offset << "~" << extent.length << dendl;
     ictx->readahead.dec_pending();
   }
 };
 
 template <typename I>
-void readahead(I *ictx, const Extents& image_extents) {
+void readahead(I *ictx, const Extents& image_extents, IOContext io_context) {
   uint64_t total_bytes = 0;
   for (auto& image_extent : image_extents) {
     total_bytes += image_extent.second;
@@ -74,8 +145,7 @@ void readahead(I *ictx, const Extents& image_extents) {
     return;
   }
 
-  uint64_t image_size = ictx->get_image_size(ictx->snap_id);
-  auto snap_id = ictx->snap_id;
+  uint64_t image_size = ictx->get_effective_image_size(ictx->snap_id);
   ictx->image_lock.unlock_shared();
 
   auto readahead_extent = ictx->readahead.update(image_extents, image_size);
@@ -86,9 +156,8 @@ void readahead(I *ictx, const Extents& image_extents) {
     ldout(ictx->cct, 20) << "(readahead logical) " << readahead_offset << "~"
                          << readahead_length << dendl;
     LightweightObjectExtents readahead_object_extents;
-    Striper::file_to_extents(ictx->cct, &ictx->layout,
-                             readahead_offset, readahead_length, 0, 0,
-                             &readahead_object_extents);
+    io::util::file_to_extents(ictx, readahead_offset, readahead_length, 0,
+                              &readahead_object_extents);
     for (auto& object_extent : readahead_object_extents) {
       ldout(ictx->cct, 20) << "(readahead) "
                            << data_object_name(ictx,
@@ -101,8 +170,7 @@ void readahead(I *ictx, const Extents& image_extents) {
                                              object_extent.length);
       auto req = io::ObjectDispatchSpec::create_read(
         ictx, io::OBJECT_DISPATCH_LAYER_NONE, object_extent.object_no,
-        object_extent.offset, object_extent.length, snap_id, 0, {},
-        &req_comp->read_data, &req_comp->extent_map, req_comp);
+        &req_comp->extents, io_context, 0, 0, {}, nullptr, req_comp);
       req->send();
     }
 
@@ -160,20 +228,22 @@ bool should_update_timestamp(const utime_t& now, const utime_t& timestamp,
 template <typename I>
 void ImageRequest<I>::aio_read(I *ictx, AioCompletion *c,
                                Extents &&image_extents,
-                               ReadResult &&read_result, int op_flags,
+                               ReadResult &&read_result, IOContext io_context,
+                               int op_flags, int read_flags,
 			       const ZTracer::Trace &parent_trace) {
   ImageReadRequest<I> req(*ictx, c, std::move(image_extents),
-                          std::move(read_result), op_flags, parent_trace);
+                          std::move(read_result), io_context, op_flags,
+                          read_flags, parent_trace);
   req.send();
 }
 
 template <typename I>
 void ImageRequest<I>::aio_write(I *ictx, AioCompletion *c,
                                 Extents &&image_extents, bufferlist &&bl,
-                                int op_flags,
+                                IOContext io_context, int op_flags,
 				const ZTracer::Trace &parent_trace) {
   ImageWriteRequest<I> req(*ictx, c, std::move(image_extents), std::move(bl),
-                           op_flags, parent_trace);
+                           io_context, op_flags, parent_trace);
   req.send();
 }
 
@@ -181,9 +251,11 @@ template <typename I>
 void ImageRequest<I>::aio_discard(I *ictx, AioCompletion *c,
                                   Extents &&image_extents,
                                   uint32_t discard_granularity_bytes,
-				  const ZTracer::Trace &parent_trace) {
+				  IOContext io_context,
+                                  const ZTracer::Trace &parent_trace) {
   ImageDiscardRequest<I> req(*ictx, c, std::move(image_extents),
-                             discard_granularity_bytes, parent_trace);
+                             discard_granularity_bytes, io_context,
+                             parent_trace);
   req.send();
 }
 
@@ -198,10 +270,12 @@ void ImageRequest<I>::aio_flush(I *ictx, AioCompletion *c,
 template <typename I>
 void ImageRequest<I>::aio_writesame(I *ictx, AioCompletion *c,
                                     Extents &&image_extents,
-                                    bufferlist &&bl, int op_flags,
+                                    bufferlist &&bl, IOContext io_context,
+                                    int op_flags,
 				    const ZTracer::Trace &parent_trace) {
   ImageWriteSameRequest<I> req(*ictx, c, std::move(image_extents),
-                               std::move(bl), op_flags, parent_trace);
+                               std::move(bl), io_context, op_flags,
+                               parent_trace);
   req.send();
 }
 
@@ -211,11 +285,12 @@ void ImageRequest<I>::aio_compare_and_write(I *ictx, AioCompletion *c,
                                             bufferlist &&cmp_bl,
                                             bufferlist &&bl,
                                             uint64_t *mismatch_offset,
-                                            int op_flags,
+                                            IOContext io_context, int op_flags,
                                             const ZTracer::Trace &parent_trace) {
   ImageCompareAndWriteRequest<I> req(*ictx, c, std::move(image_extents),
                                      std::move(cmp_bl), std::move(bl),
-                                     mismatch_offset, op_flags, parent_trace);
+                                     mismatch_offset, io_context, op_flags,
+                                     parent_trace);
   req.send();
 }
 
@@ -230,37 +305,8 @@ void ImageRequest<I>::send() {
   ldout(cct, 20) << get_request_type() << ": ictx=" << &image_ctx << ", "
                  << "completion=" << aio_comp << dendl;
 
-  int r = clip_request();
-  if (r < 0) {
-    m_aio_comp->fail(r);
-    return;
-  }
-
-  if (finish_request_early()) {
-    return;
-  }
-
-  if (m_bypass_image_cache || m_image_ctx.image_cache == nullptr) {
-    update_timestamp();
-    send_request();
-  } else {
-    send_image_cache_request();
-  }
-}
-
-template <typename I>
-int ImageRequest<I>::clip_request() {
-  std::shared_lock image_locker{m_image_ctx.image_lock};
-  for (auto &image_extent : m_image_extents) {
-    auto clip_len = image_extent.second;
-    int r = clip_io(get_image_ctx(&m_image_ctx), image_extent.first, &clip_len);
-    if (r < 0) {
-      return r;
-    }
-
-    image_extent.second = clip_len;
-  }
-  return 0;
+  update_timestamp();
+  send_request();
 }
 
 template <typename I>
@@ -318,30 +364,16 @@ void ImageRequest<I>::update_timestamp() {
 template <typename I>
 ImageReadRequest<I>::ImageReadRequest(I &image_ctx, AioCompletion *aio_comp,
                                       Extents &&image_extents,
-                                      ReadResult &&read_result, int op_flags,
-				      const ZTracer::Trace &parent_trace)
-  : ImageRequest<I>(image_ctx, aio_comp, std::move(image_extents), "read",
-		    parent_trace),
-    m_op_flags(op_flags) {
+                                      ReadResult &&read_result,
+                                      IOContext io_context, int op_flags,
+				      int read_flags,
+                                      const ZTracer::Trace &parent_trace)
+  : ImageRequest<I>(image_ctx, aio_comp, std::move(image_extents),
+                    io_context, "read", parent_trace),
+    m_op_flags(op_flags), m_read_flags(read_flags) {
   aio_comp->read_result = std::move(read_result);
 }
 
-template <typename I>
-int ImageReadRequest<I>::clip_request() {
-  int r = ImageRequest<I>::clip_request();
-  if (r < 0) {
-    return r;
-  }
-
-  uint64_t buffer_length = 0;
-  auto &image_extents = this->m_image_extents;
-  for (auto &image_extent : image_extents) {
-    buffer_length += image_extent.second;
-  }
-  this->m_aio_comp->read_result.set_clip_length(buffer_length);
-  return 0;
-}
-
 template <typename I>
 void ImageReadRequest<I>::send_request() {
   I &image_ctx = this->m_image_ctx;
@@ -350,32 +382,26 @@ void ImageReadRequest<I>::send_request() {
   auto &image_extents = this->m_image_extents;
   if (image_ctx.cache && image_ctx.readahead_max_bytes > 0 &&
       !(m_op_flags & LIBRADOS_OP_FLAG_FADVISE_RANDOM)) {
-    readahead(get_image_ctx(&image_ctx), image_extents);
-  }
-
-  librados::snap_t snap_id;
-  uint64_t buffer_ofs = 0;
-  {
-    // prevent image size from changing between computing clip and recording
-    // pending async operation
-    std::shared_lock image_locker{image_ctx.image_lock};
-    snap_id = image_ctx.snap_id;
+    readahead(get_image_ctx(&image_ctx), image_extents, this->m_io_context);
   }
 
   // map image extents to object extents
   LightweightObjectExtents object_extents;
+  uint64_t buffer_ofs = 0;
   for (auto &extent : image_extents) {
     if (extent.second == 0) {
       continue;
     }
 
-    Striper::file_to_extents(cct, &image_ctx.layout, extent.first,
-                             extent.second, 0, buffer_ofs, &object_extents);
+    util::file_to_extents(&image_ctx, extent.first, extent.second, buffer_ofs,
+                          &object_extents);
     buffer_ofs += extent.second;
   }
 
-  // issue the requests
   AioCompletion *aio_comp = this->m_aio_comp;
+  aio_comp->read_result.set_image_extents(image_extents);
+
+  // issue the requests
   aio_comp->set_request_count(object_extents.size());
   for (auto &oe : object_extents) {
     ldout(cct, 20) << data_object_name(&image_ctx, oe.object_no) << " "
@@ -383,11 +409,11 @@ void ImageReadRequest<I>::send_request() {
                    << oe.buffer_extents << dendl;
 
     auto req_comp = new io::ReadResult::C_ObjectReadRequest(
-      aio_comp, oe.offset, oe.length, std::move(oe.buffer_extents));
+      aio_comp, {{oe.offset, oe.length, std::move(oe.buffer_extents)}});
     auto req = ObjectDispatchSpec::create_read(
-      &image_ctx, OBJECT_DISPATCH_LAYER_NONE, oe.object_no, oe.offset,
-      oe.length, snap_id, m_op_flags, this->m_trace, &req_comp->bl,
-      &req_comp->extent_map, req_comp);
+      &image_ctx, OBJECT_DISPATCH_LAYER_NONE, oe.object_no,
+      &req_comp->extents, this->m_io_context, m_op_flags, m_read_flags,
+      this->m_trace, nullptr, req_comp);
     req->send();
   }
 
@@ -395,57 +421,17 @@ void ImageReadRequest<I>::send_request() {
   image_ctx.perfcounter->inc(l_librbd_rd_bytes, buffer_ofs);
 }
 
-template <typename I>
-void ImageReadRequest<I>::send_image_cache_request() {
-  I &image_ctx = this->m_image_ctx;
-  ceph_assert(image_ctx.image_cache != nullptr);
-
-  AioCompletion *aio_comp = this->m_aio_comp;
-  aio_comp->set_request_count(1);
-
-  auto *req_comp = new io::ReadResult::C_ImageReadRequest(
-    aio_comp, this->m_image_extents);
-  image_ctx.image_cache->aio_read(std::move(this->m_image_extents),
-                                  &req_comp->bl, m_op_flags,
-                                  req_comp);
-}
-
-template <typename I>
-bool AbstractImageWriteRequest<I>::finish_request_early() {
-  AioCompletion *aio_comp = this->m_aio_comp;
-  {
-    std::shared_lock image_locker{this->m_image_ctx.image_lock};
-    if (this->m_image_ctx.snap_id != CEPH_NOSNAP || this->m_image_ctx.read_only) {
-      aio_comp->fail(-EROFS);
-      return true;
-    }
-  }
-  uint64_t total_bytes = 0;
-  for (auto& image_extent : this->m_image_extents) {
-    total_bytes += image_extent.second;
-  }
-  if (total_bytes == 0) {
-    aio_comp->set_request_count(0);
-    return true;
-  }
-  return false;
-}
-
 template <typename I>
 void AbstractImageWriteRequest<I>::send_request() {
   I &image_ctx = this->m_image_ctx;
-  CephContext *cct = image_ctx.cct;
 
   bool journaling = false;
 
   AioCompletion *aio_comp = this->m_aio_comp;
-  ::SnapContext snapc;
   {
     // prevent image size from changing between computing clip and recording
     // pending async operation
     std::shared_lock image_locker{image_ctx.image_lock};
-
-    snapc = image_ctx.snapc;
     journaling = (image_ctx.journal != nullptr &&
                   image_ctx.journal->is_journal_appending());
   }
@@ -458,8 +444,8 @@ void AbstractImageWriteRequest<I>::send_request() {
     }
 
     // map to object extents
-    Striper::file_to_extents(cct, &image_ctx.layout, extent.first,
-                             extent.second, 0, clip_len, &object_extents);
+    io::util::file_to_extents(&image_ctx, extent.first, extent.second, clip_len,
+                              &object_extents);
     clip_len += extent.second;
   }
 
@@ -478,7 +464,7 @@ void AbstractImageWriteRequest<I>::send_request() {
       journal_tid = append_journal_event(m_synchronous);
     }
 
-    send_object_requests(object_extents, snapc, journal_tid);
+    send_object_requests(object_extents, this->m_io_context, journal_tid);
   }
 
   update_stats(clip_len);
@@ -486,7 +472,7 @@ void AbstractImageWriteRequest<I>::send_request() {
 
 template <typename I>
 void AbstractImageWriteRequest<I>::send_object_requests(
-    const LightweightObjectExtents &object_extents, const ::SnapContext &snapc,
+    const LightweightObjectExtents &object_extents, IOContext io_context,
     uint64_t journal_tid) {
   I &image_ctx = this->m_image_ctx;
   CephContext *cct = image_ctx.cct;
@@ -498,8 +484,8 @@ void AbstractImageWriteRequest<I>::send_object_requests(
                    << oe.offset << "~" << oe.length << " from "
                    << oe.buffer_extents << dendl;
     C_AioRequest *req_comp = new C_AioRequest(aio_comp);
-    auto request = create_object_request(oe, snapc, journal_tid, single_extent,
-                                         req_comp);
+    auto request = create_object_request(oe, io_context, journal_tid,
+                                         single_extent, req_comp);
     request->send();
   }
 }
@@ -534,21 +520,9 @@ uint64_t ImageWriteRequest<I>::append_journal_event(bool synchronous) {
   return tid;
 }
 
-template <typename I>
-void ImageWriteRequest<I>::send_image_cache_request() {
-  I &image_ctx = this->m_image_ctx;
-  ceph_assert(image_ctx.image_cache != nullptr);
-
-  AioCompletion *aio_comp = this->m_aio_comp;
-  aio_comp->set_request_count(1);
-  C_AioRequest *req_comp = new C_AioRequest(aio_comp);
-  image_ctx.image_cache->aio_write(std::move(this->m_image_extents),
-                                   std::move(m_bl), m_op_flags, req_comp);
-}
-
 template <typename I>
 ObjectDispatchSpec *ImageWriteRequest<I>::create_object_request(
-    const LightweightObjectExtent &object_extent, const ::SnapContext &snapc,
+    const LightweightObjectExtent &object_extent, IOContext io_context,
     uint64_t journal_tid, bool single_extent, Context *on_finish) {
   I &image_ctx = this->m_image_ctx;
 
@@ -563,8 +537,8 @@ ObjectDispatchSpec *ImageWriteRequest<I>::create_object_request(
 
   auto req = ObjectDispatchSpec::create_write(
     &image_ctx, OBJECT_DISPATCH_LAYER_NONE, object_extent.object_no,
-    object_extent.offset, std::move(bl), snapc, m_op_flags, journal_tid,
-    this->m_trace, on_finish);
+    object_extent.offset, std::move(bl), io_context, m_op_flags, 0,
+    std::nullopt, journal_tid, this->m_trace, on_finish);
   return req;
 }
 
@@ -594,29 +568,14 @@ uint64_t ImageDiscardRequest<I>::append_journal_event(bool synchronous) {
   return tid;
 }
 
-template <typename I>
-void ImageDiscardRequest<I>::send_image_cache_request() {
-  I &image_ctx = this->m_image_ctx;
-  ceph_assert(image_ctx.image_cache != nullptr);
-
-  AioCompletion *aio_comp = this->m_aio_comp;
-  aio_comp->set_request_count(this->m_image_extents.size());
-  for (auto &extent : this->m_image_extents) {
-    C_AioRequest *req_comp = new C_AioRequest(aio_comp);
-    image_ctx.image_cache->aio_discard(extent.first, extent.second,
-                                       this->m_discard_granularity_bytes,
-                                       req_comp);
-  }
-}
-
 template <typename I>
 ObjectDispatchSpec *ImageDiscardRequest<I>::create_object_request(
-    const LightweightObjectExtent &object_extent, const ::SnapContext &snapc,
+    const LightweightObjectExtent &object_extent, IOContext io_context,
     uint64_t journal_tid, bool single_extent, Context *on_finish) {
   I &image_ctx = this->m_image_ctx;
   auto req = ObjectDispatchSpec::create_discard(
     &image_ctx, OBJECT_DISPATCH_LAYER_NONE, object_extent.object_no,
-    object_extent.offset, object_extent.length, snapc,
+    object_extent.offset, object_extent.length, io_context,
     OBJECT_DISCARD_FLAG_DISABLE_CLONE_REMOVE, journal_tid, this->m_trace,
     on_finish);
   return req;
@@ -718,7 +677,11 @@ void ImageFlushRequest<I>::send_request() {
     });
 
   // ensure all in-flight IOs are settled if non-user flush request
-  aio_comp->async_op.flush(ctx);
+  if (m_flush_source == FLUSH_SOURCE_WRITEBACK) {
+    ctx->complete(0);
+  } else {
+    aio_comp->async_op.flush(ctx);
+  }
 
   // might be flushing during image shutdown
   if (image_ctx.perfcounter != nullptr) {
@@ -726,17 +689,6 @@ void ImageFlushRequest<I>::send_request() {
   }
 }
 
-template <typename I>
-void ImageFlushRequest<I>::send_image_cache_request() {
-  I &image_ctx = this->m_image_ctx;
-  ceph_assert(image_ctx.image_cache != nullptr);
-
-  AioCompletion *aio_comp = this->m_aio_comp;
-  aio_comp->set_request_count(1);
-  C_AioRequest *req_comp = new C_AioRequest(aio_comp);
-  image_ctx.image_cache->aio_flush(req_comp);
-}
-
 template <typename I>
 uint64_t ImageWriteSameRequest<I>::append_journal_event(bool synchronous) {
   I &image_ctx = this->m_image_ctx;
@@ -755,24 +707,9 @@ uint64_t ImageWriteSameRequest<I>::append_journal_event(bool synchronous) {
   return tid;
 }
 
-template <typename I>
-void ImageWriteSameRequest<I>::send_image_cache_request() {
-  I &image_ctx = this->m_image_ctx;
-  ceph_assert(image_ctx.image_cache != nullptr);
-
-  AioCompletion *aio_comp = this->m_aio_comp;
-  aio_comp->set_request_count(this->m_image_extents.size());
-  for (auto &extent : this->m_image_extents) {
-    C_AioRequest *req_comp = new C_AioRequest(aio_comp);
-    image_ctx.image_cache->aio_writesame(extent.first, extent.second,
-                                         std::move(m_data_bl), m_op_flags,
-                                         req_comp);
-  }
-}
-
 template <typename I>
 ObjectDispatchSpec *ImageWriteSameRequest<I>::create_object_request(
-    const LightweightObjectExtent &object_extent, const ::SnapContext &snapc,
+    const LightweightObjectExtent &object_extent, IOContext io_context,
     uint64_t journal_tid, bool single_extent, Context *on_finish) {
   I &image_ctx = this->m_image_ctx;
 
@@ -785,14 +722,14 @@ ObjectDispatchSpec *ImageWriteSameRequest<I>::create_object_request(
     req = ObjectDispatchSpec::create_write_same(
       &image_ctx, OBJECT_DISPATCH_LAYER_NONE, object_extent.object_no,
       object_extent.offset, object_extent.length, std::move(buffer_extents),
-      std::move(bl), snapc, m_op_flags, journal_tid,
+      std::move(bl), io_context, m_op_flags, journal_tid,
       this->m_trace, on_finish);
     return req;
   }
   req = ObjectDispatchSpec::create_write(
     &image_ctx, OBJECT_DISPATCH_LAYER_NONE, object_extent.object_no,
-    object_extent.offset, std::move(bl), snapc, m_op_flags, journal_tid,
-    this->m_trace, on_finish);
+    object_extent.offset, std::move(bl), io_context, m_op_flags, 0,
+    std::nullopt, journal_tid, this->m_trace, on_finish);
   return req;
 }
 
@@ -832,22 +769,9 @@ void ImageCompareAndWriteRequest<I>::assemble_extent(
   }
 }
 
-template <typename I>
-void ImageCompareAndWriteRequest<I>::send_image_cache_request() {
-  I &image_ctx = this->m_image_ctx;
-  ceph_assert(image_ctx.image_cache != nullptr);
-
-  AioCompletion *aio_comp = this->m_aio_comp;
-  aio_comp->set_request_count(1);
-  C_AioRequest *req_comp = new C_AioRequest(aio_comp);
-  image_ctx.image_cache->aio_compare_and_write(
-    std::move(this->m_image_extents), std::move(m_cmp_bl), std::move(m_bl),
-    m_mismatch_offset, m_op_flags, req_comp);
-}
-
 template <typename I>
 ObjectDispatchSpec *ImageCompareAndWriteRequest<I>::create_object_request(
-    const LightweightObjectExtent &object_extent, const ::SnapContext &snapc,
+    const LightweightObjectExtent &object_extent, IOContext io_context,
     uint64_t journal_tid, bool single_extent, Context *on_finish) {
   I &image_ctx = this->m_image_ctx;
 
@@ -857,7 +781,7 @@ ObjectDispatchSpec *ImageCompareAndWriteRequest<I>::create_object_request(
   assemble_extent(object_extent, &bl);
   auto req = ObjectDispatchSpec::create_compare_and_write(
     &image_ctx, OBJECT_DISPATCH_LAYER_NONE, object_extent.object_no,
-    object_extent.offset, std::move(m_cmp_bl), std::move(bl), snapc,
+    object_extent.offset, std::move(m_cmp_bl), std::move(bl), io_context,
     m_mismatch_offset, m_op_flags, journal_tid, this->m_trace, on_finish);
   return req;
 }
@@ -886,6 +810,63 @@ int ImageCompareAndWriteRequest<I>::prune_object_extents(
   return 0;
 }
 
+template <typename I>
+ImageListSnapsRequest<I>::ImageListSnapsRequest(
+    I& image_ctx, AioCompletion* aio_comp, Extents&& image_extents,
+    SnapIds&& snap_ids, int list_snaps_flags, SnapshotDelta* snapshot_delta,
+    const ZTracer::Trace& parent_trace)
+  : ImageRequest<I>(image_ctx, aio_comp, std::move(image_extents),
+                    image_ctx.get_data_io_context(), "list-snaps",
+                    parent_trace),
+    m_snap_ids(std::move(snap_ids)), m_list_snaps_flags(list_snaps_flags),
+    m_snapshot_delta(snapshot_delta) {
+}
+
+template <typename I>
+void ImageListSnapsRequest<I>::send_request() {
+  I &image_ctx = this->m_image_ctx;
+  CephContext *cct = image_ctx.cct;
+
+  // map image extents to object extents
+  auto &image_extents = this->m_image_extents;
+  std::map<uint64_t, Extents> object_number_extents;
+  for (auto& image_extent : image_extents) {
+    if (image_extent.second == 0) {
+      continue;
+    }
+
+    striper::LightweightObjectExtents object_extents;
+    io::util::file_to_extents(&image_ctx, image_extent.first,
+                              image_extent.second, 0, &object_extents);
+    for (auto& object_extent : object_extents) {
+      object_number_extents[object_extent.object_no].emplace_back(
+        object_extent.offset, object_extent.length);
+    }
+  }
+
+  // reassemble the deltas back into image-extents when complete
+  auto aio_comp = this->m_aio_comp;
+  aio_comp->set_request_count(1);
+  auto assemble_ctx = new C_AssembleSnapshotDeltas<I>(
+    &image_ctx, aio_comp, m_snapshot_delta);
+  auto sub_aio_comp = AioCompletion::create_and_start<
+    Context, &Context::complete>(assemble_ctx, get_image_ctx(&image_ctx),
+                                 AIO_TYPE_GENERIC);
+
+  // issue the requests
+  sub_aio_comp->set_request_count(object_number_extents.size());
+  for (auto& oe : object_number_extents) {
+    ldout(cct, 20) << data_object_name(&image_ctx, oe.first) << " "
+                   << oe.second << dendl;
+    auto ctx = new C_AioRequest(sub_aio_comp);
+    auto req = ObjectDispatchSpec::create_list_snaps(
+      &image_ctx, OBJECT_DISPATCH_LAYER_NONE, oe.first, std::move(oe.second),
+      SnapIds{m_snap_ids}, m_list_snaps_flags, this->m_trace,
+      assemble_ctx->get_snapshot_delta(oe.first), ctx);
+    req->send();
+  }
+}
+
 } // namespace io
 } // namespace librbd
 
@@ -897,3 +878,4 @@ template class librbd::io::ImageDiscardRequest<librbd::ImageCtx>;
 template class librbd::io::ImageFlushRequest<librbd::ImageCtx>;
 template class librbd::io::ImageWriteSameRequest<librbd::ImageCtx>;
 template class librbd::io::ImageCompareAndWriteRequest<librbd::ImageCtx>;
+template class librbd::io::ImageListSnapsRequest<librbd::ImageCtx>;
