@@ -8,9 +8,10 @@ namespace librbd {
 namespace object_map {
 
 enum DiffState {
-  DIFF_STATE_NONE    = 0,
-  DIFF_STATE_UPDATED = 1,
-  DIFF_STATE_HOLE    = 2
+  DIFF_STATE_HOLE         = 0, /* unchanged hole */
+  DIFF_STATE_DATA         = 1, /* unchanged data */
+  DIFF_STATE_HOLE_UPDATED = 2, /* new hole */
+  DIFF_STATE_DATA_UPDATED = 3  /* new data */
 };
 
 } // namespace object_map
