@@ -6,24 +6,22 @@
 #include "common/errno.h"
 #include "common/safe_io.h"
 #include "common/Graylog.h"
+#include "common/Journald.h"
 #include "common/valgrind.h"
 
 #include "include/ceph_assert.h"
 #include "include/compat.h"
 #include "include/on_exit.h"
+#include "include/uuid.h"
 
 #include "Entry.h"
 #include "LogClock.h"
 #include "SubsystemMap.h"
 
-#include <boost/container/vector.hpp>
-
 #include <errno.h>
 #include <fcntl.h>
-#include <limits.h>
 #include <syslog.h>
 
-#include <algorithm>
 #include <iostream>
 #include <set>
 
@@ -50,7 +48,6 @@ Log::Log(const SubsystemMap *s)
     m_recent(DEFAULT_MAX_RECENT)
 {
   m_log_buf.reserve(MAX_LOG_BUF);
-  _configure_stderr();
 }
 
 Log::~Log()
@@ -60,44 +57,8 @@ Log::~Log()
   }
 
   ceph_assert(!is_started());
-  if (m_fd >= 0) {
+  if (m_fd >= 0)
     VOID_TEMP_FAILURE_RETRY(::close(m_fd));
-    m_fd = -1;
-  }
-}
-
-void Log::_configure_stderr()
-{
-#ifndef _WIN32
-  struct stat info;
-  if (int rc = fstat(m_fd_stderr, &info); rc == -1) {
-    std::cerr << "failed to stat stderr: " << cpp_strerror(errno) << std::endl;
-    return;
-  }
-
-  if (S_ISFIFO(info.st_mode)) {
-    /* Set O_NONBLOCK on FIFO stderr file. We want to ensure atomic debug log
-     * writes so they do not get partially read by e.g. buggy container
-     * runtimes. See also IEEE Std 1003.1-2017 and Log::_log_stderr below.
-     *
-     * This isn't required on Windows.
-     */
-    int flags = fcntl(m_fd_stderr, F_GETFL);
-    if (flags == -1) {
-      std::cerr << "failed to get fcntl flags for stderr: " << cpp_strerror(errno) << std::endl;
-      return;
-    }
-    if (!(flags & O_NONBLOCK)) {
-      flags |= O_NONBLOCK;
-      flags = fcntl(m_fd_stderr, F_SETFL, flags);
-      if (flags == -1) {
-        std::cerr << "failed to set fcntl flags for stderr: " << cpp_strerror(errno) << std::endl;
-        return;
-      }
-    }
-    do_stderr_poll = true;
-  }
-#endif // !_WIN32
 }
 
 
@@ -154,10 +115,8 @@ void Log::reopen_log_file()
     return;
   }
   m_flush_mutex_holder = pthread_self();
-  if (m_fd >= 0) {
+  if (m_fd >= 0)
     VOID_TEMP_FAILURE_RETRY(::close(m_fd));
-    m_fd = -1;
-  }
   if (m_log_file.length()) {
     m_fd = ::open(m_log_file.c_str(), O_CREAT|O_WRONLY|O_APPEND|O_CLOEXEC, 0644);
     if (m_fd >= 0 && (m_uid || m_gid)) {
@@ -167,6 +126,8 @@ void Log::reopen_log_file()
 	     << std::endl;
       }
     }
+  } else {
+    m_fd = -1;
   }
   m_flush_mutex_holder = 0;
 }
@@ -205,11 +166,15 @@ void Log::set_graylog_level(int log, int crash)
   m_graylog_crash = crash;
 }
 
-void Log::start_graylog()
+void Log::start_graylog(const std::string& host,
+			const uuid_d& fsid)
 {
   std::scoped_lock lock(m_flush_mutex);
-  if (! m_graylog.get())
+  if (! m_graylog.get()) {
     m_graylog = std::make_shared<Graylog>(m_subs, "dlog");
+    m_graylog->set_hostname(host);
+    m_graylog->set_fsid(fsid);
+  }
 }
 
 
@@ -219,6 +184,27 @@ void Log::stop_graylog()
   m_graylog.reset();
 }
 
+void Log::set_journald_level(int log, int crash)
+{
+  std::scoped_lock lock(m_flush_mutex);
+  m_journald_log = log;
+  m_journald_crash = crash;
+}
+
+void Log::start_journald_logger()
+{
+  std::scoped_lock lock(m_flush_mutex);
+  if (!m_journald) {
+    m_journald = std::make_unique<JournaldLogger>(m_subs);
+  }
+}
+
+void Log::stop_journald_logger()
+{
+  std::scoped_lock lock(m_flush_mutex);
+  m_journald.reset();
+}
+
 void Log::submit_entry(Entry&& e)
 {
   std::unique_lock lock(m_queue_mutex);
@@ -271,69 +257,6 @@ void Log::_log_safe_write(std::string_view sv)
   }
 }
 
-void Log::set_stderr_fd(int fd)
-{
-  m_fd_stderr = fd;
-  _configure_stderr();
-}
-
-void Log::_log_stderr(std::string_view strv)
-{
-  if (do_stderr_poll) {
-    auto& prefix = m_log_stderr_prefix;
-    size_t const len = prefix.size() + strv.size();
-    boost::container::small_vector<char, PIPE_BUF> buf;
-    buf.resize(len+1, '\0');
-    memcpy(buf.data(), prefix.c_str(), prefix.size());
-    memcpy(buf.data()+prefix.size(), strv.data(), strv.size());
-
-    char const* const start = buf.data();
-    char const* current = start;
-    while ((size_t)(current-start) < len) {
-      auto chunk = std::min<ssize_t>(PIPE_BUF, len-(ssize_t)(current-start));
-      while (1) {
-        ssize_t rc = write(m_fd_stderr, current, chunk);
-        if (rc == chunk) {
-          current += chunk;
-          break;
-        } else if (rc > 0) {
-          /* According to IEEE Std 1003.1-2017, this cannot happen:
-           *
-           * Write requests to a pipe or FIFO shall be handled in the same way as a regular file with the following exceptions:
-           * ...
-           *   If the O_NONBLOCK flag is set ...
-           *   ...
-           *     A write request for {PIPE_BUF} or fewer bytes shall have the
-           *     following effect: if there is sufficient space available in
-           *     the pipe, write() shall transfer all the data and return the
-           *     number of bytes requested. Otherwise, write() shall transfer
-           *     no data and return -1 with errno set to [EAGAIN].
-           *
-           * In any case, handle misbehavior gracefully by incrementing current.
-           */
-          current += rc;
-          break;
-        } else if (rc == -1) {
-          if (errno == EAGAIN) {
-            struct pollfd pfd[1];
-            pfd[0].fd = m_fd_stderr;
-            pfd[0].events = POLLOUT;
-            poll(pfd, 1, -1);
-            /* ignore errors / success, just retry the write */
-          } else if (errno == EINTR) {
-            continue;
-          } else {
-            /* some other kind of error, no point logging if stderr writes fail */
-            return;
-          }
-        }
-      }
-    }
-  } else {
-    std::cerr << m_log_stderr_prefix << strv << std::endl;
-  }
-}
-
 void Log::_flush_logbuf()
 {
   if (m_log_buf.size()) {
@@ -364,6 +287,7 @@ void Log::_flush(EntryVector& t, bool crash)
     bool do_syslog = m_syslog_crash >= prio && should_log;
     bool do_stderr = m_stderr_crash >= prio && should_log;
     bool do_graylog2 = m_graylog_crash >= prio && should_log;
+    bool do_journald = m_journald_crash >= prio && should_log;
 
     if (do_fd || do_syslog || do_stderr) {
       const std::size_t cur = m_log_buf.size();
@@ -388,13 +312,13 @@ void Log::_flush(EntryVector& t, bool crash)
         syslog(LOG_USER|LOG_INFO, "%s", pos);
       }
 
+      if (do_stderr) {
+        std::cerr << m_log_stderr_prefix << std::string_view(pos, used) << std::endl;
+      }
+
       /* now add newline */
       pos[used++] = '\n';
 
-      if (do_stderr) {
-        _log_stderr(std::string_view(pos, used));
-      }
-
       if (do_fd) {
         m_log_buf.resize(cur + used);
       } else {
@@ -410,6 +334,10 @@ void Log::_flush(EntryVector& t, bool crash)
       m_graylog->log_entry(e);
     }
 
+    if (do_journald && m_journald) {
+      m_journald->log_entry(e);
+    }
+
     m_recent.push_back(std::move(e));
   }
   t.clear();
@@ -486,7 +414,9 @@ void Log::dump_recent()
   {
     char pthread_name[16] = {0}; //limited by 16B include terminating null byte.
     ceph_pthread_getname(pthread_id, pthread_name, sizeof(pthread_name));
-    _log_message(fmt::format("  {} / {}",
+    // we want the ID to be printed in the same format as we use for a log entry.
+    // The reason is easier grepping.
+    _log_message(fmt::format("  {:x} / {}",
 			     tid_to_int(pthread_id), pthread_name), true);
   }
 
