@@ -66,9 +66,9 @@ public:
   CInodeCommitOperation(int prio, int64_t po)
     : pool(po), priority(prio) {
   }
-  CInodeCommitOperation(int prio, int64_t po, file_layout_t l, uint64_t f)
-    : pool(po), priority(prio), _layout(l), _features(f) {
-      update_layout = true;
+  CInodeCommitOperation(int prio, int64_t po, file_layout_t l, uint64_t f, std::string_view s)
+    : pool(po), priority(prio), _layout(l), _features(f), _symlink(s) {
+      update_layout_symlink = true;
   }
 
   void update(ObjectOperation &op, inode_backtrace_t &bt);
@@ -77,9 +77,10 @@ public:
 private:
   int64_t pool;     ///< pool id
   int priority;
-  bool update_layout = false;
+  bool update_layout_symlink = false;
   file_layout_t _layout;
   uint64_t _features;
+  std::string_view _symlink;
 };
 
 struct CInodeCommitOperations {
