@@ -121,7 +121,6 @@ static const int MDS_TRAVERSE_RDLOCK_PATH	= (1 << 7);
 static const int MDS_TRAVERSE_XLOCK_DENTRY	= (1 << 8);
 static const int MDS_TRAVERSE_RDLOCK_AUTHLOCK	= (1 << 9);
 static const int MDS_TRAVERSE_CHECK_LOCKCACHE	= (1 << 10);
-static const int MDS_TRAVERSE_WANT_INODE	= (1 << 11);
 
 
 // flags for predirty_journal_parents()
@@ -177,7 +176,7 @@ class MDCache {
     bool path_locked = false;
     mds_rank_t hint = MDS_RANK_NONE;
     mds_rank_t checking = MDS_RANK_NONE;
-    set<mds_rank_t> checked;
+    std::set<mds_rank_t> checked;
   };
 
   friend class C_MDC_RejoinOpenInoFinish;
@@ -194,19 +193,6 @@ class MDCache {
   explicit MDCache(MDSRank *m, PurgeQueue &purge_queue_);
   ~MDCache();
 
-  void insert_taken_inos(inodeno_t ino) {
-    replay_taken_inos.insert(ino);
-  }
-  void clear_taken_inos(inodeno_t ino) {
-    replay_taken_inos.erase(ino);
-  }
-  bool test_and_clear_taken_inos(inodeno_t ino) {
-    return replay_taken_inos.erase(ino) != 0;
-  }
-  bool is_taken_inos_empty(void) {
-    return replay_taken_inos.empty();
-  }
-
   uint64_t cache_limit_memory(void) {
     return cache_memory_limit;
   }
@@ -237,6 +223,10 @@ class MDCache {
     return export_ephemeral_random_config;
   }
 
+  bool get_symlink_recovery(void) const {
+    return symlink_recovery;
+  }
+
   /**
    * Call this when you know that a CDentry is ready to be passed
    * on to StrayManager (i.e. this is a stray you've just created)
@@ -317,17 +307,17 @@ class MDCache {
   void adjust_subtree_auth(CDir *root, mds_rank_t a, mds_rank_t b=CDIR_AUTH_UNKNOWN) {
     adjust_subtree_auth(root, mds_authority_t(a,b));
   }
-  void adjust_bounded_subtree_auth(CDir *dir, const set<CDir*>& bounds, mds_authority_t auth);
-  void adjust_bounded_subtree_auth(CDir *dir, const set<CDir*>& bounds, mds_rank_t a) {
+  void adjust_bounded_subtree_auth(CDir *dir, const std::set<CDir*>& bounds, mds_authority_t auth);
+  void adjust_bounded_subtree_auth(CDir *dir, const std::set<CDir*>& bounds, mds_rank_t a) {
     adjust_bounded_subtree_auth(dir, bounds, mds_authority_t(a, CDIR_AUTH_UNKNOWN));
   }
-  void adjust_bounded_subtree_auth(CDir *dir, const vector<dirfrag_t>& bounds, const mds_authority_t &auth);
-  void adjust_bounded_subtree_auth(CDir *dir, const vector<dirfrag_t>& bounds, mds_rank_t a) {
+  void adjust_bounded_subtree_auth(CDir *dir, const std::vector<dirfrag_t>& bounds, const mds_authority_t &auth);
+  void adjust_bounded_subtree_auth(CDir *dir, const std::vector<dirfrag_t>& bounds, mds_rank_t a) {
     adjust_bounded_subtree_auth(dir, bounds, mds_authority_t(a, CDIR_AUTH_UNKNOWN));
   }
-  void map_dirfrag_set(const list<dirfrag_t>& dfs, set<CDir*>& result);
+  void map_dirfrag_set(const std::list<dirfrag_t>& dfs, std::set<CDir*>& result);
   void try_subtree_merge(CDir *root);
-  void try_subtree_merge_at(CDir *root, set<CInode*> *to_eval, bool adjust_pop=true);
+  void try_subtree_merge_at(CDir *root, std::set<CInode*> *to_eval, bool adjust_pop=true);
   void eval_subtree_root(CInode *diri);
   CDir *get_subtree_root(CDir *dir);
   CDir *get_projected_subtree_root(CDir *dir);
@@ -339,10 +329,10 @@ class MDCache {
   bool is_subtree(CDir *root) {
     return subtrees.count(root);
   }
-  void get_subtree_bounds(CDir *root, set<CDir*>& bounds);
-  void get_wouldbe_subtree_bounds(CDir *root, set<CDir*>& bounds);
-  void verify_subtree_bounds(CDir *root, const set<CDir*>& bounds);
-  void verify_subtree_bounds(CDir *root, const list<dirfrag_t>& bounds);
+  void get_subtree_bounds(CDir *root, std::set<CDir*>& bounds);
+  void get_wouldbe_subtree_bounds(CDir *root, std::set<CDir*>& bounds);
+  void verify_subtree_bounds(CDir *root, const std::set<CDir*>& bounds);
+  void verify_subtree_bounds(CDir *root, const std::list<dirfrag_t>& bounds);
 
   void project_subtree_rename(CInode *diri, CDir *olddir, CDir *newdir);
   void adjust_subtree_after_rename(CInode *diri, CDir *olddir, bool pop);
@@ -403,7 +393,7 @@ class MDCache {
     return active_requests.count(rid);
   }
   MDRequestRef request_get(metareqid_t rid);
-  void request_pin_ref(MDRequestRef& r, CInode *ref, vector<CDentry*>& trace);
+  void request_pin_ref(MDRequestRef& r, CInode *ref, std::vector<CDentry*>& trace);
   void request_finish(MDRequestRef& mdr);
   void request_forward(MDRequestRef& mdr, mds_rank_t mds, int port=0);
   void dispatch_request(MDRequestRef& mdr);
@@ -436,7 +426,7 @@ class MDCache {
 				snapid_t follows=CEPH_NOSNAP);
 
   // peers
-  void add_uncommitted_leader(metareqid_t reqid, LogSegment *ls, set<mds_rank_t> &peers, bool safe=false) {
+  void add_uncommitted_leader(metareqid_t reqid, LogSegment *ls, std::set<mds_rank_t> &peers, bool safe=false) {
     uncommitted_leaders[reqid].ls = ls;
     uncommitted_leaders[reqid].peers = peers;
     uncommitted_leaders[reqid].safe = safe;
@@ -462,7 +452,7 @@ class MDCache {
   MDPeerUpdate* get_uncommitted_peer(metareqid_t reqid, mds_rank_t leader);
   void _logged_peer_commit(mds_rank_t from, metareqid_t reqid);
 
-  void set_recovery_set(set<mds_rank_t>& s);
+  void set_recovery_set(std::set<mds_rank_t>& s);
   void handle_mds_failure(mds_rank_t who);
   void handle_mds_recovery(mds_rank_t who);
 
@@ -491,12 +481,12 @@ class MDCache {
   void finish_rollback(metareqid_t reqid, MDRequestRef& mdr);
 
   // ambiguous imports
-  void add_ambiguous_import(dirfrag_t base, const vector<dirfrag_t>& bounds);
-  void add_ambiguous_import(CDir *base, const set<CDir*>& bounds);
+  void add_ambiguous_import(dirfrag_t base, const std::vector<dirfrag_t>& bounds);
+  void add_ambiguous_import(CDir *base, const std::set<CDir*>& bounds);
   bool have_ambiguous_import(dirfrag_t base) {
     return my_ambiguous_imports.count(base);
   }
-  void get_ambiguous_import_bounds(dirfrag_t base, vector<dirfrag_t>& bounds) {
+  void get_ambiguous_import_bounds(dirfrag_t base, std::vector<dirfrag_t>& bounds) {
     ceph_assert(my_ambiguous_imports.count(base));
     bounds = my_ambiguous_imports[base];
   }
@@ -510,7 +500,7 @@ class MDCache {
   }
   
   void _move_subtree_map_bound(dirfrag_t df, dirfrag_t oldparent, dirfrag_t newparent,
-			       map<dirfrag_t,vector<dirfrag_t> >& subtrees);
+			       std::map<dirfrag_t,std::vector<dirfrag_t> >& subtrees);
   ESubtreeMap *create_subtree_map();
 
   void clean_open_file_lists();
@@ -577,16 +567,16 @@ class MDCache {
 
   void rejoin_open_ino_finish(inodeno_t ino, int ret);
   void rejoin_prefetch_ino_finish(inodeno_t ino, int ret);
-  void rejoin_open_sessions_finish(map<client_t,pair<Session*,uint64_t> >& session_map);
+  void rejoin_open_sessions_finish(std::map<client_t,std::pair<Session*,uint64_t> >& session_map);
   bool process_imported_caps();
   void choose_lock_states_and_reconnect_caps();
   void prepare_realm_split(SnapRealm *realm, client_t client, inodeno_t ino,
-			   map<client_t,ref_t<MClientSnap>>& splits);
-  void prepare_realm_merge(SnapRealm *realm, SnapRealm *parent_realm, map<client_t,ref_t<MClientSnap>>& splits);
-  void send_snaps(map<client_t,ref_t<MClientSnap>>& splits);
+			   std::map<client_t,ref_t<MClientSnap>>& splits);
+  void prepare_realm_merge(SnapRealm *realm, SnapRealm *parent_realm, std::map<client_t,ref_t<MClientSnap>>& splits);
+  void send_snaps(std::map<client_t,ref_t<MClientSnap>>& splits);
   Capability* rejoin_import_cap(CInode *in, client_t client, const cap_reconnect_t& icr, mds_rank_t frommds);
   void finish_snaprealm_reconnect(client_t client, SnapRealm *realm, snapid_t seq,
-				  map<client_t,ref_t<MClientSnap>>& updates);
+				  std::map<client_t,ref_t<MClientSnap>>& updates);
   Capability* try_reconnect_cap(CInode *in, Session *session);
   void export_remaining_imported_caps();
 
@@ -756,8 +746,8 @@ class MDCache {
 
   CDir *get_auth_container(CDir *in);
   CDir *get_export_container(CDir *dir);
-  void find_nested_exports(CDir *dir, set<CDir*>& s);
-  void find_nested_exports_under(CDir *import, CDir *dir, set<CDir*>& s);
+  void find_nested_exports(CDir *dir, std::set<CDir*>& s);
+  void find_nested_exports_under(CDir *import, CDir *dir, std::set<CDir*>& s);
 
   void init_layouts();
   void create_unlinked_system_inode(CInode *in, inodeno_t ino,
@@ -810,13 +800,8 @@ class MDCache {
    * dentry is encountered.
    * MDS_TRAVERSE_WANT_DENTRY: Caller wants tail dentry. Add a null dentry if
    * tail dentry does not exist. return 0 even tail dentry is null.
-   * MDS_TRAVERSE_WANT_INODE: Caller only wants target inode if it exists, or
-   * wants tail dentry if target inode does not exist and MDS_TRAVERSE_WANT_DENTRY
-   * is also set.
    * MDS_TRAVERSE_WANT_AUTH: Always forward request to auth MDS of target inode
    * or auth MDS of tail dentry (MDS_TRAVERSE_WANT_DENTRY is set).
-   * MDS_TRAVERSE_XLOCK_DENTRY: Caller wants to xlock tail dentry if MDS_TRAVERSE_WANT_INODE
-   * is not set or (MDS_TRAVERSE_WANT_INODE is set but target inode does not exist)
    *
    * @param pdnvec Data return parameter -- on success, contains a
    * vector of dentries. On failure, is either empty or contains the
@@ -832,19 +817,16 @@ class MDCache {
    */
   int path_traverse(MDRequestRef& mdr, MDSContextFactory& cf,
 		    const filepath& path, int flags,
-		    vector<CDentry*> *pdnvec, CInode **pin=nullptr);
-
-  int maybe_request_forward_to_auth(MDRequestRef& mdr, MDSContextFactory& cf,
-				    MDSCacheObject *p);
+		    std::vector<CDentry*> *pdnvec, CInode **pin=nullptr);
 
   CInode *cache_traverse(const filepath& path);
 
   void open_remote_dirfrag(CInode *diri, frag_t fg, MDSContext *fin);
   CInode *get_dentry_inode(CDentry *dn, MDRequestRef& mdr, bool projected=false);
 
-  bool parallel_fetch(map<inodeno_t,filepath>& pathmap, set<inodeno_t>& missing);
+  bool parallel_fetch(std::map<inodeno_t,filepath>& pathmap, std::set<inodeno_t>& missing);
   bool parallel_fetch_traverse_dir(inodeno_t ino, filepath& path, 
-				   set<CDir*>& fetch_queue, set<inodeno_t>& missing,
+				   std::set<CDir*>& fetch_queue, std::set<inodeno_t>& missing,
 				   C_GatherBuilder &gather_bld);
 
   void open_remote_dentry(CDentry *dn, bool projected, MDSContext *fin,
@@ -852,12 +834,12 @@ class MDCache {
   void _open_remote_dentry_finish(CDentry *dn, inodeno_t ino, MDSContext *fin,
 				  bool want_xlocked, int r);
 
-  void make_trace(vector<CDentry*>& trace, CInode *in);
+  void make_trace(std::vector<CDentry*>& trace, CInode *in);
 
   void kick_open_ino_peers(mds_rank_t who);
   void open_ino(inodeno_t ino, int64_t pool, MDSContext *fin,
 		bool want_replica=true, bool want_xlocked=false,
-		vector<inode_backpointer_t> *ancestors_hint=nullptr,
+		std::vector<inode_backpointer_t> *ancestors_hint=nullptr,
 		mds_rank_t auth_hint=MDS_RANK_NONE);
 
   void find_ino_peers(inodeno_t ino, MDSContext *c,
@@ -891,13 +873,13 @@ class MDCache {
   void decode_replica_inode(CInode *&in, bufferlist::const_iterator& p, CDentry *dn, MDSContext::vec& finished);
 
   void encode_replica_stray(CDentry *straydn, mds_rank_t who, bufferlist& bl);
-  void decode_replica_stray(CDentry *&straydn, CInode **in, const bufferlist &bl, mds_rank_t from);
+  void decode_replica_stray(CDentry *&straydn, const bufferlist &bl, mds_rank_t from);
 
   // -- namespace --
   void encode_remote_dentry_link(CDentry::linkage_t *dnl, bufferlist& bl);
   void decode_remote_dentry_link(CDir *dir, CDentry *dn, bufferlist::const_iterator& p);
   void send_dentry_link(CDentry *dn, MDRequestRef& mdr);
-  void send_dentry_unlink(CDentry *dn, CDentry *straydn, MDRequestRef& mdr, bool unlinking=false);
+  void send_dentry_unlink(CDentry *dn, CDentry *straydn, MDRequestRef& mdr);
 
   void wait_for_uncommitted_fragment(dirfrag_t dirfrag, MDSContext *c) {
     uncommitted_fragments.at(dirfrag).waiters.push_back(c);
@@ -931,9 +913,9 @@ class MDCache {
   // -- mdsmap --
   void handle_mdsmap(const MDSMap &mdsmap, const MDSMap &oldmap);
 
-  int dump_cache() { return dump_cache({}, nullptr); }
-  int dump_cache(std::string_view filename);
-  int dump_cache(Formatter *f);
+  int dump_cache() { return dump_cache({}, nullptr, 0); }
+  int dump_cache(std::string_view filename, double timeout);
+  int dump_cache(Formatter *f, double timeout);
   void dump_tree(CInode *in, const int cur_depth, const int max_depth, Formatter *f);
 
   void cache_status(Formatter *f);
@@ -959,7 +941,7 @@ class MDCache {
    */
   void enqueue_scrub(std::string_view path, std::string_view tag,
                      bool force, bool recursive, bool repair,
-                     bool scrub_mdsdir, Formatter *f, Context *fin);
+		     Formatter *f, Context *fin);
   void repair_inode_stats(CInode *diri);
   void repair_dirfrag_stats(CDir *dir);
   void rdlock_dirfrags_stats(CInode *diri, MDSInternalContext *fin);
@@ -989,30 +971,30 @@ class MDCache {
   // -- client caps --
   uint64_t last_cap_id = 0;
 
-  map<ceph_tid_t, discover_info_t> discovers;
+  std::map<ceph_tid_t, discover_info_t> discovers;
   ceph_tid_t discover_last_tid = 0;
 
   // waiters
-  map<int, map<inodeno_t, MDSContext::vec > > waiting_for_base_ino;
+  std::map<int, std::map<inodeno_t, MDSContext::vec > > waiting_for_base_ino;
 
-  map<inodeno_t,map<client_t, reconnected_cap_info_t> > reconnected_caps;   // inode -> client -> snap_follows,realmino
-  map<inodeno_t,map<client_t, snapid_t> > reconnected_snaprealms;  // realmino -> client -> realmseq
+  std::map<inodeno_t,std::map<client_t, reconnected_cap_info_t> > reconnected_caps;   // inode -> client -> snap_follows,realmino
+  std::map<inodeno_t,std::map<client_t, snapid_t> > reconnected_snaprealms;  // realmino -> client -> realmseq
 
   //  realm inodes
-  set<CInode*> rejoin_pending_snaprealms;
+  std::set<CInode*> rejoin_pending_snaprealms;
   // cap imports.  delayed snap parent opens.
-  map<client_t,set<CInode*> > delayed_imported_caps;
+  std::map<client_t,std::set<CInode*> > delayed_imported_caps;
 
   // subsystems
   std::unique_ptr<Migrator> migrator;
 
   bool did_shutdown_log_cap = false;
 
-  map<ceph_tid_t, find_ino_peer_info_t> find_ino_peer;
+  std::map<ceph_tid_t, find_ino_peer_info_t> find_ino_peer;
   ceph_tid_t find_ino_peer_last_tid = 0;
 
   // delayed cache expire
-  map<CDir*, expiremap> delayed_expire; // subtree root -> expire msg
+  std::map<CDir*, expiremap> delayed_expire; // subtree root -> expire msg
 
   /* Because exports may fail, this set lets us keep track of inodes that need exporting. */
   std::set<CInode *> export_pin_queue;
@@ -1027,7 +1009,7 @@ class MDCache {
   // track leader requests whose peers haven't acknowledged commit
   struct uleader {
     uleader() {}
-    set<mds_rank_t> peers;
+    std::set<mds_rank_t> peers;
     LogSegment *ls = nullptr;
     MDSContext::vec waiters;
     bool safe = false;
@@ -1045,8 +1027,8 @@ class MDCache {
 
   struct open_ino_info_t {
     open_ino_info_t() {}
-    vector<inode_backpointer_t> ancestors;
-    set<mds_rank_t> checked;
+    std::vector<inode_backpointer_t> ancestors;
+    std::set<mds_rank_t> checked;
     mds_rank_t checking = MDS_RANK_NONE;
     mds_rank_t auth_hint = MDS_RANK_NONE;
     bool check_peers = true;
@@ -1093,8 +1075,8 @@ class MDCache {
   CDir* rejoin_invent_dirfrag(dirfrag_t df);
   void handle_cache_rejoin_strong(const cref_t<MMDSCacheRejoin> &m);
   void rejoin_scour_survivor_replicas(mds_rank_t from, const cref_t<MMDSCacheRejoin> &ack,
-				      set<vinodeno_t>& acked_inodes,
-				      set<SimpleLock *>& gather_locks);
+				      std::set<vinodeno_t>& acked_inodes,
+				      std::set<SimpleLock *>& gather_locks);
   void handle_cache_rejoin_ack(const cref_t<MMDSCacheRejoin> &m);
   void rejoin_send_acks();
   void rejoin_trim_undef_inodes();
@@ -1109,8 +1091,8 @@ class MDCache {
   }
 
   void inode_remove_replica(CInode *in, mds_rank_t rep, bool rejoin,
-			    set<SimpleLock *>& gather_locks);
-  void dentry_remove_replica(CDentry *dn, mds_rank_t rep, set<SimpleLock *>& gather_locks);
+			    std::set<SimpleLock *>& gather_locks);
+  void dentry_remove_replica(CDentry *dn, mds_rank_t rep, std::set<SimpleLock *>& gather_locks);
 
   void rename_file(CDentry *srcdn, CDentry *destdn);
 
@@ -1119,7 +1101,7 @@ class MDCache {
   void _open_ino_traverse_dir(inodeno_t ino, open_ino_info_t& info, int err);
   void _open_ino_fetch_dir(inodeno_t ino, const cref_t<MMDSOpenIno> &m, CDir *dir, bool parent);
   int open_ino_traverse_dir(inodeno_t ino, const cref_t<MMDSOpenIno> &m,
-			    const vector<inode_backpointer_t>& ancestors,
+			    const std::vector<inode_backpointer_t>& ancestors,
 			    bool discover, bool want_xlocked, mds_rank_t *hint);
   void open_ino_finish(inodeno_t ino, open_ino_info_t& info, int err);
   void do_open_ino(inodeno_t ino, open_ino_info_t& info, int err);
@@ -1133,9 +1115,8 @@ class MDCache {
   void handle_discover_reply(const cref_t<MDiscoverReply> &m);
   void handle_dentry_link(const cref_t<MDentryLink> &m);
   void handle_dentry_unlink(const cref_t<MDentryUnlink> &m);
-  void handle_dentry_unlink_ack(const cref_t<MDentryUnlinkAck> &m);
 
-  int dump_cache(std::string_view fn, Formatter *f);
+  int dump_cache(std::string_view fn, Formatter *f, double timeout);
 
   void flush_dentry_work(MDRequestRef& mdr);
   /**
@@ -1152,7 +1133,7 @@ class MDCache {
   void rdlock_dirfrags_stats_work(MDRequestRef& mdr);
 
   ceph::unordered_map<inodeno_t,CInode*> inode_map;  // map of head inodes by ino
-  map<vinodeno_t, CInode*> snap_inode_map;  // map of snap inodes by ino
+  std::map<vinodeno_t, CInode*> snap_inode_map;  // map of snap inodes by ino
   CInode *root = nullptr; // root inode
   CInode *myin = nullptr; // .ceph/mds%d dir
 
@@ -1161,7 +1142,7 @@ class MDCache {
   int stray_index = 0;
   int stray_fragmenting_index = -1;
 
-  set<CInode*> base_inodes;
+  std::set<CInode*> base_inodes;
 
   std::unique_ptr<PerfCounters> logger;
 
@@ -1169,77 +1150,75 @@ class MDCache {
   std::array<xlist<ClientLease*>, client_lease_pools> client_leases{};
 
   /* subtree keys and each tree's non-recursive nested subtrees (the "bounds") */
-  map<CDir*,set<CDir*> > subtrees;
-  map<CInode*,list<pair<CDir*,CDir*> > > projected_subtree_renames;  // renamed ino -> target dir
+  std::map<CDir*,std::set<CDir*> > subtrees;
+  std::map<CInode*,std::list<std::pair<CDir*,CDir*> > > projected_subtree_renames;  // renamed ino -> target dir
 
   // -- requests --
   ceph::unordered_map<metareqid_t, MDRequestRef> active_requests;
 
   // -- recovery --
-  set<mds_rank_t> recovery_set;
+  std::set<mds_rank_t> recovery_set;
 
   // [resolve]
   // from EImportStart w/o EImportFinish during journal replay
-  map<dirfrag_t, vector<dirfrag_t> > my_ambiguous_imports;
+  std::map<dirfrag_t, std::vector<dirfrag_t> > my_ambiguous_imports;
   // from MMDSResolves
-  map<mds_rank_t, map<dirfrag_t, vector<dirfrag_t> > > other_ambiguous_imports;
+  std::map<mds_rank_t, std::map<dirfrag_t, std::vector<dirfrag_t> > > other_ambiguous_imports;
 
-  map<CInode*, int> uncommitted_peer_rename_olddir;  // peer: preserve the non-auth dir until seeing commit.
-  map<CInode*, int> uncommitted_peer_unlink;  // peer: preserve the unlinked inode until seeing commit.
+  std::map<CInode*, int> uncommitted_peer_rename_olddir;  // peer: preserve the non-auth dir until seeing commit.
+  std::map<CInode*, int> uncommitted_peer_unlink;  // peer: preserve the unlinked inode until seeing commit.
 
-  map<metareqid_t, uleader> uncommitted_leaders;         // leader: req -> peer set
-  map<metareqid_t, upeer> uncommitted_peers;  // peer: preserve the peer req until seeing commit.
+  std::map<metareqid_t, uleader> uncommitted_leaders;         // leader: req -> peer set
+  std::map<metareqid_t, upeer> uncommitted_peers;  // peer: preserve the peer req until seeing commit.
 
-  set<metareqid_t> pending_leaders;
-  map<int, set<metareqid_t> > ambiguous_peer_updates;
+  std::set<metareqid_t> pending_leaders;
+  std::map<int, std::set<metareqid_t> > ambiguous_peer_updates;
 
   bool resolves_pending = false;
-  set<mds_rank_t> resolve_gather;	// nodes i need resolves from
-  set<mds_rank_t> resolve_ack_gather;	// nodes i need a resolve_ack from
-  set<version_t> resolve_snapclient_commits;
-  map<metareqid_t, mds_rank_t> resolve_need_rollback;  // rollbacks i'm writing to the journal
-  map<mds_rank_t, cref_t<MMDSResolve>> delayed_resolve;
+  std::set<mds_rank_t> resolve_gather;	// nodes i need resolves from
+  std::set<mds_rank_t> resolve_ack_gather;	// nodes i need a resolve_ack from
+  std::set<version_t> resolve_snapclient_commits;
+  std::map<metareqid_t, mds_rank_t> resolve_need_rollback;  // rollbacks i'm writing to the journal
+  std::map<mds_rank_t, cref_t<MMDSResolve>> delayed_resolve;
 
   // [rejoin]
   bool rejoins_pending = false;
-  set<mds_rank_t> rejoin_gather;      // nodes from whom i need a rejoin
-  set<mds_rank_t> rejoin_sent;        // nodes i sent a rejoin to
-  set<mds_rank_t> rejoin_ack_sent;    // nodes i sent a rejoin to
-  set<mds_rank_t> rejoin_ack_gather;  // nodes from whom i need a rejoin ack
-  map<mds_rank_t,map<inodeno_t,map<client_t,Capability::Import> > > rejoin_imported_caps;
-  map<inodeno_t,pair<mds_rank_t,map<client_t,Capability::Export> > > rejoin_peer_exports;
+  std::set<mds_rank_t> rejoin_gather;      // nodes from whom i need a rejoin
+  std::set<mds_rank_t> rejoin_sent;        // nodes i sent a rejoin to
+  std::set<mds_rank_t> rejoin_ack_sent;    // nodes i sent a rejoin to
+  std::set<mds_rank_t> rejoin_ack_gather;  // nodes from whom i need a rejoin ack
+  std::map<mds_rank_t,std::map<inodeno_t,std::map<client_t,Capability::Import> > > rejoin_imported_caps;
+  std::map<inodeno_t,std::pair<mds_rank_t,std::map<client_t,Capability::Export> > > rejoin_peer_exports;
 
-  map<client_t,entity_inst_t> rejoin_client_map;
-  map<client_t,client_metadata_t> rejoin_client_metadata_map;
-  map<client_t,pair<Session*,uint64_t> > rejoin_session_map;
+  std::map<client_t,entity_inst_t> rejoin_client_map;
+  std::map<client_t,client_metadata_t> rejoin_client_metadata_map;
+  std::map<client_t,std::pair<Session*,uint64_t> > rejoin_session_map;
 
-  map<inodeno_t,pair<mds_rank_t,map<client_t,cap_reconnect_t> > > cap_exports; // ino -> target, client -> capex
+  std::map<inodeno_t,std::pair<mds_rank_t,std::map<client_t,cap_reconnect_t> > > cap_exports; // ino -> target, client -> capex
 
-  map<inodeno_t,map<client_t,map<mds_rank_t,cap_reconnect_t> > > cap_imports;  // ino -> client -> frommds -> capex
-  set<inodeno_t> cap_imports_missing;
-  map<inodeno_t, MDSContext::vec > cap_reconnect_waiters;
+  std::map<inodeno_t,std::map<client_t,std::map<mds_rank_t,cap_reconnect_t> > > cap_imports;  // ino -> client -> frommds -> capex
+  std::set<inodeno_t> cap_imports_missing;
+  std::map<inodeno_t, MDSContext::vec > cap_reconnect_waiters;
   int cap_imports_num_opening = 0;
 
-  set<CInode*> rejoin_undef_inodes;
-  set<CInode*> rejoin_potential_updated_scatterlocks;
-  set<CDir*>   rejoin_undef_dirfrags;
-  map<mds_rank_t, set<CInode*> > rejoin_unlinked_inodes;
+  std::set<CInode*> rejoin_undef_inodes;
+  std::set<CInode*> rejoin_potential_updated_scatterlocks;
+  std::set<CDir*>   rejoin_undef_dirfrags;
+  std::map<mds_rank_t, std::set<CInode*> > rejoin_unlinked_inodes;
 
-  vector<CInode*> rejoin_recover_q, rejoin_check_q;
-  list<SimpleLock*> rejoin_eval_locks;
+  std::vector<CInode*> rejoin_recover_q, rejoin_check_q;
+  std::list<SimpleLock*> rejoin_eval_locks;
   MDSContext::vec rejoin_waiters;
 
   std::unique_ptr<MDSContext> rejoin_done;
   std::unique_ptr<MDSContext> resolve_done;
 
   ceph_tid_t open_ino_last_tid = 0;
-  map<inodeno_t,open_ino_info_t> opening_inodes;
+  std::map<inodeno_t,open_ino_info_t> opening_inodes;
 
   StrayManager stray_manager;
 
  private:
-  std::set<inodeno_t> replay_taken_inos; // the inos have been taken when replaying
-
   // -- fragmenting --
   struct ufragment {
     ufragment() {}
@@ -1259,7 +1238,7 @@ class MDCache {
     std::vector<CDir*> dirs;
     std::vector<CDir*> resultfrags;
     MDRequestRef mdr;
-    set<mds_rank_t> notify_ack_waiting;
+    std::set<mds_rank_t> notify_ack_waiting;
     bool finishing = false;
 
     // for deadlock detection
@@ -1269,7 +1248,7 @@ class MDCache {
     int num_remote_waiters = 0;	// number of remote authpin waiters
   };
 
-  typedef map<dirfrag_t,fragment_info_t>::iterator fragment_info_iterator;
+  typedef std::map<dirfrag_t,fragment_info_t>::iterator fragment_info_iterator;
 
   friend class EFragment;
   friend class C_MDC_FragmentFrozen;
@@ -1306,7 +1285,7 @@ class MDCache {
 			    MDSContext::vec& waiters,
 			    bool replay);
   CDir *force_dir_fragment(CInode *diri, frag_t fg, bool replay=true);
-  void get_force_dirfrag_bound_set(const vector<dirfrag_t>& dfs, set<CDir*>& bounds);
+  void get_force_dirfrag_bound_set(const std::vector<dirfrag_t>& dfs, std::set<CDir*>& bounds);
 
   bool can_fragment(CInode *diri, const std::vector<CDir*>& dirs);
   void fragment_freeze_dirs(const std::vector<CDir*>& dirs);
@@ -1340,12 +1319,15 @@ class MDCache {
   bool export_ephemeral_random_config;
   unsigned export_ephemeral_dist_frag_bits;
 
+  // Stores the symlink target on the file object's head
+  bool symlink_recovery;
+
   // File size recovery
   RecoveryQueue recovery_queue;
 
   // shutdown
-  set<inodeno_t> shutdown_exporting_strays;
-  pair<dirfrag_t, string> shutdown_export_next;
+  std::set<inodeno_t> shutdown_exporting_strays;
+  std::pair<dirfrag_t, std::string> shutdown_export_next;
 
   bool opening_root = false, open = false;
   MDSContext::vec waiting_for_open;
@@ -1353,9 +1335,9 @@ class MDCache {
   // -- snaprealms --
   SnapRealm *global_snaprealm = nullptr;
 
-  map<dirfrag_t, ufragment> uncommitted_fragments;
+  std::map<dirfrag_t, ufragment> uncommitted_fragments;
 
-  map<dirfrag_t,fragment_info_t> fragments;
+  std::map<dirfrag_t,fragment_info_t> fragments;
 
   DecayCounter trim_counter;
 
