@@ -36,7 +36,10 @@ enum {
 #define dout_subsys ceph_subsys_mds
 #undef dout_prefix
 #define dout_prefix _prefix(_dout, mds)
-static ostream& _prefix(std::ostream *_dout, MDSRank *mds) {
+
+using namespace std;
+
+static std::ostream& _prefix(std::ostream *_dout, MDSRank *mds) {
   return *_dout << "mds." << mds->get_nodeid() << ".openfiles ";
 }
 
@@ -268,14 +271,13 @@ public:
 
 void OpenFileTable::_commit_finish(int r, uint64_t log_seq, MDSContext *fin)
 {
-  dout(10) << __func__ << " log_seq " << log_seq << " committed_log_seq " << committed_log_seq
-           << " committing_log_seq " << committing_log_seq << dendl;
+  dout(10) << __func__ << " log_seq " << log_seq << dendl;
   if (r < 0) {
     mds->handle_write_error(r);
     return;
   }
 
-  ceph_assert(log_seq == committing_log_seq);
+  ceph_assert(log_seq <= committing_log_seq);
   ceph_assert(log_seq >= committed_log_seq);
   committed_log_seq = log_seq;
   num_pending_commit--;
@@ -334,8 +336,7 @@ void OpenFileTable::_journal_finish(int r, uint64_t log_seq, MDSContext *c,
 
 void OpenFileTable::commit(MDSContext *c, uint64_t log_seq, int op_prio)
 {
-  dout(10) << __func__ << " log_seq " << log_seq << " committing_log_seq:"
-          << committing_log_seq << dendl;
+  dout(10) << __func__ << " log_seq " << log_seq << dendl;
 
   ceph_assert(num_pending_commit == 0);
   num_pending_commit++;
@@ -1058,12 +1059,6 @@ void OpenFileTable::_prefetch_dirfrags()
     CInode *diri = mdcache->get_inode(ino);
     if (!diri)
       continue;
-
-    if (!diri->is_dir()) {
-      dout(10) << " " << *diri << " is not dir" << dendl;
-      continue;
-    }
-
     if (diri->state_test(CInode::STATE_REJOINUNDEF))
       continue;
 
@@ -1095,7 +1090,7 @@ void OpenFileTable::_prefetch_dirfrags()
       ceph_assert(dir->get_inode()->dirfragtree.is_leaf(dir->get_frag()));
     dir->fetch(gather.new_sub());
 
-    if (!(++num_opening_dirfrags % mds->heartbeat_reset_grace()))
+    if (!(++num_opening_dirfrags % 1000))
       mds->heartbeat_reset();
   }
 
@@ -1169,7 +1164,7 @@ void OpenFileTable::_prefetch_inodes()
       mdcache->open_ino(ino, pool, fin, false);
     }
 
-    if (!(num_opening_inodes % mds->heartbeat_reset_grace()))
+    if (!(num_opening_inodes % 1000))
       mds->heartbeat_reset();
   }
 
