@@ -16,30 +16,31 @@
 #define CEPH_MEXPORTDIRACK_H
 
 #include "MExportDir.h"
-#include "msg/Message.h"
+#include "messages/MMDSOp.h"
 
-class MExportDirAck : public SafeMessage {
+class MExportDirAck final : public MMDSOp {
 public:
   dirfrag_t dirfrag;
-  bufferlist imported_caps;
+  ceph::buffer::list imported_caps;
 
   dirfrag_t get_dirfrag() const { return dirfrag; }
   
 protected:
-  MExportDirAck() : SafeMessage{MSG_MDS_EXPORTDIRACK} {}
+  MExportDirAck() : MMDSOp{MSG_MDS_EXPORTDIRACK} {}
   MExportDirAck(dirfrag_t df, uint64_t tid) :
-    SafeMessage{MSG_MDS_EXPORTDIRACK}, dirfrag(df) {
+    MMDSOp{MSG_MDS_EXPORTDIRACK}, dirfrag(df) {
     set_tid(tid);
   }
-  ~MExportDirAck() override {}
+  ~MExportDirAck() final {}
 
 public:
   std::string_view get_type_name() const override { return "ExAck"; }
-    void print(ostream& o) const override {
+  void print(std::ostream& o) const override {
     o << "export_ack(" << dirfrag << ")";
   }
 
   void decode_payload() override {
+    using ceph::decode;
     auto p = payload.cbegin();
     decode(dirfrag, p);
     decode(imported_caps, p);
@@ -52,6 +53,8 @@ public:
 private:
   template<class T, typename... Args>
   friend boost::intrusive_ptr<T> ceph::make_message(Args&&... args);
+  template<class T, typename... Args>
+  friend MURef<T> crimson::make_message(Args&&... args);
 };
 
 #endif
