@@ -22,17 +22,18 @@
  * The mgr digest is a way for the mgr to subscribe to things
  * other than the cluster maps, which are needed by 
  */
-class MMgrDigest : public Message {
+class MMgrDigest final : public Message {
 public:
-  bufferlist mon_status_json;
-  bufferlist health_json;
+  ceph::buffer::list mon_status_json;
+  ceph::buffer::list health_json;
 
   std::string_view get_type_name() const override { return "mgrdigest"; }
-  void print(ostream& out) const override {
+  void print(std::ostream& out) const override {
     out << get_type_name();
   }
 
   void decode_payload() override {
+    using ceph::decode;
     auto p = payload.cbegin();
     decode(mon_status_json, p);
     decode(health_json, p);
@@ -46,12 +47,14 @@ public:
 private:
   MMgrDigest() :
     Message{MSG_MGR_DIGEST} {}
-  ~MMgrDigest() override {}
+  ~MMgrDigest() final {}
 
   using RefCountedObject::put;
   using RefCountedObject::get;
   template<class T, typename... Args>
   friend boost::intrusive_ptr<T> ceph::make_message(Args&&... args);
+  template<class T, typename... Args>
+  friend MURef<T> crimson::make_message(Args&&... args);
 };
 
 #endif
