@@ -17,9 +17,9 @@
 
 #include "messages/PaxosServiceMessage.h"
 
-class MOSDMarkMeDown : public PaxosServiceMessage {
+class MOSDMarkMeDown final : public PaxosServiceMessage {
 private:
-  static constexpr int HEAD_VERSION = 4;
+  static constexpr int HEAD_VERSION = 3;
   static constexpr int COMPAT_VERSION = 3;
 
  public:
@@ -28,7 +28,6 @@ private:
   entity_addrvec_t target_addrs;
   epoch_t epoch = 0;
   bool request_ack = false;          // ack requested
-  bool down_and_dead = false;        // mark down and dead
 
   MOSDMarkMeDown()
     : PaxosServiceMessage{MSG_OSD_MARK_ME_DOWN, 0,
@@ -39,54 +38,28 @@ private:
 			  HEAD_VERSION, COMPAT_VERSION},
       fsid(fs), target_osd(osd), target_addrs(av),
       epoch(e), request_ack(request_ack) {}
-  MOSDMarkMeDown(const uuid_d &fs, int osd, const entity_addrvec_t& av,
-		 epoch_t e, bool request_ack, bool down_and_dead)
-    : PaxosServiceMessage{MSG_OSD_MARK_ME_DOWN, e,
-			  HEAD_VERSION, COMPAT_VERSION},
-      fsid(fs), target_osd(osd), target_addrs(av),
-      epoch(e), request_ack(request_ack), down_and_dead(down_and_dead) {}
  private:
-  ~MOSDMarkMeDown() override {}
+  ~MOSDMarkMeDown() final {}
 
 public: 
   epoch_t get_epoch() const { return epoch; }
 
   void decode_payload() override {
+    using ceph::decode;
     auto p = payload.cbegin();
     paxos_decode(p);
-    if (header.version <= 2) {
-      decode(fsid, p);
-      entity_inst_t i;
-      decode(i, p);
-      target_osd = i.name.num();
-      target_addrs = entity_addrvec_t(i.addr);
-      decode(epoch, p);
-      decode(request_ack, p);
-      return;
-    }
+    assert(header.version >= 3);
     decode(fsid, p);
     decode(target_osd, p);
     decode(target_addrs, p);
     decode(epoch, p);
     decode(request_ack, p);
-    if(header.version >= 4)
-      decode(down_and_dead, p);
   }
 
   void encode_payload(uint64_t features) override {
     using ceph::encode;
     paxos_encode();
-    if (!HAVE_FEATURE(features, SERVER_NAUTILUS)) {
-      header.version = 2;
-      header.compat_version = 2;
-      encode(fsid, payload);
-      encode(entity_inst_t(entity_name_t::OSD(target_osd),
-			   target_addrs.legacy_addr()),
-	     payload, features);
-      encode(epoch, payload);
-      encode(request_ack, payload);
-      return;
-    }
+    assert(HAVE_FEATURE(features, SERVER_NAUTILUS));
     header.version = HEAD_VERSION;
     header.compat_version = COMPAT_VERSION;
     encode(fsid, payload);
@@ -94,14 +67,12 @@ public:
     encode(target_addrs, payload, features);
     encode(epoch, payload);
     encode(request_ack, payload);
-    encode(down_and_dead, payload);
   }
 
   std::string_view get_type_name() const override { return "MOSDMarkMeDown"; }
-  void print(ostream& out) const override {
+  void print(std::ostream& out) const override {
     out << "MOSDMarkMeDown("
 	<< "request_ack=" << request_ack
-	<< ", down_and_dead=" << down_and_dead
 	<< ", osd." << target_osd
 	<< ", " << target_addrs
 	<< ", fsid=" << fsid
