@@ -10,14 +10,14 @@ class MOSDPGCreated : public PaxosServiceMessage {
 public:
   pg_t pgid;
   MOSDPGCreated()
-    : PaxosServiceMessage(MSG_OSD_PG_CREATED, 0)
+    : PaxosServiceMessage{MSG_OSD_PG_CREATED, 0}
   {}
   MOSDPGCreated(pg_t pgid)
-    : PaxosServiceMessage(MSG_OSD_PG_CREATED, 0),
+    : PaxosServiceMessage{MSG_OSD_PG_CREATED, 0},
       pgid(pgid)
   {}
-  const char *get_type_name() const override { return "pg_created"; }
-  void print(ostream& out) const override {
+  std::string_view get_type_name() const override { return "pg_created"; }
+  void print(std::ostream& out) const override {
     out << "osd_pg_created(" << pgid << ")";
   }
   void encode_payload(uint64_t features) override {
@@ -26,8 +26,12 @@ public:
     encode(pgid, payload);
   }
   void decode_payload() override {
-    auto p = payload.begin();
+    using ceph::decode;
+    auto p = payload.cbegin();
     paxos_decode(p);
     decode(pgid, p);
   }
+private:
+  template<class T, typename... Args>
+  friend boost::intrusive_ptr<T> ceph::make_message(Args&&... args);
 };
