@@ -16,20 +16,21 @@
 #ifndef CEPH_MSTATFS_H
 #define CEPH_MSTATFS_H
 
+#include <optional>
 #include <sys/statvfs.h>    /* or <sys/statfs.h> */
 #include "messages/PaxosServiceMessage.h"
 
-class MStatfs : public PaxosServiceMessage {
+class MStatfs final : public PaxosServiceMessage {
 private:
   static constexpr int HEAD_VERSION = 2;
   static constexpr int COMPAT_VERSION = 1;
 
 public:
   uuid_d fsid;
-  boost::optional<int64_t> data_pool;
+  std::optional<int64_t> data_pool;
 
   MStatfs() : PaxosServiceMessage{CEPH_MSG_STATFS, 0, HEAD_VERSION, COMPAT_VERSION} {}
-  MStatfs(const uuid_d& f, ceph_tid_t t, boost::optional<int64_t> _data_pool,
+  MStatfs(const uuid_d& f, ceph_tid_t t, std::optional<int64_t> _data_pool,
 	  version_t v)
     : PaxosServiceMessage{CEPH_MSG_STATFS, v, HEAD_VERSION, COMPAT_VERSION},
       fsid(f), data_pool(_data_pool) {
@@ -37,7 +38,7 @@ public:
   }
 
 private:
-  ~MStatfs() override {}
+  ~MStatfs() final {}
 
 public:
   std::string_view get_type_name() const override { return "statfs"; }
@@ -60,7 +61,7 @@ public:
     if (header.version >= 2) {
       decode(data_pool, p);
     } else {
-      data_pool = boost::optional<int64_t> ();
+      data_pool = std::optional<int64_t> ();
     }
   }
 private:
