@@ -18,27 +18,32 @@
 
 class MStatfsReply : public Message {
 public:
-  struct ceph_mon_statfs_reply h;
+  struct ceph_mon_statfs_reply h{};
 
-  MStatfsReply() : Message(CEPH_MSG_STATFS_REPLY) {}
-  MStatfsReply(uuid_d &f, ceph_tid_t t, epoch_t epoch) : Message(CEPH_MSG_STATFS_REPLY) {
+  MStatfsReply() : Message{CEPH_MSG_STATFS_REPLY} {}
+  MStatfsReply(uuid_d &f, ceph_tid_t t, epoch_t epoch)
+    : Message{CEPH_MSG_STATFS_REPLY} {
     memcpy(&h.fsid, f.bytes(), sizeof(h.fsid));
     header.tid = t;
     h.version = epoch;
   }
 
-  const char *get_type_name() const override { return "statfs_reply"; }
-  void print(ostream& out) const override {
+  std::string_view get_type_name() const override { return "statfs_reply"; }
+  void print(std::ostream& out) const override {
     out << "statfs_reply(" << header.tid << ")";
   }
 
   void encode_payload(uint64_t features) override {
-    ::encode(h, payload);
+    using ceph::encode;
+    encode(h, payload);
   }
   void decode_payload() override {
-    bufferlist::iterator p = payload.begin();
-    ::decode(h, p);
+    auto p = payload.cbegin();
+    decode(h, p);
   }
+private:
+  template<class T, typename... Args>
+  friend boost::intrusive_ptr<T> ceph::make_message(Args&&... args);
 };
 
 #endif
