@@ -27,7 +27,6 @@
 #include "mon/MonCommand.h"
 
 #include "messages/MMgrOpen.h"
-#include "messages/MMgrUpdate.h"
 #include "messages/MMgrClose.h"
 #include "messages/MMgrConfigure.h"
 #include "messages/MMonMgrReport.h"
@@ -46,7 +45,16 @@
 #define dout_subsys ceph_subsys_mgr
 #undef dout_prefix
 #define dout_prefix *_dout << "mgr.server " << __func__ << " "
+
 using namespace TOPNSPC::common;
+
+using std::list;
+using std::ostringstream;
+using std::string;
+using std::stringstream;
+using std::vector;
+using std::unique_ptr;
+
 namespace {
   template <typename Map>
   bool map_compare(Map const &lhs, Map const &rhs) {
@@ -259,8 +267,6 @@ bool DaemonServer::ms_dispatch2(const ref_t<Message>& m)
       return handle_report(ref_cast<MMgrReport>(m));
     case MSG_MGR_OPEN:
       return handle_open(ref_cast<MMgrOpen>(m));
-    case MSG_MGR_UPDATE:
-      return handle_update(ref_cast<MMgrUpdate>(m));
     case MSG_MGR_CLOSE:
       return handle_close(ref_cast<MMgrClose>(m));
     case MSG_COMMAND:
@@ -525,49 +531,6 @@ bool DaemonServer::handle_open(const ref_t<MMgrOpen>& m)
   return true;
 }
 
-bool DaemonServer::handle_update(const ref_t<MMgrUpdate>& m)
-{
-  DaemonKey key;
-  if (!m->service_name.empty()) {
-    key.type = m->service_name;
-  } else {
-    key.type = ceph_entity_type_name(m->get_connection()->get_peer_type());
-  }
-  key.name = m->daemon_name;
-
-  dout(10) << "from " << m->get_connection() << " " << key << dendl;
-
-  if (m->get_connection()->get_peer_type() == entity_name_t::TYPE_CLIENT &&
-      m->service_name.empty()) {
-    // Clients should not be sending us update request
-    dout(10) << "rejecting update request from non-daemon client " << m->daemon_name
-	     << dendl;
-    clog->warn() << "rejecting report from non-daemon client " << m->daemon_name
-		 << " at " << m->get_connection()->get_peer_addrs();
-    m->get_connection()->mark_down();
-    return true;
-  }
-
-
-  {
-    std::unique_lock locker(lock);
-
-    DaemonStatePtr daemon;
-    // Look up the DaemonState
-    if (daemon_state.exists(key)) {
-      dout(20) << "updating existing DaemonState for " << key << dendl;
-
-      daemon = daemon_state.get(key);
-      if (m->need_metadata_update &&
-          !m->daemon_metadata.empty()) {
-        daemon_state.update_metadata(daemon, m->daemon_metadata);
-      }
-    }
-  }
-
-  return true;
-}
-
 bool DaemonServer::handle_close(const ref_t<MMgrClose>& m)
 {
   std::lock_guard l(lock);
@@ -1094,7 +1057,7 @@ bool DaemonServer::_handle_command(
     if (boost::algorithm::ends_with(prefix, "_json")) {
       format = "json";
     } else {
-      cmd_getval(cmdctx->cmdmap, "format", format, string("plain"));
+      format = cmd_getval_or<string>(cmdctx->cmdmap, "format", "plain");
     }
     f.reset(Formatter::create(format));
   }
@@ -1112,7 +1075,7 @@ bool DaemonServer::_handle_command(
 
     auto dump_cmd = [&cmdnum, &f, m](const MonCommand &mc){
       ostringstream secname;
-      secname << "cmd" << setfill('0') << std::setw(3) << cmdnum;
+      secname << "cmd" << std::setfill('0') << std::setw(3) << cmdnum;
       dump_cmddesc_to_json(&f, m->get_connection()->get_features(),
                            secname.str(), mc.cmdstring, mc.helpstring,
                            mc.module, mc.req_perms, 0);
@@ -1283,20 +1246,13 @@ bool DaemonServer::_handle_command(
       return true;
     }
     for (auto& con : p->second) {
-      if (HAVE_FEATURE(con->get_features(), SERVER_MIMIC)) {
-	vector<spg_t> pgs = { spgid };
-	con->send_message(new MOSDScrub2(monc->get_fsid(),
-					 epoch,
-					 pgs,
-					 scrubop == "repair",
-					 scrubop == "deep-scrub"));
-      } else {
-	vector<pg_t> pgs = { pgid };
-	con->send_message(new MOSDScrub(monc->get_fsid(),
-					pgs,
-					scrubop == "repair",
-					scrubop == "deep-scrub"));
-      }
+      assert(HAVE_FEATURE(con->get_features(), SERVER_OCTOPUS));
+      vector<spg_t> pgs = { spgid };
+      con->send_message(new MOSDScrub2(monc->get_fsid(),
+				       epoch,
+				       pgs,
+				       scrubop == "repair",
+				       scrubop == "deep-scrub"));
     }
     ss << "instructing pg " << spgid << " on osd." << acting_primary
        << " to " << scrubop;
@@ -1463,8 +1419,7 @@ bool DaemonServer::_handle_command(
     bool dry_run =
       prefix == "osd test-reweight-by-pg" ||
       prefix == "osd test-reweight-by-utilization";
-    int64_t oload;
-    cmd_getval(cmdctx->cmdmap, "oload", oload, int64_t(120));
+    int64_t oload = cmd_getval_or<int64_t>(cmdctx->cmdmap, "oload", 120);
     set<int64_t> pools;
     vector<string> poolnames;
     cmd_getval(cmdctx->cmdmap, "pools", poolnames);
@@ -1498,7 +1453,7 @@ bool DaemonServer::_handle_command(
       return true;
     }
     bool no_increasing = false;
-    cmd_getval(cmdctx->cmdmap, "no_increasing", no_increasing);
+    cmd_getval_compat_cephbool(cmdctx->cmdmap, "no_increasing", no_increasing);
     string out_str;
     mempool::osdmap::map<int32_t, uint32_t> new_weights;
     r = cluster_state.with_osdmap_and_pgmap([&](const OSDMap &osdmap, const PGMap& pgmap) {
@@ -2639,6 +2594,8 @@ void DaemonServer::send_report()
 		 << std::dec << dendl;
             continue;
           }
+	  dout(20) << " + " << state->key << " "
+		   << metric << dendl;
           tie(acc, std::ignore) = accumulated.emplace(metric.get_type(),
               std::move(collector));
         }
@@ -2823,7 +2780,7 @@ void DaemonServer::adjust_pgs()
 	      dout(10) << "pool " << i.first
 		       << " pg_num " << p.get_pg_num()
 		       << " - pgp_num " << p.get_pgp_num()
-		       << " gap >= max_pg_num_change " << max_jump
+		       << " gap > max_pg_num_change " << max_jump
 		       << " - must scale pgp_num first"
 		       << dendl;
 	    } else {
@@ -2879,7 +2836,7 @@ void DaemonServer::adjust_pgs()
 	    // single adjustment that's more than half of the
 	    // max_misplaced, to somewhat limit the magnitude of
 	    // our potential error here.
-	    int next;
+	    unsigned next;
 	    static constexpr unsigned MAX_NUM_OBJECTS_PER_PG_FOR_LEAP = 1;
 	    pool_stat_t s = pg_map.get_pg_pool_sum_stat(i.first);
 	    if (aggro ||
@@ -2980,19 +2937,13 @@ void DaemonServer::got_service_map()
       if (pending_service_map.epoch == 0) {
 	// we just started up
 	dout(10) << "got initial map e" << service_map.epoch << dendl;
-	ceph_assert(pending_service_map_dirty == 0);
-	pending_service_map = service_map;
-	pending_service_map.epoch = service_map.epoch + 1;
-      } else if (pending_service_map.epoch <= service_map.epoch) {
-	// we just started up but got one more not our own map
-	dout(10) << "got newer initial map e" << service_map.epoch << dendl;
-	ceph_assert(pending_service_map_dirty == 0);
 	pending_service_map = service_map;
 	pending_service_map.epoch = service_map.epoch + 1;
       } else {
-	// we already active and therefore must have persisted it,
+	// we we already active and therefore must have persisted it,
 	// which means ours is the same or newer.
 	dout(10) << "got updated map e" << service_map.epoch << dendl;
+	ceph_assert(pending_service_map.epoch > service_map.epoch);
       }
     });
 
@@ -3133,11 +3084,6 @@ int DaemonServer::remove_mds_perf_query(MetricQueryID query_id)
   return mds_perf_metric_collector.remove_query(query_id);
 }
 
-void DaemonServer::reregister_mds_perf_queries()
-{
-  mds_perf_metric_collector.reregister_queries();
-}
-
 int DaemonServer::get_mds_perf_counters(MDSPerfCollector *collector)
 {
   return mds_perf_metric_collector.get_counters(collector);
