@@ -4,6 +4,7 @@
 #pragma once
 
 #include <map>
+#include <optional>
 #include <ostream>
 #include <string>
 
@@ -54,14 +55,14 @@ struct OptionMask {
     }
     return r;
   }
-  void dump(Formatter *f) const;
+  void dump(ceph::Formatter *f) const;
 };
 
 struct MaskedOption {
-  string raw_value;               ///< raw, unparsed, unvalidated value
+  std::string raw_value;               ///< raw, unparsed, unvalidated value
   const Option *opt;              ///< the option
   OptionMask mask;
-  unique_ptr<const Option> unknown_opt; ///< if fabricated for an unknown option
+  std::unique_ptr<const Option> unknown_opt; ///< if fabricated for an unknown option
 
   MaskedOption(const Option *o, bool fab=false) : opt(o) {
     if (fab) {
@@ -80,9 +81,9 @@ struct MaskedOption {
   /// return a precision metric (smaller is more precise)
   int get_precision(const CrushWrapper *crush);
 
-  friend ostream& operator<<(ostream& out, const MaskedOption& o);
+  friend std::ostream& operator<<(std::ostream& out, const MaskedOption& o);
 
-  void dump(Formatter *f) const;
+  void dump(ceph::Formatter *f) const;
 };
 
 struct Section {
@@ -91,14 +92,14 @@ struct Section {
   void clear() {
     options.clear();
   }
-  void dump(Formatter *f) const;
+  void dump(ceph::Formatter *f) const;
   std::string get_minimal_conf() const;
 };
 
 struct ConfigMap {
   Section global;
-  std::map<std::string,Section> by_type;
-  std::map<std::string,Section> by_id;
+  std::map<std::string,Section, std::less<>> by_type;
+  std::map<std::string,Section, std::less<>> by_id;
   std::list<std::unique_ptr<Option>> stray_options;
 
   Section *find_section(const std::string& name) {
@@ -121,14 +122,18 @@ struct ConfigMap {
     by_id.clear();
     stray_options.clear();
   }
-  void dump(Formatter *f) const;
+  void dump(ceph::Formatter *f) const;
   std::map<std::string,std::string,std::less<>> generate_entity_map(
     const EntityName& name,
-    const map<std::string,std::string>& crush_location,
+    const std::map<std::string,std::string>& crush_location,
     const CrushWrapper *crush,
     const std::string& device_class,
-    std::map<std::string,pair<std::string,const MaskedOption*>> *src=0);
+    std::map<std::string,std::pair<std::string,const MaskedOption*>> *src=0);
 
+  void parse_key(
+    const std::string& key,
+    std::string *name,
+    std::string *who);
   static bool parse_mask(
     const std::string& in,
     std::string *section,
@@ -139,11 +144,11 @@ struct ConfigMap {
 struct ConfigChangeSet {
   version_t version;
   utime_t stamp;
-  string name;
+  std::string name;
 
   // key -> (old value, new value)
-  map<string,pair<boost::optional<string>,boost::optional<string>>> diff;
+  std::map<std::string,std::pair<std::optional<std::string>,std::optional<std::string>>> diff;
 
-  void dump(Formatter *f) const;
-  void print(ostream& out) const;
+  void dump(ceph::Formatter *f) const;
+  void print(std::ostream& out) const;
 };
