@@ -24,6 +24,9 @@
 #include "ConfigMonitor.h"
 #include "HealthMonitor.h"
 
+#include "common/TextTable.h"
+#include "include/stringify.h"
+
 #include "MgrMonitor.h"
 
 #define MGR_METADATA_PREFIX "mgr_metadata"
@@ -32,28 +35,42 @@
 #undef dout_prefix
 #define dout_prefix _prefix(_dout, mon, map)
 using namespace TOPNSPC::common;
-static ostream& _prefix(std::ostream *_dout, Monitor *mon,
+
+using std::dec;
+using std::hex;
+using std::list;
+using std::map;
+using std::make_pair;
+using std::ostream;
+using std::ostringstream;
+using std::pair;
+using std::set;
+using std::string;
+using std::stringstream;
+using std::to_string;
+using std::vector;
+
+using ceph::bufferlist;
+using ceph::decode;
+using ceph::encode;
+using ceph::ErasureCodeInterfaceRef;
+using ceph::ErasureCodeProfile;
+using ceph::Formatter;
+using ceph::JSONFormatter;
+using ceph::make_message;
+using ceph::mono_clock;
+using ceph::mono_time;
+
+static ostream& _prefix(std::ostream *_dout, Monitor &mon,
 			const MgrMap& mgrmap) {
-  return *_dout << "mon." << mon->name << "@" << mon->rank
-		<< "(" << mon->get_state_name()
+  return *_dout << "mon." << mon.name << "@" << mon.rank
+		<< "(" << mon.get_state_name()
 		<< ").mgr e" << mgrmap.get_epoch() << " ";
 }
 
 // the system treats always_on_modules as if they provide built-in functionality
 // by ensuring that they are always enabled.
 const static std::map<uint32_t, std::set<std::string>> always_on_modules = {
-  {
-    CEPH_RELEASE_NAUTILUS, {
-      "crash",
-      "status",
-      "progress",
-      "balancer",
-      "devicehealth",
-      "orchestrator_cli",
-      "rbd_support",
-      "volumes",
-    }
-  },
   {
     CEPH_RELEASE_OCTOPUS, {
       "crash",
@@ -67,7 +84,35 @@ const static std::map<uint32_t, std::set<std::string>> always_on_modules = {
       "pg_autoscaler",
       "telemetry",
     }
-  }
+  },
+  {
+    CEPH_RELEASE_PACIFIC, {
+      "crash",
+      "status",
+      "progress",
+      "balancer",
+      "devicehealth",
+      "orchestrator",
+      "rbd_support",
+      "volumes",
+      "pg_autoscaler",
+      "telemetry",
+    }
+  },
+  {
+    CEPH_RELEASE_QUINCY, {
+      "crash",
+      "status",
+      "progress",
+      "balancer",
+      "devicehealth",
+      "orchestrator",
+      "rbd_support",
+      "volumes",
+      "pg_autoscaler",
+      "telemetry",
+    }
+  },
 };
 
 // Prefix for mon store of active mgr's command descriptions
@@ -170,7 +215,7 @@ void MgrMonitor::update_from_paxos(bool *need_bootstrap)
       dout(4) << "mkfs or daemon transitioned to available, loading commands"
 	      << dendl;
       bufferlist loaded_commands;
-      int r = mon->store->get(command_descs_prefix, "", loaded_commands);
+      int r = mon.store->get(command_descs_prefix, "", loaded_commands);
       if (r < 0) {
         derr << "Failed to load mgr commands: " << cpp_strerror(r) << dendl;
       } else {
@@ -229,9 +274,9 @@ void MgrMonitor::update_from_paxos(bool *need_bootstrap)
   }
   // force ConfigMonitor to refresh, since it uses const Option *
   // pointers into our mgr_module_options (which we just rebuilt).
-  mon->configmon()->load_config();
+  mon.configmon()->load_config();
 
-  if (!mon->is_init()) {
+  if (!mon.is_init()) {
     // feed our pet MgrClient, unless we are in Monitor::[pre]init()
     prime_mgr_client();
   }
@@ -240,7 +285,7 @@ void MgrMonitor::update_from_paxos(bool *need_bootstrap)
 void MgrMonitor::prime_mgr_client()
 {
   dout(10) << __func__ << dendl;
-  mon->mgr_client.ms_dispatch2(make_message<MMgrMap>(map));
+  mon.mgr_client.ms_dispatch2(make_message<MMgrMap>(map));
 }
 
 void MgrMonitor::create_pending()
@@ -255,8 +300,8 @@ health_status_t MgrMonitor::should_warn_about_mgr_down()
   // we warn if we have osds AND we've exceeded the grace period
   // which means a new mon cluster and be HEALTH_OK indefinitely as long as
   // no OSDs are ever created.
-  if (mon->osdmon()->osdmap.get_num_osds() > 0 &&
-       now > mon->monmap->created + g_conf().get_val<int64_t>("mon_mgr_mkfs_grace")) {
+  if (mon.osdmon()->osdmap.get_num_osds() > 0 &&
+       now > mon.monmap->created + g_conf().get_val<int64_t>("mon_mgr_mkfs_grace")) {
     health_status_t level = HEALTH_WARN;
     if (first_seen_inactive != utime_t() &&
 	now - first_seen_inactive > g_conf().get_val<int64_t>("mon_mgr_inactive_grace")) {
@@ -273,12 +318,12 @@ void MgrMonitor::post_paxos_update()
   if (digest_event) {
     bool send = false;
     if (prev_health_checks.empty()) {
-      prev_health_checks.resize(mon->paxos_service.size());
+      prev_health_checks.resize(mon.paxos_service.size());
       send = true;
     }
-    ceph_assert(prev_health_checks.size() == mon->paxos_service.size());
+    ceph_assert(prev_health_checks.size() == mon.paxos_service.size());
     for (auto i = 0u; i < prev_health_checks.size(); i++) {
-      const auto& curr = mon->paxos_service[i]->get_health_checks();
+      const auto& curr = mon.paxos_service[i]->get_health_checks();
       if (!send && curr != prev_health_checks[i]) {
         send = true;
       }
@@ -289,7 +334,7 @@ void MgrMonitor::post_paxos_update()
         send_digests();
       } else {
         cancel_timer();
-        wait_for_active_ctx(new C_MonContext{mon, [this](int) {
+        wait_for_active_ctx(new C_MonContext{&mon, [this](int) {
           send_digests();
         }});
       }
@@ -301,7 +346,7 @@ void MgrMonitor::encode_pending(MonitorDBStore::TransactionRef t)
 {
   dout(10) << __func__ << " " << pending_map << dendl;
   bufferlist bl;
-  pending_map.encode(bl, mon->get_quorum_con_features());
+  pending_map.encode(bl, mon.get_quorum_con_features());
   put_version(t, pending_map.epoch, bl);
   put_last_committed(t, pending_map.epoch);
 
@@ -353,9 +398,9 @@ bool MgrMonitor::check_caps(MonOpRequestRef op, const uuid_d& fsid)
     dout(1) << __func__ << " insufficient caps " << session->caps << dendl;
     return false;
   }
-  if (fsid != mon->monmap->fsid) {
+  if (fsid != mon.monmap->fsid) {
     dout(1) << __func__ << " op fsid " << fsid
-	    << " != " << mon->monmap->fsid << dendl;
+	    << " != " << mon.monmap->fsid << dendl;
     return false;
   }
   return true;
@@ -372,12 +417,12 @@ bool MgrMonitor::preprocess_query(MonOpRequestRef op)
 	return preprocess_command(op);
       } catch (const bad_cmd_get& e) {
       bufferlist bl;
-      mon->reply_command(op, -EINVAL, e.what(), bl, get_last_committed());
+      mon.reply_command(op, -EINVAL, e.what(), bl, get_last_committed());
       return true;
     }
 
     default:
-      mon->no_reply(op);
+      mon.no_reply(op);
       derr << "Unhandled message type " << m->get_type() << dendl;
       return true;
   }
@@ -395,12 +440,12 @@ bool MgrMonitor::prepare_update(MonOpRequestRef op)
 	return prepare_command(op);
       } catch (const bad_cmd_get& e) {
 	bufferlist bl;
-	mon->reply_command(op, -EINVAL, e.what(), bl, get_last_committed());
+	mon.reply_command(op, -EINVAL, e.what(), bl, get_last_committed());
 	return true;
       }
 
     default:
-      mon->no_reply(op);
+      mon.no_reply(op);
       derr << "Unhandled message type " << m->get_type() << dendl;
       return true;
   }
@@ -416,9 +461,9 @@ public:
     mm(a), op(c) {}
   void finish(int r) override {
     if (r >= 0) {
-      // Success 
+      // Success
     } else if (r == -ECANCELED) {
-      mm->mon->no_reply(op);
+      mm->mon.no_reply(op);
     } else {
       mm->dispatch(op);        // try again
     }
@@ -428,7 +473,7 @@ public:
 bool MgrMonitor::preprocess_beacon(MonOpRequestRef op)
 {
   auto m = op->get_req<MMgrBeacon>();
-  mon->no_reply(op); // we never reply to beacons
+  mon.no_reply(op); // we never reply to beacons
   dout(4) << "beacon from " << m->get_gid() << dendl;
 
   if (!check_caps(op, m->get_fsid())) {
@@ -450,12 +495,12 @@ bool MgrMonitor::prepare_beacon(MonOpRequestRef op)
       && m->get_gid() != pending_map.active_gid)
   {
     dout(4) << "Active daemon restart (mgr." << m->get_name() << ")" << dendl;
-    mon->clog->info() << "Active manager daemon " << m->get_name()
+    mon.clog->info() << "Active manager daemon " << m->get_name()
                       << " restarted";
-    if (!mon->osdmon()->is_writeable()) {
+    if (!mon.osdmon()->is_writeable()) {
       dout(1) << __func__ << ":  waiting for osdmon writeable to"
-                 " blacklist old instance." << dendl;
-      mon->osdmon()->wait_for_writeable(op, new C_RetryMessage(this, op));
+                 " blocklist old instance." << dendl;
+      mon.osdmon()->wait_for_writeable(op, new C_RetryMessage(this, op));
       return false;
     }
     drop_active();
@@ -466,7 +511,7 @@ bool MgrMonitor::prepare_beacon(MonOpRequestRef op)
     const MgrMap::StandbyInfo &s = i.second;
     if (s.name == m->get_name() && s.gid != m->get_gid()) {
       dout(4) << "Standby daemon restart (mgr." << m->get_name() << ")" << dendl;
-      mon->clog->debug() << "Standby manager daemon " << m->get_name()
+      mon.clog->debug() << "Standby manager daemon " << m->get_name()
                          << " restarted";
       drop_standby(i.first);
       break;
@@ -496,7 +541,7 @@ bool MgrMonitor::prepare_beacon(MonOpRequestRef op)
 
     if (pending_map.get_available() != m->get_available()) {
       dout(4) << "available " << m->get_gid() << dendl;
-      mon->clog->info() << "Manager daemon " << pending_map.active_name
+      mon.clog->info() << "Manager daemon " << pending_map.active_name
                         << " is now available";
 
       // This beacon should include command descriptions
@@ -546,7 +591,7 @@ bool MgrMonitor::prepare_beacon(MonOpRequestRef op)
     encode(m->get_metadata(), pending_metadata[m->get_name()]);
     pending_metadata_rm.erase(m->get_name());
 
-    mon->clog->info() << "Activating manager daemon "
+    mon.clog->info() << "Activating manager daemon "
                       << pending_map.active_name;
 
     updated = true;
@@ -565,7 +610,7 @@ bool MgrMonitor::prepare_beacon(MonOpRequestRef op)
       }
     } else {
       dout(10) << "new standby " << m->get_gid() << dendl;
-      mon->clog->debug() << "Standby manager daemon " << m->get_name()
+      mon.clog->debug() << "Standby manager daemon " << m->get_name()
                          << " started";
       pending_map.standbys[m->get_gid()] = {m->get_gid(), m->get_name(),
 					    m->get_available_modules(),
@@ -589,9 +634,9 @@ bool MgrMonitor::prepare_beacon(MonOpRequestRef op)
 void MgrMonitor::check_subs()
 {
   const std::string type = "mgrmap";
-  if (mon->session_map.subs.count(type) == 0)
+  if (mon.session_map.subs.count(type) == 0)
     return;
-  for (auto sub : *(mon->session_map.subs[type])) {
+  for (auto sub : *(mon.session_map.subs[type])) {
     check_sub(sub);
   }
 }
@@ -604,7 +649,7 @@ void MgrMonitor::check_sub(Subscription *sub)
 	       << " " << sub->session->con->get_peer_addr() << dendl;
       sub->session->con->send_message2(make_message<MMgrMap>(map));
       if (sub->onetime) {
-        mon->session_map.remove_sub(sub);
+        mon.session_map.remove_sub(sub);
       } else {
         sub->next = map.get_epoch() + 1;
       }
@@ -630,7 +675,7 @@ void MgrMonitor::send_digests()
   cancel_timer();
 
   const std::string type = "mgrdigest";
-  if (mon->session_map.subs.count(type) == 0) {
+  if (mon.session_map.subs.count(type) == 0) {
     prev_health_checks.clear();
     return;
   }
@@ -641,17 +686,17 @@ void MgrMonitor::send_digests()
   }
   dout(10) << __func__ << dendl;
 
-  for (auto sub : *(mon->session_map.subs[type])) {
+  for (auto sub : *(mon.session_map.subs[type])) {
     dout(10) << __func__ << " sending digest to subscriber " << sub->session->con
 	     << " " << sub->session->con->get_peer_addr() << dendl;
     auto mdigest = make_message<MMgrDigest>();
 
     JSONFormatter f;
-    mon->healthmon()->get_health_status(true, &f, nullptr, nullptr, nullptr);
+    mon.healthmon()->get_health_status(true, &f, nullptr, nullptr, nullptr);
     f.flush(mdigest->health_json);
     f.reset();
 
-    mon->get_mon_status(&f);
+    mon.get_mon_status(&f);
     f.flush(mdigest->mon_status_json);
     f.reset();
 
@@ -659,9 +704,9 @@ void MgrMonitor::send_digests()
   }
 
 timer:
-  digest_event = mon->timer.add_event_after(
+  digest_event = mon.timer.add_event_after(
     g_conf().get_val<int64_t>("mon_mgr_digest_period"),
-    new C_MonContext{mon, [this](int) {
+    new C_MonContext{&mon, [this](int) {
       send_digests();
   }});
 }
@@ -669,20 +714,18 @@ timer:
 void MgrMonitor::cancel_timer()
 {
   if (digest_event) {
-    mon->timer.cancel_event(digest_event);
+    mon.timer.cancel_event(digest_event);
     digest_event = nullptr;
   }
 }
 
 void MgrMonitor::on_active()
 {
-  if (!mon->is_leader()) {
-    return;
-  }
-  mon->clog->debug() << "mgrmap e" << map.epoch << ": " << map;
-  if (!HAVE_FEATURE(mon->get_quorum_con_features(), SERVER_NAUTILUS)) {
+  if (!mon.is_leader()) {
     return;
   }
+  mon.clog->debug() << "mgrmap e" << map.epoch << ": " << map;
+  assert(HAVE_FEATURE(mon.get_quorum_con_features(), SERVER_NAUTILUS));
   if (pending_map.always_on_modules == always_on_modules) {
     return;
   }
@@ -695,7 +738,7 @@ void MgrMonitor::on_active()
 
 void MgrMonitor::tick()
 {
-  if (!is_active() || !mon->is_leader())
+  if (!is_active() || !mon.is_leader())
     return;
 
   const auto now = ceph::coarse_mono_clock::now();
@@ -756,25 +799,25 @@ void MgrMonitor::tick()
 
   if (pending_map.active_gid != 0
       && last_beacon.at(pending_map.active_gid) < cutoff
-      && mon->osdmon()->is_writeable()) {
+      && mon.osdmon()->is_writeable()) {
     const std::string old_active_name = pending_map.active_name;
     drop_active();
     propose = true;
     dout(4) << "Dropping active" << pending_map.active_gid << dendl;
     if (promote_standby()) {
       dout(4) << "Promoted standby " << pending_map.active_gid << dendl;
-      mon->clog->info() << "Manager daemon " << old_active_name
+      mon.clog->info() << "Manager daemon " << old_active_name
                         << " is unresponsive, replacing it with standby"
                         << " daemon " << pending_map.active_name;
     } else {
       dout(4) << "Active is laggy but have no standbys to replace it" << dendl;
-      mon->clog->info() << "Manager daemon " << old_active_name
+      mon.clog->info() << "Manager daemon " << old_active_name
                         << " is unresponsive.  No standby daemons available.";
     }
   } else if (pending_map.active_gid == 0) {
     if (promote_standby()) {
       dout(4) << "Promoted standby " << pending_map.active_gid << dendl;
-      mon->clog->info() << "Activating manager daemon "
+      mon.clog->info() << "Activating manager daemon "
                       << pending_map.active_name;
       propose = true;
     }
@@ -789,6 +832,15 @@ void MgrMonitor::tick()
     propose = true;
   }
 
+  // obsolete modules?
+  if (mon.monmap->min_mon_release >= ceph_release_t::octopus &&
+      pending_map.module_enabled("orchestrator_cli")) {
+    dout(10) << " disabling obsolete/renamed 'orchestrator_cli'" << dendl;
+    // we don't need to enable 'orchestrator' because it's now always-on
+    pending_map.modules.erase("orchestrator_cli");
+    propose = true;
+  }
+
   if (propose) {
     propose_pending();
   }
@@ -828,7 +880,7 @@ bool MgrMonitor::promote_standby()
 
 void MgrMonitor::drop_active()
 {
-  ceph_assert(mon->osdmon()->is_writeable());
+  ceph_assert(mon.osdmon()->is_writeable());
 
   if (last_beacon.count(pending_map.active_gid) > 0) {
     last_beacon.erase(pending_map.active_gid);
@@ -836,17 +888,17 @@ void MgrMonitor::drop_active()
 
   ceph_assert(pending_map.active_gid > 0);
   auto until = ceph_clock_now();
-  until += g_conf().get_val<double>("mon_mgr_blacklist_interval");
-  dout(5) << "blacklisting previous mgr." << pending_map.active_name << "."
+  until += g_conf().get_val<double>("mon_mgr_blocklist_interval");
+  dout(5) << "blocklisting previous mgr." << pending_map.active_name << "."
           << pending_map.active_gid << " ("
           << pending_map.active_addrs << ")" << dendl;
-  auto blacklist_epoch = mon->osdmon()->blacklist(pending_map.active_addrs, until);
+  auto blocklist_epoch = mon.osdmon()->blocklist(pending_map.active_addrs, until);
 
-  /* blacklist RADOS clients in use by the mgr */
+  /* blocklist RADOS clients in use by the mgr */
   for (const auto& a : pending_map.clients) {
-    mon->osdmon()->blacklist(a, until);
+    mon.osdmon()->blocklist(a, until);
   }
-  request_proposal(mon->osdmon());
+  request_proposal(mon.osdmon());
 
   pending_metadata_rm.insert(pending_map.active_name);
   pending_metadata.erase(pending_map.active_name);
@@ -858,7 +910,7 @@ void MgrMonitor::drop_active()
   pending_map.active_addrs = entity_addrvec_t();
   pending_map.services.clear();
   pending_map.clients.clear();
-  pending_map.last_failure_osd_epoch = blacklist_epoch;
+  pending_map.last_failure_osd_epoch = blocklist_epoch;
 
   // So that when new active mgr subscribes to mgrdigest, it will
   // get an immediate response instead of waiting for next timer
@@ -886,29 +938,40 @@ bool MgrMonitor::preprocess_command(MonOpRequestRef op)
   cmdmap_t cmdmap;
   if (!cmdmap_from_json(m->cmd, &cmdmap, ss)) {
     string rs = ss.str();
-    mon->reply_command(op, -EINVAL, rs, rdata, get_last_committed());
+    mon.reply_command(op, -EINVAL, rs, rdata, get_last_committed());
     return true;
   }
 
   MonSession *session = op->get_session();
   if (!session) {
-    mon->reply_command(op, -EACCES, "access denied", rdata,
+    mon.reply_command(op, -EACCES, "access denied", rdata,
 		       get_last_committed());
     return true;
   }
 
-  string format;
-  cmd_getval(cmdmap, "format", format);
-  boost::scoped_ptr<Formatter> f(Formatter::create(format, "json-pretty",
-						   "json-pretty"));
+  string format = cmd_getval_or<string>(cmdmap, "format", "plain");
+  boost::scoped_ptr<Formatter> f(Formatter::create(format));
 
   string prefix;
   cmd_getval(cmdmap, "prefix", prefix);
   int r = 0;
 
-  if (prefix == "mgr dump") {
-    int64_t epoch = 0;
-    cmd_getval(cmdmap, "epoch", epoch, (int64_t)map.get_epoch());
+  if (prefix == "mgr stat") {
+    if (!f) {
+      f.reset(Formatter::create(format, "json-pretty", "json-pretty"));
+    }
+    f->open_object_section("stat");
+    f->dump_unsigned("epoch", map.get_epoch());
+    f->dump_bool("available", map.get_available());
+    f->dump_string("active_name", map.get_active_name());
+    f->dump_unsigned("num_standby", map.get_num_standby());
+    f->close_section();
+    f->flush(rdata);
+  } else if (prefix == "mgr dump") {
+    if (!f) {
+      f.reset(Formatter::create(format, "json-pretty", "json-pretty"));
+    }
+    int64_t epoch = cmd_getval_or<int64_t>(cmdmap, "epoch", map.get_epoch());
     if (epoch == (int64_t)map.get_epoch()) {
       f->dump_object("mgrmap", map);
     } else {
@@ -926,36 +989,67 @@ bool MgrMonitor::preprocess_command(MonOpRequestRef op)
     }
     f->flush(rdata);
   } else if (prefix == "mgr module ls") {
-    f->open_object_section("modules");
-    {
-      f->open_array_section("always_on_modules");
-      for (auto& p : map.get_always_on_modules()) {
-        f->dump_string("module", p);
+    if (f) {
+      f->open_object_section("modules");
+      {
+        f->open_array_section("always_on_modules");
+        for (auto& p : map.get_always_on_modules()) {
+          f->dump_string("module", p);
+        }
+        f->close_section();
+        f->open_array_section("enabled_modules");
+        for (auto& p : map.modules) {
+          if (map.get_always_on_modules().count(p) > 0)
+            continue;
+          // We only show the name for enabled modules.  The any errors
+          // etc will show up as a health checks.
+          f->dump_string("module", p);
+        }
+        f->close_section();
+        f->open_array_section("disabled_modules");
+        for (auto& p : map.available_modules) {
+          if (map.modules.count(p.name) == 0 &&
+            map.get_always_on_modules().count(p.name) == 0) {
+            // For disabled modules, we show the full info if the detail
+            // parameter is enabled, to give a hint about whether enabling it will work
+            p.dump(f.get());
+          }
+        }
+        f->close_section();
       }
       f->close_section();
-      f->open_array_section("enabled_modules");
+      f->flush(rdata);
+    } else {
+      TextTable tbl;
+      tbl.define_column("MODULE", TextTable::LEFT, TextTable::LEFT);
+      tbl.define_column("      ", TextTable::LEFT, TextTable::LEFT);
+
+      for (auto& p : map.get_always_on_modules()) {
+        tbl << p;
+        tbl << "on (always on)";
+        tbl << TextTable::endrow;
+      }
       for (auto& p : map.modules) {
         if (map.get_always_on_modules().count(p) > 0)
           continue;
-        // We only show the name for enabled modules.  The any errors
-        // etc will show up as a health checks.
-        f->dump_string("module", p);
+        tbl << p;
+        tbl << "on";
+        tbl << TextTable::endrow;
       }
-      f->close_section();
-      f->open_array_section("disabled_modules");
       for (auto& p : map.available_modules) {
         if (map.modules.count(p.name) == 0 &&
             map.get_always_on_modules().count(p.name) == 0) {
-          // For disabled modules, we show the full info, to
-          // give a hint about whether enabling it will work
-          p.dump(f.get());
+          tbl << p.name;
+          tbl << "-";
+          tbl << TextTable::endrow;
         }
       }
-      f->close_section();
+      rdata.append(stringify(tbl));
     }
-    f->close_section();
-    f->flush(rdata);
   } else if (prefix == "mgr services") {
+    if (!f) {
+      f.reset(Formatter::create(format, "json-pretty", "json-pretty"));
+    }
     f->open_object_section("services");
     for (const auto &i : map.services) {
       f->dump_string(i.first.c_str(), i.second);
@@ -963,6 +1057,9 @@ bool MgrMonitor::preprocess_command(MonOpRequestRef op)
     f->close_section();
     f->flush(rdata);
   } else if (prefix == "mgr metadata") {
+    if (!f) {
+      f.reset(Formatter::create(format, "json-pretty", "json-pretty"));
+    }
     string name;
     cmd_getval(cmdmap, "who", name);
     if (name.size() > 0 && !map.have_name(name)) {
@@ -970,9 +1067,6 @@ bool MgrMonitor::preprocess_command(MonOpRequestRef op)
       r = -ENOENT;
       goto reply;
     }
-    string format;
-    cmd_getval(cmdmap, "format", format);
-    boost::scoped_ptr<Formatter> f(Formatter::create(format, "json-pretty", "json-pretty"));
     if (name.size()) {
       f->open_object_section("mgr_metadata");
       f->dump_string("name", name);
@@ -1001,10 +1095,16 @@ bool MgrMonitor::preprocess_command(MonOpRequestRef op)
     }
     f->flush(rdata);
   } else if (prefix == "mgr versions") {
+    if (!f) {
+      f.reset(Formatter::create(format, "json-pretty", "json-pretty"));
+    }
     count_metadata("ceph_version", f.get());
     f->flush(rdata);
     r = 0;
   } else if (prefix == "mgr count-metadata") {
+    if (!f) {
+      f.reset(Formatter::create(format, "json-pretty", "json-pretty"));
+    }
     string field;
     cmd_getval(cmdmap, "property", field);
     count_metadata(field, f.get());
@@ -1017,7 +1117,7 @@ bool MgrMonitor::preprocess_command(MonOpRequestRef op)
 reply:
   string rs;
   getline(ss, rs);
-  mon->reply_command(op, r, rs, rdata, get_last_committed());
+  mon.reply_command(op, r, rs, rdata, get_last_committed());
   return true;
 }
 
@@ -1031,18 +1131,17 @@ bool MgrMonitor::prepare_command(MonOpRequestRef op)
   cmdmap_t cmdmap;
   if (!cmdmap_from_json(m->cmd, &cmdmap, ss)) {
     string rs = ss.str();
-    mon->reply_command(op, -EINVAL, rs, rdata, get_last_committed());
+    mon.reply_command(op, -EINVAL, rs, rdata, get_last_committed());
     return true;
   }
 
   MonSession *session = op->get_session();
   if (!session) {
-    mon->reply_command(op, -EACCES, "access denied", rdata, get_last_committed());
+    mon.reply_command(op, -EACCES, "access denied", rdata, get_last_committed());
     return true;
   }
 
-  string format;
-  cmd_getval(cmdmap, "format", format, string("plain"));
+  string format = cmd_getval_or<string>(cmdmap, "format", "plain");
   boost::scoped_ptr<Formatter> f(Formatter::create(format));
 
   string prefix;
@@ -1066,8 +1165,8 @@ bool MgrMonitor::prepare_command(MonOpRequestRef op)
     if (!err.empty()) {
       // Does not parse as a gid, treat it as a name
       if (pending_map.active_name == who) {
-        if (!mon->osdmon()->is_writeable()) {
-          mon->osdmon()->wait_for_writeable(op, new C_RetryMessage(this, op));
+        if (!mon.osdmon()->is_writeable()) {
+          mon.osdmon()->wait_for_writeable(op, new C_RetryMessage(this, op));
           return false;
         }
         drop_active();
@@ -1089,8 +1188,8 @@ bool MgrMonitor::prepare_command(MonOpRequestRef op)
       }
     } else {
       if (pending_map.active_gid == gid) {
-        if (!mon->osdmon()->is_writeable()) {
-          mon->osdmon()->wait_for_writeable(op, new C_RetryMessage(this, op));
+        if (!mon.osdmon()->is_writeable()) {
+          mon.osdmon()->wait_for_writeable(op, new C_RetryMessage(this, op));
           return false;
         }
         drop_active();
@@ -1117,10 +1216,10 @@ bool MgrMonitor::prepare_command(MonOpRequestRef op)
       ss << "module '" << module << "' is already enabled (always-on)";
       goto out;
     }
-    string force;
-    cmd_getval(cmdmap, "force", force);
+    bool force = false;
+    cmd_getval_compat_cephbool(cmdmap, "force", force);
     if (!pending_map.all_support_module(module) &&
-	force != "--force") {
+	!force) {
       ss << "all mgr daemons do not support module '" << module << "', pass "
 	 << "--force to force enablement";
       r = -ENOENT;
@@ -1128,7 +1227,7 @@ bool MgrMonitor::prepare_command(MonOpRequestRef op)
     }
 
     std::string can_run_error;
-    if (force != "--force" && !pending_map.can_run_module(module, &can_run_error)) {
+    if (!force && !pending_map.can_run_module(module, &can_run_error)) {
       ss << "module '" << module << "' reports that it cannot run on the active "
             "manager daemon: " << can_run_error << " (pass --force to force "
             "enablement)";
@@ -1159,8 +1258,8 @@ bool MgrMonitor::prepare_command(MonOpRequestRef op)
       r = 0;
       goto out;
     }
-    if (!pending_map.any_supports_module(module)) {
-      ss << "module '" << module << "' does not exist";
+    if (!pending_map.modules.count(module)) {
+      ss << "module '" << module << "' is not enabled";
     }
     pending_map.modules.erase(module);
   } else {
@@ -1181,7 +1280,7 @@ out:
     return true;
   } else {
     // reply immediately
-    mon->reply_command(op, r, rs, rdata, get_last_committed());
+    mon.reply_command(op, r, rs, rdata, get_last_committed());
     return false;
   }
 }
@@ -1202,14 +1301,14 @@ int MgrMonitor::load_metadata(const string& name, std::map<string, string>& m,
 			      ostream *err) const
 {
   bufferlist bl;
-  int r = mon->store->get(MGR_METADATA_PREFIX, name, bl);
+  int r = mon.store->get(MGR_METADATA_PREFIX, name, bl);
   if (r < 0)
     return r;
   try {
     auto p = bl.cbegin();
     decode(m, p);
   }
-  catch (buffer::error& e) {
+  catch (ceph::buffer::error& e) {
     if (err)
       *err << "mgr." << name << " metadata is corrupt";
     return -EIO;
@@ -1243,6 +1342,18 @@ void MgrMonitor::count_metadata(const string& field, Formatter *f)
   f->close_section();
 }
 
+void MgrMonitor::get_versions(std::map<string, list<string> > &versions)
+{
+  std::set<string> ls = map.get_all_names();
+  for (auto& name : ls) {
+    std::map<string,string> meta;
+    load_metadata(name, meta, nullptr);
+    auto p = meta.find("ceph_version_short");
+    if (p == meta.end()) continue;
+    versions[p->second].push_back(string("mgr.") + name);
+  }
+}
+
 int MgrMonitor::dump_metadata(const string& name, Formatter *f, ostream *err)
 {
   std::map<string,string> m;
