@@ -28,8 +28,8 @@ class MgrMonitor: public PaxosService
   MgrMap pending_map;
   bool ever_had_active_mgr = false;
 
-  std::map<std::string, bufferlist> pending_metadata;
-  std::set<std::string>             pending_metadata_rm;
+  std::map<std::string, ceph::buffer::list> pending_metadata;
+  std::set<std::string> pending_metadata_rm;
 
   std::map<std::string,Option> mgr_module_options;
   std::list<std::string> misc_option_strings;
@@ -71,7 +71,7 @@ class MgrMonitor: public PaxosService
   std::vector<MonCommand> pending_command_descs;
 
 public:
-  MgrMonitor(Monitor *mn, Paxos *p, const string& service_name)
+  MgrMonitor(Monitor &mn, Paxos &p, const std::string& service_name)
     : PaxosService(mn, p, service_name)
   {}
   ~MgrMonitor() override {}
@@ -84,7 +84,7 @@ public:
   const std::map<std::string,Option>& get_mgr_module_options() {
     return mgr_module_options;
   }
-  const Option *find_module_option(const string& name);
+  const Option *find_module_option(const std::string& name);
 
   bool in_use() const { return map.epoch > 0; }
 
@@ -93,7 +93,7 @@ public:
   void prime_mgr_client();
 
   void create_initial() override;
-  void get_store_prefixes(std::set<string>& s) const override;
+  void get_store_prefixes(std::set<std::string>& s) const override;
   void update_from_paxos(bool *need_bootstrap) override;
   void post_paxos_update() override;
   void create_pending() override;
@@ -119,16 +119,17 @@ public:
 
   void tick() override;
 
-  void print_summary(Formatter *f, std::ostream *ss) const;
+  void print_summary(ceph::Formatter *f, std::ostream *ss) const;
 
   const std::vector<MonCommand> &get_command_descs() const;
 
-  int load_metadata(const string& name, std::map<string, string>& m,
-		    ostream *err) const;
-  int dump_metadata(const string& name, Formatter *f, ostream *err);
-  void print_nodes(Formatter *f) const;
-  void count_metadata(const string& field, Formatter *f);
-  void count_metadata(const string& field, std::map<string,int> *out);
+  int load_metadata(const std::string& name, std::map<std::string, std::string>& m,
+		    std::ostream *err) const;
+  int dump_metadata(const std::string& name, ceph::Formatter *f, std::ostream *err);
+  void print_nodes(ceph::Formatter *f) const;
+  void count_metadata(const std::string& field, ceph::Formatter *f);
+  void count_metadata(const std::string& field, std::map<std::string,int> *out);
+  void get_versions(std::map<std::string, std::list<std::string>> &versions);
 
   // When did the mon last call into our tick() method?  Used for detecting
   // when the mon was not updating us for some period (e.g. during slow
