@@ -4,6 +4,7 @@
 #include "MonMap.h"
 
 #include <algorithm>
+#include <sstream>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -24,10 +25,12 @@
 #include "common/errno.h"
 #include "common/dout.h"
 #include "common/Clock.h"
+#include "mon/health_check.h"
 
 using std::list;
 using std::map;
 using std::ostream;
+using std::ostringstream;
 using std::set;
 using std::string;
 using std::vector;
@@ -35,15 +38,33 @@ using std::vector;
 using ceph::DNSResolver;
 using ceph::Formatter;
 
+#ifdef WITH_SEASTAR
+namespace {
+  seastar::logger& logger()
+  {
+    return crimson::get_logger(ceph_subsys_monc);
+  }
+}
+#endif
+
 void mon_info_t::encode(ceph::buffer::list& bl, uint64_t features) const
 {
-  uint8_t v = 4;
+  uint8_t v = 5;
+  uint8_t min_v = 1;
+  if (!crush_loc.empty()) {
+    // we added crush_loc in version 5, but need to let old clients decode it
+    // so just leave the min_v at version 1. Monitors are protected
+    // from misunderstandings about location because setting it is blocked
+    // on FEATURE_PINGING
+    min_v = 1;
+  }
   if (!HAVE_FEATURE(features, SERVER_NAUTILUS)) {
     v = 2;
   }
-  ENCODE_START(v, 1, bl);
+  ENCODE_START(v, min_v, bl);
   encode(name, bl);
   if (v < 3) {
+    ceph_assert(min_v == 1);
     auto a = public_addrs.legacy_addr();
     if (a != entity_addr_t()) {
       encode(a, bl, features);
@@ -59,12 +80,13 @@ void mon_info_t::encode(ceph::buffer::list& bl, uint64_t features) const
   }
   encode(priority, bl);
   encode(weight, bl);
+  encode(crush_loc, bl);
   ENCODE_FINISH(bl);
 }
 
 void mon_info_t::decode(ceph::buffer::list::const_iterator& p)
 {
-  DECODE_START(4, p);
+  DECODE_START(5, p);
   decode(name, p);
   decode(public_addrs, p);
   if (struct_v >= 2) {
@@ -73,6 +95,9 @@ void mon_info_t::decode(ceph::buffer::list::const_iterator& p)
   if (struct_v >= 4) {
     decode(weight, p);
   }
+  if (struct_v >= 5) {
+    decode(crush_loc, p);
+  }
   DECODE_FINISH(p);
 }
 
@@ -81,7 +106,8 @@ void mon_info_t::print(ostream& out) const
   out << "mon." << name
       << " addrs " << public_addrs
       << " priority " << priority
-      << " weight " << weight;
+      << " weight " << weight
+      << " crush location " << crush_loc;
 }
 
 namespace {
@@ -186,7 +212,7 @@ void MonMap::encode(ceph::buffer::list& blist, uint64_t con_features) const
     return;
   }
 
-  ENCODE_START(7, 6, blist);
+  ENCODE_START(9, 6, blist);
   ceph::encode_raw(fsid, blist);
   encode(epoch, blist);
   encode(last_changed, blist);
@@ -196,13 +222,20 @@ void MonMap::encode(ceph::buffer::list& blist, uint64_t con_features) const
   encode(mon_info, blist, con_features);
   encode(ranks, blist);
   encode(min_mon_release, blist);
+  encode(removed_ranks, blist);
+  uint8_t t = strategy;
+  encode(t, blist);
+  encode(disallowed_leaders, blist);
+  encode(stretch_mode_enabled, blist);
+  encode(tiebreaker_mon, blist);
+  encode(stretch_marked_down_mons, blist);
   ENCODE_FINISH(blist);
 }
 
 void MonMap::decode(ceph::buffer::list::const_iterator& p)
 {
   map<string,entity_addr_t> mon_addr;
-  DECODE_START_LEGACY_COMPAT_LEN_16(7, 3, 3, p);
+  DECODE_START_LEGACY_COMPAT_LEN_16(9, 3, 3, p);
   ceph::decode_raw(fsid, p);
   decode(epoch, p);
   if (struct_v == 1) {
@@ -244,6 +277,22 @@ void MonMap::decode(ceph::buffer::list::const_iterator& p)
   } else {
     min_mon_release = infer_ceph_release_from_mon_features(persistent_features);
   }
+  if (struct_v >= 8) {
+    decode(removed_ranks, p);
+    uint8_t t;
+    decode(t, p);
+    strategy = static_cast<election_strategy>(t);
+    decode(disallowed_leaders, p);
+  }
+  if (struct_v >= 9) {
+    decode(stretch_mode_enabled, p);
+    decode(tiebreaker_mon, p);
+    decode(stretch_marked_down_mons, p);
+  } else {
+    stretch_mode_enabled = false;
+    tiebreaker_mon = "";
+    stretch_marked_down_mons.clear();
+  }
   calc_addr_mons();
   DECODE_FINISH(p);
 }
@@ -328,11 +377,26 @@ void MonMap::print(ostream& out) const
   out << "fsid " << fsid << "\n";
   out << "last_changed " << last_changed << "\n";
   out << "created " << created << "\n";
-  out << "min_mon_release " << ceph::to_integer<unsigned>(min_mon_release)
+  out << "min_mon_release " << to_integer<unsigned>(min_mon_release)
       << " (" << min_mon_release << ")\n";
+  out << "election_strategy: " << strategy << "\n";
+  if (stretch_mode_enabled) {
+    out << "stretch_mode_enabled " << stretch_mode_enabled << "\n";
+    out << "tiebreaker_mon " << tiebreaker_mon << "\n";
+  }
+  if (stretch_mode_enabled ||
+      !disallowed_leaders.empty()) {
+    out << "disallowed_leaders " << disallowed_leaders << "\n";
+  }
   unsigned i = 0;
   for (auto p = ranks.begin(); p != ranks.end(); ++p) {
-    out << i++ << ": " << get_addrs(*p) << " mon." << *p << "\n";
+    const auto &mi = mon_info.find(*p);
+    ceph_assert(mi != mon_info.end());
+    out << i++ << ": " << mi->second.public_addrs << " mon." << *p;
+    if (!mi->second.crush_loc.empty()) {
+      out << "; crush_location " << mi->second.crush_loc;
+    }
+    out << "\n";
   }
 }
 
@@ -342,8 +406,12 @@ void MonMap::dump(Formatter *f) const
   f->dump_stream("fsid") <<  fsid;
   last_changed.gmtime(f->dump_stream("modified"));
   created.gmtime(f->dump_stream("created"));
-  f->dump_unsigned("min_mon_release", ceph::to_integer<unsigned>(min_mon_release));
-  f->dump_string("min_mon_release_name", ceph::to_string(min_mon_release));
+  f->dump_unsigned("min_mon_release", to_integer<unsigned>(min_mon_release));
+  f->dump_string("min_mon_release_name", to_string(min_mon_release));
+  f->dump_int ("election_strategy", strategy);
+  f->dump_stream("disallowed_leaders: ") << disallowed_leaders;
+  f->dump_bool("stretch_mode", stretch_mode_enabled);
+  f->dump_string("tiebreaker_mon", tiebreaker_mon);
   f->open_object_section("features");
   persistent_features.dump(f, "persistent");
   optional_features.dump(f, "optional");
@@ -360,6 +428,9 @@ void MonMap::dump(Formatter *f) const
     f->dump_stream("public_addr") << get_addrs(*p).get_legacy_str();
     f->dump_unsigned("priority", get_priority(*p));
     f->dump_unsigned("weight", get_weight(*p));
+    const auto &mi = mon_info.find(*p);
+    // we don't need to assert this validity as all the get_* functions did
+    f->dump_stream("crush_location") << mi->second.crush_loc;
     f->close_section();
   }
   f->close_section();
@@ -368,7 +439,7 @@ void MonMap::dump(Formatter *f) const
 void MonMap::dump_summary(Formatter *f) const
 {
   f->dump_unsigned("epoch", epoch);
-  f->dump_string("min_mon_release_name", ceph::to_string(min_mon_release));
+  f->dump_string("min_mon_release_name", to_string(min_mon_release));
   f->dump_unsigned("num_mons", ranks.size());
 }
 
@@ -606,7 +677,7 @@ int MonMap::init_with_config_file(const ConfigProxy& conf,
     // there on a weird port is a v1 address, and do not handle
     // addrvecs.
     entity_addr_t addr;
-    if (!addr.parse(val.c_str(), nullptr, entity_addr_t::TYPE_LEGACY)) {
+    if (!addr.parse(val, entity_addr_t::TYPE_LEGACY)) {
       errout << "unable to parse address for mon." << mon_name
              << ": addr='" << val << "'" << std::endl;
       continue;
@@ -646,12 +717,33 @@ int MonMap::init_with_config_file(const ConfigProxy& conf,
   return 0;
 }
 
+void MonMap::check_health(health_check_map_t *checks) const
+{
+  if (stretch_mode_enabled) {
+    list<string> detail;
+    for (auto& p : mon_info) {
+      if (p.second.crush_loc.empty()) {
+	ostringstream ss;
+	ss << "mon " << p.first << " has no location set while in stretch mode";
+	detail.push_back(ss.str());
+      }
+    }
+    if (!detail.empty()) {
+      ostringstream ss;
+      ss << detail.size() << " monitor(s) have no location set while in stretch mode"
+	 << "; this may cause issues with failover, OSD connections, netsplit handling, etc";
+      auto& d = checks->add("MON_LOCATION_NOT_SET", HEALTH_WARN,
+			    ss.str(), detail.size());
+      d.detail.swap(detail);
+    }
+  }
+}
+
 #ifdef WITH_SEASTAR
 
-using namespace seastar;
-
-future<> MonMap::read_monmap(const std::string& monmap)
+seastar::future<> MonMap::read_monmap(const std::string& monmap)
 {
+  using namespace seastar;
   return open_file_dma(monmap, open_flags::ro).then([this] (file f) {
     return f.size().then([this, f = std::move(f)](size_t s) {
       return do_with(make_file_input_stream(f), [this, s](input_stream<char>& in) {
@@ -666,8 +758,9 @@ future<> MonMap::read_monmap(const std::string& monmap)
   });
 }
 
-future<> MonMap::init_with_dns_srv(bool for_mkfs, const std::string& name)
+seastar::future<> MonMap::init_with_dns_srv(bool for_mkfs, const std::string& name)
 {
+  logger().debug("{}: for_mkfs={}, name={}", __func__, for_mkfs, name);
   string domain;
   string service = name;
   // check if domain is also provided and extract it from srv_name
@@ -679,7 +772,7 @@ future<> MonMap::init_with_dns_srv(bool for_mkfs, const std::string& name)
   return seastar::net::dns::get_srv_records(
       seastar::net::dns_resolver::srv_proto::tcp,
       service, domain).then([this](seastar::net::dns_resolver::srv_records records) {
-    return parallel_for_each(records, [this](auto record) {
+    return seastar::parallel_for_each(records, [this](auto record) {
       return seastar::net::dns::resolve_name(record.target).then(
           [record,this](seastar::net::inet_address a) {
 	// the resolved address does not contain ceph specific info like nonce
@@ -701,30 +794,44 @@ future<> MonMap::init_with_dns_srv(bool for_mkfs, const std::string& name)
                             record.priority,
                             record.weight,
                             false);
+      }).handle_exception_type([t=record.target](const std::system_error& e) {
+        logger().debug("{}: unable to resolve name for {}: {}",
+                       "init_with_dns_srv", t, e);
       });
     });
-  }).handle_exception_type([](const std::system_error& e) {
+  }).handle_exception_type([name](const std::system_error& e) {
+    logger().debug("{}: unable to get monitor info from DNS SRV with {}: {}",
+                 "init_with_dns_srv", name, e);
     // ignore DNS failures
     return seastar::make_ready_future<>();
   });
 }
 
-seastar::future<> MonMap::build_monmap(const crimson::common::ConfigProxy& conf,
-				       bool for_mkfs)
+bool MonMap::maybe_init_with_mon_host(const std::string& mon_host,
+                                      const bool for_mkfs)
 {
-  // -m foo?
-  if (const auto mon_host = conf.get_val<std::string>("mon_host");
-      !mon_host.empty()) {
+  if (!mon_host.empty()) {
     if (auto ret = init_with_ips(mon_host, for_mkfs, "noname-"); ret == 0) {
-      return make_ready_future<>();
+      return true;
     }
     // TODO: resolve_addrs() is a blocking call
     if (auto ret = init_with_hosts(mon_host, for_mkfs, "noname-"); ret == 0) {
-      return make_ready_future<>();
+      return true;
     } else {
       throw std::runtime_error(cpp_strerror(ret));
     }
   }
+  return false;
+}
+
+seastar::future<> MonMap::build_monmap(const crimson::common::ConfigProxy& conf,
+				       bool for_mkfs)
+{
+  logger().debug("{}: for_mkfs={}", __func__, for_mkfs);
+  // -m foo?
+  if (maybe_init_with_mon_host(conf.get_val<std::string>("mon_host"), for_mkfs)) {
+    return seastar::make_ready_future<>();
+  }
 
   // What monitors are in the config file?
   ostringstream errout;
@@ -732,7 +839,7 @@ seastar::future<> MonMap::build_monmap(const crimson::common::ConfigProxy& conf,
     throw std::runtime_error(errout.str());
   }
   if (size() > 0) {
-    return make_ready_future<>();
+    return seastar::make_ready_future<>();
   }
   // no info found from conf options lets try use DNS SRV records
   const string srv_name = conf.get_val<std::string>("mon_dns_srv_name");
@@ -743,8 +850,14 @@ seastar::future<> MonMap::build_monmap(const crimson::common::ConfigProxy& conf,
   });
 }
 
-future<> MonMap::build_initial(const crimson::common::ConfigProxy& conf, bool for_mkfs)
+seastar::future<> MonMap::build_initial(const crimson::common::ConfigProxy& conf, bool for_mkfs)
 {
+  // mon_host_override?
+  if (maybe_init_with_mon_host(conf.get_val<std::string>("mon_host_override"),
+                               for_mkfs)) {
+    return seastar::make_ready_future<>();
+  }
+
   // file?
   if (const auto monmap = conf.get_val<std::string>("monmap");
       !monmap.empty()) {
@@ -785,6 +898,8 @@ int MonMap::init_with_dns_srv(CephContext* cct,
 			      bool for_mkfs,
                               std::ostream& errout)
 {
+  lgeneric_dout(cct, 1) << __func__ << " srv_name: " << srv_name << dendl;
+
   string domain;
   // check if domain is also provided and extract it from srv_name
   size_t idx = srv_name.find("_");
@@ -815,6 +930,7 @@ int MonMap::init_with_dns_srv(CephContext* cct,
 
 int MonMap::build_initial(CephContext *cct, bool for_mkfs, ostream& errout)
 {
+  lgeneric_dout(cct, 1) << __func__ << " for_mkfs: " << for_mkfs << dendl;
   const auto& conf = cct->_conf;
 
   // mon_host_override?
@@ -880,6 +996,7 @@ int MonMap::build_initial(CephContext *cct, bool for_mkfs, ostream& errout)
     errout << "no monitors specified to connect to." << std::endl;
     return -ENOENT;
   }
+  strategy = static_cast<election_strategy>(conf.get_val<uint64_t>("mon_election_default_strategy"));
   created = ceph_clock_now();
   last_changed = created;
   calc_legacy_ranks();
