@@ -190,7 +190,6 @@
 #include "messages/MMgrDigest.h"
 #include "messages/MMgrReport.h"
 #include "messages/MMgrOpen.h"
-#include "messages/MMgrUpdate.h"
 #include "messages/MMgrClose.h"
 #include "messages/MMgrConfigure.h"
 #include "messages/MMonMgrReport.h"
@@ -894,10 +893,6 @@ Message *decode_message(CephContext *cct,
     m = make_message<MMgrOpen>();
     break;
 
-  case MSG_MGR_UPDATE:
-    m = make_message<MMgrUpdate>();
-    break;
-
   case MSG_MGR_CLOSE:
     m = make_message<MMgrClose>();
     break;
