@@ -5,8 +5,10 @@
 
 #include <map>
 
+#include "include/Context.h"
 #include "include/int_types.h"
 #include "include/buffer.h"
+
 #include "osd/osd_types.h"
 
 #define OPS_PER_PTR 32
@@ -167,14 +169,7 @@ public:
     ceph_le32 dest_cid;
     ceph_le32 dest_oid;               //OP_CLONE, OP_CLONERANGE
     ceph_le64 dest_off;               //OP_CLONERANGE
-    union {
-	struct {
-	  ceph_le32 hint_type;          //OP_COLL_HINT
-	} __attribute__ ((packed));
-	struct {
-	  ceph_le32 alloc_hint_flags;   //OP_SETALLOCHINT
-	} __attribute__ ((packed));
-    } __attribute__ ((packed));
+    ceph_le32 hint;                   //OP_COLL_HINT,OP_SETALLOCHINT
     ceph_le64 expected_object_size;   //OP_SETALLOCHINT
     ceph_le64 expected_write_size;    //OP_SETALLOCHINT
     ceph_le32 split_bits;             //OP_SPLIT_COLLECTION2,OP_COLL_SET_BITS,
@@ -190,11 +185,11 @@ public:
     ceph_le32 fadvise_flags;
 
     TransactionData() noexcept :
-      ops(init_le64(0)),
-      largest_data_len(init_le32(0)),
-      largest_data_off(init_le32(0)),
-      largest_data_off_in_data_bl(init_le32(0)),
-      fadvise_flags(init_le32(0)) { }
+      ops(0),
+      largest_data_len(0),
+      largest_data_off(0),
+      largest_data_off_in_data_bl(0),
+      fadvise_flags(0) { }
 
     // override default move operations to reset default values
     TransactionData(TransactionData&& other) noexcept :
@@ -362,7 +357,7 @@ public:
   }
   static Context *collect_all_contexts(
     Transaction& t) {
-    list<Context*> contexts;
+    std::list<Context*> contexts;
     contexts.splice(contexts.end(), t.on_applied);
     contexts.splice(contexts.end(), t.on_commit);
     contexts.splice(contexts.end(), t.on_applied_sync);
@@ -748,6 +743,10 @@ public:
     uint32_t get_fadvise_flags() const {
 	return t->get_fadvise_flags();
     }
+
+    const std::vector<ghobject_t> &get_objects() const {
+      return objects;
+    }
   };
 
   iterator begin() {
@@ -913,7 +912,9 @@ public:
     data.ops = data.ops + 1;
   }
   /// Set multiple xattrs of an object
-  void setattrs(const coll_t& cid, const ghobject_t& oid, const std::map<std::string,ceph::buffer::ptr>& attrset) {
+  void setattrs(const coll_t& cid,
+		const ghobject_t& oid,
+		const std::map<std::string,ceph::buffer::ptr,std::less<>>& attrset) {
     using ceph::encode;
     Op* _op = _get_next_op();
     _op->op = OP_SETATTRS;
@@ -923,7 +924,9 @@ public:
     data.ops = data.ops + 1;
   }
   /// Set multiple xattrs of an object
-  void setattrs(const coll_t& cid, const ghobject_t& oid, const std::map<std::string,ceph::buffer::list>& attrset) {
+  void setattrs(const coll_t& cid,
+		const ghobject_t& oid,
+		const std::map<std::string,ceph::buffer::list,std::less<>>& attrset) {
     using ceph::encode;
     Op* _op = _get_next_op();
     _op->op = OP_SETATTRS;
@@ -1023,7 +1026,7 @@ public:
     Op* _op = _get_next_op();
     _op->op = OP_COLL_HINT;
     _op->cid = _get_coll_id(cid);
-    _op->hint_type = type;
+    _op->hint = type;
     encode(hint, data_bl);
     data.ops = data.ops + 1;
   }
@@ -1257,7 +1260,7 @@ public:
     _op->oid = _get_object_id(oid);
     _op->expected_object_size = expected_object_size;
     _op->expected_write_size = expected_write_size;
-    _op->alloc_hint_flags = flags;
+    _op->hint = flags;
     data.ops = data.ops + 1;
   }
 
