@@ -13,7 +13,7 @@
 StupidAllocator::StupidAllocator(CephContext* cct,
                                  int64_t capacity,
                                  int64_t _block_size,
-                                 const std::string& name)
+                                 std::string_view name)
   : Allocator(name, capacity, _block_size),
     cct(cct), num_free(0),
     free(10)
@@ -299,7 +299,7 @@ void StupidAllocator::dump()
   }
 }
 
-void StupidAllocator::foreach(std::function<void(uint64_t offset, uint64_t length)> notify)
+void StupidAllocator::dump(std::function<void(uint64_t offset, uint64_t length)> notify)
 {
   std::lock_guard l(lock);
   for (unsigned bin = 0; bin < free.size(); ++bin) {
