@@ -23,7 +23,7 @@ import threading
 import uuid
 
 from collections import abc
-from typing import Any, Callable, Dict, Generic, List, Optional, Sequence, Tuple, Union
+from typing import cast, Any, Callable, Dict, Generic, List, Optional, Sequence, Tuple, Union
 
 if sys.version_info >= (3, 8):
     from typing import get_args, get_origin
@@ -147,10 +147,7 @@ class CephArgtype(object):
         probably also override __str__() to give a more user-friendly
         'name/type' description for use in command format help messages.
         """
-        a = ''
-        if hasattr(self, 'typeargs'):
-            a = self.typeargs
-        return '{0}(\'{1}\')'.format(self.__class__.__name__, a)
+        return self.__class__.__name__
 
     def __str__(self):
         """
@@ -168,32 +165,34 @@ class CephArgtype(object):
         return []
 
     @staticmethod
-    def _compound_type_to_argdesc(tp, attrs):
+    def _compound_type_to_argdesc(tp, attrs, positional):
         # generate argdesc from Sequence[T], Tuple[T,..] and Optional[T]
         orig_type = get_origin(tp)
         type_args = get_args(tp)
         if orig_type in (abc.Sequence, Sequence, List, list):
             assert len(type_args) == 1
             attrs['n'] = 'N'
-            return CephArgtype.to_argdesc(type_args[0], attrs)
+            return CephArgtype.to_argdesc(type_args[0], attrs, positional=positional)
         elif orig_type is Tuple:
             assert len(type_args) >= 1
             inner_tp = type_args[0]
             assert type_args.count(inner_tp) == len(type_args), \
                 f'all elements in {tp} should be identical'
             attrs['n'] = str(len(type_args))
-            return CephArgtype.to_argdesc(inner_tp, attrs)
+            return CephArgtype.to_argdesc(inner_tp, attrs, positional=positional)
         elif get_origin(tp) is Union:
             # should be Union[t, NoneType]
             assert len(type_args) == 2 and isinstance(None, type_args[1])
-            return CephArgtype.to_argdesc(type_args[0], attrs, True)
+            return CephArgtype.to_argdesc(type_args[0], attrs, True, positional)
         else:
             raise ValueError(f"unknown type '{tp}': '{attrs}'")
 
     @staticmethod
-    def to_argdesc(tp, attrs, has_default=False):
+    def to_argdesc(tp, attrs, has_default=False, positional=True):
         if has_default:
             attrs['req'] = 'false'
+        if not positional:
+            attrs['positional'] = 'false'
         CEPH_ARG_TYPES = {
             str: CephString,
             int: CephInt,
@@ -208,7 +207,7 @@ class CephArgtype(object):
             elif isinstance(tp, type) and issubclass(tp, enum.Enum):
                 return CephChoices(tp=tp).argdesc(attrs)
             else:
-                return CephArgtype._compound_type_to_argdesc(tp, attrs)
+                return CephArgtype._compound_type_to_argdesc(tp, attrs, positional)
 
     def argdesc(self, attrs):
         attrs['type'] = type(self).__name__
@@ -219,6 +218,8 @@ class CephArgtype(object):
         orig_type = get_origin(tp)
         type_args = get_args(tp)
         if orig_type in (abc.Sequence, Sequence, List, list):
+            if v is None:
+                return None
             return [CephArgtype.cast_to(type_args[0], e) for e in v]
         elif orig_type is Tuple:
             return tuple(CephArgtype.cast_to(type_args[0], e) for e in v)
@@ -254,8 +255,7 @@ class CephInt(CephArgtype):
         if range == '':
             self.range = list()
         else:
-            self.range = list(range.split('|'))
-            self.range = [int(x) for x in self.range]
+            self.range = [int(x) for x in range.split('|')]
 
     def valid(self, s, partial=False):
         try:
@@ -281,7 +281,7 @@ class CephInt(CephArgtype):
 
     def argdesc(self, attrs):
         if self.range:
-            attrs['range'] = '|'.join(self.range)
+            attrs['range'] = '|'.join(str(v) for v in self.range)
         return super().argdesc(attrs)
 
 
@@ -294,8 +294,7 @@ class CephFloat(CephArgtype):
         if range == '':
             self.range = list()
         else:
-            self.range = list(range.split('|'))
-            self.range = [float(x) for x in self.range]
+            self.range = [float(x) for x in range.split('|')]
 
     def valid(self, s, partial=False):
         try:
@@ -320,7 +319,7 @@ class CephFloat(CephArgtype):
 
     def argdesc(self, attrs):
         if self.range:
-            attrs['range'] = '|'.join(self.range)
+            attrs['range'] = '|'.join(str(v) for v in self.range)
         return super().argdesc(attrs)
 
 
@@ -340,20 +339,20 @@ class CephString(CephArgtype):
             [c for c in printable if re.match(goodchars, c)]
         )
 
-    def valid(self, s, partial=False):
+    def valid(self, s: str, partial: bool = False) -> None:
         sset = set(s)
         if self.goodset and not sset <= self.goodset:
             raise ArgumentFormat("invalid chars {0} in {1}".
                                  format(''.join(sset - self.goodset), s))
         self.val = s
 
-    def __str__(self):
+    def __str__(self) -> str:
         b = ''
         if self.goodchars:
             b += '(goodchars {0})'.format(self.goodchars)
         return '<string{0}>'.format(b)
 
-    def complete(self, s):
+    def complete(self, s) -> List[str]:
         if s == '':
             return []
         else:
@@ -369,13 +368,13 @@ class CephSocketpath(CephArgtype):
     """
     Admin socket path; check that it's readable and S_ISSOCK
     """
-    def valid(self, s, partial=False):
+    def valid(self, s: str, partial: bool = False) -> None:
         mode = os.stat(s).st_mode
         if not stat.S_ISSOCK(mode):
             raise ArgumentValid('socket path {0} is not a socket'.format(s))
         self.val = s
 
-    def __str__(self):
+    def __str__(self) -> str:
         return '<admin-socket-path>'
 
 
@@ -386,6 +385,7 @@ class CephIPAddr(CephArgtype):
     def valid(self, s, partial=False):
         # parse off port, use socket to validate addr
         type = 6
+        p: Optional[str] = None
         if s.startswith('['):
             type = 6
         elif s.find('.') != -1:
@@ -412,7 +412,7 @@ class CephIPAddr(CephArgtype):
                     raise ArgumentFormat('{0} missing terminating ]'.format(s))
                 if s[end + 1] == ':':
                     try:
-                        p = int(s[end + 2])
+                        p = s[end + 2]
                     except ValueError:
                         raise ArgumentValid('{0}: bad port number'.format(s))
                 a = s[1:end]
@@ -437,7 +437,7 @@ class CephEntityAddr(CephIPAddr):
     """
     EntityAddress, that is, IP address[/nonce]
     """
-    def valid(self, s, partial=False):
+    def valid(self, s: str, partial: bool = False) -> None:
         nonce = None
         if '/' in s:
             ip, nonce = s.split('/')
@@ -457,7 +457,7 @@ class CephEntityAddr(CephIPAddr):
                 )
         self.val = s
 
-    def __str__(self):
+    def __str__(self) -> str:
         return '<EntityAddr>'
 
 
@@ -465,7 +465,7 @@ class CephPoolname(CephArgtype):
     """
     Pool name; very little utility
     """
-    def __str__(self):
+    def __str__(self) -> str:
         return '<poolname>'
 
 
@@ -474,7 +474,7 @@ class CephObjectname(CephArgtype):
     Object name.  Maybe should be combined with Pool name as they're always
     present in pairs, and then could be checked for presence
     """
-    def __str__(self):
+    def __str__(self) -> str:
         return '<objectname>'
 
 
@@ -485,15 +485,15 @@ class CephPgid(CephArgtype):
     def valid(self, s, partial=False):
         if s.find('.') == -1:
             raise ArgumentFormat('pgid has no .')
-        poolid, pgnum = s.split('.', 1)
+        poolid_s, pgnum_s = s.split('.', 1)
         try:
-            poolid = int(poolid)
+            poolid = int(poolid_s)
         except ValueError:
             raise ArgumentFormat('pool {0} not integer'.format(poolid))
         if poolid < 0:
             raise ArgumentFormat('pool {0} < 0'.format(poolid))
         try:
-            pgnum = int(pgnum, 16)
+            pgnum = int(pgnum_s, 16)
         except ValueError:
             raise ArgumentFormat('pgnum {0} not hex integer'.format(pgnum))
         self.val = s
@@ -510,9 +510,9 @@ class CephName(CephArgtype):
 
     Also accept '*'
     """
-    def __init__(self):
-        self.nametype = None
-        self.nameid = None
+    def __init__(self) -> None:
+        self.nametype: Optional[str] = None
+        self.nameid: Optional[str] = None
 
     def valid(self, s, partial=False):
         if s == '*':
@@ -535,9 +535,9 @@ class CephName(CephArgtype):
             if t == 'osd':
                 if i != '*':
                     try:
-                        i = int(i)
+                        int(i)
                     except ValueError:
-                        raise ArgumentFormat('osd id ' + i + ' not integer')
+                        raise ArgumentFormat(f'osd id {i} not integer')
             self.nametype = t
         self.val = s
         self.nameid = i
@@ -553,8 +553,8 @@ class CephOsdName(CephArgtype):
     osd.<id>, or <id>, or *, where id is a base10 int
     """
     def __init__(self):
-        self.nametype = None
-        self.nameid = None
+        self.nametype: Optional[str] = None
+        self.nameid: Optional[int] = None
 
     def valid(self, s, partial=False):
         if s == '*':
@@ -568,14 +568,14 @@ class CephOsdName(CephArgtype):
             t = 'osd'
             i = s
         try:
-            i = int(i)
+            v = int(i)
         except ValueError:
-            raise ArgumentFormat('osd id ' + i + ' not integer')
-        if i < 0:
-            raise ArgumentFormat('osd id {0} is less than 0'.format(i))
+            raise ArgumentFormat(f'osd id {i} not integer')
+        if v < 0:
+            raise ArgumentFormat(f'osd id {v} is less than 0')
         self.nametype = t
-        self.nameid = i
-        self.val = i
+        self.nameid = v
+        self.val = v
 
     def __str__(self):
         return '<osdname (id|osd.id)>'
@@ -704,14 +704,14 @@ class CephUUID(CephArgtype):
     """
     CephUUID: pretty self-explanatory
     """
-    def valid(self, s, partial=False):
+    def valid(self, s: str, partial: bool = False) -> None:
         try:
             uuid.UUID(s)
         except Exception as e:
             raise ArgumentFormat('invalid UUID {0}: {1}'.format(s, e))
         self.val = s
 
-    def __str__(self):
+    def __str__(self) -> str:
         return '<uuid>'
 
 
@@ -719,10 +719,10 @@ class CephPrefix(CephArgtype):
     """
     CephPrefix: magic type for "all the first n fixed strings"
     """
-    def __init__(self, prefix=''):
+    def __init__(self, prefix: str = '') -> None:
         self.prefix = prefix
 
-    def valid(self, s, partial=False):
+    def valid(self, s: str, partial: bool = False) -> None:
         try:
             s = str(s)
             if isinstance(s, bytes):
@@ -745,10 +745,10 @@ class CephPrefix(CephArgtype):
 
         raise ArgumentPrefix("no match for {0}".format(s))
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.prefix
 
-    def complete(self, s):
+    def complete(self, s) -> List[str]:
         if self.prefix.startswith(s):
             return [self.prefix.rstrip(' ')]
         else:
@@ -758,7 +758,8 @@ class CephPrefix(CephArgtype):
 class argdesc(object):
     """
     argdesc(typename, name='name', n=numallowed|N,
-            req=False, helptext=helptext, **kwargs (type-specific))
+            req=False, positional=True,
+            helptext=helptext, **kwargs (type-specific))
 
     validation rules:
     typename: type(**kwargs) will be constructed
@@ -767,6 +768,7 @@ class argdesc(object):
     name is used for parse errors and for constructing JSON output
     n is a numeric literal or 'n|N', meaning "at least one, but maybe more"
     req=False means the argument need not be present in the list
+    positional=False means the argument name must be specified, e.g. "--myoption value"
     helptext is the associated help for the command
     anything else are arguments to pass to the type constructor.
 
@@ -775,15 +777,19 @@ class argdesc(object):
     valid() will later be called with input to validate against it,
     and will store the validated value in self.instance.val for extraction.
     """
-    def __init__(self, t, name=None, n=1, req=True, **kwargs):
+    def __init__(self, t, name=None, n=1, req=True, positional=True, **kwargs) -> None:
         if isinstance(t, basestring):
             self.t = CephPrefix
             self.typeargs = {'prefix': t}
             self.req = True
+            self.positional = True
         else:
             self.t = t
             self.typeargs = kwargs
             self.req = req in (True, 'True', 'true')
+            self.positional = positional in (True, 'True', 'true')
+            if not positional:
+                assert not req
 
         self.name = name
         self.N = (n in ['n', 'N'])
@@ -828,34 +834,61 @@ class argdesc(object):
         like str(), but omit parameter names (except for CephString,
         which really needs them)
         """
-        if self.t == CephBool:
-            chunk = "--{0}".format(self.name.replace("_", "-"))
-        elif self.t == CephPrefix:
-            chunk = str(self.instance)
-        elif self.t == CephChoices:
-            if self.name == 'format':
-                chunk = f'--{self.name} {{{str(self.instance)}}}'
-            else:
+        if self.positional:
+            if self.t == CephBool:
+                chunk = "--{0}".format(self.name.replace("_", "-"))
+            elif self.t == CephPrefix:
                 chunk = str(self.instance)
-        elif self.t == CephOsdName:
-            # it just so happens all CephOsdName commands are named 'id' anyway,
-            # so <id|osd.id> is perfect.
-            chunk = '<id|osd.id>'
-        elif self.t == CephName:
-            # CephName commands similarly only have one arg of the
-            # type, so <type.id> is good.
-            chunk = '<type.id>'
-        elif self.t == CephInt:
-            chunk = '<{0}:int>'.format(self.name)
-        elif self.t == CephFloat:
-            chunk = '<{0}:float>'.format(self.name)
+            elif self.t == CephChoices:
+                if self.name == 'format':
+                    # this is for talking to legacy clusters only; new clusters
+                    # should properly mark format args as non-positional.
+                    chunk = f'--{self.name} {{{str(self.instance)}}}'
+                else:
+                    chunk = f'<{self.name}:{self.instance}>'
+            elif self.t == CephOsdName:
+                # it just so happens all CephOsdName commands are named 'id' anyway,
+                # so <id|osd.id> is perfect.
+                chunk = '<id|osd.id>'
+            elif self.t == CephName:
+                # CephName commands similarly only have one arg of the
+                # type, so <type.id> is good.
+                chunk = '<type.id>'
+            elif self.t == CephInt:
+                chunk = '<{0}:int>'.format(self.name)
+            elif self.t == CephFloat:
+                chunk = '<{0}:float>'.format(self.name)
+            else:
+                chunk = '<{0}>'.format(self.name)
+            s = chunk
+            if self.N:
+                s += '...'
+            if not self.req:
+                s = '[' + s + ']'
         else:
-            chunk = '<{0}>'.format(self.name)
-        s = chunk
-        if self.N:
-            s += '...'
-        if not self.req:
-            s = '[' + s + ']'
+            # non-positional
+            if self.t == CephBool:
+                chunk = "--{0}".format(self.name.replace("_", "-"))
+            elif self.t == CephPrefix:
+                chunk = str(self.instance)
+            elif self.t == CephChoices:
+                chunk = f'--{self.name} {{{str(self.instance)}}}'
+            elif self.t == CephOsdName:
+                chunk = f'--{self.name} <id|osd.id>'
+            elif self.t == CephName:
+                chunk = f'--{self.name} <type.id>'
+            elif self.t == CephInt:
+                chunk = f'--{self.name} <int>'
+            elif self.t == CephFloat:
+                chunk = f'--{self.name} <float>'
+            else:
+                chunk = f'--{self.name} <value>'
+            s = chunk
+            if self.N:
+                s += '...'
+            if not self.req:  # req should *always* be false
+                s = '[' + s + ']'
+
         return s
 
     def complete(self, s):
@@ -877,14 +910,7 @@ def descsort_key(sh):
     return concise_sig(sh['sig'])
 
 
-def descsort(sh1, sh2):
-    """
-    Deprecated; use (key=descsort_key) instead of (cmp=descsort)
-    """
-    return cmp(descsort_key(sh1), descsort_key(sh2))
-
-
-def parse_funcsig(sig: Sequence[Union[str, Dict[str, str]]]) -> List[argdesc]:
+def parse_funcsig(sig: Sequence[Union[str, Dict[str, Any]]]) -> List[argdesc]:
     """
     parse a single descriptor (array of strings or dicts) into a
     dict of function descriptor/validators (objects of CephXXX type)
@@ -917,12 +943,13 @@ def parse_funcsig(sig: Sequence[Union[str, Dict[str, str]]]) -> List[argdesc]:
 
         kwargs = dict()
         for key, val in desc.items():
-            if key not in ['type', 'name', 'n', 'req']:
+            if key not in ['type', 'name', 'n', 'req', 'positional']:
                 kwargs[key] = val
         newsig.append(argdesc(t,
                               name=desc.get('name', None),
                               n=desc.get('n', 1),
                               req=desc.get('req', True),
+                              positional=desc.get('positional', True),
                               **kwargs))
     return newsig
 
@@ -980,7 +1007,12 @@ def parse_json_funcsigs(s: str,
     return sigdict
 
 
-def validate_one(word, desc, is_kwarg, partial=False):
+ArgValT = Union[bool, int, float, str, Tuple[str, str]]
+
+def validate_one(word: str,
+                 desc,
+                 is_kwarg: bool,
+                 partial: bool = False) -> List[ArgValT]:
     """
     validate_one(word, desc, is_kwarg, partial=False)
 
@@ -1005,7 +1037,9 @@ def validate_one(word, desc, is_kwarg, partial=False):
     return vals
 
 
-def matchnum(args, signature, partial=False):
+def matchnum(args: List[str],
+             signature: List[argdesc],
+             partial: bool = False) -> int:
     """
     matchnum(s, signature, partial=False)
 
@@ -1053,7 +1087,7 @@ ValidatedArg = Union[bool, int, float, str,
 ValidatedArgs = Dict[str, ValidatedArg]
 
 
-def store_arg(desc: argdesc, args: List[ValidatedArg], d: ValidatedArgs):
+def store_arg(desc: argdesc, args: Sequence[ValidatedArg], d: ValidatedArgs):
     '''
     Store argument described by, and held in, thanks to valid(),
     desc into the dictionary d, keyed by desc.name.  Three cases:
@@ -1081,7 +1115,7 @@ def store_arg(desc: argdesc, args: List[ValidatedArg], d: ValidatedArgs):
 
 def validate(args: List[str],
              signature: Sequence[argdesc],
-             flags: Optional[int] = 0,
+             flags: int = 0,
              partial: Optional[bool] = False) -> ValidatedArgs:
     """
     validate(args, signature, flags=0, partial=False)
@@ -1105,11 +1139,11 @@ def validate(args: List[str],
     mysig = copy.deepcopy(signature)
     reqsiglen = len([desc for desc in mysig if desc.req])
     matchcnt = 0
-    d = dict()
+    d: ValidatedArgs = dict()
     save_exception = None
 
-    arg_descs_by_name = dict([desc.name, desc] for desc in mysig
-                             if desc.t != CephPrefix)
+    arg_descs_by_name: Dict[str, argdesc] = \
+        dict((desc.name, desc) for desc in mysig if desc.t != CephPrefix)
 
     # Special case: detect "injectargs" (legacy way of modifying daemon
     # configs) and permit "--" string arguments if so.
@@ -1121,24 +1155,24 @@ def validate(args: List[str],
 
         while desc.numseen < desc.n:
             if myargs:
-                myarg = myargs.pop(0)
+                myarg: Optional[str] = myargs.pop(0)
             else:
                 myarg = None
 
             # no arg, but not required?  Continue consuming mysig
             # in case there are later required args
-            if myarg in (None, []) and not desc.req:
-                break
+            if myarg in (None, []):
+                if not desc.req:
+                    break
+                # did we already get this argument (as a named arg, earlier?)
+                if desc.name in d:
+                    break
 
             # A keyword argument?
             if myarg:
                 # argdesc for the keyword argument, if we find one
                 kwarg_desc = None
 
-                # Track whether we need to push value back onto
-                # myargs in the case that this isn't a valid k=v
-                consumed_next = False
-
                 # Try both styles of keyword argument
                 kwarg_match = re.match(KWARG_EQUALS, myarg)
                 if kwarg_match:
@@ -1176,6 +1210,10 @@ def validate(args: List[str],
                     store_arg(kwarg_desc, args, d)
                     continue
 
+            if not desc.positional:
+                # No more positional args!
+                raise ArgumentValid(f"Unexpected argument '{myarg}'")
+
             # Don't handle something as a positional argument if it
             # has a leading "--" unless it's a CephChoices (used for
             # "--yes-i-really-mean-it")
@@ -1225,6 +1263,7 @@ def validate(args: List[str],
                 break
 
             # Have an arg; validate it
+            assert myarg is not None
             try:
                 args = validate_one(myarg, desc, False)
             except ArgumentError as e:
@@ -1267,7 +1306,7 @@ def validate(args: List[str],
 
 
 def validate_command(sigdict: Dict[str, Dict[str, Any]],
-                     args: Sequence[str],
+                     args: List[str],
                      verbose: Optional[bool] = False) -> ValidatedArgs:
     """
     Parse positional arguments into a parameter dict, according to
@@ -1287,19 +1326,19 @@ def validate_command(sigdict: Dict[str, Dict[str, Any]],
     """
     if verbose:
         print("validate_command: " + " ".join(args), file=sys.stderr)
-    found = []
+    found: Optional[Dict[str, Any]] = None
     valid_dict = {}
 
     # look for best match, accumulate possibles in bestcmds
     # (so we can maybe give a more-useful error message)
-    best_match_cnt = 0
-    bestcmds = []
+    best_match_cnt = 0.0
+    bestcmds: List[Dict[str, Any]] = []
     for cmd in sigdict.values():
         flags = cmd.get('flags', 0)
         if flags & Flag.OBSOLETE:
             continue
         sig = cmd['sig']
-        matched = matchnum(args, sig, partial=True)
+        matched: float = matchnum(args, sig, partial=True)
         if (matched >= math.floor(best_match_cnt) and
             matched == matchnum(args, sig, partial=False)):
             # prefer those fully matched over partial patch
@@ -1329,7 +1368,7 @@ def validate_command(sigdict: Dict[str, Dict[str, Any]],
         print("bestcmds_sorted: ", file=sys.stderr)
         pprint.PrettyPrinter(stream=sys.stderr).pprint(bestcmds_sorted)
 
-    ex = None
+    ex: Optional[ArgumentError] = None
     # for everything in bestcmds, look for a true match
     for cmd in bestcmds_sorted:
         sig = cmd['sig']
@@ -1377,7 +1416,7 @@ def validate_command(sigdict: Dict[str, Dict[str, Any]],
     return valid_dict
 
 
-def find_cmd_target(childargs: List[str]) -> Tuple[str, str]:
+def find_cmd_target(childargs: List[str]) -> Tuple[str, Optional[str]]:
     """
     Using a minimal validation, figure out whether the command
     should be sent to a monitor or an osd.  We do this before even
@@ -1397,6 +1436,7 @@ def find_cmd_target(childargs: List[str]) -> Tuple[str, str]:
             # if this fails, something is horribly wrong, as it just
             # validated successfully above
             name.valid(valid_dict['target'])
+            assert name.nametype is not None
             return name.nametype, name.nameid
 
     sig = parse_funcsig(['tell', {'name': 'pgid', 'type': 'CephPgid'}])
@@ -1407,7 +1447,9 @@ def find_cmd_target(childargs: List[str]) -> Tuple[str, str]:
     else:
         if len(valid_dict) == 2:
             # pg doesn't need revalidation; the string is fine
-            return 'pg', valid_dict['pgid']
+            pgid = valid_dict['pgid']
+            assert isinstance(pgid, str)
+            return 'pg', pgid
 
     # If we reached this far it must mean that so far we've been unable to
     # obtain a proper target from childargs.  This may mean that we are not
@@ -1431,6 +1473,8 @@ def find_cmd_target(childargs: List[str]) -> Tuple[str, str]:
         # CephName.valid() raises on validation error; find_cmd_target()'s
         # caller should handle them
         name.valid(childargs[1])
+        assert name.nametype is not None
+        assert name.nameid is not None
         return name.nametype, name.nameid
 
     sig = parse_funcsig(['pg', {'name': 'pgid', 'type': 'CephPgid'}])
@@ -1440,7 +1484,9 @@ def find_cmd_target(childargs: List[str]) -> Tuple[str, str]:
         pass
     else:
         if len(valid_dict) == 2:
-            return 'pg', valid_dict['pgid']
+            pgid = valid_dict['pgid']
+            assert isinstance(pgid, str)
+            return 'pg', pgid
 
     return 'mon', ''
 
@@ -1460,8 +1506,8 @@ class RadosThread(threading.Thread):
             self.exception = e
 
 
-def run_in_thread(func: Callable[[Any, Any], int],
-                  *args: Any, **kwargs: Any) -> int:
+def run_in_thread(func: Callable[[Any, Any], Tuple[int, bytes, str]],
+                  *args: Any, **kwargs: Any) -> Tuple[int, bytes, str]:
     timeout = kwargs.pop('timeout', 0)
     if timeout == 0 or timeout is None:
         # python threading module will just get blocked if timeout is `None`,
@@ -1509,8 +1555,8 @@ def send_command_retry(*args: Any, **kwargs: Any) -> Tuple[int, bytes, str]:
 
 
 def send_command(cluster,
-                 target: Optional[Tuple[str, str]] = ('mon', ''),
-                 cmd: Optional[List[str]] = None,
+                 target: Tuple[str, Optional[str]] = ('mon', ''),
+                 cmd: Optional[str] = None,
                  inbuf: Optional[bytes] = b'',
                  timeout: Optional[int] = 0,
                  verbose: Optional[bool] = False) -> Tuple[int, bytes, str]:
@@ -1525,16 +1571,16 @@ def send_command(cluster,
 
     If target is osd.N, send command to that osd (except for pgid cmds)
     """
-    cmd = cmd or []
     try:
         if target[0] == 'osd':
             osdid = target[1]
+            assert osdid is not None
 
             if verbose:
                 print('submit {0} to osd.{1}'.format(cmd, osdid),
                       file=sys.stderr)
             ret, outbuf, outs = run_in_thread(
-                cluster.osd_command, osdid, cmd, inbuf, timeout=timeout)
+                cluster.osd_command, int(osdid), cmd, inbuf, timeout=timeout)
 
         elif target[0] == 'mgr':
             name = ''     # non-None empty string means "current active mgr"
@@ -1609,9 +1655,9 @@ def send_command(cluster,
 
 
 def json_command(cluster,
-                 target: Optional[Tuple[str, str]] = ('mon', ''),
+                 target: Tuple[str, Optional[str]] = ('mon', ''),
                  prefix: Optional[str] = None,
-                 argdict: Optional[Dict[str, str]] = None,
+                 argdict: Optional[ValidatedArgs] = None,
                  inbuf: Optional[bytes] = b'',
                  timeout: Optional[int] = 0,
                  verbose: Optional[bool] = False) -> Tuple[int, bytes, str]:
@@ -1626,14 +1672,14 @@ def json_command(cluster,
     :param prefix: String to inject into command arguments as 'prefix'
     :param argdict: Command arguments
     """
-    cmddict = {}
+    cmddict: ValidatedArgs = {}
     if prefix:
         cmddict.update({'prefix': prefix})
 
     if argdict:
         cmddict.update(argdict)
         if 'target' in argdict:
-            target = argdict.get('target')
+            target = cast(Tuple[str, str], argdict['target'])
 
     try:
         if target[0] == 'osd':
@@ -1641,7 +1687,7 @@ def json_command(cluster,
             osdtarget = '{0}.{1}'.format(*target)
             # prefer target from cmddict if present and valid
             if 'target' in cmddict:
-                osdtarget = cmddict.pop('target')
+                osdtarget = cast(str, cmddict.pop('target'))
             try:
                 osdtarg.valid(osdtarget)
                 target = ('osd', osdtarg.nameid)
