@@ -1,25 +1,97 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import
-from typing import Any, cast, Dict, Iterable, List, Optional, Union
 
 import time
+from typing import Any, Dict, Iterable, List, Optional, Union, cast
+
 import cherrypy
 
-from . import ApiController, ControllerDoc, RESTController, Endpoint, ReadPermission, Task, \
-    UiApiController
 from .. import mgr
 from ..security import Scope
 from ..services.ceph_service import CephService
-from ..services.rbd import RbdConfiguration
 from ..services.exception import handle_send_command_error
-from ..tools import str_to_bool, TaskManager
+from ..services.rbd import RbdConfiguration
+from ..tools import TaskManager, str_to_bool
+from . import APIDoc, APIRouter, Endpoint, EndpointDoc, ReadPermission, \
+    RESTController, Task, UIRouter
+
+POOL_SCHEMA = ([{
+    "pool": (int, "pool id"),
+    "pool_name": (str, "pool name"),
+    "flags": (int, ""),
+    "flags_names": (str, "flags name"),
+    "type": (str, "type of pool"),
+    "size": (int, "pool size"),
+    "min_size": (int, ""),
+    "crush_rule": (str, ""),
+    "object_hash": (int, ""),
+    "pg_autoscale_mode": (str, ""),
+    "pg_num": (int, ""),
+    "pg_placement_num": (int, ""),
+    "pg_placement_num_target": (int, ""),
+    "pg_num_target": (int, ""),
+    "pg_num_pending": (int, ""),
+    "last_pg_merge_meta": ({
+        "ready_epoch": (int, ""),
+        "last_epoch_started": (int, ""),
+        "last_epoch_clean": (int, ""),
+        "source_pgid": (str, ""),
+        "source_version": (str, ""),
+        "target_version": (str, ""),
+    }, ""),
+    "auid": (int, ""),
+    "snap_mode": (str, ""),
+    "snap_seq": (int, ""),
+    "snap_epoch": (int, ""),
+    "pool_snaps": ([str], ""),
+    "quota_max_bytes": (int, ""),
+    "quota_max_objects": (int, ""),
+    "tiers": ([str], ""),
+    "tier_of": (int, ""),
+    "read_tier": (int, ""),
+    "write_tier": (int, ""),
+    "cache_mode": (str, ""),
+    "target_max_bytes": (int, ""),
+    "target_max_objects": (int, ""),
+    "cache_target_dirty_ratio_micro": (int, ""),
+    "cache_target_dirty_high_ratio_micro": (int, ""),
+    "cache_target_full_ratio_micro": (int, ""),
+    "cache_min_flush_age": (int, ""),
+    "cache_min_evict_age": (int, ""),
+    "erasure_code_profile": (str, ""),
+    "hit_set_params": ({
+        "type": (str, "")
+    }, ""),
+    "hit_set_period": (int, ""),
+    "hit_set_count": (int, ""),
+    "use_gmt_hitset": (bool, ""),
+    "min_read_recency_for_promote": (int, ""),
+    "min_write_recency_for_promote": (int, ""),
+    "hit_set_grade_decay_rate": (int, ""),
+    "hit_set_search_last_n": (int, ""),
+    "grade_table": ([str], ""),
+    "stripe_width": (int, ""),
+    "expected_num_objects": (int, ""),
+    "fast_read": (bool, ""),
+    "options": ({
+        "pg_num_min": (int, ""),
+        "pg_num_max": (int, "")
+    }, ""),
+    "application_metadata": ([str], ""),
+    "create_time": (str, ""),
+    "last_change": (str, ""),
+    "last_force_op_resend": (str, ""),
+    "last_force_op_resend_prenautilus": (str, ""),
+    "last_force_op_resend_preluminous": (str, ""),
+    "removed_snaps": ([str], "")
+}])
 
 
 def pool_task(name, metadata, wait_for=2.0):
     return Task("pool/{}".format(name), metadata, wait_for)
 
 
-@ApiController('/pool', Scope.POOL)
+@APIRouter('/pool', Scope.POOL)
+@APIDoc("Get pool details by pool name", "Pool")
 class Pool(RESTController):
 
     @staticmethod
@@ -58,6 +130,12 @@ class Pool(RESTController):
 
         return [cls._serialize_pool(pool, attrs) for pool in pools]
 
+    @EndpointDoc("Display Pool List",
+                 parameters={
+                     'attrs': (str, 'Pool Attributes'),
+                     'stats': (bool, 'Pool Stats')
+                 },
+                 responses={200: POOL_SCHEMA})
     def list(self, attrs=None, stats=False):
         return self._pool_list(attrs, stats)
 
@@ -206,8 +284,8 @@ class Pool(RESTController):
         return RbdConfiguration(pool_name).list()
 
 
-@UiApiController('/pool', Scope.POOL)
-@ControllerDoc("Dashboard UI helper function; not part of the public API", "PoolUi")
+@UIRouter('/pool', Scope.POOL)
+@APIDoc("Dashboard UI helper function; not part of the public API", "PoolUi")
 class PoolUi(Pool):
     @Endpoint()
     @ReadPermission
