@@ -2,10 +2,10 @@ import { HttpClientTestingModule } from '@angular/common/http/testing';
 import { ComponentFixture, TestBed } from '@angular/core/testing';
 import { RouterTestingModule } from '@angular/router/testing';
 
-import { TabsModule } from 'ngx-bootstrap/tabs';
+import { NgbNavModule } from '@ng-bootstrap/ng-bootstrap';
 
-import { configureTestBed, i18nProviders } from '../../../../testing/unit-test-helper';
-import { SharedModule } from '../../../shared/shared.module';
+import { SharedModule } from '~/app/shared/shared.module';
+import { configureTestBed } from '~/testing/unit-test-helper';
 import { RbdTabsComponent } from '../rbd-tabs/rbd-tabs.component';
 import { RbdPerformanceComponent } from './rbd-performance.component';
 
@@ -14,9 +14,8 @@ describe('RbdPerformanceComponent', () => {
   let fixture: ComponentFixture<RbdPerformanceComponent>;
 
   configureTestBed({
-    imports: [HttpClientTestingModule, RouterTestingModule, SharedModule, TabsModule.forRoot()],
-    declarations: [RbdPerformanceComponent, RbdTabsComponent],
-    providers: i18nProviders
+    imports: [HttpClientTestingModule, RouterTestingModule, SharedModule, NgbNavModule],
+    declarations: [RbdPerformanceComponent, RbdTabsComponent]
   });
 
   beforeEach(() => {
