@@ -1,40 +1,40 @@
 import { Component, Input, OnChanges, OnInit, TemplateRef, ViewChild } from '@angular/core';
 import { Validators } from '@angular/forms';
 
-import { I18n } from '@ngx-translate/i18n-polyfill';
 import {
   ITreeOptions,
-  TREE_ACTIONS,
   TreeComponent,
   TreeModel,
-  TreeNode
-} from 'angular-tree-component';
-import * as _ from 'lodash';
-import * as moment from 'moment';
-import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
+  TreeNode,
+  TREE_ACTIONS
+} from '@circlon/angular-tree-component';
+import { NgbModalRef } from '@ng-bootstrap/ng-bootstrap';
+import _ from 'lodash';
+import moment from 'moment';
 
-import { CephfsService } from '../../../shared/api/cephfs.service';
-import { ConfirmationModalComponent } from '../../../shared/components/confirmation-modal/confirmation-modal.component';
-import { CriticalConfirmationModalComponent } from '../../../shared/components/critical-confirmation-modal/critical-confirmation-modal.component';
-import { FormModalComponent } from '../../../shared/components/form-modal/form-modal.component';
-import { ActionLabelsI18n } from '../../../shared/constants/app.constants';
-import { Icons } from '../../../shared/enum/icons.enum';
-import { NotificationType } from '../../../shared/enum/notification-type.enum';
-import { CdValidators } from '../../../shared/forms/cd-validators';
-import { CdFormModalFieldConfig } from '../../../shared/models/cd-form-modal-field-config';
-import { CdTableAction } from '../../../shared/models/cd-table-action';
-import { CdTableColumn } from '../../../shared/models/cd-table-column';
-import { CdTableSelection } from '../../../shared/models/cd-table-selection';
+import { CephfsService } from '~/app/shared/api/cephfs.service';
+import { ConfirmationModalComponent } from '~/app/shared/components/confirmation-modal/confirmation-modal.component';
+import { CriticalConfirmationModalComponent } from '~/app/shared/components/critical-confirmation-modal/critical-confirmation-modal.component';
+import { FormModalComponent } from '~/app/shared/components/form-modal/form-modal.component';
+import { ActionLabelsI18n } from '~/app/shared/constants/app.constants';
+import { Icons } from '~/app/shared/enum/icons.enum';
+import { NotificationType } from '~/app/shared/enum/notification-type.enum';
+import { CdValidators } from '~/app/shared/forms/cd-validators';
+import { CdFormModalFieldConfig } from '~/app/shared/models/cd-form-modal-field-config';
+import { CdTableAction } from '~/app/shared/models/cd-table-action';
+import { CdTableColumn } from '~/app/shared/models/cd-table-column';
+import { CdTableSelection } from '~/app/shared/models/cd-table-selection';
 import {
   CephfsDir,
   CephfsQuotas,
   CephfsSnapshot
-} from '../../../shared/models/cephfs-directory-models';
-import { Permission } from '../../../shared/models/permissions';
-import { CdDatePipe } from '../../../shared/pipes/cd-date.pipe';
-import { DimlessBinaryPipe } from '../../../shared/pipes/dimless-binary.pipe';
-import { AuthStorageService } from '../../../shared/services/auth-storage.service';
-import { NotificationService } from '../../../shared/services/notification.service';
+} from '~/app/shared/models/cephfs-directory-models';
+import { Permission } from '~/app/shared/models/permissions';
+import { CdDatePipe } from '~/app/shared/pipes/cd-date.pipe';
+import { DimlessBinaryPipe } from '~/app/shared/pipes/dimless-binary.pipe';
+import { AuthStorageService } from '~/app/shared/services/auth-storage.service';
+import { ModalService } from '~/app/shared/services/modal.service';
+import { NotificationService } from '~/app/shared/services/notification.service';
 
 class QuotaSetting {
   row: {
@@ -57,7 +57,7 @@ class QuotaSetting {
   styleUrls: ['./cephfs-directories.component.scss']
 })
 export class CephfsDirectoriesComponent implements OnInit, OnChanges {
-  @ViewChild(TreeComponent, { static: false })
+  @ViewChild(TreeComponent)
   treeComponent: TreeComponent;
   @ViewChild('origin', { static: true })
   originTmpl: TemplateRef<any>;
@@ -65,7 +65,7 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
   @Input()
   id: number;
 
-  private modalRef: BsModalRef;
+  private modalRef: NgbModalRef;
   private dirs: CephfsDir[];
   private nodeIds: { [path: string]: CephfsDir };
   private requestedPaths: string[];
@@ -106,10 +106,9 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
 
   constructor(
     private authStorageService: AuthStorageService,
-    private modalService: BsModalService,
+    private modalService: ModalService,
     private cephfsService: CephfsService,
     private cdDatePipe: CdDatePipe,
-    private i18n: I18n,
     private actionLabels: ActionLabelsI18n,
     private notificationService: NotificationService,
     private dimlessBinaryPipe: DimlessBinaryPipe
@@ -140,18 +139,18 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
       columns: [
         {
           prop: 'row.name',
-          name: this.i18n('Name'),
+          name: $localize`Name`,
           flexGrow: 1
         },
         {
           prop: 'row.value',
-          name: this.i18n('Value'),
+          name: $localize`Value`,
           sortable: false,
           flexGrow: 1
         },
         {
           prop: 'row.originPath',
-          name: this.i18n('Origin'),
+          name: $localize`Origin`,
           sortable: false,
           cellTemplate: this.originTmpl,
           flexGrow: 1
@@ -194,18 +193,18 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
       columns: [
         {
           prop: 'name',
-          name: this.i18n('Name'),
+          name: $localize`Name`,
           flexGrow: 1
         },
         {
           prop: 'path',
-          name: this.i18n('Path'),
+          name: $localize`Path`,
           isHidden: true,
           flexGrow: 2
         },
         {
           prop: 'created',
-          name: this.i18n('Created'),
+          name: $localize`Created`,
           flexGrow: 1,
           pipe: this.cdDatePipe
         }
@@ -220,7 +219,8 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
           icon: Icons.add,
           permission: 'create',
           canBePrimary: (selection) => !selection.hasSelection,
-          click: () => this.createSnapshot()
+          click: () => this.createSnapshot(),
+          disable: () => this.disableCreateSnapshot()
         },
         {
           name: this.actionLabels.DELETE,
@@ -234,6 +234,16 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
     };
   }
 
+  private disableCreateSnapshot(): string | boolean {
+    const folders = this.selectedDir.path.split('/').slice(1);
+    // With deph of 4 or more we have the subvolume files/folders for which we cannot create
+    // a snapshot. Somehow, you can create a snapshot of the subvolume but not its files.
+    if (folders.length >= 4 && folders[0] === 'volumes') {
+      return $localize`Cannot create snapshots for files/folders in the subvolume ${folders[2]}`;
+    }
+    return false;
+  }
+
   ngOnChanges() {
     this.selectedDir = undefined;
     this.dirs = [];
@@ -351,7 +361,7 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
     }
     return {
       row: {
-        name: quotaKey === 'max_bytes' ? this.i18n('Max size') : this.i18n('Max files'),
+        name: quotaKey === 'max_bytes' ? $localize`Max size` : $localize`Max files`,
         value: valueConvertFn(value),
         originPath: value ? dir.path : ''
       },
@@ -416,33 +426,28 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
     const key = selection.quotaKey;
     const value = selection.dirValue;
     this.modalService.show(FormModalComponent, {
-      initialState: {
-        titleText: this.getModalQuotaTitle(
-          value === 0 ? this.actionLabels.SET : this.actionLabels.UPDATE,
-          path
-        ),
-        message: nextMax.value
-          ? this.i18n('The inherited {{quotaValue}} is the maximum value to be used.', {
-              quotaValue: this.getQuotaValueFromPathMsg(nextMax.value, nextMax.path)
-            })
-          : undefined,
-        fields: [this.getQuotaFormField(selection.row.name, key, value, nextMax.value)],
-        submitButtonText: this.i18n('Save'),
-        onSubmit: (values: CephfsQuotas) => this.updateQuota(values)
-      }
+      titleText: this.getModalQuotaTitle(
+        value === 0 ? this.actionLabels.SET : this.actionLabels.UPDATE,
+        path
+      ),
+      message: nextMax.value
+        ? $localize`The inherited ${this.getQuotaValueFromPathMsg(
+            nextMax.value,
+            nextMax.path
+          )} is the maximum value to be used.`
+        : undefined,
+      fields: [this.getQuotaFormField(selection.row.name, key, value, nextMax.value)],
+      submitButtonText: $localize`Save`,
+      onSubmit: (values: CephfsQuotas) => this.updateQuota(values)
     });
   }
 
   private getModalQuotaTitle(action: string, path: string): string {
-    return this.i18n("{{action}} CephFS {{quotaName}} quota for '{{path}}'", {
-      action,
-      quotaName: this.getQuotaName(),
-      path
-    });
+    return $localize`${action} CephFS ${this.getQuotaName()} quota for '${path}'`;
   }
 
   private getQuotaName(): string {
-    return this.isBytesQuotaSelected() ? this.i18n('size') : this.i18n('files');
+    return this.isBytesQuotaSelected() ? $localize`size` : $localize`files`;
   }
 
   private isBytesQuotaSelected(): boolean {
@@ -450,11 +455,9 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
   }
 
   private getQuotaValueFromPathMsg(value: number, path: string): string {
-    return this.i18n("{{quotaName}} quota {{value}} from '{{path}}'", {
-      value: this.isBytesQuotaSelected() ? this.dimlessBinaryPipe.transform(value) : value,
-      quotaName: this.getQuotaName(),
-      path
-    });
+    value = this.isBytesQuotaSelected() ? this.dimlessBinaryPipe.transform(value) : value;
+
+    return $localize`${this.getQuotaName()} quota ${value} from '${path}'`;
   }
 
   private getQuotaFormField(
@@ -478,8 +481,8 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
     };
     if (!isBinary) {
       field.errors = {
-        min: this.i18n(`Value has to be at least {{value}} or more`, { value: 0 }),
-        max: this.i18n(`Value has to be at most {{value}} or less`, { value: maxValue })
+        min: $localize`Value has to be at least 0 or more`,
+        max: $localize`Value has to be at most ${maxValue} or less`
       };
     }
     return field;
@@ -493,8 +496,8 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
         ? this.actionLabels.SET
         : values[key] === 0
         ? this.actionLabels.UNSET
-        : this.i18n('Updated');
-    this.cephfsService.updateQuota(this.id, path, values).subscribe(() => {
+        : $localize`Updated`;
+    this.cephfsService.quota(this.id, path, values).subscribe(() => {
       if (onSuccess) {
         onSuccess();
       }
@@ -513,26 +516,22 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
     const nextMax = selection.nextTreeMaximum;
     const dirValue = selection.dirValue;
 
+    const quotaValue = this.getQuotaValueFromPathMsg(nextMax.value, nextMax.path);
+    const conclusion =
+      nextMax.value > 0
+        ? nextMax.value > dirValue
+          ? $localize`in order to inherit ${quotaValue}`
+          : $localize`which isn't used because of the inheritance of ${quotaValue}`
+        : $localize`in order to have no quota on the directory`;
+
     this.modalRef = this.modalService.show(ConfirmationModalComponent, {
-      initialState: {
-        titleText: this.getModalQuotaTitle(this.actionLabels.UNSET, path),
-        buttonText: this.actionLabels.UNSET,
-        description: this.i18n(`{{action}} {{quotaValue}} {{conclusion}}.`, {
-          action: this.actionLabels.UNSET,
-          quotaValue: this.getQuotaValueFromPathMsg(dirValue, path),
-          conclusion:
-            nextMax.value > 0
-              ? nextMax.value > dirValue
-                ? this.i18n('in order to inherit {{quotaValue}}', {
-                    quotaValue: this.getQuotaValueFromPathMsg(nextMax.value, nextMax.path)
-                  })
-                : this.i18n("which isn't used because of the inheritance of {{quotaValue}}", {
-                    quotaValue: this.getQuotaValueFromPathMsg(nextMax.value, nextMax.path)
-                  })
-              : this.i18n('in order to have no quota on the directory')
-        }),
-        onSubmit: () => this.updateQuota({ [key]: 0 }, () => this.modalRef.hide())
-      }
+      titleText: this.getModalQuotaTitle(this.actionLabels.UNSET, path),
+      buttonText: this.actionLabels.UNSET,
+      description: $localize`${this.actionLabels.UNSET} ${this.getQuotaValueFromPathMsg(
+        dirValue,
+        path
+      )} ${conclusion}.`,
+      onSubmit: () => this.updateQuota({ [key]: 0 }, () => this.modalRef.close())
     });
   }
 
@@ -540,30 +539,25 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
     // Create a snapshot. Auto-generate a snapshot name by default.
     const path = this.selectedDir.path;
     this.modalService.show(FormModalComponent, {
-      initialState: {
-        titleText: this.i18n('Create Snapshot'),
-        message: this.i18n('Please enter the name of the snapshot.'),
-        fields: [
-          {
-            type: 'text',
-            name: 'name',
-            value: `${moment().toISOString(true)}`,
-            required: true
-          }
-        ],
-        submitButtonText: this.i18n('Create Snapshot'),
-        onSubmit: (values: CephfsSnapshot) => {
-          this.cephfsService.mkSnapshot(this.id, path, values.name).subscribe((name) => {
-            this.notificationService.show(
-              NotificationType.success,
-              this.i18n('Created snapshot "{{name}}" for "{{path}}"', {
-                name: name,
-                path: path
-              })
-            );
-            this.forceDirRefresh();
-          });
+      titleText: $localize`Create Snapshot`,
+      message: $localize`Please enter the name of the snapshot.`,
+      fields: [
+        {
+          type: 'text',
+          name: 'name',
+          value: `${moment().toISOString(true)}`,
+          required: true
         }
+      ],
+      submitButtonText: $localize`Create Snapshot`,
+      onSubmit: (values: CephfsSnapshot) => {
+        this.cephfsService.mkSnapshot(this.id, path, values.name).subscribe((name) => {
+          this.notificationService.show(
+            NotificationType.success,
+            $localize`Created snapshot '${name}' for '${path}'`
+          );
+          this.forceDirRefresh();
+        });
       }
     });
   }
@@ -673,13 +667,9 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
 
   deleteSnapshotModal() {
     this.modalRef = this.modalService.show(CriticalConfirmationModalComponent, {
-      initialState: {
-        itemDescription: this.i18n('CephFs Snapshot'),
-        itemNames: this.snapshot.selection.selected.map(
-          (snapshot: CephfsSnapshot) => snapshot.name
-        ),
-        submitAction: () => this.deleteSnapshot()
-      }
+      itemDescription: $localize`CephFs Snapshot`,
+      itemNames: this.snapshot.selection.selected.map((snapshot: CephfsSnapshot) => snapshot.name),
+      submitAction: () => this.deleteSnapshot()
     });
   }
 
@@ -690,14 +680,11 @@ export class CephfsDirectoriesComponent implements OnInit, OnChanges {
       this.cephfsService.rmSnapshot(this.id, path, name).subscribe(() => {
         this.notificationService.show(
           NotificationType.success,
-          this.i18n('Deleted snapshot "{{name}}" for "{{path}}"', {
-            name: name,
-            path: path
-          })
+          $localize`Deleted snapshot '${name}' for '${path}'`
         );
       });
     });
-    this.modalRef.hide();
+    this.modalRef.close();
     this.forceDirRefresh();
   }
 
