@@ -3,17 +3,14 @@ import { ComponentFixture, TestBed } from '@angular/core/testing';
 import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
 import { RouterTestingModule } from '@angular/router/testing';
 
-import { NgBootstrapFormValidationModule } from 'ng-bootstrap-form-validation';
-import { BsDropdownModule } from 'ngx-bootstrap/dropdown';
-import { TabsModule } from 'ngx-bootstrap/tabs';
 import { ToastrModule } from 'ngx-toastr';
 
-import { configureTestBed, i18nProviders } from '../../../../../testing/unit-test-helper';
-import { CoreModule } from '../../../../core/core.module';
-import { Permissions } from '../../../../shared/models/permissions';
-import { SharedModule } from '../../../../shared/shared.module';
-import { CephModule } from '../../../ceph.module';
-import { CephSharedModule } from '../../../shared/ceph-shared.module';
+import { CephModule } from '~/app/ceph/ceph.module';
+import { CephSharedModule } from '~/app/ceph/shared/ceph-shared.module';
+import { CoreModule } from '~/app/core/core.module';
+import { Permissions } from '~/app/shared/models/permissions';
+import { SharedModule } from '~/app/shared/shared.module';
+import { configureTestBed, TabHelper } from '~/testing/unit-test-helper';
 import { HostDetailsComponent } from './host-details.component';
 
 describe('HostDetailsComponent', () => {
@@ -24,18 +21,13 @@ describe('HostDetailsComponent', () => {
     imports: [
       BrowserAnimationsModule,
       HttpClientTestingModule,
-      TabsModule.forRoot(),
-      BsDropdownModule.forRoot(),
-      NgBootstrapFormValidationModule.forRoot(),
       RouterTestingModule,
       CephModule,
       CoreModule,
       CephSharedModule,
       SharedModule,
       ToastrModule.forRoot()
-    ],
-    declarations: [],
-    providers: [i18nProviders]
+    ]
   });
 
   beforeEach(() => {
@@ -59,17 +51,17 @@ describe('HostDetailsComponent', () => {
     });
 
     it('should recognize a tabset child', () => {
-      const tabsetChild = component.tabsetChild;
+      const tabsetChild = TabHelper.getNgbNav(fixture);
       expect(tabsetChild).toBeDefined();
     });
 
     it('should show tabs', () => {
-      expect(component.tabsetChild.tabs.map((t) => t.heading)).toEqual([
+      expect(TabHelper.getTextContents(fixture)).toEqual([
         'Devices',
-        'Device health',
-        'Inventory',
+        'Physical Disks',
         'Daemons',
-        'Performance Details'
+        'Performance Details',
+        'Device health'
       ]);
     });
   });
