@@ -2,24 +2,16 @@ import { CommonModule } from '@angular/common';
 import { NgModule } from '@angular/core';
 import { ReactiveFormsModule } from '@angular/forms';
 
-import { NgBootstrapFormValidationModule } from 'ng-bootstrap-form-validation';
-import { TabsModule } from 'ngx-bootstrap/tabs';
+import { NgbNavModule } from '@ng-bootstrap/ng-bootstrap';
 
-import { AppRoutingModule } from '../../../app-routing.module';
-import { SharedModule } from '../../../shared/shared.module';
+import { AppRoutingModule } from '~/app/app-routing.module';
+import { SharedModule } from '~/app/shared/shared.module';
 import { MgrModuleDetailsComponent } from './mgr-module-details/mgr-module-details.component';
 import { MgrModuleFormComponent } from './mgr-module-form/mgr-module-form.component';
 import { MgrModuleListComponent } from './mgr-module-list/mgr-module-list.component';
 
 @NgModule({
-  imports: [
-    AppRoutingModule,
-    CommonModule,
-    ReactiveFormsModule,
-    SharedModule,
-    TabsModule.forRoot(),
-    NgBootstrapFormValidationModule
-  ],
+  imports: [AppRoutingModule, CommonModule, ReactiveFormsModule, SharedModule, NgbNavModule],
   declarations: [MgrModuleListComponent, MgrModuleFormComponent, MgrModuleDetailsComponent]
 })
 export class MgrModulesModule {}
