@@ -1,23 +1,20 @@
 import { Pipe, PipeTransform } from '@angular/core';
 
-import { I18n } from '@ngx-translate/i18n-polyfill';
-import * as _ from 'lodash';
+import _ from 'lodash';
 
 @Pipe({
   name: 'mgrSummary'
 })
 export class MgrSummaryPipe implements PipeTransform {
-  constructor(private i18n: I18n) {}
-
   transform(value: any): any {
     if (!value) {
       return '';
     }
 
-    let activeCount = this.i18n('n/a');
+    let activeCount = $localize`n/a`;
     const activeTitleText = _.isUndefined(value.active_name)
       ? ''
-      : `${this.i18n('active daemon')}: ${value.active_name}`;
+      : `${$localize`active daemon`}: ${value.active_name}`;
     // There is always one standbyreplay to replace active daemon, if active one is down
     if (activeTitleText.length > 0) {
       activeCount = '1';
@@ -25,11 +22,11 @@ export class MgrSummaryPipe implements PipeTransform {
     const standbyHoverText = value.standbys.map((s: any): string => s.name).join(', ');
     const standbyTitleText = !standbyHoverText
       ? ''
-      : `${this.i18n('standby daemons')}: ${standbyHoverText}`;
+      : `${$localize`standby daemons`}: ${standbyHoverText}`;
     const standbyCount = value.standbys.length;
     const mgrSummary = [
       {
-        content: `${activeCount} ${this.i18n('active')}`,
+        content: `${activeCount} ${$localize`active`}`,
         class: 'popover-info',
         titleText: activeTitleText
       }
@@ -41,7 +38,7 @@ export class MgrSummaryPipe implements PipeTransform {
       titleText: ''
     });
     mgrSummary.push({
-      content: `${standbyCount} ${this.i18n('standby')}`,
+      content: `${standbyCount} ${$localize`standby`}`,
       class: 'popover-info',
       titleText: standbyTitleText
     });
