@@ -1,5 +1,6 @@
 import { HttpClientTestingModule } from '@angular/common/http/testing';
-import { ComponentFixture, TestBed } from '@angular/core/testing';
+import { ComponentFixture, fakeAsync, TestBed, tick } from '@angular/core/testing';
+import { By } from '@angular/platform-browser';
 import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
 import { RouterTestingModule } from '@angular/router/testing';
 
@@ -7,6 +8,8 @@ import { NgbNavModule } from '@ng-bootstrap/ng-bootstrap';
 import { of } from 'rxjs';
 
 import { PerformanceCounterModule } from '~/app/ceph/performance-counter/performance-counter.module';
+import { RgwDaemon } from '~/app/ceph/rgw/models/rgw-daemon';
+import { RgwDaemonService } from '~/app/shared/api/rgw-daemon.service';
 import { RgwSiteService } from '~/app/shared/api/rgw-site.service';
 import { Permissions } from '~/app/shared/models/permissions';
 import { AuthStorageService } from '~/app/shared/services/auth-storage.service';
@@ -20,7 +23,18 @@ describe('RgwDaemonListComponent', () => {
   let fixture: ComponentFixture<RgwDaemonListComponent>;
   let getPermissionsSpy: jasmine.Spy;
   let getRealmsSpy: jasmine.Spy;
+  let listDaemonsSpy: jest.SpyInstance;
   const permissions = new Permissions({ grafana: ['read'] });
+  const daemon: RgwDaemon = {
+    id: '8000',
+    service_map_id: '4803',
+    version: 'ceph version',
+    server_hostname: 'ceph',
+    realm_name: 'realm1',
+    zonegroup_name: 'zg1-realm1',
+    zone_name: 'zone1-zg1-realm1',
+    default: true
+  };
 
   const expectTabsAndHeading = (length: number, heading: string) => {
     const tabs = TabHelper.getTextContents(fixture);
@@ -45,6 +59,9 @@ describe('RgwDaemonListComponent', () => {
     getPermissionsSpy.and.returnValue(new Permissions({}));
     getRealmsSpy = spyOn(TestBed.inject(RgwSiteService), 'get');
     getRealmsSpy.and.returnValue(of([]));
+    listDaemonsSpy = jest
+      .spyOn(TestBed.inject(RgwDaemonService), 'list')
+      .mockReturnValue(of([daemon]));
     fixture = TestBed.createComponent(RgwDaemonListComponent);
     component = fixture.componentInstance;
   });
@@ -54,6 +71,18 @@ describe('RgwDaemonListComponent', () => {
     expect(component).toBeTruthy();
   });
 
+  it('should show a row with daemon info', fakeAsync(() => {
+    fixture.detectChanges();
+    tick();
+    expect(listDaemonsSpy).toHaveBeenCalledTimes(1);
+    expect(component.daemons).toEqual([daemon]);
+    expect(fixture.debugElement.query(By.css('cd-table')).nativeElement.textContent).toContain(
+      'total 1'
+    );
+
+    fixture.destroy();
+  }));
+
   it('should only show Daemons List tab', () => {
     fixture.detectChanges();
 
