@@ -2,20 +2,16 @@
 """
 ceph dashboard mgr plugin (based on CherryPy)
 """
-from __future__ import absolute_import
-
 import collections
 import errno
 import logging
 import os
-import socket
 import ssl
 import sys
 import tempfile
 import threading
 import time
 from typing import TYPE_CHECKING, Optional
-from urllib.parse import urlparse
 
 if TYPE_CHECKING:
     if sys.version_info >= (3, 8):
@@ -23,8 +19,8 @@ if TYPE_CHECKING:
     else:
         from typing_extensions import Literal
 
-from mgr_module import CLIReadCommand, CLIWriteCommand, HandleCommandResult, \
-    MgrModule, MgrStandbyModule, NotifyType, Option, _get_localized_key
+from mgr_module import CLIWriteCommand, HandleCommandResult, MgrModule, \
+    MgrStandbyModule, NotifyType, Option, _get_localized_key
 from mgr_util import ServerConfigException, build_url, \
     create_self_signed_cert, get_default_addr, verify_tls_files
 
@@ -121,7 +117,6 @@ class CherryPyConfig(object):
 
         # Initialize custom handlers.
         cherrypy.tools.authenticate = AuthManagerTool()
-        self.configure_cors()
         cherrypy.tools.plugin_hooks_filter_request = cherrypy.Tool(
             'before_handler',
             lambda: PLUGIN_MANAGER.hook.filter_request_before_handler(request=cherrypy.request),
@@ -180,9 +175,9 @@ class CherryPyConfig(object):
             context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
             context.load_cert_chain(cert_fname, pkey_fname)
             if sys.version_info >= (3, 7):
-                context.minimum_version = ssl.TLSVersion.TLSv1_3
+                context.minimum_version = ssl.TLSVersion.TLSv1_2
             else:
-                context.options |= ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1 | ssl.OP_NO_TLSv1_2
+                context.options |= ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1
 
             config['server.ssl_module'] = 'builtin'
             config['server.ssl_certificate'] = cert_fname
@@ -224,70 +219,6 @@ class CherryPyConfig(object):
                 self.log.info("Configured CherryPy, starting engine...")  # type: ignore
                 return uri
 
-    def configure_cors(self):
-        """
-        Allow CORS requests if the cross_origin_url option is set.
-        """
-        cross_origin_url = mgr.get_localized_module_option('cross_origin_url', '')
-        if cross_origin_url:
-            cherrypy.tools.CORS = cherrypy.Tool('before_handler', self.cors_tool)
-            config = {
-                'tools.CORS.on': True,
-            }
-            self.update_cherrypy_config(config)
-
-    def cors_tool(self):
-        '''
-        Handle both simple and complex CORS requests
-
-        Add CORS headers to each response. If the request is a CORS preflight
-        request swap out the default handler with a simple, single-purpose handler
-        that verifies the request and provides a valid CORS response.
-        '''
-        req_head = cherrypy.request.headers
-        resp_head = cherrypy.response.headers
-
-        # Always set response headers necessary for 'simple' CORS.
-        req_header_cross_origin_url = req_head.get('Access-Control-Allow-Origin')
-        cross_origin_urls = mgr.get_localized_module_option('cross_origin_url', '')
-        cross_origin_url_list = [url.strip() for url in cross_origin_urls.split(',')]
-        if req_header_cross_origin_url in cross_origin_url_list:
-            resp_head['Access-Control-Allow-Origin'] = req_header_cross_origin_url
-        resp_head['Access-Control-Expose-Headers'] = 'GET, POST'
-        resp_head['Access-Control-Allow-Credentials'] = 'true'
-
-        # Non-simple CORS preflight request; short-circuit the normal handler.
-        if cherrypy.request.method == 'OPTIONS':
-            req_header_origin_url = req_head.get('Origin')
-            if req_header_origin_url in cross_origin_url_list:
-                resp_head['Access-Control-Allow-Origin'] = req_header_origin_url
-            ac_method = req_head.get('Access-Control-Request-Method', None)
-
-            allowed_methods = ['GET', 'POST', 'PUT']
-            allowed_headers = [
-                'Content-Type',
-                'Authorization',
-                'Accept',
-                'Access-Control-Allow-Origin'
-            ]
-
-            if ac_method and ac_method in allowed_methods:
-                resp_head['Access-Control-Allow-Methods'] = ', '.join(allowed_methods)
-                resp_head['Access-Control-Allow-Headers'] = ', '.join(allowed_headers)
-
-                resp_head['Connection'] = 'keep-alive'
-                resp_head['Access-Control-Max-Age'] = '3600'
-
-            # CORS requests should short-circuit the other tools.
-            cherrypy.response.body = ''.encode('utf8')
-            cherrypy.response.status = 200
-            cherrypy.serving.request.handler = None
-
-            # Needed to avoid the auth_tool check.
-            if cherrypy.request.config.get('tools.sessions.on', False):
-                cherrypy.session['token'] = True
-            return True
-
 
 if TYPE_CHECKING:
     SslConfigKey = Literal['crt', 'key']
@@ -337,9 +268,7 @@ class Module(MgrModule, CherryPyConfig):
         Option(name='standby_behaviour', type='str', default='redirect',
                enum_allowed=['redirect', 'error']),
         Option(name='standby_error_status_code', type='int', default=500,
-               min=400, max=599),
-        Option(name='redirect_resolve_ip_addr', type='bool', default=False),
-        Option(name='cross_origin_url', type='str', default=''),
+               min=400, max=599)
     ]
     MODULE_OPTIONS.extend(options_schema_list())
     for options in PLUGIN_MANAGER.hook.get_options() or []:
@@ -376,7 +305,15 @@ class Module(MgrModule, CherryPyConfig):
     @classmethod
     def get_frontend_path(cls):
         current_dir = os.path.dirname(os.path.abspath(__file__))
-        return os.path.join(current_dir, 'frontend/dist')
+        path = os.path.join(current_dir, 'frontend/dist')
+        if os.path.exists(path):
+            return path
+        else:
+            path = os.path.join(current_dir,
+                                '../../../../build',
+                                'src/pybind/mgr/dashboard',
+                                'frontend/dist')
+            return os.path.abspath(path)
 
     def serve(self):
 
@@ -480,39 +417,6 @@ class Module(MgrModule, CherryPyConfig):
 
         return 0, 'RGW credentials configured', ''
 
-    @CLIWriteCommand("dashboard set-login-banner")
-    def set_login_banner(self, inbuf: str):
-        '''
-        Set the custom login banner read from -i <file>
-        '''
-        item_label = 'login banner file'
-        if inbuf is None:
-            return HandleCommandResult(
-                -errno.EINVAL,
-                stderr=f'Please specify the {item_label} with "-i" option'
-            )
-        mgr.set_store('custom_login_banner', inbuf)
-        return HandleCommandResult(stdout=f'{item_label} added')
-
-    @CLIReadCommand("dashboard get-login-banner")
-    def get_login_banner(self):
-        '''
-        Get the custom login banner text
-        '''
-        banner_text = mgr.get_store('custom_login_banner')
-        if banner_text is None:
-            return HandleCommandResult(stdout='No login banner set')
-        else:
-            return HandleCommandResult(stdout=banner_text)
-
-    @CLIWriteCommand("dashboard unset-login-banner")
-    def unset_login_banner(self):
-        '''
-        Unset the custom login banner
-        '''
-        mgr.set_store('custom_login_banner', None)
-        return HandleCommandResult(stdout='Login banner removed')
-
     def handle_command(self, inbuf, cmd):
         # pylint: disable=too-many-return-statements
         res = handle_option_command(cmd, inbuf)
@@ -580,21 +484,7 @@ class StandbyModule(MgrStandbyModule, CherryPyConfig):
             def default(self, *args, **kwargs):
                 if module.get_module_option('standby_behaviour', 'redirect') == 'redirect':
                     active_uri = module.get_active_uri()
-
-                    if cherrypy.request.path_info.startswith('/api/prometheus_receiver'):
-                        module.log.debug("Suppressed redirecting alert to active '%s'",
-                                         active_uri)
-                        cherrypy.response.status = 204
-                        return None
-
                     if active_uri:
-                        if module.get_module_option('redirect_resolve_ip_addr'):
-                            p_result = urlparse(active_uri)
-                            hostname = str(p_result.hostname)
-                            fqdn_netloc = p_result.netloc.replace(
-                                hostname, socket.getfqdn(hostname))
-                            active_uri = p_result._replace(netloc=fqdn_netloc).geturl()
-
                         module.log.info("Redirecting to active '%s'", active_uri)
                         raise cherrypy.HTTPRedirect(active_uri)
                     else:
