@@ -11,14 +11,8 @@ start_ceph() {
     set -x
 
     # Create an Object Gateway User
-    ./bin/radosgw-admin user create --uid=dev --display-name=Developer --system
-    # Obtain and set access and secret key for the previously created user. $() is safer than backticks `..`
-    RGW_ACCESS_KEY_FILE="/tmp/rgw-user-access-key.txt"
-    printf "$(./bin/radosgw-admin user info --uid=dev | jq -r .keys[0].access_key)" > "${RGW_ACCESS_KEY_FILE}"
-    ./bin/ceph dashboard set-rgw-api-access-key -i "${RGW_ACCESS_KEY_FILE}"
-    RGW_SECRET_KEY_FILE="/tmp/rgw-user-secret-key.txt"
-    printf "$(./bin/radosgw-admin user info --uid=dev | jq -r .keys[0].secret_key)" > "${RGW_SECRET_KEY_FILE}"
-    ./bin/ceph dashboard set-rgw-api-secret-key -i "${RGW_SECRET_KEY_FILE}"
+    ./bin/ceph dashboard set-rgw-credentials
+
     # Set SSL verify to False
     ./bin/ceph dashboard set-rgw-api-ssl-verify False
 
@@ -85,7 +79,7 @@ DASH_DIR=`pwd`
 cd ../../../../${BUILD_DIR}
 FULL_PATH_BUILD_DIR=`pwd`
 
-[[ "$(command -v npm)" == '' ]] && . ${FULL_PATH_BUILD_DIR}/src/pybind/mgr/dashboard/node-env/bin/activate
+[[ "$(command -v npm)" == '' ]] && . ${FULL_PATH_BUILD_DIR}/src/pybind/mgr/dashboard/frontend/node-env/bin/activate
 
 : ${CYPRESS_CACHE_FOLDER:="${FULL_PATH_BUILD_DIR}/src/pybind/mgr/dashboard/cypress"}
 
@@ -105,6 +99,7 @@ rm -f cypress/reports/results-*.xml || true
 case "$DEVICE" in
     docker)
         failed=0
+        CYPRESS_VERSION=$(cat package.json | grep '"cypress"' | grep -o "[0-9]\.[0-9]\.[0-9]")
         docker run \
             -v $(pwd):/e2e \
             -w /e2e \
@@ -113,7 +108,7 @@ case "$DEVICE" in
             --env CYPRESS_LOGIN_PWD \
             --name=e2e \
             --network=host \
-            cypress/included:5.1.0 || failed=1
+            cypress/included:${CYPRESS_VERSION} || failed=1
         stop $failed
         ;;
     *)
