@@ -1,30 +1,23 @@
 # -*- coding: utf-8 -*-
 # pylint: disable=too-many-return-statements,too-many-branches
-from __future__ import absolute_import
 
-import os
 import errno
 import json
 import logging
+import os
 import threading
 import warnings
-
-import six
-from six.moves.urllib import parse
+from urllib import parse
 
 from .. import mgr
 from ..tools import prepare_url_prefix
 
-
-if six.PY2:
-    FileNotFoundError = IOError  # pylint: disable=redefined-builtin
-
 logger = logging.getLogger('sso')
 
 try:
-    from onelogin.saml2.settings import OneLogin_Saml2_Settings as Saml2Settings
     from onelogin.saml2.errors import OneLogin_Saml2_Error as Saml2Error
     from onelogin.saml2.idp_metadata_parser import OneLogin_Saml2_IdPMetadataParser as Saml2Parser
+    from onelogin.saml2.settings import OneLogin_Saml2_Settings as Saml2Settings
 
     python_saml_imported = True
 except ImportError:
@@ -143,6 +136,7 @@ def _get_optional_attr(cmd, attr, default):
 
 
 def handle_sso_command(cmd):
+    ret = -errno.ENOSYS, '', ''
     if cmd['prefix'] not in ['dashboard sso enable saml2',
                              'dashboard sso disable',
                              'dashboard sso status',
@@ -150,23 +144,22 @@ def handle_sso_command(cmd):
                              'dashboard sso setup saml2']:
         return -errno.ENOSYS, '', ''
 
+    if not python_saml_imported:
+        return -errno.EPERM, '', 'Required library not found: `python3-saml`'
+
     if cmd['prefix'] == 'dashboard sso disable':
         mgr.SSO_DB.protocol = ''
         mgr.SSO_DB.save()
         return 0, 'SSO is "disabled".', ''
 
-    if not python_saml_imported:
-        return -errno.EPERM, '', 'Required library not found: `python3-saml`'
-
     if cmd['prefix'] == 'dashboard sso enable saml2':
-        try:
-            Saml2Settings(mgr.SSO_DB.saml2.onelogin_settings)
-        except Saml2Error:
-            return -errno.EPERM, '', 'Single Sign-On is not configured: ' \
-                          'use `ceph dashboard sso setup saml2`'
-        mgr.SSO_DB.protocol = 'saml2'
-        mgr.SSO_DB.save()
-        return 0, 'SSO is "enabled" with "SAML2" protocol.', ''
+        configured = _is_sso_configured()
+        if configured:
+            mgr.SSO_DB.protocol = 'saml2'
+            mgr.SSO_DB.save()
+            return 0, 'SSO is "enabled" with "SAML2" protocol.', ''
+        return -errno.EPERM, '', 'Single Sign-On is not configured: ' \
+            'use `ceph dashboard sso setup saml2`'
 
     if cmd['prefix'] == 'dashboard sso status':
         if mgr.SSO_DB.protocol == 'saml2':
@@ -178,86 +171,123 @@ def handle_sso_command(cmd):
         return 0, json.dumps(mgr.SSO_DB.saml2.to_dict()), ''
 
     if cmd['prefix'] == 'dashboard sso setup saml2':
-        ceph_dashboard_base_url = cmd['ceph_dashboard_base_url']
-        idp_metadata = cmd['idp_metadata']
-        idp_username_attribute = _get_optional_attr(cmd, 'idp_username_attribute', 'uid')
-        idp_entity_id = _get_optional_attr(cmd, 'idp_entity_id', None)
-        sp_x_509_cert_path = _get_optional_attr(cmd, 'sp_x_509_cert', '')
-        sp_private_key_path = _get_optional_attr(cmd, 'sp_private_key', '')
-        if sp_x_509_cert_path and not sp_private_key_path:
-            return -errno.EINVAL, '', 'Missing parameter `sp_private_key`.'
-        if not sp_x_509_cert_path and sp_private_key_path:
-            return -errno.EINVAL, '', 'Missing parameter `sp_x_509_cert`.'
-        has_sp_cert = sp_x_509_cert_path != "" and sp_private_key_path != ""
-        if has_sp_cert:
-            try:
-                with open(sp_x_509_cert_path, 'r', encoding='utf-8') as f:
-                    sp_x_509_cert = f.read()
-            except FileNotFoundError:
-                return -errno.EINVAL, '', '`{}` not found.'.format(sp_x_509_cert_path)
-            try:
-                with open(sp_private_key_path, 'r', encoding='utf-8') as f:
-                    sp_private_key = f.read()
-            except FileNotFoundError:
-                return -errno.EINVAL, '', '`{}` not found.'.format(sp_private_key_path)
-        else:
-            sp_x_509_cert = ''
-            sp_private_key = ''
-
-        if os.path.isfile(idp_metadata):
-            warnings.warn(
-                "Please prepend 'file://' to indicate a local SAML2 IdP file", DeprecationWarning)
-            with open(idp_metadata, 'r', encoding='utf-8') as f:
-                idp_settings = Saml2Parser.parse(f.read(), entity_id=idp_entity_id)
-        elif parse.urlparse(idp_metadata)[0] in ('http', 'https', 'file'):
-            idp_settings = Saml2Parser.parse_remote(
-                url=idp_metadata, validate_cert=False, entity_id=idp_entity_id)
-        else:
-            idp_settings = Saml2Parser.parse(idp_metadata, entity_id=idp_entity_id)
-
-        url_prefix = prepare_url_prefix(mgr.get_module_option('url_prefix', default=''))
-        settings = {
-            'sp': {
-                'entityId': '{}{}/auth/saml2/metadata'.format(ceph_dashboard_base_url, url_prefix),
-                'assertionConsumerService': {
-                    'url': '{}{}/auth/saml2'.format(ceph_dashboard_base_url, url_prefix),
-                    'binding': "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
-                },
-                'attributeConsumingService': {
-                    'serviceName': "Ceph Dashboard",
-                    "serviceDescription": "Ceph Dashboard Service",
-                    "requestedAttributes": [
-                        {
-                            "name": idp_username_attribute,
-                            "isRequired": True
-                        }
-                    ]
-                },
-                'singleLogoutService': {
-                    'url': '{}{}/auth/saml2/logout'.format(ceph_dashboard_base_url, url_prefix),
-                    'binding': 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect'
-                },
-                "x509cert": sp_x_509_cert,
-                "privateKey": sp_private_key
-            },
-            'security': {
-                "nameIdEncrypted": has_sp_cert,
-                "authnRequestsSigned": has_sp_cert,
-                "logoutRequestSigned": has_sp_cert,
-                "logoutResponseSigned": has_sp_cert,
-                "signMetadata": has_sp_cert,
-                "wantMessagesSigned": has_sp_cert,
-                "wantAssertionsSigned": has_sp_cert,
-                "wantAssertionsEncrypted": has_sp_cert,
-                "wantNameIdEncrypted": False,  # Not all Identity Providers support this.
-                "metadataValidUntil": '',
-                "wantAttributeStatement": False
-            }
-        }
-        settings = Saml2Parser.merge_settings(settings, idp_settings)
-        mgr.SSO_DB.saml2.onelogin_settings = settings
-        mgr.SSO_DB.protocol = 'saml2'
-        mgr.SSO_DB.save()
-        return 0, json.dumps(mgr.SSO_DB.saml2.onelogin_settings), ''
+        ret = _handle_saml_setup(cmd)
+        return ret
 
     return -errno.ENOSYS, '', ''
+
+
+def _is_sso_configured():
+    configured = True
+    try:
+        Saml2Settings(mgr.SSO_DB.saml2.onelogin_settings)
+    except Saml2Error:
+        configured = False
+    return configured
+
+
+def _handle_saml_setup(cmd):
+    err, sp_x_509_cert, sp_private_key, has_sp_cert = _read_saml_files(cmd)
+    if err:
+        ret = -errno.EINVAL, '', err
+    else:
+        _set_saml_settings(cmd, sp_x_509_cert, sp_private_key, has_sp_cert)
+        ret = 0, json.dumps(mgr.SSO_DB.saml2.onelogin_settings), ''
+    return ret
+
+
+def _read_saml_files(cmd):
+    sp_x_509_cert_path = _get_optional_attr(cmd, 'sp_x_509_cert', '')
+    sp_private_key_path = _get_optional_attr(cmd, 'sp_private_key', '')
+    has_sp_cert = sp_x_509_cert_path != "" and sp_private_key_path != ""
+    sp_x_509_cert = ''
+    sp_private_key = ''
+    err = None
+    if sp_x_509_cert_path and not sp_private_key_path:
+        err = 'Missing parameter `sp_private_key`.'
+    elif not sp_x_509_cert_path and sp_private_key_path:
+        err = 'Missing parameter `sp_x_509_cert`.'
+    elif has_sp_cert:
+        sp_x_509_cert, err = _try_read_file(sp_x_509_cert_path)
+        sp_private_key, err = _try_read_file(sp_private_key_path)
+    return err, sp_x_509_cert, sp_private_key, has_sp_cert
+
+
+def _try_read_file(path):
+    res = ""
+    ret = ""
+    try:
+        with open(path, 'r', encoding='utf-8') as f:
+            res = f.read()
+    except FileNotFoundError:
+        ret = '`{}` not found.'.format(path)
+    return res, ret
+
+
+def _set_saml_settings(cmd, sp_x_509_cert, sp_private_key, has_sp_cert):
+    ceph_dashboard_base_url = cmd['ceph_dashboard_base_url']
+    idp_metadata = cmd['idp_metadata']
+    idp_username_attribute = _get_optional_attr(
+        cmd, 'idp_username_attribute', 'uid')
+    idp_entity_id = _get_optional_attr(cmd, 'idp_entity_id', None)
+    idp_settings = _parse_saml_settings(idp_metadata, idp_entity_id)
+
+    url_prefix = prepare_url_prefix(
+        mgr.get_module_option('url_prefix', default=''))
+    settings = {
+        'sp': {
+            'entityId': '{}{}/auth/saml2/metadata'.format(ceph_dashboard_base_url, url_prefix),
+            'assertionConsumerService': {
+                'url': '{}{}/auth/saml2'.format(ceph_dashboard_base_url, url_prefix),
+                'binding': "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
+            },
+            'attributeConsumingService': {
+                'serviceName': "Ceph Dashboard",
+                "serviceDescription": "Ceph Dashboard Service",
+                "requestedAttributes": [
+                    {
+                        "name": idp_username_attribute,
+                        "isRequired": True
+                    }
+                ]
+            },
+            'singleLogoutService': {
+                'url': '{}{}/auth/saml2/logout'.format(ceph_dashboard_base_url, url_prefix),
+                'binding': 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect'
+            },
+            "x509cert": sp_x_509_cert,
+            "privateKey": sp_private_key
+        },
+        'security': {
+            "nameIdEncrypted": has_sp_cert,
+            "authnRequestsSigned": has_sp_cert,
+            "logoutRequestSigned": has_sp_cert,
+            "logoutResponseSigned": has_sp_cert,
+            "signMetadata": has_sp_cert,
+            "wantMessagesSigned": has_sp_cert,
+            "wantAssertionsSigned": has_sp_cert,
+            "wantAssertionsEncrypted": has_sp_cert,
+            # Not all Identity Providers support this.
+            "wantNameIdEncrypted": False,
+            "metadataValidUntil": '',
+            "wantAttributeStatement": False
+        }
+    }
+    settings = Saml2Parser.merge_settings(settings, idp_settings)
+    mgr.SSO_DB.saml2.onelogin_settings = settings
+    mgr.SSO_DB.protocol = 'saml2'
+    mgr.SSO_DB.save()
+
+
+def _parse_saml_settings(idp_metadata, idp_entity_id):
+    if os.path.isfile(idp_metadata):
+        warnings.warn(
+            "Please prepend 'file://' to indicate a local SAML2 IdP file", DeprecationWarning)
+        with open(idp_metadata, 'r', encoding='utf-8') as f:
+            idp_settings = Saml2Parser.parse(f.read(), entity_id=idp_entity_id)
+    elif parse.urlparse(idp_metadata)[0] in ('http', 'https', 'file'):
+        idp_settings = Saml2Parser.parse_remote(
+            url=idp_metadata, validate_cert=False, entity_id=idp_entity_id)
+    else:
+        idp_settings = Saml2Parser.parse(idp_metadata, entity_id=idp_entity_id)
+    return idp_settings
