@@ -1,24 +1,21 @@
 # -*- coding: utf-8 -*-
 # pylint: disable=dangerous-default-value,too-many-public-methods
-from __future__ import absolute_import
 
 import errno
 import json
 import tempfile
 import time
 import unittest
-
 from datetime import datetime, timedelta
 
 from mgr_module import ERROR_MSG_EMPTY_INPUT_FILE
 
-from . import CmdException, CLICommandTestMixin  # pylint: disable=no-name-in-module
 from .. import mgr
-from ..security import Scope, Permission
-from ..services.access_control import load_access_control_db, \
-                                      password_hash, AccessControlDB, \
-                                      SYSTEM_ROLES, PasswordPolicy
+from ..security import Permission, Scope
+from ..services.access_control import SYSTEM_ROLES, AccessControlDB, \
+    PasswordPolicy, load_access_control_db, password_hash
 from ..settings import Settings
+from ..tests import CLICommandTestMixin, CmdException
 
 
 class AccessControlTest(unittest.TestCase, CLICommandTestMixin):
@@ -157,7 +154,10 @@ class AccessControlTest(unittest.TestCase, CLICommandTestMixin):
     def test_show_system_role(self):
         role = self.exec_cmd('ac-role-show', rolename="read-only")
         self.assertEqual(role['name'], 'read-only')
-        self.assertEqual(role['description'], 'Read-Only')
+        self.assertEqual(
+            role['description'],
+            'allows read permission for all security scope except dashboard settings and config-opt'
+        )
 
     def test_delete_system_role(self):
         with self.assertRaises(CmdException) as ctx:
@@ -392,7 +392,7 @@ class AccessControlTest(unittest.TestCase, CLICommandTestMixin):
             uroles.sort()
             user = self.exec_cmd('ac-user-add-roles', username=username,
                                  roles=[role])
-            self.assertDictContainsSubset({'roles': uroles}, user)
+            self.assertLessEqual(uroles, user['roles'])
         self.validate_persistent_user(username, uroles)
         self.assertGreaterEqual(user['lastUpdate'], user_orig['lastUpdate'])
 
@@ -400,8 +400,8 @@ class AccessControlTest(unittest.TestCase, CLICommandTestMixin):
         user_orig = self.test_create_user()
         user = self.exec_cmd('ac-user-add-roles', username="admin",
                              roles=['pool-manager', 'block-manager'])
-        self.assertDictContainsSubset(
-            {'roles': ['block-manager', 'pool-manager']}, user)
+        self.assertLessEqual(['block-manager', 'pool-manager'],
+                             user['roles'])
         self.validate_persistent_user('admin', ['block-manager',
                                                 'pool-manager'])
         self.assertGreaterEqual(user['lastUpdate'], user_orig['lastUpdate'])
@@ -428,14 +428,13 @@ class AccessControlTest(unittest.TestCase, CLICommandTestMixin):
         user_orig = self.test_create_user()
         user = self.exec_cmd('ac-user-add-roles', username="admin",
                              roles=['pool-manager'])
-        self.assertDictContainsSubset(
-            {'roles': ['pool-manager']}, user)
+        self.assertLessEqual(['pool-manager'], user['roles'])
         self.validate_persistent_user('admin', ['pool-manager'])
         self.assertGreaterEqual(user['lastUpdate'], user_orig['lastUpdate'])
         user2 = self.exec_cmd('ac-user-set-roles', username="admin",
                               roles=['rgw-manager', 'block-manager'])
-        self.assertDictContainsSubset(
-            {'roles': ['block-manager', 'rgw-manager']}, user2)
+        self.assertLessEqual(['block-manager', 'rgw-manager'],
+                             user2['roles'])
         self.validate_persistent_user('admin', ['block-manager',
                                                 'rgw-manager'])
         self.assertGreaterEqual(user2['lastUpdate'], user['lastUpdate'])
@@ -462,8 +461,7 @@ class AccessControlTest(unittest.TestCase, CLICommandTestMixin):
         self.test_add_user_roles()
         user = self.exec_cmd('ac-user-del-roles', username="admin",
                              roles=['pool-manager'])
-        self.assertDictContainsSubset(
-            {'roles': ['block-manager']}, user)
+        self.assertLessEqual(['block-manager'], user['roles'])
         self.validate_persistent_user('admin', ['block-manager'])
 
     def test_del_user_roles_not_existent_user(self):
@@ -612,7 +610,7 @@ class AccessControlTest(unittest.TestCase, CLICommandTestMixin):
                           force_password=True)
 
         self.assertEqual(ctx.exception.retcode, -errno.EINVAL)
-        self.assertEqual(str(ctx.exception), ERROR_MSG_EMPTY_INPUT_FILE)
+        self.assertIn(ERROR_MSG_EMPTY_INPUT_FILE, str(ctx.exception))
 
     def test_set_user_password_hash(self):
         user_orig = self.test_create_user()
