@@ -6,7 +6,7 @@ from orchestrator import Orchestrator as OrchestratorBase
 
 from ..controllers.orchestrator import Orchestrator
 from ..services.orchestrator import OrchFeature
-from . import ControllerTestCase  # pylint: disable=no-name-in-module
+from ..tests import ControllerTestCase
 
 
 class OrchestratorControllerTest(ControllerTestCase):
@@ -15,8 +15,6 @@ class OrchestratorControllerTest(ControllerTestCase):
 
     @classmethod
     def setup_server(cls):
-        # pylint: disable=protected-access
-        Orchestrator._cp_config['tools.authenticate.on'] = False
         cls.setup_controllers([Orchestrator])
 
     @mock.patch('dashboard.controllers.orchestrator.OrchClient.instance')
