@@ -1,11 +1,13 @@
+from typing import Any
+
 from mgr_module import CLIReadCommand, HandleCommandResult, MgrModule
 
 
 class Module(MgrModule):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
-    def self_test(self):
+    def self_test(self) -> None:
         r = self.get('io_rate')
         assert('pg_stats_delta' in r)
         assert('stamp_delta' in r['pg_stats_delta'])
@@ -16,7 +18,7 @@ class Module(MgrModule):
         assert('num_read' in r['pg_stats_delta']['stat_sum'])
 
     @CLIReadCommand('iostat', poll=True)
-    def iostat(self, width: int = 80, print_header: bool = False):
+    def iostat(self, width: int = 80, print_header: bool = False) -> HandleCommandResult:
         """
         Get IO rates
         """
@@ -54,7 +56,7 @@ class Module(MgrModule):
             str(rd_ops),
             str(wr_ops),
             str(total_ops)
-            ]
+        ]
         ret += self.get_pretty_row(elems, width)
 
         return HandleCommandResult(stdout=ret)
