@@ -154,7 +154,6 @@ class CephfsConnectionPool(object):
             self.fs.conf_set("client_mount_uid", "0")
             self.fs.conf_set("client_mount_gid", "0")
             self.fs.conf_set("client_check_pool_perm", "false")
-            self.fs.conf_set("client_quota", "false")
             logger.debug("CephFS initializing...")
             self.fs.init()
             logger.debug("CephFS mounting...")
@@ -290,10 +289,16 @@ class CephfsConnectionPool(object):
 class CephfsClient(Generic[Module_T]):
     def __init__(self, mgr: Module_T):
         self.mgr = mgr
+        self.stopping = Event()
         self.connection_pool = CephfsConnectionPool(self.mgr)
 
+    def is_stopping(self) -> bool:
+        return self.stopping.is_set()
+
     def shutdown(self) -> None:
         logger.info("shutting down")
+        # first, note that we're shutting down
+        self.stopping.set()
         # second, delete all libcephfs handles from connection pool
         self.connection_pool.del_all_connections()
 
@@ -316,15 +321,6 @@ class CephfsClient(Generic[Module_T]):
             return fs['mdsmap']['metadata_pool']
         return None
 
-    def get_all_filesystems(self) -> List[str]:
-        fs_list: List[str] = []
-        fs_map = self.mgr.get('fs_map')
-        if fs_map['filesystems']:
-            for fs in fs_map['filesystems']:
-                fs_list.append(fs['mdsmap']['fs_name'])
-        return fs_list
-
-
 
 @contextlib.contextmanager
 def open_filesystem(fsc: CephfsClient, fs_name: str) -> Generator["cephfs.LibCephFS", None, None]:
@@ -336,6 +332,10 @@ def open_filesystem(fsc: CephfsClient, fs_name: str) -> Generator["cephfs.LibCep
     :param fs_name: fs name
     :return: yields a fs handle (ceph filesystem handle)
     """
+    if fsc.is_stopping():
+        raise CephfsConnectionException(-errno.ESHUTDOWN,
+                                        "shutdown in progress")
+
     fs_handle = fsc.connection_pool.get_fs_handle(fs_name)
     try:
         yield fs_handle
