@@ -1,23 +1,37 @@
 import re
 
+from typing import Any, Dict, Optional, Tuple, TYPE_CHECKING, Union
+
+
 GLOBAL_POOL_KEY = (None, None)
 
 class NotAuthorizedError(Exception):
     pass
 
 
-def is_authorized(module, pool, namespace):
+if TYPE_CHECKING:
+    from rbd_support.module import Module
+
+
+def is_authorized(module: 'Module',
+                  pool: Optional[str],
+                  namespace: Optional[str]) -> bool:
     return module.is_authorized({"pool": pool or '',
                                  "namespace": namespace or ''})
 
 
-def authorize_request(module, pool, namespace):
+def authorize_request(module: 'Module',
+                      pool: Optional[str],
+                      namespace: Optional[str]) -> None:
     if not is_authorized(module, pool, namespace):
         raise NotAuthorizedError("not authorized on pool={}, namespace={}".format(
             pool, namespace))
 
 
-def extract_pool_key(pool_spec):
+PoolKeyT = Union[Tuple[str, str], Tuple[None, None]]
+
+
+def extract_pool_key(pool_spec: Optional[str]) -> PoolKeyT:
     if not pool_spec:
         return GLOBAL_POOL_KEY
 
@@ -27,7 +41,7 @@ def extract_pool_key(pool_spec):
     return (match.group(1), match.group(2) or '')
 
 
-def get_rbd_pools(module):
+def get_rbd_pools(module: 'Module') -> Dict[int, str]:
     osd_map = module.get('osd_map')
     return {pool['pool']: pool['pool_name'] for pool in osd_map['pools']
             if 'rbd' in pool.get('application_metadata', {})}
