@@ -7,6 +7,7 @@ import traceback
 
 from datetime import datetime
 from threading import Condition, Lock, Thread
+from typing import Any, Dict, List, Optional, Tuple
 
 from .common import get_rbd_pools
 from .schedule import LevelSpec, Interval, StartTime, Schedule, Schedules
@@ -15,13 +16,12 @@ from .schedule import LevelSpec, Interval, StartTime, Schedule, Schedules
 class TrashPurgeScheduleHandler:
     MODULE_OPTION_NAME = "trash_purge_schedule"
     SCHEDULE_OID = "rbd_trash_purge_schedule"
-    REFRESH_DELAY_SECONDS = 60.0
 
     lock = Lock()
     condition = Condition(lock)
     thread = None
 
-    def __init__(self, module):
+    def __init__(self, module: Any) -> None:
         self.module = module
         self.log = module.log
         self.last_refresh_pools = datetime(1970, 1, 1)
@@ -31,15 +31,15 @@ class TrashPurgeScheduleHandler:
         self.thread = Thread(target=self.run)
         self.thread.start()
 
-    def run(self):
+    def run(self) -> None:
         try:
             self.log.info("TrashPurgeScheduleHandler: starting")
             while True:
-                refresh_delay = self.refresh_pools()
+                self.refresh_pools()
                 with self.lock:
                     (ns_spec, wait_time) = self.dequeue()
                     if not ns_spec:
-                        self.condition.wait(min(wait_time, refresh_delay))
+                        self.condition.wait(min(wait_time, 60))
                         continue
                 pool_id, namespace = ns_spec
                 self.trash_purge(pool_id, namespace)
@@ -50,7 +50,7 @@ class TrashPurgeScheduleHandler:
             self.log.fatal("Fatal runtime error: {}\n{}".format(
                 ex, traceback.format_exc()))
 
-    def trash_purge(self, pool_id, namespace):
+    def trash_purge(self, pool_id: str, namespace: str) -> None:
         try:
             with self.module.rados.open_ioctx2(int(pool_id)) as ioctx:
                 ioctx.set_namespace(namespace)
@@ -59,37 +59,30 @@ class TrashPurgeScheduleHandler:
             self.log.error("exception when purgin {}/{}: {}".format(
                 pool_id, namespace, e))
 
-
-    def init_schedule_queue(self):
-        self.queue = {}
-        self.pools = {}
+    def init_schedule_queue(self) -> None:
+        self.queue: Dict[str, List[Tuple[str, str]]] = {}
+        # pool_id => {namespace => pool_name}
+        self.pools: Dict[str, Dict[str, str]] = {}
         self.refresh_pools()
-        self.log.debug("TrashPurgeScheduleHandler: queue is initialized")
+        self.log.debug("scheduler queue is initialized")
 
-    def load_schedules(self):
+    def load_schedules(self) -> None:
         self.log.info("TrashPurgeScheduleHandler: load_schedules")
 
         schedules = Schedules(self)
         schedules.load()
-        self.schedules = schedules
+        with self.lock:
+            self.schedules = schedules
 
-    def refresh_pools(self):
-        elapsed = (datetime.now() - self.last_refresh_pools).total_seconds()
-        if elapsed < self.REFRESH_DELAY_SECONDS:
-            return self.REFRESH_DELAY_SECONDS - elapsed
+    def refresh_pools(self) -> None:
+        if (datetime.now() - self.last_refresh_pools).seconds < 60:
+            return
 
         self.log.debug("TrashPurgeScheduleHandler: refresh_pools")
 
-        with self.lock:
-            self.load_schedules()
-            if not self.schedules:
-                self.log.debug("TrashPurgeScheduleHandler: no schedules")
-                self.pools = {}
-                self.queue = {}
-                self.last_refresh_pools = datetime.now()
-                return self.REFRESH_DELAY_SECONDS
+        self.load_schedules()
 
-        pools = {}
+        pools: Dict[str, Dict[str, str]] = {}
 
         for pool_id, pool_name in get_rbd_pools(self.module).items():
             if not self.schedules.intersects(
@@ -103,9 +96,8 @@ class TrashPurgeScheduleHandler:
             self.pools = pools
 
         self.last_refresh_pools = datetime.now()
-        return self.REFRESH_DELAY_SECONDS
 
-    def load_pool(self, ioctx, pools):
+    def load_pool(self, ioctx: rados.Ioctx, pools: Dict[str, Dict[str, str]]) -> None:
         pool_id = str(ioctx.get_pool_id())
         pool_name = ioctx.get_pool_name()
         pools[pool_id] = {}
@@ -124,26 +116,27 @@ class TrashPurgeScheduleHandler:
         for namespace in pool_namespaces:
             pools[pool_id][namespace] = pool_name
 
-    def rebuild_queue(self):
-        now = datetime.now()
+    def rebuild_queue(self) -> None:
+        with self.lock:
+            now = datetime.now()
 
-        # don't remove from queue "due" images
-        now_string = datetime.strftime(now, "%Y-%m-%d %H:%M:00")
+            # don't remove from queue "due" images
+            now_string = datetime.strftime(now, "%Y-%m-%d %H:%M:00")
 
-        for schedule_time in list(self.queue):
-            if schedule_time > now_string:
-                del self.queue[schedule_time]
+            for schedule_time in list(self.queue):
+                if schedule_time > now_string:
+                    del self.queue[schedule_time]
 
-        if not self.schedules:
-            return
+            if not self.schedules:
+                return
 
-        for pool_id, namespaces in self.pools.items():
-            for namespace in namespaces:
-                self.enqueue(now, pool_id, namespace)
+            for pool_id, namespaces in self.pools.items():
+                for namespace in namespaces:
+                    self.enqueue(now, pool_id, namespace)
 
-        self.condition.notify()
+            self.condition.notify()
 
-    def refresh_queue(self, current_pools):
+    def refresh_queue(self, current_pools: Dict[str, Dict[str, str]]) -> None:
         now = datetime.now()
 
         for pool_id, namespaces in self.pools.items():
@@ -160,27 +153,24 @@ class TrashPurgeScheduleHandler:
 
         self.condition.notify()
 
-    def enqueue(self, now, pool_id, namespace):
+    def enqueue(self, now: datetime, pool_id: str, namespace: str) -> None:
+
         schedule = self.schedules.find(pool_id, namespace)
         if not schedule:
-            self.log.debug(
-                "TrashPurgeScheduleHandler: no schedule for {}/{}".format(
-                    pool_id, namespace))
             return
 
         schedule_time = schedule.next_run(now)
         if schedule_time not in self.queue:
             self.queue[schedule_time] = []
-        self.log.debug(
-            "TrashPurgeScheduleHandler: scheduling {}/{} at {}".format(
-                pool_id, namespace, schedule_time))
+        self.log.debug("schedule {}/{} at {}".format(
+            pool_id, namespace, schedule_time))
         ns_spec = (pool_id, namespace)
         if ns_spec not in self.queue[schedule_time]:
             self.queue[schedule_time].append((pool_id, namespace))
 
-    def dequeue(self):
+    def dequeue(self) -> Tuple[Optional[Tuple[str, str]], float]:
         if not self.queue:
-            return None, 1000
+            return None, 1000.0
 
         now = datetime.now()
         schedule_time = sorted(self.queue)[0]
@@ -194,13 +184,9 @@ class TrashPurgeScheduleHandler:
         namespace = namespaces.pop(0)
         if not namespaces:
             del self.queue[schedule_time]
-        return namespace, 0
-
-    def remove_from_queue(self, pool_id, namespace):
-        self.log.debug(
-            "TrashPurgeScheduleHandler: descheduling {}/{}".format(
-                pool_id, namespace))
+        return namespace, 0.0
 
+    def remove_from_queue(self, pool_id: str, namespace: str) -> None:
         empty_slots = []
         for schedule_time, namespaces in self.queue.items():
             if (pool_id, namespace) in namespaces:
@@ -210,42 +196,46 @@ class TrashPurgeScheduleHandler:
         for schedule_time in empty_slots:
             del self.queue[schedule_time]
 
-    def add_schedule(self, level_spec, interval, start_time):
+    def add_schedule(self,
+                     level_spec: LevelSpec,
+                     interval: str,
+                     start_time: Optional[str]) -> Tuple[int, str, str]:
         self.log.debug(
-            "TrashPurgeScheduleHandler: add_schedule: level_spec={}, interval={}, start_time={}".format(
+            "add_schedule: level_spec={}, interval={}, start_time={}".format(
                 level_spec.name, interval, start_time))
 
-        # TODO: optimize to rebuild only affected part of the queue
         with self.lock:
             self.schedules.add(level_spec, interval, start_time)
-            self.rebuild_queue()
-        return 0, "", ""
-
-    def remove_schedule(self, level_spec, interval, start_time):
-        self.log.debug(
-            "TrashPurgeScheduleHandler: remove_schedule: level_spec={}, interval={}, start_time={}".format(
-                level_spec.name, interval, start_time))
 
         # TODO: optimize to rebuild only affected part of the queue
-        with self.lock:
-            self.schedules.remove(level_spec, interval, start_time)
-            self.rebuild_queue()
+        self.rebuild_queue()
         return 0, "", ""
 
-    def list(self, level_spec):
+    def remove_schedule(self,
+                        level_spec: LevelSpec,
+                        interval: Optional[str],
+                        start_time: Optional[str]) -> Tuple[int, str, str]:
         self.log.debug(
-            "TrashPurgeScheduleHandler: list: level_spec={}".format(
-                level_spec.name))
+            "remove_schedule: level_spec={}, interval={}, start_time={}".format(
+                level_spec.name, interval, start_time))
+
+        with self.lock:
+            self.schedules.remove(level_spec, interval, start_time)
+
+        # TODO: optimize to rebuild only affected part of the queue
+        self.rebuild_queue()
+        return 0, "", ""
+
+    def list(self, level_spec: LevelSpec) -> Tuple[int, str, str]:
+        self.log.debug("list: level_spec={}".format(level_spec.name))
 
         with self.lock:
             result = self.schedules.to_list(level_spec)
 
         return 0, json.dumps(result, indent=4, sort_keys=True), ""
 
-    def status(self, level_spec):
-        self.log.debug(
-            "TrashPurgeScheduleHandler: status: level_spec={}".format(
-                level_spec.name))
+    def status(self, level_spec: LevelSpec) -> Tuple[int, str, str]:
+        self.log.debug("status: level_spec={}".format(level_spec.name))
 
         scheduled = []
         with self.lock:
@@ -262,26 +252,3 @@ class TrashPurgeScheduleHandler:
                     })
         return 0, json.dumps({'scheduled' : scheduled}, indent=4,
                              sort_keys=True), ""
-
-    def handle_command(self, inbuf, prefix, cmd):
-        level_spec_name = cmd.get('level_spec', "")
-
-        try:
-            level_spec = LevelSpec.from_name(self, level_spec_name,
-                                             allow_image_level=False)
-        except ValueError as e:
-            return -errno.EINVAL, '', "Invalid level spec {}: {}".format(
-                level_spec_name, e)
-
-        if prefix == 'add':
-            return self.add_schedule(level_spec, cmd['interval'],
-                                     cmd.get('start_time'))
-        elif prefix == 'remove':
-            return self.remove_schedule(level_spec, cmd.get('interval'),
-                                        cmd.get('start_time'))
-        elif prefix == 'list':
-            return self.list(level_spec)
-        elif prefix == 'status':
-            return self.status(level_spec)
-
-        raise NotImplementedError(cmd['prefix'])
