@@ -1,5 +1,4 @@
 # type: ignore
-from __future__ import absolute_import
 
 import json
 import logging
@@ -73,7 +72,10 @@ if 'UNITTEST' in os.environ:
             except FileNotFoundError:
                 val = None
             mo = [o for o in self.MODULE_OPTIONS if o['name'] == key]
-            if len(mo) == 1:
+            if len(mo) >= 1:  # >= 1, cause self.MODULE_OPTIONS. otherwise it
+                #               fails when importing multiple modules.
+                if 'default' in mo and val is None:
+                    val = mo[0]['default']
                 if val is not None:
                     cls = {
                         'str': str,
@@ -99,7 +101,9 @@ if 'UNITTEST' in os.environ:
             return self.mock_store_get('_ceph_get', data_name, mock.MagicMock())
 
         def _ceph_send_command(self, res, svc_type, svc_id, command, tag, inbuf):
+
             cmd = json.loads(command)
+            getattr(self, '_mon_commands_sent', []).append(cmd)
 
             # Mocking the config store is handy sometimes:
             def config_get():
@@ -115,6 +119,10 @@ if 'UNITTEST' in os.environ:
                 self.mock_store_set('config', f'{cmd["who"]}/{cmd["name"]}', cmd['value'])
                 return ''
 
+            def config_rm():
+                self.mock_store_set('config', f'{cmd["who"]}/{cmd["name"]}', None)
+                return ''
+
             def config_dump():
                 r = []
                 for prefix, value in self.mock_store_prefix('config', '').items():
@@ -133,12 +141,26 @@ if 'UNITTEST' in os.environ:
                 outb = config_set()
             elif cmd['prefix'] == 'config dump':
                 outb = config_dump()
+            elif cmd['prefix'] == 'config rm':
+                outb = config_rm()
             elif hasattr(self, '_mon_command_mock_' + cmd['prefix'].replace(' ', '_')):
                 a = getattr(self, '_mon_command_mock_' + cmd['prefix'].replace(' ', '_'))
                 outb = a(cmd)
 
             res.complete(0, outb, '')
 
+        def _ceph_get_foreign_option(self, entity, name):
+            who = entity.split('.')
+            whos = ['global'] + ['.'.join(who[:i+1]) for i in range(len(who))]
+            for attepmt in reversed(whos):
+                val = self.mock_store_get('config', f'{attepmt}/{name}', None)
+                if val is not None:
+                    return val
+            return None
+
+        def assert_issued_mon_command(self, command):
+            assert command in self._mon_commands_sent, self._mon_commands_sent
+
         @property
         def _logger(self):
             return logging.getLogger(__name__)
@@ -148,18 +170,20 @@ if 'UNITTEST' in os.environ:
             pass
 
         def __init__(self, *args):
+            self._mon_commands_sent = []
             if not hasattr(self, '_store'):
                 self._store = {}
 
 
-            if self.__class__.__name__ not in M_classes:
+            if self.__class__ not in M_classes:
                 # call those only once.
                 self._register_commands('')
                 self._register_options('')
-                M_classes.add(self.__class__.__name__)
+                M_classes.add(self.__class__)
 
             super(M, self).__init__()
             self._ceph_get_version = mock.Mock()
+            self._ceph_get_ceph_conf_path = mock.MagicMock()
             self._ceph_get_option = mock.MagicMock()
             self._ceph_get_context = mock.MagicMock()
             self._ceph_register_client = mock.MagicMock()
@@ -188,9 +212,17 @@ if 'UNITTEST' in os.environ:
                     return msg
                 return '[errno {0}] {1}'.format(self.errno, msg)
 
+        class MockObjectNotFound(Exception):
+            pass
 
         sys.modules.update({
-            'rados': mock.MagicMock(Error=MockRadosError, OSError=MockRadosError),
+            'rados': mock.MagicMock(
+                Error=MockRadosError,
+                OSError=MockRadosError,
+                ObjectNotFound=MockObjectNotFound),
             'rbd': mock.Mock(),
             'cephfs': mock.Mock(),
-        })
\ No newline at end of file
+        })
+
+    # Unconditionally mock the rados objects when we're imported
+    mock_ceph_modules()  # type: ignore
