@@ -2,7 +2,7 @@ import enum
 import yaml
 
 from ceph.deployment.inventory import Device
-from ceph.deployment.service_spec import ServiceSpec, PlacementSpec, CustomConfig
+from ceph.deployment.service_spec import ServiceSpec, PlacementSpec
 from ceph.deployment.hostspec import SpecValidationError
 
 try:
@@ -149,8 +149,8 @@ class DriveGroupSpec(ServiceSpec):
         "db_slots", "wal_slots", "block_db_size", "placement", "service_id", "service_type",
         "data_devices", "db_devices", "wal_devices", "journal_devices",
         "data_directories", "osds_per_device", "objectstore", "osd_id_claims",
-        "journal_size", "unmanaged", "filter_logic", "preview_only", "extra_container_args",
-        "extra_entrypoint_args", "data_allocate_fraction", "method", "crush_device_class",
+        "journal_size", "unmanaged", "filter_logic", "preview_only",
+        "data_allocate_fraction", "method"
     ]
 
     def __init__(self,
@@ -174,21 +174,14 @@ class DriveGroupSpec(ServiceSpec):
                  unmanaged=False,  # type: bool
                  filter_logic='AND',  # type: str
                  preview_only=False,  # type: bool
-                 extra_container_args=None,  # type: Optional[List[str]]
-                 extra_entrypoint_args: Optional[List[str]] = None,
                  data_allocate_fraction=None,  # type: Optional[float]
                  method=None,  # type: Optional[OSDMethod]
-                 crush_device_class=None,  # type: Optional[str]
-                 custom_configs=None,  # type: Optional[List[CustomConfig]]
                  ):
         assert service_type is None or service_type == 'osd'
         super(DriveGroupSpec, self).__init__('osd', service_id=service_id,
                                              placement=placement,
                                              unmanaged=unmanaged,
-                                             preview_only=preview_only,
-                                             extra_container_args=extra_container_args,
-                                             extra_entrypoint_args=extra_entrypoint_args,
-                                             custom_configs=custom_configs)
+                                             preview_only=preview_only)
 
         #: A :class:`ceph.deployment.drive_group.DeviceSelection`
         self.data_devices = data_devices
@@ -246,9 +239,6 @@ class DriveGroupSpec(ServiceSpec):
 
         self.method = method
 
-        #: Crush device class to assign to OSDs
-        self.crush_device_class = crush_device_class
-
     @classmethod
     def _from_json_impl(cls, json_drive_group):
         # type: (dict) -> DriveGroupSpec
@@ -344,9 +334,13 @@ class DriveGroupSpec(ServiceSpec):
                 'filter_logic must be either <AND> or <OR>')
 
         if self.method not in [None, 'lvm', 'raw']:
-            raise DriveGroupValidationError(self.service_id, 'method must be one of None, lvm, raw')
+            raise DriveGroupValidationError(
+                self.service_id,
+                'method must be one of None, lvm, raw')
         if self.method == 'raw' and self.objectstore == 'filestore':
-            raise DriveGroupValidationError(self.service_id, 'method raw only supports bluestore')
+            raise DriveGroupValidationError(
+                self.service_id,
+                'method raw only supports bluestore')
 
 
 yaml.add_representer(DriveGroupSpec, DriveGroupSpec.yaml_representer)
