@@ -1,25 +1,28 @@
 import logging
 
 try:
-    from typing import Optional
+    from typing import Optional, List
 except ImportError:
     pass
 
-from ceph.deployment.drive_group import DriveGroupSpec
 from ceph.deployment.drive_selection.selector import DriveSelection
 
 logger = logging.getLogger(__name__)
 
 
+# TODO refactor this to a DriveSelection method
 class to_ceph_volume(object):
 
     def __init__(self,
-                 spec,  # type: DriveGroupSpec
-                 selection  # type: DriveSelection
+                 selection,  # type: DriveSelection
+                 osd_id_claims=None,  # type: Optional[List[str]]
+                 preview=False  # type: bool
                  ):
 
-        self.spec = spec
         self.selection = selection
+        self.spec = selection.spec
+        self.preview = preview
+        self.osd_id_claims = osd_id_claims
 
     def run(self):
         # type: () -> Optional[str]
@@ -32,7 +35,17 @@ class to_ceph_volume(object):
         if not data_devices:
             return None
 
-        if self.spec.objectstore == 'filestore':
+        cmd = ""
+        if self.spec.method == 'raw':
+            assert self.spec.objectstore == 'bluestore'
+            cmd = "raw prepare --bluestore"
+            cmd += " --data {}".format(" ".join(data_devices))
+            if db_devices:
+                cmd += " --block.db {}".format(" ".join(db_devices))
+            if wal_devices:
+                cmd += " --block.wal {}".format(" ".join(wal_devices))
+
+        elif self.spec.objectstore == 'filestore':
             cmd = "lvm batch --no-auto"
 
             cmd += " {}".format(" ".join(data_devices))
@@ -46,17 +59,7 @@ class to_ceph_volume(object):
 
             cmd += " --filestore"
 
-        # HORRIBLE HACK
-        if self.spec.objectstore == 'bluestore' and \
-           not self.spec.encrypted and \
-           not self.spec.osds_per_device and \
-           len(data_devices) == 1 and \
-           not db_devices and \
-           not wal_devices:
-            cmd = "lvm prepare --bluestore --data %s --no-systemd" % (' '.join(data_devices))
-            return cmd
-
-        if self.spec.objectstore == 'bluestore':
+        elif self.spec.objectstore == 'bluestore':
 
             cmd = "lvm batch --no-auto {}".format(" ".join(data_devices))
 
@@ -78,7 +81,18 @@ class to_ceph_volume(object):
         if self.spec.osds_per_device:
             cmd += " --osds-per-device {}".format(self.spec.osds_per_device)
 
-        cmd += " --yes"
-        cmd += " --no-systemd"
+        if self.spec.data_allocate_fraction:
+            cmd += " --data-allocate-fraction {}".format(self.spec.data_allocate_fraction)
+
+        if self.osd_id_claims:
+            cmd += " --osd-ids {}".format(" ".join(self.osd_id_claims))
+
+        if self.spec.method != 'raw':
+            cmd += " --yes"
+            cmd += " --no-systemd"
+
+        if self.preview:
+            cmd += " --report"
+            cmd += " --format json"
 
         return cmd
