@@ -1,5 +1,5 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
-// vim: ts=8 sw=2 smarttab
+// vim: ts=8 sw=2 smarttab ft=cpp
 
 #include <string>
 #include <vector>
@@ -20,12 +20,12 @@
 #include "rgw_rest_s3.h"
 #include "rgw_auth_s3.h"
 
-#include "common/ceph_crypto_cms.h"
-#include "common/armor.h"
+#include "common/ceph_crypto.h"
 #include "common/Cond.h"
 
 #define dout_subsys ceph_subsys_rgw
 
+using namespace std;
 
 namespace rgw {
 namespace auth {
@@ -37,29 +37,8 @@ TokenEngine::is_applicable(const std::string& token) const noexcept
   return ! token.empty() && ! cct->_conf->rgw_keystone_url.empty();
 }
 
-TokenEngine::token_envelope_t
-TokenEngine::decode_pki_token(const std::string& token) const
-{
-  ceph::buffer::list token_body_bl;
-  int ret = rgw_decode_b64_cms(cct, token, token_body_bl);
-  if (ret < 0) {
-    ldout(cct, 20) << "cannot decode pki token" << dendl;
-    throw ret;
-  } else {
-    ldout(cct, 20) << "successfully decoded pki token" << dendl;
-  }
-
-  TokenEngine::token_envelope_t token_body;
-  ret = token_body.parse(cct, token, token_body_bl, config.get_api_version());
-  if (ret < 0) {
-    throw ret;
-  }
-
-  return token_body;
-}
-
 boost::optional<TokenEngine::token_envelope_t>
-TokenEngine::get_from_keystone(const std::string& token) const
+TokenEngine::get_from_keystone(const DoutPrefixProvider* dpp, const std::string& token) const
 {
   /* Unfortunately, we can't use the short form of "using" here. It's because
    * we're aliasing a class' member, not namespace. */
@@ -85,7 +64,7 @@ TokenEngine::get_from_keystone(const std::string& token) const
   }
 
   std::string admin_token;
-  if (rgw::keystone::Service::get_admin_token(cct, token_cache, config,
+  if (rgw::keystone::Service::get_admin_token(dpp, cct, token_cache, config,
                                               admin_token) < 0) {
     throw -EINVAL;
   }
@@ -95,7 +74,7 @@ TokenEngine::get_from_keystone(const std::string& token) const
 
   validate.set_url(url);
 
-  int ret = validate.process();
+  int ret = validate.process(null_yield);
   if (ret < 0) {
     throw ret;
   }
@@ -113,16 +92,16 @@ TokenEngine::get_from_keystone(const std::string& token) const
       validate.get_http_status() ==
           /* Most likely: non-existent token supplied by the client. */
           RGWValidateKeystoneToken::HTTP_STATUS_NOTFOUND) {
-    ldout(cct, 5) << "Failed keystone auth from " << url << " with "
+    ldpp_dout(dpp, 5) << "Failed keystone auth from " << url << " with "
                   << validate.get_http_status() << dendl;
     return boost::none;
   }
 
-  ldout(cct, 20) << "received response status=" << validate.get_http_status()
+  ldpp_dout(dpp, 20) << "received response status=" << validate.get_http_status()
                  << ", body=" << token_body_bl.c_str() << dendl;
 
   TokenEngine::token_envelope_t token_body;
-  ret = token_body.parse(cct, token, token_body_bl, config.get_api_version());
+  ret = token_body.parse(dpp, cct, token, token_body_bl, config.get_api_version());
   if (ret < 0) {
     throw ret;
   }
@@ -205,7 +184,8 @@ TokenEngine::get_acl_strategy(const TokenEngine::token_envelope_t& token) const
 }
 
 TokenEngine::result_t
-TokenEngine::authenticate(const std::string& token,
+TokenEngine::authenticate(const DoutPrefixProvider* dpp,
+                          const std::string& token,
                           const req_state* const s) const
 {
   boost::optional<TokenEngine::token_envelope_t> t;
@@ -213,7 +193,7 @@ TokenEngine::authenticate(const std::string& token,
   /* This will be initialized on the first call to this method. In C++11 it's
    * also thread-safe. */
   static const struct RolesCacher {
-    RolesCacher(CephContext* const cct) {
+    explicit RolesCacher(CephContext* const cct) {
       get_str_vec(cct->_conf->rgw_keystone_accepted_roles, plain);
       get_str_vec(cct->_conf->rgw_keystone_accepted_admin_roles, admin);
 
@@ -229,33 +209,27 @@ TokenEngine::authenticate(const std::string& token,
     return result_t::deny();
   }
 
-  /* Token ID is a concept that makes dealing with PKI tokens more effective.
-   * Instead of storing several kilobytes, a short hash can be burried. */
+  /* Token ID is a legacy of supporting the service-side validation
+   * of PKI/PKIz token type which are already-removed-in-OpenStack.
+   * The idea was to bury in cache only a short hash instead of few
+   * kilobytes. RadosGW doesn't do the local validation anymore. */
   const auto& token_id = rgw_get_token_id(token);
-  ldout(cct, 20) << "token_id=" << token_id << dendl;
+  ldpp_dout(dpp, 20) << "token_id=" << token_id << dendl;
 
   /* Check cache first. */
   t = token_cache.find(token_id);
   if (t) {
-    ldout(cct, 20) << "cached token.project.id=" << t->get_project_id()
+    ldpp_dout(dpp, 20) << "cached token.project.id=" << t->get_project_id()
                    << dendl;
     auto apl = apl_factory->create_apl_remote(cct, s, get_acl_strategy(*t),
                                               get_creds_info(*t, roles.admin));
     return result_t::grant(std::move(apl));
   }
 
-  /* Retrieve token. */
-  if (rgw_is_pki_token(token)) {
-    try {
-      t = decode_pki_token(token);
-    } catch (...) {
-      /* Last resort. */
-      t = get_from_keystone(token);
-    }
-  } else {
-    /* Can't decode, just go to the Keystone server for validation. */
-    t = get_from_keystone(token);
-  }
+  /* Not in cache. Go to the Keystone for validation. This happens even
+   * for the legacy PKI/PKIz token types. That's it, after the PKI/PKIz
+   * RadosGW-side validation has been removed, we always ask Keystone. */
+  t = get_from_keystone(dpp, token);
 
   if (! t) {
     return result_t::deny(-EACCES);
@@ -263,7 +237,7 @@ TokenEngine::authenticate(const std::string& token,
 
   /* Verify expiration. */
   if (t->expired()) {
-    ldout(cct, 0) << "got expired token: " << t->get_project_name()
+    ldpp_dout(dpp, 0) << "got expired token: " << t->get_project_name()
                   << ":" << t->get_user_name()
                   << " expired: " << t->get_expires() << dendl;
     return result_t::deny(-EPERM);
@@ -272,7 +246,7 @@ TokenEngine::authenticate(const std::string& token,
   /* Check for necessary roles. */
   for (const auto& role : roles.plain) {
     if (t->has_role(role) == true) {
-      ldout(cct, 0) << "validated token: " << t->get_project_name()
+      ldpp_dout(dpp, 0) << "validated token: " << t->get_project_name()
                     << ":" << t->get_user_name()
                     << " expires: " << t->get_expires() << dendl;
       token_cache.add(token_id, *t);
@@ -282,8 +256,8 @@ TokenEngine::authenticate(const std::string& token,
     }
   }
 
-  ldout(cct, 0) << "user does not hold a matching role; required roles: "
-                << g_conf->rgw_keystone_accepted_roles << dendl;
+  ldpp_dout(dpp, 0) << "user does not hold a matching role; required roles: "
+                << g_conf()->rgw_keystone_accepted_roles << dendl;
 
   return result_t::deny(-EPERM);
 }
@@ -293,9 +267,9 @@ TokenEngine::authenticate(const std::string& token,
  * Try to validate S3 auth against keystone s3token interface
  */
 std::pair<boost::optional<rgw::keystone::TokenEnvelope>, int>
-EC2Engine::get_from_keystone(const boost::string_view& access_key_id,
+EC2Engine::get_from_keystone(const DoutPrefixProvider* dpp, const std::string_view& access_key_id,
                              const std::string& string_to_sign,
-                             const boost::string_view& signature) const
+                             const std::string_view& signature) const
 {
   /* prepare keystone url */
   std::string keystone_url = config.get_endpoint_url();
@@ -304,7 +278,7 @@ EC2Engine::get_from_keystone(const boost::string_view& access_key_id,
   }
 
   const auto api_version = config.get_api_version();
-  if (config.get_api_version() == rgw::keystone::ApiVersion::VER_3) {
+  if (api_version == rgw::keystone::ApiVersion::VER_3) {
     keystone_url.append("v3/s3tokens");
   } else {
     keystone_url.append("v2.0/s3tokens");
@@ -312,10 +286,10 @@ EC2Engine::get_from_keystone(const boost::string_view& access_key_id,
 
   /* get authentication token for Keystone. */
   std::string admin_token;
-  int ret = rgw::keystone::Service::get_admin_token(cct, token_cache, config,
+  int ret = rgw::keystone::Service::get_admin_token(dpp, cct, token_cache, config,
                                                     admin_token);
   if (ret < 0) {
-    ldout(cct, 2) << "s3 keystone: cannot get token for keystone access"
+    ldpp_dout(dpp, 2) << "s3 keystone: cannot get token for keystone access"
                   << dendl;
     throw ret;
   }
@@ -351,9 +325,9 @@ EC2Engine::get_from_keystone(const boost::string_view& access_key_id,
   validate.set_send_length(os.str().length());
 
   /* send request */
-  ret = validate.process();
+  ret = validate.process(null_yield);
   if (ret < 0) {
-    ldout(cct, 2) << "s3 keystone: token validation ERROR: "
+    ldpp_dout(dpp, 2) << "s3 keystone: token validation ERROR: "
                   << token_body_bl.c_str() << dendl;
     throw ret;
   }
@@ -369,9 +343,9 @@ EC2Engine::get_from_keystone(const boost::string_view& access_key_id,
 
   /* now parse response */
   rgw::keystone::TokenEnvelope token_envelope;
-  ret = token_envelope.parse(cct, std::string(), token_body_bl, api_version);
+  ret = token_envelope.parse(dpp, cct, std::string(), token_body_bl, api_version);
   if (ret < 0) {
-    ldout(cct, 2) << "s3 keystone: token parsing failed, ret=0" << ret
+    ldpp_dout(dpp, 2) << "s3 keystone: token parsing failed, ret=0" << ret
                   << dendl;
     throw ret;
   }
@@ -379,6 +353,141 @@ EC2Engine::get_from_keystone(const boost::string_view& access_key_id,
   return std::make_pair(std::move(token_envelope), 0);
 }
 
+std::pair<boost::optional<std::string>, int> EC2Engine::get_secret_from_keystone(const DoutPrefixProvider* dpp,
+                                                                                 const std::string& user_id,
+                                                                                 const std::string_view& access_key_id) const
+{
+  /*  Fetch from /users/{USER_ID}/credentials/OS-EC2/{ACCESS_KEY_ID} */
+  /* Should return json with response key "credential" which contains entry "secret"*/
+
+  /* prepare keystone url */
+  std::string keystone_url = config.get_endpoint_url();
+  if (keystone_url.empty()) {
+    return make_pair(boost::none, -EINVAL);
+  }
+
+  const auto api_version = config.get_api_version();
+  if (api_version == rgw::keystone::ApiVersion::VER_3) {
+    keystone_url.append("v3/");
+  } else {
+    keystone_url.append("v2.0/");
+  }
+  keystone_url.append("users/");
+  keystone_url.append(user_id);
+  keystone_url.append("/credentials/OS-EC2/");
+  keystone_url.append(std::string(access_key_id));
+
+  /* get authentication token for Keystone. */
+  std::string admin_token;
+  int ret = rgw::keystone::Service::get_admin_token(dpp, cct, token_cache, config,
+                                                    admin_token);
+  if (ret < 0) {
+    ldpp_dout(dpp, 2) << "s3 keystone: cannot get token for keystone access"
+                  << dendl;
+    return make_pair(boost::none, ret);
+  }
+
+  using RGWGetAccessSecret
+    = rgw::keystone::Service::RGWKeystoneHTTPTransceiver;
+
+  /* The container for plain response obtained from Keystone.*/
+  ceph::bufferlist token_body_bl;
+  RGWGetAccessSecret secret(cct, "GET", keystone_url, &token_body_bl);
+
+  /* set required headers for keystone request */
+  secret.append_header("X-Auth-Token", admin_token);
+
+  /* check if we want to verify keystone's ssl certs */
+  secret.set_verify_ssl(cct->_conf->rgw_keystone_verify_ssl);
+
+  /* send request */
+  ret = secret.process(null_yield);
+  if (ret < 0) {
+    ldpp_dout(dpp, 2) << "s3 keystone: secret fetching error: "
+                  << token_body_bl.c_str() << dendl;
+    return make_pair(boost::none, ret);
+  }
+
+  /* if the supplied signature is wrong, we will get 401 from Keystone */
+  if (secret.get_http_status() ==
+          decltype(secret)::HTTP_STATUS_NOTFOUND) {
+    return make_pair(boost::none, -EINVAL);
+  }
+
+  /* now parse response */
+
+  JSONParser parser;
+  if (! parser.parse(token_body_bl.c_str(), token_body_bl.length())) {
+    ldpp_dout(dpp, 0) << "Keystone credential parse error: malformed json" << dendl;
+    return make_pair(boost::none, -EINVAL);
+  }
+
+  JSONObjIter credential_iter = parser.find_first("credential");
+  std::string secret_string;
+
+  try {
+    if (!credential_iter.end()) {
+      JSONDecoder::decode_json("secret", secret_string, *credential_iter, true);
+    } else {
+      ldpp_dout(dpp, 0) << "Keystone credential not present in return from server" << dendl;
+      return make_pair(boost::none, -EINVAL);
+    }
+  } catch (const JSONDecoder::err& err) {
+    ldpp_dout(dpp, 0) << "Keystone credential parse error: " << err.what() << dendl;
+    return make_pair(boost::none, -EINVAL);
+  }
+
+  return make_pair(secret_string, 0);
+}
+
+/*
+ * Try to get a token for S3 authentication, using a secret cache if available
+ */
+std::pair<boost::optional<rgw::keystone::TokenEnvelope>, int>
+EC2Engine::get_access_token(const DoutPrefixProvider* dpp,
+			    const std::string_view& access_key_id,
+                            const std::string& string_to_sign,
+                            const std::string_view& signature,
+			    const signature_factory_t& signature_factory) const
+{
+  using server_signature_t = VersionAbstractor::server_signature_t;
+  boost::optional<rgw::keystone::TokenEnvelope> token;
+  int failure_reason;
+
+  /* Get a token from the cache if one has already been stored */
+  boost::optional<boost::tuple<rgw::keystone::TokenEnvelope, std::string>>
+    t = secret_cache.find(std::string(access_key_id));
+
+  /* Check that credentials can correctly be used to sign data */
+  if (t) {
+    std::string sig(signature);
+    server_signature_t server_signature = signature_factory(cct, t->get<1>(), string_to_sign);
+    if (sig.compare(server_signature) == 0) {
+      return std::make_pair(t->get<0>(), 0);
+    } else {
+      ldpp_dout(dpp, 0) << "Secret string does not correctly sign payload, cache miss" << dendl;
+    }
+  } else {
+    ldpp_dout(dpp, 0) << "No stored secret string, cache miss" << dendl;
+  }
+
+  /* No cached token, token expired, or secret invalid: fall back to keystone */
+  std::tie(token, failure_reason) = get_from_keystone(dpp, access_key_id, string_to_sign, signature);
+
+  if (token) {
+    /* Fetch secret from keystone for the access_key_id */
+    boost::optional<std::string> secret;
+    std::tie(secret, failure_reason) = get_secret_from_keystone(dpp, token->get_user_id(), access_key_id);
+
+    if (secret) {
+      /* Add token, secret pair to cache, and set timeout */
+      secret_cache.add(std::string(access_key_id), *token, *secret);
+    }
+  }
+
+  return std::make_pair(token, failure_reason);
+}
+
 EC2Engine::acl_strategy_t
 EC2Engine::get_acl_strategy(const EC2Engine::token_envelope_t&) const
 {
@@ -418,18 +527,21 @@ EC2Engine::get_creds_info(const EC2Engine::token_envelope_t& token,
 }
 
 rgw::auth::Engine::result_t EC2Engine::authenticate(
-  const boost::string_view& access_key_id,
-  const boost::string_view& signature,
+  const DoutPrefixProvider* dpp,
+  const std::string_view& access_key_id,
+  const std::string_view& signature,
+  const std::string_view& session_token,
   const string_to_sign_t& string_to_sign,
-  const signature_factory_t&,
+  const signature_factory_t& signature_factory,
   const completer_factory_t& completer_factory,
   /* Passthorugh only! */
-  const req_state* s) const
+  const req_state* s,
+  optional_yield y) const
 {
   /* This will be initialized on the first call to this method. In C++11 it's
    * also thread-safe. */
   static const struct RolesCacher {
-    RolesCacher(CephContext* const cct) {
+    explicit RolesCacher(CephContext* const cct) {
       get_str_vec(cct->_conf->rgw_keystone_accepted_roles, plain);
       get_str_vec(cct->_conf->rgw_keystone_accepted_admin_roles, admin);
 
@@ -444,14 +556,14 @@ rgw::auth::Engine::result_t EC2Engine::authenticate(
   boost::optional<token_envelope_t> t;
   int failure_reason;
   std::tie(t, failure_reason) = \
-    get_from_keystone(access_key_id, string_to_sign, signature);
+    get_access_token(dpp, access_key_id, string_to_sign, signature, signature_factory);
   if (! t) {
     return result_t::deny(failure_reason);
   }
 
   /* Verify expiration. */
   if (t->expired()) {
-    ldout(cct, 0) << "got expired token: " << t->get_project_name()
+    ldpp_dout(dpp, 0) << "got expired token: " << t->get_project_name()
                   << ":" << t->get_user_name()
                   << " expired: " << t->get_expires() << dendl;
     return result_t::deny();
@@ -467,13 +579,13 @@ rgw::auth::Engine::result_t EC2Engine::authenticate(
   }
 
   if (! found) {
-    ldout(cct, 5) << "s3 keystone: user does not hold a matching role;"
+    ldpp_dout(dpp, 5) << "s3 keystone: user does not hold a matching role;"
                      " required roles: "
                   << cct->_conf->rgw_keystone_accepted_roles << dendl;
     return result_t::deny();
   } else {
     /* everything seems fine, continue with this user */
-    ldout(cct, 5) << "s3 keystone: validated token: " << t->get_project_name()
+    ldpp_dout(dpp, 5) << "s3 keystone: validated token: " << t->get_project_name()
                   << ":" << t->get_user_name()
                   << " expires: " << t->get_expires() << dendl;
 
@@ -483,6 +595,63 @@ rgw::auth::Engine::result_t EC2Engine::authenticate(
   }
 }
 
+bool SecretCache::find(const std::string& token_id,
+                       SecretCache::token_envelope_t& token,
+		       std::string &secret)
+{
+  std::lock_guard<std::mutex> l(lock);
+
+  map<std::string, secret_entry>::iterator iter = secrets.find(token_id);
+  if (iter == secrets.end()) {
+    return false;
+  }
+
+  secret_entry& entry = iter->second;
+  secrets_lru.erase(entry.lru_iter);
+
+  const utime_t now = ceph_clock_now();
+  if (entry.token.expired() || now > entry.expires) {
+    secrets.erase(iter);
+    return false;
+  }
+  token = entry.token;
+  secret = entry.secret;
+
+  secrets_lru.push_front(token_id);
+  entry.lru_iter = secrets_lru.begin();
+
+  return true;
+}
+
+void SecretCache::add(const std::string& token_id,
+                      const SecretCache::token_envelope_t& token,
+		      const std::string& secret)
+{
+  std::lock_guard<std::mutex> l(lock);
+
+  map<string, secret_entry>::iterator iter = secrets.find(token_id);
+  if (iter != secrets.end()) {
+    secret_entry& e = iter->second;
+    secrets_lru.erase(e.lru_iter);
+  }
+
+  const utime_t now = ceph_clock_now();
+  secrets_lru.push_front(token_id);
+  secret_entry& entry = secrets[token_id];
+  entry.token = token;
+  entry.secret = secret;
+  entry.expires = now + s3_token_expiry_length;
+  entry.lru_iter = secrets_lru.begin();
+
+  while (secrets_lru.size() > max) {
+    list<string>::reverse_iterator riter = secrets_lru.rbegin();
+    iter = secrets.find(*riter);
+    assert(iter != secrets.end());
+    secrets.erase(iter);
+    secrets_lru.pop_back();
+  }
+}
+
 }; /* namespace keystone */
 }; /* namespace auth */
 }; /* namespace rgw */
