@@ -79,7 +79,7 @@ class SecretCache {
     token_envelope_t token;
     std::string secret;
     utime_t expires;
-    list<std::string>::iterator lru_iter;
+    std::list<std::string>::iterator lru_iter;
   };
 
   const boost::intrusive_ptr<CephContext> cct;
@@ -138,8 +138,7 @@ class EC2Engine : public rgw::auth::s3::AWSEngine {
   /* Helper methods. */
   acl_strategy_t get_acl_strategy(const token_envelope_t& token) const;
   auth_info_t get_creds_info(const token_envelope_t& token,
-                             const std::vector<std::string>& admin_roles,
-                             const std::string& access_key_id
+                             const std::vector<std::string>& admin_roles
                             ) const noexcept;
   std::pair<boost::optional<token_envelope_t>, int>
   get_from_keystone(const DoutPrefixProvider* dpp,
@@ -151,8 +150,7 @@ class EC2Engine : public rgw::auth::s3::AWSEngine {
                    const std::string_view& access_key_id,
                    const std::string& string_to_sign,
                    const std::string_view& signature,
-		   const signature_factory_t& signature_factory,
-                   bool ignore_signature) const;
+		   const signature_factory_t& signature_factory) const;
   result_t authenticate(const DoutPrefixProvider* dpp,
                         const std::string_view& access_key_id,
                         const std::string_view& signature,
