@@ -7,6 +7,8 @@
 #define dout_context g_ceph_context
 #define dout_subsys ceph_subsys_rgw
 
+using namespace std;
+
 int rgw_opa_authorize(RGWOp *& op,
                       req_state * const s)
 {
@@ -31,6 +33,7 @@ int rgw_opa_authorize(RGWOp *& op,
   /* set required headers for OPA request */
   req.append_header("X-Auth-Token", opa_token);
   req.append_header("Content-Type", "application/json");
+  req.append_header("Expect", "100-continue");
 
   /* check if we want to verify OPA server SSL certificate */
   req.set_verify_ssl(s->cct->_conf->rgw_opa_verify_ssl);
@@ -44,10 +47,10 @@ int rgw_opa_authorize(RGWOp *& op,
   jf.dump_string("decoded_uri", s->decoded_uri.c_str());
   jf.dump_string("params", s->info.request_params.c_str());
   jf.dump_string("request_uri_aws4", s->info.request_uri_aws4.c_str());
-  jf.dump_string("object_name", s->object.name.c_str());
+  jf.dump_string("object_name", s->object->get_name().c_str());
   jf.dump_string("subuser", s->auth.identity->get_subuser().c_str());
   jf.dump_object("user_info", s->user->get_info());
-  jf.dump_object("bucket_info", s->bucket_info);
+  jf.dump_object("bucket_info", s->bucket->get_info());
   jf.close_section();
   jf.close_section();
 
