@@ -5,27 +5,29 @@
 
 #include "rgw_auth.h"
 #include "rgw_auth_filters.h"
+#include "rgw_rest.h"
 
 class RGWHandler_REST_IAM : public RGWHandler_REST {
   const rgw::auth::StrategyRegistry& auth_registry;
-  const string& post_body;
+  const std::string& post_body;
   RGWOp *op_post() override;
   void rgw_iam_parse_input();
 public:
 
   static int init_from_header(struct req_state *s, int default_formatter, bool configurable_format);
 
-  RGWHandler_REST_IAM(const rgw::auth::StrategyRegistry& auth_registry, const string& post_body="")
+  RGWHandler_REST_IAM(const rgw::auth::StrategyRegistry& auth_registry,
+		      const std::string& post_body="")
     : RGWHandler_REST(),
       auth_registry(auth_registry),
       post_body(post_body) {}
   ~RGWHandler_REST_IAM() override = default;
 
-  int init(rgw::sal::RGWRadosStore *store,
+  int init(rgw::sal::Store* store,
            struct req_state *s,
            rgw::io::BasicClient *cio) override;
-  int authorize(const DoutPrefixProvider* dpp) override;
-  int postauth_init() override { return 0; }
+  int authorize(const DoutPrefixProvider* dpp, optional_yield y) override;
+  int postauth_init(optional_yield y) override { return 0; }
 };
 
 class RGWRESTMgr_IAM : public RGWRESTMgr {
@@ -39,7 +41,8 @@ public:
     return this;
   }
 
-  RGWHandler_REST* get_handler(struct req_state*,
+  RGWHandler_REST* get_handler(rgw::sal::Store* store,
+			       struct req_state*,
                                const rgw::auth::StrategyRegistry&,
                                const std::string&) override;
 };
