@@ -1,10 +1,11 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
-// vim: ts=8 sw=2 smarttab
+// vim: ts=8 sw=2 smarttab ft=cpp
 
-#ifndef CEPH_RGW_REST_SWIFT_H
-#define CEPH_RGW_REST_SWIFT_H
+#pragma once
 #define TIME_BUF_SIZE 128
 
+#include <string_view>
+
 #include <boost/optional.hpp>
 #include <boost/utility/typed_in_place_factory.hpp>
 
@@ -13,7 +14,6 @@
 #include "rgw_swift_auth.h"
 #include "rgw_http_errors.h"
 
-#include <boost/utility/string_ref.hpp>
 
 class RGWGetObj_ObjStore_SWIFT : public RGWGetObj_ObjStore {
   int custom_http_ret = 0;
@@ -21,9 +21,9 @@ public:
   RGWGetObj_ObjStore_SWIFT() {}
   ~RGWGetObj_ObjStore_SWIFT() override {}
 
-  int verify_permission() override;
-  int get_params() override;
-  int send_response_data_error() override;
+  int verify_permission(optional_yield y) override;
+  int get_params(optional_yield y) override;
+  int send_response_data_error(optional_yield y) override;
   int send_response_data(bufferlist& bl, off_t ofs, off_t len) override;
 
   void set_custom_http_response(const int http_ret) {
@@ -39,7 +39,7 @@ class RGWListBuckets_ObjStore_SWIFT : public RGWListBuckets_ObjStore {
   bool need_stats;
   bool wants_reversed;
   std::string prefix;
-  std::vector<RGWUserBuckets> reverse_buffer;
+  std::vector<rgw::sal::BucketList> reverse_buffer;
 
   uint64_t get_default_max() const override {
     return 0;
@@ -52,12 +52,12 @@ public:
   }
   ~RGWListBuckets_ObjStore_SWIFT() override {}
 
-  int get_params() override;
-  void handle_listing_chunk(RGWUserBuckets&& buckets) override;
+  int get_params(optional_yield y) override;
+  void handle_listing_chunk(rgw::sal::BucketList&& buckets) override;
   void send_response_begin(bool has_buckets) override;
-  void send_response_data(RGWUserBuckets& buckets) override;
-  void send_response_data_reversed(RGWUserBuckets& buckets);
-  void dump_bucket_entry(const RGWBucketEnt& obj);
+  void send_response_data(rgw::sal::BucketList& buckets) override;
+  void send_response_data_reversed(rgw::sal::BucketList& buckets);
+  void dump_bucket_entry(const rgw::sal::Bucket& obj);
   void send_response_end() override;
 
   bool should_get_stats() override { return need_stats; }
@@ -65,26 +65,26 @@ public:
 };
 
 class RGWListBucket_ObjStore_SWIFT : public RGWListBucket_ObjStore {
-  string path;
+  std::string path;
 public:
   RGWListBucket_ObjStore_SWIFT() {
     default_max = 10000;
   }
   ~RGWListBucket_ObjStore_SWIFT() override {}
 
-  int get_params() override;
+  int get_params(optional_yield y) override;
   void send_response() override;
   bool need_container_stats() override { return true; }
 };
 
 class RGWStatAccount_ObjStore_SWIFT : public RGWStatAccount_ObjStore {
-  map<string, bufferlist> attrs;
+  std::map<std::string, bufferlist> attrs;
 public:
   RGWStatAccount_ObjStore_SWIFT() {
   }
   ~RGWStatAccount_ObjStore_SWIFT() override {}
 
-  void execute() override;
+  void execute(optional_yield y) override;
   void send_response() override;
 };
 
@@ -103,7 +103,7 @@ public:
   RGWCreateBucket_ObjStore_SWIFT() {}
   ~RGWCreateBucket_ObjStore_SWIFT() override {}
 
-  int get_params() override;
+  int get_params(optional_yield y) override;
   void send_response() override;
 };
 
@@ -116,13 +116,15 @@ public:
 };
 
 class RGWPutObj_ObjStore_SWIFT : public RGWPutObj_ObjStore {
-  string lo_etag;
+  std::string lo_etag;
 public:
   RGWPutObj_ObjStore_SWIFT() {}
   ~RGWPutObj_ObjStore_SWIFT() override {}
 
-  int verify_permission() override;
-  int get_params() override;
+  int update_slo_segment_size(rgw_slo_entry& entry);
+
+  int verify_permission(optional_yield y) override;
+  int get_params(optional_yield y) override;
   void send_response() override;
 };
 
@@ -131,7 +133,7 @@ public:
   RGWPutMetadataAccount_ObjStore_SWIFT() {}
   ~RGWPutMetadataAccount_ObjStore_SWIFT() override {}
 
-  int get_params() override;
+  int get_params(optional_yield y) override;
   void send_response() override;
 };
 
@@ -140,7 +142,7 @@ public:
   RGWPutMetadataBucket_ObjStore_SWIFT() {}
   ~RGWPutMetadataBucket_ObjStore_SWIFT() override {}
 
-  int get_params() override;
+  int get_params(optional_yield y) override;
   void send_response() override;
 };
 
@@ -149,7 +151,7 @@ public:
   RGWPutMetadataObject_ObjStore_SWIFT() {}
   ~RGWPutMetadataObject_ObjStore_SWIFT() override {}
 
-  int get_params() override;
+  int get_params(optional_yield y) override;
   void send_response() override;
   bool need_object_expiration() override { return true; }
 };
@@ -159,8 +161,8 @@ public:
   RGWDeleteObj_ObjStore_SWIFT() {}
   ~RGWDeleteObj_ObjStore_SWIFT() override {}
 
-  int verify_permission() override;
-  int get_params() override;
+  int verify_permission(optional_yield y) override;
+  int get_params(optional_yield y) override;
   bool need_object_expiration() override { return true; }
   void send_response() override;
 };
@@ -174,7 +176,7 @@ public:
   ~RGWCopyObj_ObjStore_SWIFT() override {}
 
   int init_dest_policy() override;
-  int get_params() override;
+  int get_params(optional_yield y) override;
   void send_response() override;
   void send_partial_response(off_t ofs) override;
 };
@@ -233,21 +235,21 @@ protected:
   struct info
   {
     bool is_admin_info;
-    function<void (Formatter&, const md_config_t&, RGWRados&)> list_data;
+    std::function<void (Formatter&, const ConfigProxy&, rgw::sal::Store*)> list_data;
   };
 
-  static const vector<pair<string, struct info>> swift_info;
+  static const std::vector<std::pair<std::string, struct info>> swift_info;
 public:
   RGWInfo_ObjStore_SWIFT() {}
   ~RGWInfo_ObjStore_SWIFT() override {}
 
-  void execute() override;
+  void execute(optional_yield y) override;
   void send_response() override;
-  static void list_swift_data(Formatter& formatter, const md_config_t& config, RGWRados& store);
-  static void list_tempauth_data(Formatter& formatter, const md_config_t& config, RGWRados& store);
-  static void list_tempurl_data(Formatter& formatter, const md_config_t& config, RGWRados& store);
-  static void list_slo_data(Formatter& formatter, const md_config_t& config, RGWRados& store);
-  static bool is_expired(const std::string& expires, CephContext* cct);
+  static void list_swift_data(Formatter& formatter, const ConfigProxy& config, rgw::sal::Store* store);
+  static void list_tempauth_data(Formatter& formatter, const ConfigProxy& config, rgw::sal::Store* store);
+  static void list_tempurl_data(Formatter& formatter, const ConfigProxy& config, rgw::sal::Store* store);
+  static void list_slo_data(Formatter& formatter, const ConfigProxy& config, rgw::sal::Store* store);
+  static bool is_expired(const std::string& expires, const DoutPrefixProvider* dpp);
 };
 
 
@@ -271,11 +273,11 @@ public:
   RGWFormPost() = default;
   ~RGWFormPost() = default;
 
-  void init(RGWRados* store,
+  void init(rgw::sal::Store* store,
             req_state* s,
             RGWHandler* dialect_handler) override;
 
-  int get_params() override;
+  int get_params(optional_yield y) override;
   int get_data(ceph::bufferlist& bl, bool& again) override;
   void send_response() override;
 
@@ -295,11 +297,11 @@ public:
   SignatureHelper() = default;
 
   const char* calc(const std::string& key,
-                   const boost::string_ref& path_info,
-                   const boost::string_ref& redirect,
-                   const boost::string_ref& max_file_size,
-                   const boost::string_ref& max_file_count,
-                   const boost::string_ref& expires) {
+                   const std::string_view& path_info,
+                   const std::string_view& redirect,
+                   const std::string_view& max_file_size,
+                   const std::string_view& max_file_count,
+                   const std::string_view& expires) {
     using ceph::crypto::HMACSHA1;
     using UCHARPTR = const unsigned char*;
 
@@ -342,22 +344,22 @@ public:
 
 
 class RGWSwiftWebsiteHandler {
-  RGWRados* const store;
+  rgw::sal::Store* const store;
   req_state* const s;
   RGWHandler_REST* const handler;
 
   bool is_web_mode() const;
   bool can_be_website_req() const;
   bool is_web_dir() const;
-  bool is_index_present(const std::string& index);
+  bool is_index_present(const std::string& index) const;
 
-  int serve_errordoc(int http_ret, std::string error_doc);
+  int serve_errordoc(int http_ret, std::string error_doc, optional_yield y);
 
   RGWOp* get_ws_redirect_op();
   RGWOp* get_ws_index_op();
   RGWOp* get_ws_listing_op();
 public:
-  RGWSwiftWebsiteHandler(RGWRados* const store,
+  RGWSwiftWebsiteHandler(rgw::sal::Store* const store,
                          req_state* const s,
                          RGWHandler_REST* const handler)
     : store(store),
@@ -366,7 +368,8 @@ public:
   }
 
   int error_handler(const int err_no,
-                    std::string* const error_content);
+                    std::string* const error_content,
+		    optional_yield y);
   int retarget_bucket(RGWOp* op, RGWOp** new_op);
   int retarget_object(RGWOp* op, RGWOp** new_op);
 };
@@ -378,23 +381,23 @@ class RGWHandler_REST_SWIFT : public RGWHandler_REST {
 protected:
   const rgw::auth::Strategy& auth_strategy;
 
-  virtual bool is_acl_op() {
+  virtual bool is_acl_op() const {
     return false;
   }
 
-  static int init_from_header(struct req_state* s,
+  static int init_from_header(rgw::sal::Store* store, struct req_state* s,
                               const std::string& frontend_prefix);
 public:
-  RGWHandler_REST_SWIFT(const rgw::auth::Strategy& auth_strategy)
+  explicit RGWHandler_REST_SWIFT(const rgw::auth::Strategy& auth_strategy)
     : auth_strategy(auth_strategy) {
   }
   ~RGWHandler_REST_SWIFT() override = default;
 
-  int validate_bucket_name(const string& bucket);
+  int validate_bucket_name(const std::string& bucket);
 
-  int init(RGWRados *store, struct req_state *s, rgw::io::BasicClient *cio) override;
-  int authorize() override;
-  int postauth_init() override;
+  int init(rgw::sal::Store* store, struct req_state *s, rgw::io::BasicClient *cio) override;
+  int authorize(const DoutPrefixProvider *dpp, optional_yield y) override;
+  int postauth_init(optional_yield y) override;
 
   RGWAccessControlPolicy *alloc_policy() { return nullptr; /* return new RGWAccessControlPolicy_SWIFT; */ }
   void free_policy(RGWAccessControlPolicy *policy) { delete policy; }
@@ -417,7 +420,7 @@ class RGWHandler_REST_Bucket_SWIFT : public RGWHandler_REST_SWIFT {
    * initialization (see the init() method). */
   boost::optional<RGWSwiftWebsiteHandler> website_handler;
 protected:
-  bool is_obj_update_op() override {
+  bool is_obj_update_op() const override {
     return s->op == OP_POST;
   }
 
@@ -432,15 +435,15 @@ public:
   using RGWHandler_REST_SWIFT::RGWHandler_REST_SWIFT;
   ~RGWHandler_REST_Bucket_SWIFT() override = default;
 
-  int error_handler(int err_no, std::string *error_content) override {
-    return website_handler->error_handler(err_no, error_content);
+  int error_handler(int err_no, std::string *error_content, optional_yield y) override {
+    return website_handler->error_handler(err_no, error_content, y);
   }
 
-  int retarget(RGWOp* op, RGWOp** new_op) override {
+  int retarget(RGWOp* op, RGWOp** new_op, optional_yield) override {
     return website_handler->retarget_bucket(op, new_op);
   }
 
-  int init(RGWRados* const store,
+  int init(rgw::sal::Store* const store,
            struct req_state* const s,
            rgw::io::BasicClient* const cio) override {
     website_handler = boost::in_place<RGWSwiftWebsiteHandler>(store, s, this);
@@ -453,7 +456,7 @@ class RGWHandler_REST_Obj_SWIFT : public RGWHandler_REST_SWIFT {
    * initialization (see the init() method). */
   boost::optional<RGWSwiftWebsiteHandler> website_handler;
 protected:
-  bool is_obj_update_op() override {
+  bool is_obj_update_op() const override {
     return s->op == OP_POST;
   }
 
@@ -470,15 +473,16 @@ public:
   using RGWHandler_REST_SWIFT::RGWHandler_REST_SWIFT;
   ~RGWHandler_REST_Obj_SWIFT() override = default;
 
-  int error_handler(int err_no, std::string *error_content) override {
-    return website_handler->error_handler(err_no, error_content);
+  int error_handler(int err_no, std::string *error_content,
+		    optional_yield y) override {
+    return website_handler->error_handler(err_no, error_content, y);
   }
 
-  int retarget(RGWOp* op, RGWOp** new_op) override {
+  int retarget(RGWOp* op, RGWOp** new_op, optional_yield) override {
     return website_handler->retarget_object(op, new_op);
   }
 
-  int init(RGWRados* const store,
+  int init(rgw::sal::Store* const store,
            struct req_state* const s,
            rgw::io::BasicClient* const cio) override {
     website_handler = boost::in_place<RGWSwiftWebsiteHandler>(store, s, this);
@@ -498,7 +502,8 @@ public:
   RGWRESTMgr_SWIFT() = default;
   ~RGWRESTMgr_SWIFT() override = default;
 
-  RGWHandler_REST *get_handler(struct req_state *s,
+  RGWHandler_REST *get_handler(rgw::sal::Store* store,
+			       struct req_state *s,
                                const rgw::auth::StrategyRegistry& auth_registry,
                                const std::string& frontend_prefix) override;
 };
@@ -531,7 +536,7 @@ public:
     return new RGWGetCrossDomainPolicy_ObjStore_SWIFT();
   }
 
-  int init(RGWRados* const store,
+  int init(rgw::sal::Store* const store,
            struct req_state* const state,
            rgw::io::BasicClient* const cio) override {
     state->dialect = "swift";
@@ -541,15 +546,15 @@ public:
     return RGWHandler::init(store, state, cio);
   }
 
-  int authorize() override {
+  int authorize(const DoutPrefixProvider *dpp, optional_yield) override {
     return 0;
   }
 
-  int postauth_init() override {
+  int postauth_init(optional_yield) override {
     return 0;
   }
 
-  int read_permissions(RGWOp *) override {
+  int read_permissions(RGWOp *, optional_yield y) override {
     return 0;
   }
 
@@ -569,7 +574,8 @@ public:
   RGWRESTMgr_SWIFT_CrossDomain() = default;
   ~RGWRESTMgr_SWIFT_CrossDomain() override = default;
 
-  RGWHandler_REST* get_handler(struct req_state* const s,
+  RGWHandler_REST* get_handler(rgw::sal::Store* store,
+			       struct req_state* const s,
                                const rgw::auth::StrategyRegistry&,
                                const std::string&) override {
     s->prot_flags |= RGW_REST_SWIFT;
@@ -587,7 +593,7 @@ public:
     return new RGWGetHealthCheck_ObjStore_SWIFT();
   }
 
-  int init(RGWRados* const store,
+  int init(rgw::sal::Store* const store,
            struct req_state* const state,
            rgw::io::BasicClient* const cio) override {
     state->dialect = "swift";
@@ -597,15 +603,15 @@ public:
     return RGWHandler::init(store, state, cio);
   }
 
-  int authorize() override {
+  int authorize(const DoutPrefixProvider *dpp, optional_yield y) override {
     return 0;
   }
 
-  int postauth_init() override {
+  int postauth_init(optional_yield) override {
     return 0;
   }
 
-  int read_permissions(RGWOp *) override {
+  int read_permissions(RGWOp *, optional_yield y) override {
     return 0;
   }
 
@@ -625,7 +631,8 @@ public:
   RGWRESTMgr_SWIFT_HealthCheck() = default;
   ~RGWRESTMgr_SWIFT_HealthCheck() override = default;
 
-  RGWHandler_REST* get_handler(struct req_state* const s,
+  RGWHandler_REST* get_handler(rgw::sal::Store* store,
+			       struct req_state* const s,
                                const rgw::auth::StrategyRegistry&,
                                const std::string&) override {
     s->prot_flags |= RGW_REST_SWIFT;
@@ -643,7 +650,7 @@ public:
     return new RGWInfo_ObjStore_SWIFT();
   }
 
-  int init(RGWRados* const store,
+  int init(rgw::sal::Store* const store,
            struct req_state* const state,
            rgw::io::BasicClient* const cio) override {
     state->dialect = "swift";
@@ -653,15 +660,15 @@ public:
     return RGWHandler::init(store, state, cio);
   }
 
-  int authorize() override {
+  int authorize(const DoutPrefixProvider *dpp, optional_yield) override {
     return 0;
   }
 
-  int postauth_init() override {
+  int postauth_init(optional_yield) override {
     return 0;
   }
 
-  int read_permissions(RGWOp *) override {
+  int read_permissions(RGWOp *, optional_yield y) override {
     return 0;
   }
 };
@@ -671,9 +678,8 @@ public:
   RGWRESTMgr_SWIFT_Info() = default;
   ~RGWRESTMgr_SWIFT_Info() override = default;
 
-  RGWHandler_REST *get_handler(struct req_state* s,
+  RGWHandler_REST *get_handler(rgw::sal::Store* store,
+			       struct req_state* s,
                                const rgw::auth::StrategyRegistry& auth_registry,
                                const std::string& frontend_prefix) override;
 };
-
-#endif
