@@ -15,8 +15,16 @@
 
 #define dout_subsys ceph_subsys_rgw
 
+using namespace std;
+
 static string notify_oid_prefix = "notify";
 
+RGWSI_Notify::~RGWSI_Notify()
+{
+  shutdown();
+}
+
+
 class RGWWatcher : public DoutPrefixProvider , public librados::WatchCtx2 {
   CephContext *cct;
   RGWSI_Notify *svc;
@@ -24,7 +32,6 @@ class RGWWatcher : public DoutPrefixProvider , public librados::WatchCtx2 {
   RGWSI_RADOS::Obj obj;
   uint64_t watch_handle;
   int register_ret{0};
-  bool unregister_done{false};
   librados::AioCompletion *register_completion{nullptr};
 
   class C_ReinitWatch : public Context {
@@ -71,30 +78,27 @@ public:
     obj.notify_ack(notify_id, cookie, reply_bl);
   }
   void handle_error(uint64_t cookie, int err) override {
-    lderr(cct) << "RGWWatcher::handle_error cookie " << cookie
+    ldpp_dout(this, -1) << "RGWWatcher::handle_error cookie " << cookie
 			<< " err " << cpp_strerror(err) << dendl;
     svc->remove_watcher(index);
     svc->schedule_context(new C_ReinitWatch(this));
   }
 
   void reinit() {
-    if(!unregister_done) {
-      int ret = unregister_watch();
-      if (ret < 0) {
-        ldout(cct, 0) << "ERROR: unregister_watch() returned ret=" << ret << dendl;
-      }
+    int ret = unregister_watch();
+    if (ret < 0) {
+      ldout(cct, 0) << "ERROR: unregister_watch() returned ret=" << ret << dendl;
+      return;
     }
-    int ret = register_watch();
+    ret = register_watch();
     if (ret < 0) {
       ldout(cct, 0) << "ERROR: register_watch() returned ret=" << ret << dendl;
-      svc->schedule_context(new C_ReinitWatch(this));
       return;
     }
   }
 
   int unregister_watch() {
     int r = svc->unwatch(obj, watch_handle);
-    unregister_done = true;
     if (r < 0) {
       return r;
     }
@@ -131,7 +135,6 @@ public:
       return r;
     }
     svc->add_watcher(index);
-    unregister_done = false;
     return 0;
   }
 
@@ -141,7 +144,6 @@ public:
       return r;
     }
     svc->add_watcher(index);
-    unregister_done = false;
     return 0;
   }
 };
@@ -245,8 +247,7 @@ void RGWSI_Notify::finalize_watch()
 {
   for (int i = 0; i < num_watchers; i++) {
     RGWWatcher *watcher = watchers[i];
-    if (watchers_set.find(i) != watchers_set.end())
-      watcher->unregister_watch();
+    watcher->unregister_watch();
     delete watcher;
   }
 
@@ -271,15 +272,11 @@ int RGWSI_Notify::do_start(optional_yield y, const DoutPrefixProvider *dpp)
     return r;
   }
 
-  inject_notify_timeout_probability =
-    cct->_conf.get_val<double>("rgw_inject_notify_timeout_probability");
-  max_notify_retries = cct->_conf.get_val<uint64_t>("rgw_max_notify_retries");
-
   control_pool = zone_svc->get_zone_params().control_pool;
 
   int ret = init_watch(dpp, y);
   if (ret < 0) {
-    lderr(cct) << "ERROR: failed to initialize watch: " << cpp_strerror(-ret) << dendl;
+    ldpp_dout(dpp, -1) << "ERROR: failed to initialize watch: " << cpp_strerror(-ret) << dendl;
     return ret;
   }
 
@@ -307,11 +304,6 @@ void RGWSI_Notify::shutdown()
   finalized = true;
 }
 
-RGWSI_Notify::~RGWSI_Notify()
-{
-  shutdown();
-}
-
 int RGWSI_Notify::unwatch(RGWSI_RADOS::Obj& obj, uint64_t watch_handle)
 {
   int r = obj.unwatch(watch_handle);
@@ -398,69 +390,19 @@ int RGWSI_Notify::distribute(const DoutPrefixProvider *dpp, const string& key,
   return 0;
 }
 
-namespace librados {
-
-static std::ostream& operator<<(std::ostream& out, const notify_timeout_t& t)
-{
-  return out << t.notifier_id << ':' << t.cookie;
-}
-
-} // namespace librados
-
-using timeout_vector = std::vector<librados::notify_timeout_t>;
-
-static timeout_vector decode_timeouts(const bufferlist& bl)
-{
-  using ceph::decode;
-  auto p = bl.begin();
-
-  // decode and discard the acks
-  uint32_t num_acks;
-  decode(num_acks, p);
-  for (auto i = 0u; i < num_acks; ++i) {
-    std::pair<uint64_t, uint64_t> id;
-    decode(id, p);
-    // discard the payload
-    uint32_t blen;
-    decode(blen, p);
-    p += blen;
-  }
-
-  // decode and return the timeouts
-  uint32_t num_timeouts;
-  decode(num_timeouts, p);
-
-  timeout_vector timeouts;
-  for (auto i = 0u; i < num_timeouts; ++i) {
-    std::pair<uint64_t, uint64_t> id;
-    decode(id, p);
-    timeouts.push_back({id.first, id.second});
-  }
-  return timeouts;
-}
-
 int RGWSI_Notify::robust_notify(const DoutPrefixProvider *dpp,
                                 RGWSI_RADOS::Obj& notify_obj,
 				const RGWCacheNotifyInfo& cni,
                                 optional_yield y)
 {
-  bufferlist bl, rbl;
+  bufferlist bl;
   encode(cni, bl);
 
   // First, try to send, without being fancy about it.
-  auto r = notify_obj.notify(dpp, bl, 0, &rbl, y);
+  auto r = notify_obj.notify(dpp, bl, 0, nullptr, y);
 
   if (r < 0) {
-    timeout_vector timeouts;
-    try {
-      timeouts = decode_timeouts(rbl);
-    } catch (const buffer::error& e) {
-      ldpp_dout(dpp, 0) << "robust_notify failed to decode notify response: "
-          << e.what() << dendl;
-    }
-
     ldpp_dout(dpp, 1) << __PRETTY_FUNCTION__ << ":" << __LINE__
-		      << " Watchers " << timeouts << " did not respond."
 		      << " Notify failed on object " << cni.obj << ": "
 		      << cpp_strerror(-r) << dendl;
   }
@@ -468,7 +410,7 @@ int RGWSI_Notify::robust_notify(const DoutPrefixProvider *dpp,
   // If we timed out, get serious.
   if (r == -ETIMEDOUT) {
     RGWCacheNotifyInfo info;
-    info.op = REMOVE_OBJ;
+    info.op = INVALIDATE_OBJ;
     info.obj = cni.obj;
     bufferlist retrybl;
     encode(info, retrybl);
@@ -479,19 +421,10 @@ int RGWSI_Notify::robust_notify(const DoutPrefixProvider *dpp,
       ldpp_dout(dpp, 1) << __PRETTY_FUNCTION__ << ":" << __LINE__
 			<< " Invalidating obj=" << info.obj << " tries="
 			<< tries << dendl;
-      r = notify_obj.notify(dpp, retrybl, 0, &rbl, y);
+      r = notify_obj.notify(dpp, bl, 0, nullptr, y);
       if (r < 0) {
-        timeout_vector timeouts;
-        try {
-          timeouts = decode_timeouts(rbl);
-        } catch (const buffer::error& e) {
-          ldpp_dout(dpp, 0) << "robust_notify failed to decode notify response: "
-              << e.what() << dendl;
-        }
-
 	ldpp_dout(dpp, 1) << __PRETTY_FUNCTION__ << ":" << __LINE__
-			  << " Watchers " << timeouts << " did not respond."
-			  << " Invalidation attempt " << tries << " failed: "
+			  << " invalidation attempt " << tries << " failed: "
 			  << cpp_strerror(-r) << dendl;
       }
     }
