@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-3-Clause
  *
- * Copyright (c) 2016-2018 Solarflare Communications Inc.
- * All rights reserved.
+ * Copyright(c) 2019-2020 Xilinx, Inc.
+ * Copyright(c) 2016-2019 Solarflare Communications Inc.
  *
  * This software was jointly developed between OKTET Labs (under contract
  * for Solarflare) and Solarflare Communications, Inc.
@@ -25,12 +25,14 @@
 /* Log PMD message, automatically add prefix and \n */
 #define sfc_panic(sa, fmt, args...) \
 	do {								\
-		const struct sfc_adapter *_sa = (sa);			\
+		const struct sfc_adapter_shared *_sas;			\
 									\
-		rte_panic("sfc " PCI_PRI_FMT " #%" PRIu8 ": " fmt "\n",	\
-			  _sa->pci_addr.domain, _sa->pci_addr.bus,	\
-			  _sa->pci_addr.devid, _sa->pci_addr.function,	\
-			  _sa->port_id, ##args);			\
+		_sas = (sa)->priv.shared;				\
+		rte_panic("sfc " PCI_PRI_FMT				\
+			  " #%" PRIu16 ": " fmt "\n",			\
+			  _sas->pci_addr.domain, _sas->pci_addr.bus,	\
+			  _sas->pci_addr.devid, _sas->pci_addr.function,\
+			  _sas->port_id, ##args);			\
 	} while (0)
 
 #endif /* _SFC_DEBUG_H_ */
