@@ -44,7 +44,7 @@
 #include "unit/lib/json_mock.c"
 #include "common/lib/test_env.c"
 
-struct spdk_iscsi_globals g_spdk_iscsi;
+struct spdk_iscsi_globals g_iscsi;
 
 const char *config_file;
 
@@ -71,10 +71,10 @@ DEFINE_STUB(spdk_sock_is_ipv6, bool, (struct spdk_sock *sock), false);
 
 DEFINE_STUB(spdk_sock_is_ipv4, bool, (struct spdk_sock *sock), false);
 
-DEFINE_STUB(spdk_iscsi_portal_grp_find_by_tag,
+DEFINE_STUB(iscsi_portal_grp_find_by_tag,
 	    struct spdk_iscsi_portal_grp *, (int tag), NULL);
 
-DEFINE_STUB(spdk_iscsi_init_grp_find_by_tag, struct spdk_iscsi_init_grp *,
+DEFINE_STUB(iscsi_init_grp_find_by_tag, struct spdk_iscsi_init_grp *,
 	    (int tag), NULL);
 
 struct spdk_scsi_lun *
@@ -102,51 +102,48 @@ spdk_scsi_dev_add_lun(struct spdk_scsi_dev *dev, const char *bdev_name, int lun_
 static void
 add_lun_test_cases(void)
 {
-	struct spdk_iscsi_tgt_node tgtnode;
+	struct spdk_iscsi_tgt_node tgtnode = {};
 	int lun_id = 0;
 	char *bdev_name = NULL;
-	struct spdk_scsi_dev scsi_dev;
+	struct spdk_scsi_dev scsi_dev = {};
 	int rc;
 
-	memset(&tgtnode, 0, sizeof(struct spdk_iscsi_tgt_node));
-	memset(&scsi_dev, 0, sizeof(struct spdk_scsi_dev));
-
 	/* case 1 */
 	tgtnode.num_active_conns = 1;
 
-	rc = spdk_iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
+	rc = iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
 	CU_ASSERT(rc != 0);
 
 	/* case 2 */
 	tgtnode.num_active_conns = 0;
 	lun_id = -2;
 
-	rc = spdk_iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
+	rc = iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
 	CU_ASSERT(rc != 0);
 
 	/* case 3 */
 	lun_id = SPDK_SCSI_DEV_MAX_LUN;
 
-	rc = spdk_iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
+	rc = iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
 	CU_ASSERT(rc != 0);
 
 	/* case 4 */
 	lun_id = -1;
 	tgtnode.dev = NULL;
 
-	rc = spdk_iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
+	rc = iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
 	CU_ASSERT(rc != 0);
 
 	/* case 5 */
 	tgtnode.dev = &scsi_dev;
 
-	rc = spdk_iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
+	rc = iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
 	CU_ASSERT(rc != 0);
 
 	/* case 6 */
 	bdev_name = "LUN0";
 
-	rc = spdk_iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
+	rc = iscsi_tgt_node_add_lun(&tgtnode, bdev_name, lun_id);
 	CU_ASSERT(rc == 0);
 }
 
@@ -344,42 +341,38 @@ allow_ipv4_invalid(void)
 static void
 node_access_allowed(void)
 {
-	struct spdk_iscsi_tgt_node tgtnode;
-	struct spdk_iscsi_portal_grp pg;
-	struct spdk_iscsi_init_grp ig;
-	struct spdk_iscsi_conn conn;
-	struct spdk_iscsi_portal portal;
-	struct spdk_iscsi_initiator_name iname;
-	struct spdk_iscsi_initiator_netmask imask;
-	struct spdk_scsi_dev scsi_dev;
+	struct spdk_iscsi_tgt_node tgtnode = {};
+	struct spdk_iscsi_portal_grp pg = {};
+	struct spdk_iscsi_init_grp ig = {};
+	struct spdk_iscsi_conn conn = {};
+	struct spdk_iscsi_portal portal = {};
+	struct spdk_iscsi_initiator_name iname = {};
+	struct spdk_iscsi_initiator_netmask imask = {};
+	struct spdk_scsi_dev scsi_dev = {};
 	struct spdk_iscsi_pg_map *pg_map;
 	char *iqn, *addr;
 	bool result;
 
 	/* portal group initialization */
-	memset(&pg, 0, sizeof(struct spdk_iscsi_portal_grp));
 	pg.tag = 1;
 
 	/* initiator group initialization */
-	memset(&ig, 0, sizeof(struct spdk_iscsi_init_grp));
 	ig.tag = 1;
 
 	ig.ninitiators = 1;
-	iname.name = "iqn.2017-10.spdk.io:0001";
+	snprintf(iname.name, sizeof(iname.name), "iqn.2017-10.spdk.io:0001");
 	TAILQ_INIT(&ig.initiator_head);
 	TAILQ_INSERT_TAIL(&ig.initiator_head, &iname, tailq);
 
 	ig.nnetmasks = 1;
-	imask.mask = "192.168.2.0/24";
+	snprintf(imask.mask, sizeof(imask.mask), "192.168.2.0/24");
 	TAILQ_INIT(&ig.netmask_head);
 	TAILQ_INSERT_TAIL(&ig.netmask_head, &imask, tailq);
 
 	/* target initialization */
-	memset(&tgtnode, 0, sizeof(struct spdk_iscsi_tgt_node));
-	tgtnode.name = "iqn.2017-10.spdk.io:0001";
+	snprintf(tgtnode.name, sizeof(tgtnode.name), "iqn.2017-10.spdk.io:0001");
 	TAILQ_INIT(&tgtnode.pg_map_head);
 
-	memset(&scsi_dev, 0, sizeof(struct spdk_scsi_dev));
 	snprintf(scsi_dev.name, sizeof(scsi_dev.name), "iqn.2017-10.spdk.io:0001");
 	tgtnode.dev = &scsi_dev;
 
@@ -387,19 +380,17 @@ node_access_allowed(void)
 	iscsi_pg_map_add_ig_map(pg_map, &ig);
 
 	/* portal initialization */
-	memset(&portal, 0, sizeof(struct spdk_iscsi_portal));
 	portal.group = &pg;
-	portal.host = "192.168.2.0";
-	portal.port = "3260";
+	snprintf(portal.host, sizeof(portal.host), "192.168.2.0");
+	snprintf(portal.port, sizeof(portal.port), "3260");
 
 	/* input for UT */
-	memset(&conn, 0, sizeof(struct spdk_iscsi_conn));
 	conn.portal = &portal;
 
 	iqn = "iqn.2017-10.spdk.io:0001";
 	addr = "192.168.2.1";
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == true);
 
 	iscsi_pg_map_delete_ig_map(pg_map, &ig);
@@ -409,27 +400,25 @@ node_access_allowed(void)
 static void
 node_access_denied_by_empty_netmask(void)
 {
-	struct spdk_iscsi_tgt_node tgtnode;
-	struct spdk_iscsi_portal_grp pg;
-	struct spdk_iscsi_init_grp ig;
-	struct spdk_iscsi_conn conn;
-	struct spdk_iscsi_portal portal;
-	struct spdk_iscsi_initiator_name iname;
-	struct spdk_scsi_dev scsi_dev;
+	struct spdk_iscsi_tgt_node tgtnode = {};
+	struct spdk_iscsi_portal_grp pg = {};
+	struct spdk_iscsi_init_grp ig = {};
+	struct spdk_iscsi_conn conn = {};
+	struct spdk_iscsi_portal portal = {};
+	struct spdk_iscsi_initiator_name iname = {};
+	struct spdk_scsi_dev scsi_dev = {};
 	struct spdk_iscsi_pg_map *pg_map;
 	char *iqn, *addr;
 	bool result;
 
 	/* portal group initialization */
-	memset(&pg, 0, sizeof(struct spdk_iscsi_portal_grp));
 	pg.tag = 1;
 
 	/* initiator group initialization */
-	memset(&ig, 0, sizeof(struct spdk_iscsi_init_grp));
 	ig.tag = 1;
 
 	ig.ninitiators = 1;
-	iname.name = "iqn.2017-10.spdk.io:0001";
+	snprintf(iname.name, sizeof(iname.name), "iqn.2017-10.spdk.io:0001");
 	TAILQ_INIT(&ig.initiator_head);
 	TAILQ_INSERT_TAIL(&ig.initiator_head, &iname, tailq);
 
@@ -437,11 +426,9 @@ node_access_denied_by_empty_netmask(void)
 	TAILQ_INIT(&ig.netmask_head);
 
 	/* target initialization */
-	memset(&tgtnode, 0, sizeof(struct spdk_iscsi_tgt_node));
-	tgtnode.name = "iqn.2017-10.spdk.io:0001";
+	snprintf(tgtnode.name, sizeof(tgtnode.name), "iqn.2017-10.spdk.io:0001");
 	TAILQ_INIT(&tgtnode.pg_map_head);
 
-	memset(&scsi_dev, 0, sizeof(struct spdk_scsi_dev));
 	snprintf(scsi_dev.name, sizeof(scsi_dev.name), "iqn.2017-10.spdk.io:0001");
 	tgtnode.dev = &scsi_dev;
 
@@ -449,19 +436,17 @@ node_access_denied_by_empty_netmask(void)
 	iscsi_pg_map_add_ig_map(pg_map, &ig);
 
 	/* portal initialization */
-	memset(&portal, 0, sizeof(struct spdk_iscsi_portal));
 	portal.group = &pg;
-	portal.host = "192.168.2.0";
-	portal.port = "3260";
+	snprintf(portal.host, sizeof(portal.host), "192.168.2.0");
+	snprintf(portal.port, sizeof(portal.port), "3260");
 
 	/* input for UT */
-	memset(&conn, 0, sizeof(struct spdk_iscsi_conn));
 	conn.portal = &portal;
 
 	iqn = "iqn.2017-10.spdk.io:0001";
 	addr = "192.168.3.1";
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == false);
 
 	iscsi_pg_map_delete_ig_map(pg_map, &ig);
@@ -477,56 +462,47 @@ node_access_denied_by_empty_netmask(void)
 static void
 node_access_multi_initiator_groups_cases(void)
 {
-	struct spdk_iscsi_tgt_node tgtnode;
-	struct spdk_iscsi_conn conn;
-	struct spdk_iscsi_portal_grp pg;
-	struct spdk_iscsi_portal portal;
-	struct spdk_iscsi_init_grp ig1, ig2;
-	struct spdk_iscsi_initiator_name iname1, iname2;
-	struct spdk_iscsi_initiator_netmask imask1, imask2;
-	struct spdk_scsi_dev scsi_dev;
+	struct spdk_iscsi_tgt_node tgtnode = {};
+	struct spdk_iscsi_conn conn = {};
+	struct spdk_iscsi_portal_grp pg = {};
+	struct spdk_iscsi_portal portal = {};
+	struct spdk_iscsi_init_grp ig1 = {}, ig2 = {};
+	struct spdk_iscsi_initiator_name iname1 = {}, iname2 = {};
+	struct spdk_iscsi_initiator_netmask imask1 = {}, imask2 = {};
+	struct spdk_scsi_dev scsi_dev = {};
 	struct spdk_iscsi_pg_map *pg_map;
 	char *iqn, *addr;
 	bool result;
 
 	/* target initialization */
-	memset(&tgtnode, 0, sizeof(struct spdk_iscsi_tgt_node));
-	tgtnode.name = IQN1;
+	snprintf(tgtnode.name, sizeof(tgtnode.name), IQN1);
 	TAILQ_INIT(&tgtnode.pg_map_head);
 
-	memset(&scsi_dev, 0, sizeof(struct spdk_scsi_dev));
 	snprintf(scsi_dev.name, sizeof(scsi_dev.name), IQN1);
 	tgtnode.dev = &scsi_dev;
 
 	/* initiator group initialization */
-	memset(&ig1, 0, sizeof(struct spdk_iscsi_init_grp));
 	ig1.tag = 1;
 	TAILQ_INIT(&ig1.initiator_head);
 	TAILQ_INIT(&ig1.netmask_head);
 
 	ig1.ninitiators = 1;
-	iname1.name = NULL;
 	TAILQ_INSERT_TAIL(&ig1.initiator_head, &iname1, tailq);
 
 	ig1.nnetmasks = 1;
-	imask1.mask = NULL;
 	TAILQ_INSERT_TAIL(&ig1.netmask_head, &imask1, tailq);
 
-	memset(&ig2, 0, sizeof(struct spdk_iscsi_init_grp));
 	ig2.tag = 2;
 	TAILQ_INIT(&ig2.initiator_head);
 	TAILQ_INIT(&ig2.netmask_head);
 
 	ig2.ninitiators = 1;
-	iname2.name = NULL;
 	TAILQ_INSERT_TAIL(&ig2.initiator_head, &iname2, tailq);
 
 	ig2.nnetmasks = 1;
-	imask2.mask = NULL;
 	TAILQ_INSERT_TAIL(&ig2.netmask_head, &imask2, tailq);
 
 	/* portal group initialization */
-	memset(&pg, 0, sizeof(struct spdk_iscsi_portal_grp));
 	pg.tag = 1;
 
 	pg_map = iscsi_tgt_node_add_pg_map(&tgtnode, &pg);
@@ -534,13 +510,11 @@ node_access_multi_initiator_groups_cases(void)
 	iscsi_pg_map_add_ig_map(pg_map, &ig2);
 
 	/* portal initialization */
-	memset(&portal, 0, sizeof(struct spdk_iscsi_portal));
 	portal.group = &pg;
-	portal.host = IP1;
-	portal.port = "3260";
+	snprintf(portal.host, sizeof(portal.host), IP1);
+	snprintf(portal.port, sizeof(portal.port), "3260");
 
 	/* connection initialization */
-	memset(&conn, 0, sizeof(struct spdk_iscsi_conn));
 	conn.portal = &portal;
 
 	iqn = IQN1;
@@ -557,9 +531,9 @@ node_access_multi_initiator_groups_cases(void)
 	 * | denied    | -       | -         | -       | denied  |
 	 * +-------------------------------------------+---------+
 	 */
-	iname1.name = NO_IQN1;
+	snprintf(iname1.name, sizeof(iname1.name), NO_IQN1);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == false);
 
 	/*
@@ -573,10 +547,10 @@ node_access_multi_initiator_groups_cases(void)
 	 * | allowed   | allowed | -         | -       | allowed |
 	 * +-------------------------------------------+---------+
 	 */
-	iname1.name = IQN1;
-	imask1.mask = IP1;
+	snprintf(iname1.name, sizeof(iname1.name), IQN1);
+	snprintf(imask1.mask, sizeof(imask1.mask), IP1);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == true);
 
 	/*
@@ -590,11 +564,11 @@ node_access_multi_initiator_groups_cases(void)
 	 * | allowed   | denied  | denied   | -        | denied  |
 	 * +-------------------------------------------+---------+
 	 */
-	iname1.name = IQN1;
-	imask1.mask = IP2;
-	iname2.name = NO_IQN1;
+	snprintf(iname1.name, sizeof(iname1.name), IQN1);
+	snprintf(imask1.mask, sizeof(imask1.mask), IP2);
+	snprintf(iname2.name, sizeof(iname2.name), NO_IQN1);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == false);
 
 	/*
@@ -608,12 +582,12 @@ node_access_multi_initiator_groups_cases(void)
 	 * | allowed   | denied  | allowed   | allowed | allowed |
 	 * +-------------------------------------------+---------+
 	 */
-	iname1.name = IQN1;
-	imask1.mask = IP2;
-	iname2.name = IQN1;
-	imask2.mask = IP1;
+	snprintf(iname1.name, sizeof(iname1.name), IQN1);
+	snprintf(imask1.mask, sizeof(imask1.mask), IP2);
+	snprintf(iname2.name, sizeof(iname2.name), IQN1);
+	snprintf(imask2.mask, sizeof(imask2.mask), IP1);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == true);
 
 	/*
@@ -627,12 +601,12 @@ node_access_multi_initiator_groups_cases(void)
 	 * | allowed   | denied  | allowed     | denied  | denied  |
 	 * +---------------------------------------------+---------+
 	 */
-	iname1.name = IQN1;
-	imask1.mask = IP2;
-	iname2.name = IQN1;
-	imask2.mask = IP2;
+	snprintf(iname1.name, sizeof(iname1.name), IQN1);
+	snprintf(imask1.mask, sizeof(imask1.mask), IP2);
+	snprintf(iname2.name, sizeof(iname2.name), IQN1);
+	snprintf(imask2.mask, sizeof(imask2.mask), IP2);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == false);
 
 	/*
@@ -646,11 +620,11 @@ node_access_multi_initiator_groups_cases(void)
 	 * | allowed   | denied  | not found   | -       | denied  |
 	 * +---------------------------------------------+---------+
 	 */
-	iname1.name = IQN1;
-	imask1.mask = IP2;
-	iname2.name = IQN2;
+	snprintf(iname1.name, sizeof(iname1.name), IQN1);
+	snprintf(imask1.mask, sizeof(imask1.mask), IP2);
+	snprintf(iname2.name, sizeof(iname2.name), IQN2);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == false);
 
 	/*
@@ -664,10 +638,10 @@ node_access_multi_initiator_groups_cases(void)
 	 * | not found   | -       | denied    | -       | denied  |
 	 * +---------------------------------------------+---------+
 	 */
-	iname1.name = IQN2;
-	iname2.name = NO_IQN1;
+	snprintf(iname1.name, sizeof(iname1.name), IQN2);
+	snprintf(iname2.name, sizeof(iname2.name), NO_IQN1);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == false);
 
 	/*
@@ -681,11 +655,11 @@ node_access_multi_initiator_groups_cases(void)
 	 * | not found   | -       | allowed   | allowed | allowed |
 	 * +---------------------------------------------+---------+
 	 */
-	iname1.name = IQN2;
-	iname2.name = IQN1;
-	imask2.mask = IP1;
+	snprintf(iname1.name, sizeof(iname1.name), IQN2);
+	snprintf(iname2.name, sizeof(iname2.name), IQN1);
+	snprintf(imask2.mask, sizeof(imask2.mask), IP1);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == true);
 
 	/*
@@ -699,11 +673,11 @@ node_access_multi_initiator_groups_cases(void)
 	 * | not found   | -       | allowed   | denied  | denied  |
 	 * +---------------------------------------------+---------+
 	 */
-	iname1.name = IQN2;
-	iname2.name = IQN1;
-	imask2.mask = IP2;
+	snprintf(iname1.name, sizeof(iname1.name), IQN2);
+	snprintf(iname2.name, sizeof(iname2.name), IQN1);
+	snprintf(imask2.mask, sizeof(imask2.mask), IP2);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == false);
 
 	/*
@@ -717,10 +691,10 @@ node_access_multi_initiator_groups_cases(void)
 	 * | not found   | -       | not found | -       | denied  |
 	 * +---------------------------------------------+---------+
 	 */
-	iname1.name = IQN2;
-	iname2.name = IQN2;
+	snprintf(iname1.name, sizeof(iname1.name), IQN2);
+	snprintf(iname2.name, sizeof(iname2.name), IQN2);
 
-	result = spdk_iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
+	result = iscsi_tgt_node_access(&conn, &tgtnode, iqn, addr);
 	CU_ASSERT(result == false);
 
 	iscsi_pg_map_delete_ig_map(pg_map, &ig1);
@@ -731,35 +705,29 @@ node_access_multi_initiator_groups_cases(void)
 static void
 allow_iscsi_name_multi_maps_case(void)
 {
-	struct spdk_iscsi_tgt_node tgtnode;
-	struct spdk_iscsi_portal_grp pg1, pg2;
-	struct spdk_iscsi_init_grp ig;
-	struct spdk_iscsi_initiator_name iname;
+	struct spdk_iscsi_tgt_node tgtnode = {};
+	struct spdk_iscsi_portal_grp pg1 = {}, pg2 = {};
+	struct spdk_iscsi_init_grp ig = {};
+	struct spdk_iscsi_initiator_name iname = {};
 	struct spdk_iscsi_pg_map *pg_map1, *pg_map2;
-	struct spdk_scsi_dev scsi_dev;
+	struct spdk_scsi_dev scsi_dev = {};
 	char *iqn;
 	bool result;
 
 	/* target initialization */
-	memset(&tgtnode, 0, sizeof(struct spdk_iscsi_tgt_node));
 	TAILQ_INIT(&tgtnode.pg_map_head);
 
-	memset(&scsi_dev, 0, sizeof(struct spdk_scsi_dev));
 	snprintf(scsi_dev.name, sizeof(scsi_dev.name), IQN1);
 	tgtnode.dev = &scsi_dev;
 
 	/* initiator group initialization */
-	memset(&ig, 0, sizeof(struct spdk_iscsi_init_grp));
 	TAILQ_INIT(&ig.initiator_head);
 
 	ig.ninitiators = 1;
-	iname.name = NULL;
 	TAILQ_INSERT_TAIL(&ig.initiator_head, &iname, tailq);
 
 	/* portal group initialization */
-	memset(&pg1, 0, sizeof(struct spdk_iscsi_portal_grp));
 	pg1.tag = 1;
-	memset(&pg2, 0, sizeof(struct spdk_iscsi_portal_grp));
 	pg2.tag = 1;
 
 	pg_map1 = iscsi_tgt_node_add_pg_map(&tgtnode, &pg1);
@@ -770,12 +738,12 @@ allow_iscsi_name_multi_maps_case(void)
 	/* test for IG1 <-> PG1, PG2 case */
 	iqn = IQN1;
 
-	iname.name = IQN1;
+	snprintf(iname.name, sizeof(iname.name), IQN1);
 
 	result = iscsi_tgt_node_allow_iscsi_name(&tgtnode, iqn);
 	CU_ASSERT(result == true);
 
-	iname.name = IQN2;
+	snprintf(iname.name, sizeof(iname.name), IQN2);
 
 	result = iscsi_tgt_node_allow_iscsi_name(&tgtnode, iqn);
 	CU_ASSERT(result == false);
@@ -788,39 +756,39 @@ allow_iscsi_name_multi_maps_case(void)
 
 /*
  * static bool
- * spdk_iscsi_check_chap_params(bool disable_chap, bool require_chap,
+ * iscsi_check_chap_params(bool disable_chap, bool require_chap,
  *                              bool mutual_chap, int chap_group);
  */
 static void
 chap_param_test_cases(void)
 {
 	/* Auto */
-	CU_ASSERT(spdk_iscsi_check_chap_params(false, false, false, 0) == true);
+	CU_ASSERT(iscsi_check_chap_params(false, false, false, 0) == true);
 
 	/* None */
-	CU_ASSERT(spdk_iscsi_check_chap_params(true, false, false, 0) == true);
+	CU_ASSERT(iscsi_check_chap_params(true, false, false, 0) == true);
 
 	/* CHAP */
-	CU_ASSERT(spdk_iscsi_check_chap_params(false, true, false, 0) == true);
+	CU_ASSERT(iscsi_check_chap_params(false, true, false, 0) == true);
 
 	/* CHAP Mutual */
-	CU_ASSERT(spdk_iscsi_check_chap_params(false, true, true, 0) == true);
+	CU_ASSERT(iscsi_check_chap_params(false, true, true, 0) == true);
 
 	/* Check mutual exclusiveness of disabled and required */
-	CU_ASSERT(spdk_iscsi_check_chap_params(true, true, false, 0) == false);
+	CU_ASSERT(iscsi_check_chap_params(true, true, false, 0) == false);
 
 	/* Mutual requires Required */
-	CU_ASSERT(spdk_iscsi_check_chap_params(false, false, true, 0) == false);
+	CU_ASSERT(iscsi_check_chap_params(false, false, true, 0) == false);
 
 	/* Remaining combinations */
-	CU_ASSERT(spdk_iscsi_check_chap_params(true, false, true, 0) == false);
-	CU_ASSERT(spdk_iscsi_check_chap_params(true, true, true, 0) == false);
+	CU_ASSERT(iscsi_check_chap_params(true, false, true, 0) == false);
+	CU_ASSERT(iscsi_check_chap_params(true, true, true, 0) == false);
 
 	/* Valid auth group ID */
-	CU_ASSERT(spdk_iscsi_check_chap_params(false, false, false, 1) == true);
+	CU_ASSERT(iscsi_check_chap_params(false, false, false, 1) == true);
 
 	/* Invalid auth group ID */
-	CU_ASSERT(spdk_iscsi_check_chap_params(false, false, false, -1) == false);
+	CU_ASSERT(iscsi_check_chap_params(false, false, false, -1) == false);
 }
 
 int
@@ -834,40 +802,27 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	if (CU_initialize_registry() != CUE_SUCCESS) {
-		return CU_get_error();
-	}
+	CU_set_error_action(CUEA_ABORT);
+	CU_initialize_registry();
 
 	config_file = argv[1];
 
 	suite = CU_add_suite("iscsi_target_node_suite", NULL, NULL);
-	if (suite == NULL) {
-		CU_cleanup_registry();
-		return CU_get_error();
-	}
 
-	if (
-		CU_add_test(suite, "add lun test cases", add_lun_test_cases) == NULL
-		|| CU_add_test(suite, "config file fail cases", config_file_fail_cases) == NULL
-		|| CU_add_test(suite, "allow any allowed case", allow_any_allowed) == NULL
-		|| CU_add_test(suite, "allow ipv6 allowed case", allow_ipv6_allowed) == NULL
-		|| CU_add_test(suite, "allow ipv6 denied case", allow_ipv6_denied) == NULL
-		|| CU_add_test(suite, "allow ipv6 invalid case", allow_ipv6_invalid) == NULL
-		|| CU_add_test(suite, "allow ipv4 allowed case", allow_ipv4_allowed) == NULL
-		|| CU_add_test(suite, "allow ipv4 denied case", allow_ipv4_denied) == NULL
-		|| CU_add_test(suite, "allow ipv4 invalid case", allow_ipv4_invalid) == NULL
-		|| CU_add_test(suite, "node access allowed case", node_access_allowed) == NULL
-		|| CU_add_test(suite, "node access denied case (empty netmask)",
-			       node_access_denied_by_empty_netmask) == NULL
-		|| CU_add_test(suite, "node access multiple initiator groups cases",
-			       node_access_multi_initiator_groups_cases) == NULL
-		|| CU_add_test(suite, "allow iscsi name case",
-			       allow_iscsi_name_multi_maps_case) == NULL
-		|| CU_add_test(suite, "chap param test cases", chap_param_test_cases) == NULL
-	) {
-		CU_cleanup_registry();
-		return CU_get_error();
-	}
+	CU_ADD_TEST(suite, add_lun_test_cases);
+	CU_ADD_TEST(suite, config_file_fail_cases);
+	CU_ADD_TEST(suite, allow_any_allowed);
+	CU_ADD_TEST(suite, allow_ipv6_allowed);
+	CU_ADD_TEST(suite, allow_ipv6_denied);
+	CU_ADD_TEST(suite, allow_ipv6_invalid);
+	CU_ADD_TEST(suite, allow_ipv4_allowed);
+	CU_ADD_TEST(suite, allow_ipv4_denied);
+	CU_ADD_TEST(suite, allow_ipv4_invalid);
+	CU_ADD_TEST(suite, node_access_allowed);
+	CU_ADD_TEST(suite, node_access_denied_by_empty_netmask);
+	CU_ADD_TEST(suite, node_access_multi_initiator_groups_cases);
+	CU_ADD_TEST(suite, allow_iscsi_name_multi_maps_case);
+	CU_ADD_TEST(suite, chap_param_test_cases);
 
 	CU_basic_set_mode(CU_BRM_VERBOSE);
 	CU_basic_run_tests();
