@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2014 Red Hat <contact@redhat.com>
  *
- * LGPL2.1 (see COPYING-LGPL2.1) or later
+ * LGPL-2.1 (see COPYING-LGPL2.1) or later
  */
 
 #include <gtest/gtest.h>
@@ -28,7 +28,7 @@ public:
 };
 
 typedef ::testing::Types<TestParams<2> > BitVectorTypes;
-TYPED_TEST_CASE(BitVectorTest, BitVectorTypes);
+TYPED_TEST_SUITE(BitVectorTest, BitVectorTypes);
 
 TYPED_TEST(BitVectorTest, resize) {
   typename TestFixture::bit_vector_t bit_vector;
@@ -143,12 +143,12 @@ TYPED_TEST(BitVectorTest, partial_decode_encode) {
   }
 
   bufferlist bl;
-  ::encode(bit_vector, bl);
+  encode(bit_vector, bl);
   bit_vector.clear();
 
   bufferlist header_bl;
   header_bl.substr_of(bl, 0, bit_vector.get_header_length());
-  bufferlist::iterator header_it = header_bl.begin();
+  auto header_it = header_bl.cbegin();
   bit_vector.decode_header(header_it);
 
   uint64_t object_byte_offset;
@@ -175,7 +175,7 @@ TYPED_TEST(BitVectorTest, partial_decode_encode) {
                                      &footer_byte_length);
     ASSERT_TRUE(footer_byte_offset + footer_byte_length <= bl.length());
     footer_bl.substr_of(bl, footer_byte_offset, footer_byte_length);
-    bufferlist::iterator footer_it = footer_bl.begin();
+    auto footer_it = footer_bl.cbegin();
     bit_vector.decode_data_crcs(footer_it, it->first);
 
     uint64_t element_offset = it->first;
@@ -188,7 +188,7 @@ TYPED_TEST(BitVectorTest, partial_decode_encode) {
     bufferlist data_bl;
     data_bl.substr_of(bl, bit_vector.get_header_length() + data_byte_offset,
 		      byte_length);
-    bufferlist::iterator data_it = data_bl.begin();
+    auto data_it = data_bl.cbegin();
     bit_vector.decode_data(data_it, data_byte_offset);
 
     data_bl.clear();
@@ -225,8 +225,8 @@ TYPED_TEST(BitVectorTest, partial_decode_encode) {
     updated_bl.append(full_footer);
     ASSERT_EQ(bl, updated_bl);
 
-    bufferlist::iterator updated_it = updated_bl.begin();
-    ::decode(bit_vector, updated_it);
+    auto updated_it = updated_bl.cbegin();
+    decode(bit_vector, updated_it);
   }
 }
 
@@ -239,7 +239,7 @@ TYPED_TEST(BitVectorTest, header_crc) {
   bufferlist footer;
   bit_vector.encode_footer(footer);
 
-  bufferlist::iterator it = footer.begin();
+  auto it = footer.cbegin();
   bit_vector.decode_footer(it);
 
   bit_vector.resize(1);
@@ -266,7 +266,7 @@ TYPED_TEST(BitVectorTest, data_crc) {
   bufferlist data;
   bit_vector1.encode_data(data, data_byte_offset, byte_length);
 
-  bufferlist::iterator data_it = data.begin();
+  auto data_it = data.cbegin();
   bit_vector1.decode_data(data_it, data_byte_offset);
 
   bit_vector2[bit_vector2.size() - 1] = 1;
