@@ -22,13 +22,14 @@
 #include "include/stringify.h"
 #include "erasure-code/lrc/ErasureCodeLrc.h"
 #include "global/global_context.h"
-#include "common/config.h"
+#include "common/config_proxy.h"
 #include "gtest/gtest.h"
 
+using namespace std;
 
 TEST(ErasureCodeLrc, parse_rule)
 {
-  ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+  ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
   EXPECT_EQ("default", lrc.rule_root);
   EXPECT_EQ("host", lrc.rule_steps.front().type);
 
@@ -130,7 +131,7 @@ TEST(ErasureCodeTest, create_rule)
 
   c->finalize();
 
-  ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+  ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
   EXPECT_EQ(0, lrc.create_rule("rule1", *c, &cerr));
 
   ErasureCodeProfile profile;
@@ -171,7 +172,7 @@ TEST(ErasureCodeTest, create_rule)
 
 TEST(ErasureCodeLrc, parse_kml)
 {
-  ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+  ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
   ErasureCodeProfile profile;
   EXPECT_EQ(0, lrc.parse_kml(profile, &cerr));
   profile["k"] = "4";
@@ -246,7 +247,7 @@ TEST(ErasureCodeLrc, parse_kml)
 
 TEST(ErasureCodeLrc, layers_description)
 {
-  ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+  ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
   ErasureCodeProfile profile;
 
   json_spirit::mArray description;
@@ -275,7 +276,7 @@ TEST(ErasureCodeLrc, layers_description)
 TEST(ErasureCodeLrc, layers_parse)
 {
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
 
     const char *description_string ="[ 0 ]";
@@ -287,7 +288,7 @@ TEST(ErasureCodeLrc, layers_parse)
   }
 
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
 
     const char *description_string ="[ [ 0 ] ]";
@@ -299,7 +300,7 @@ TEST(ErasureCodeLrc, layers_parse)
   }
 
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
 
     const char *description_string ="[ [ \"\", 0 ] ]";
@@ -315,7 +316,7 @@ TEST(ErasureCodeLrc, layers_parse)
   // profile.
   //
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
 
     const char *description_string ="[ [ \"\", { \"a\": \"b\" }, \"ignored\" ] ]";
@@ -331,7 +332,7 @@ TEST(ErasureCodeLrc, layers_parse)
   // profile.
   //
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
 
     const char *description_string ="[ [ \"\", \"a=b c=d\" ] ]";
@@ -348,7 +349,7 @@ TEST(ErasureCodeLrc, layers_parse)
 TEST(ErasureCodeLrc, layers_sanity_checks)
 {
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
     profile["mapping"] =
 	    "__DDD__DD";
@@ -362,7 +363,7 @@ TEST(ErasureCodeLrc, layers_sanity_checks)
     EXPECT_EQ(0, lrc.init(profile, &cerr));
   }
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
     const char *description_string =
       "[ "
@@ -371,7 +372,7 @@ TEST(ErasureCodeLrc, layers_sanity_checks)
     EXPECT_EQ(ERROR_LRC_MAPPING, lrc.init(profile, &cerr));
   }
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
     profile["mapping"] = "";
     const char *description_string =
@@ -381,7 +382,7 @@ TEST(ErasureCodeLrc, layers_sanity_checks)
     EXPECT_EQ(ERROR_LRC_LAYERS_COUNT, lrc.init(profile, &cerr));
   }
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
     profile["mapping"] =
 	    "DD";
@@ -399,11 +400,11 @@ TEST(ErasureCodeLrc, layers_sanity_checks)
 TEST(ErasureCodeLrc, layers_init)
 {
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
 
     const char* env = getenv("CEPH_LIB");
-    string directory(env ? env : ".libs");
+    string directory(env ? env : "lib");
     string description_string = 
       "[ " 
       "  [ \"_cDDD_cDD_\", \"directory=" + directory + "\" ]," 
@@ -422,7 +423,7 @@ TEST(ErasureCodeLrc, layers_init)
 
 TEST(ErasureCodeLrc, init)
 {
-  ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+  ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
   ErasureCodeProfile profile;
   profile["mapping"] =
     "__DDD__DD";
@@ -438,7 +439,7 @@ TEST(ErasureCodeLrc, init)
 
 TEST(ErasureCodeLrc, init_kml)
 {
-  ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+  ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
   ErasureCodeProfile profile;
   profile["k"] = "4";
   profile["m"] = "2";
@@ -451,7 +452,7 @@ TEST(ErasureCodeLrc, minimum_to_decode)
 {
   // trivial : no erasures, the minimum is want_to_read
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
     profile["mapping"] =
       "__DDD__DD";
@@ -469,12 +470,12 @@ TEST(ErasureCodeLrc, minimum_to_decode)
     available_chunks.insert(1);
     available_chunks.insert(2);
     set<int> minimum;
-    EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+    EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     EXPECT_EQ(want_to_read, minimum);
   }
   // locally repairable erasure
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
     profile["mapping"] =
 	    "__DDD__DD_";
@@ -499,7 +500,7 @@ TEST(ErasureCodeLrc, minimum_to_decode)
 	available_chunks.insert(i);
       // _____DDDDc can recover c
       set<int> minimum;
-      EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+      EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
       set<int> expected_minimum;
       expected_minimum.insert(5);
       expected_minimum.insert(6);
@@ -514,7 +515,7 @@ TEST(ErasureCodeLrc, minimum_to_decode)
       for (int i = 1; i < (int)lrc.get_chunk_count(); i++)
 	available_chunks.insert(i);
       set<int> minimum;
-      EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+      EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
       set<int> expected_minimum;
       expected_minimum.insert(2);
       expected_minimum.insert(3);
@@ -524,7 +525,7 @@ TEST(ErasureCodeLrc, minimum_to_decode)
   }
   // implicit parity required
   {
-    ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+    ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
     ErasureCodeProfile profile;
     profile["mapping"] =
 	    "__DDD__DD";
@@ -555,7 +556,7 @@ TEST(ErasureCodeLrc, minimum_to_decode)
       // missing             (7)
       // missing             (8)
       set<int> minimum;
-      EXPECT_EQ(-EIO, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+      EXPECT_EQ(-EIO, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     }
     //
     // We want to read chunk 8 and encoding was done with
@@ -594,7 +595,7 @@ TEST(ErasureCodeLrc, minimum_to_decode)
       // missing             (7)
       // missing             (8)
       set<int> minimum;
-      EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+      EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
       EXPECT_EQ(available_chunks, minimum);
     }
   }
@@ -602,7 +603,7 @@ TEST(ErasureCodeLrc, minimum_to_decode)
 
 TEST(ErasureCodeLrc, encode_decode)
 {
-  ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+  ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
   ErasureCodeProfile profile;
   profile["mapping"] =
     "__DD__DD";
@@ -615,7 +616,7 @@ TEST(ErasureCodeLrc, encode_decode)
   profile["layers"] = description_string;
   EXPECT_EQ(0, lrc.init(profile, &cerr));
   EXPECT_EQ(4U, lrc.get_data_chunk_count());
-  unsigned int chunk_size = g_conf->osd_pool_erasure_code_stripe_unit;
+  unsigned int chunk_size = g_conf().get_val<Option::size_t>("osd_pool_erasure_code_stripe_unit");
   unsigned int stripe_width = lrc.get_data_chunk_count() * chunk_size;
   EXPECT_EQ(chunk_size, lrc.get_chunk_size(stripe_width));
   set<int> want_to_encode;
@@ -623,7 +624,10 @@ TEST(ErasureCodeLrc, encode_decode)
   for (unsigned int i = 0; i < lrc.get_chunk_count(); ++i) {
     want_to_encode.insert(i);
     bufferptr ptr(buffer::create_page_aligned(chunk_size));
-    encoded[i].push_front(ptr);
+    bufferlist tmp;
+    tmp.push_back(ptr);
+    tmp.claim_append(encoded[i]);
+    encoded[i].swap(tmp);
   }
   const vector<int> &mapping = lrc.get_chunk_mapping();
   char c = 'A';
@@ -648,14 +652,14 @@ TEST(ErasureCodeLrc, encode_decode)
     available_chunks.insert(5);
     available_chunks.insert(6);
     set<int> minimum;
-    EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+    EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     // only need three chunks from the second local layer
     EXPECT_EQ(3U, minimum.size());
     EXPECT_EQ(1U, minimum.count(4));
     EXPECT_EQ(1U, minimum.count(5));
     EXPECT_EQ(1U, minimum.count(6));
     map<int, bufferlist> decoded;
-    EXPECT_EQ(0, lrc.decode(want_to_read, chunks, &decoded));
+    EXPECT_EQ(0, lrc._decode(want_to_read, chunks, &decoded));
     string s(chunk_size, 'D');
     EXPECT_EQ(s, string(decoded[7].c_str(), chunk_size));
   }
@@ -675,12 +679,12 @@ TEST(ErasureCodeLrc, encode_decode)
     available_chunks.insert(6);
     available_chunks.insert(7);
     set<int> minimum;
-    EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+    EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     EXPECT_EQ(5U, minimum.size());
     EXPECT_EQ(available_chunks, minimum);
 
     map<int, bufferlist> decoded;
-    EXPECT_EQ(0, lrc.decode(want_to_read, encoded, &decoded));
+    EXPECT_EQ(0, lrc._decode(want_to_read, encoded, &decoded));
     string s(chunk_size, 'A');
     EXPECT_EQ(s, string(decoded[2].c_str(), chunk_size));
   }
@@ -701,7 +705,7 @@ TEST(ErasureCodeLrc, encode_decode)
     encoded.erase(3);
     encoded.erase(6);
     set<int> minimum;
-    EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+    EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     EXPECT_EQ(4U, minimum.size());
     // only need two chunks from the first local layer
     EXPECT_EQ(1U, minimum.count(0));
@@ -714,7 +718,7 @@ TEST(ErasureCodeLrc, encode_decode)
     EXPECT_EQ(1U, minimum.count(5));
 
     map<int, bufferlist> decoded;
-    EXPECT_EQ(0, lrc.decode(want_to_read, encoded, &decoded));
+    EXPECT_EQ(0, lrc._decode(want_to_read, encoded, &decoded));
     {
       string s(chunk_size, 'B');
       EXPECT_EQ(s, string(decoded[3].c_str(), chunk_size));
@@ -732,7 +736,7 @@ TEST(ErasureCodeLrc, encode_decode)
 
 TEST(ErasureCodeLrc, encode_decode_2)
 {
-  ErasureCodeLrc lrc(g_conf->get_val<std::string>("erasure_code_dir"));
+  ErasureCodeLrc lrc(g_conf().get_val<std::string>("erasure_code_dir"));
   ErasureCodeProfile profile;
   profile["mapping"] =
     "DD__DD__";
@@ -745,7 +749,7 @@ TEST(ErasureCodeLrc, encode_decode_2)
   profile["layers"] = description_string;
   EXPECT_EQ(0, lrc.init(profile, &cerr));
   EXPECT_EQ(4U, lrc.get_data_chunk_count());
-  unsigned int chunk_size = g_conf->osd_pool_erasure_code_stripe_unit;
+  unsigned int chunk_size = g_conf().get_val<Option::size_t>("osd_pool_erasure_code_stripe_unit");
   unsigned int stripe_width = lrc.get_data_chunk_count() * chunk_size;
   EXPECT_EQ(chunk_size, lrc.get_chunk_size(stripe_width));
   set<int> want_to_encode;
@@ -753,7 +757,10 @@ TEST(ErasureCodeLrc, encode_decode_2)
   for (unsigned int i = 0; i < lrc.get_chunk_count(); ++i) {
     want_to_encode.insert(i);
     bufferptr ptr(buffer::create_page_aligned(chunk_size));
-    encoded[i].push_front(ptr);
+    bufferlist tmp;
+    tmp.push_back(ptr);
+    tmp.claim_append(encoded[i]);
+    encoded[i].swap(tmp);
   }
   const vector<int> &mapping = lrc.get_chunk_mapping();
   char c = 'A';
@@ -784,7 +791,7 @@ TEST(ErasureCodeLrc, encode_decode_2)
     available_chunks.insert(6);
     available_chunks.insert(7);
     set<int> minimum;
-    EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+    EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     EXPECT_EQ(4U, minimum.size());
     EXPECT_EQ(1U, minimum.count(1));
     EXPECT_EQ(1U, minimum.count(4));
@@ -792,7 +799,7 @@ TEST(ErasureCodeLrc, encode_decode_2)
     EXPECT_EQ(1U, minimum.count(6));
 
     map<int, bufferlist> decoded;
-    EXPECT_EQ(0, lrc.decode(want_to_read, chunks, &decoded));
+    EXPECT_EQ(0, lrc._decode(want_to_read, chunks, &decoded));
     string s(chunk_size, 'A');
     EXPECT_EQ(s, string(decoded[0].c_str(), chunk_size));
   }
@@ -813,7 +820,7 @@ TEST(ErasureCodeLrc, encode_decode_2)
     available_chunks.insert(6);
     available_chunks.insert(7);
     set<int> minimum;
-    EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+    EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     EXPECT_EQ(5U, minimum.size());
     EXPECT_EQ(1U, minimum.count(1));
     EXPECT_EQ(1U, minimum.count(3));
@@ -822,7 +829,7 @@ TEST(ErasureCodeLrc, encode_decode_2)
     EXPECT_EQ(1U, minimum.count(7));
 
     map<int, bufferlist> decoded;
-    EXPECT_EQ(0, lrc.decode(want_to_read, chunks, &decoded));
+    EXPECT_EQ(0, lrc._decode(want_to_read, chunks, &decoded));
     {
       string s(chunk_size, 'A');
       EXPECT_EQ(s, string(decoded[0].c_str(), chunk_size));
@@ -857,7 +864,7 @@ TEST(ErasureCodeLrc, encode_decode_2)
     available_chunks.insert(6);
     available_chunks.insert(7);
     set<int> minimum;
-    EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+    EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     EXPECT_EQ(5U, minimum.size());
     EXPECT_EQ(1U, minimum.count(1));
     EXPECT_EQ(1U, minimum.count(3));
@@ -866,7 +873,7 @@ TEST(ErasureCodeLrc, encode_decode_2)
     EXPECT_EQ(1U, minimum.count(7));
 
     map<int, bufferlist> decoded;
-    EXPECT_EQ(0, lrc.decode(want_to_read, chunks, &decoded));
+    EXPECT_EQ(0, lrc._decode(want_to_read, chunks, &decoded));
     {
       string s(chunk_size, 'A');
       EXPECT_EQ(s, string(decoded[0].c_str(), chunk_size));
@@ -900,11 +907,11 @@ TEST(ErasureCodeLrc, encode_decode_2)
     available_chunks.insert(5);
     available_chunks.insert(7);
     set<int> minimum;
-    EXPECT_EQ(0, lrc.minimum_to_decode(want_to_read, available_chunks, &minimum));
+    EXPECT_EQ(0, lrc._minimum_to_decode(want_to_read, available_chunks, &minimum));
     EXPECT_EQ(available_chunks, minimum);
 
     map<int, bufferlist> decoded;
-    EXPECT_EQ(0, lrc.decode(want_to_read, chunks, &decoded));
+    EXPECT_EQ(0, lrc._decode(want_to_read, chunks, &decoded));
   }
 }
 
