@@ -7,12 +7,14 @@
 #include "test/rbd_mirror/test_fixture.h"
 #include "test/librados_test_stub/LibradosTestStub.h"
 #include "common/WorkQueue.h"
+#include "librbd/asio/ContextWQ.h"
 #include <boost/shared_ptr.hpp>
 #include <gmock/gmock.h>
-#include "include/assert.h"
+#include "include/ceph_assert.h"
 
 namespace librados {
 class TestRadosClient;
+class MockTestMemCluster;
 class MockTestMemIoCtxImpl;
 class MockTestMemRadosClient;
 }
@@ -30,10 +32,14 @@ ACTION_P(CompleteContext, r) {
 }
 
 ACTION_P2(CompleteContext, wq, r) {
-  ContextWQ *context_wq = reinterpret_cast<ContextWQ *>(wq);
+  auto context_wq = reinterpret_cast<librbd::asio::ContextWQ *>(wq);
   context_wq->queue(arg0, r);
 }
 
+ACTION_P(GetReference, ref_object) {
+  ref_object->get();
+}
+
 MATCHER_P(ContentsEqual, bl, "") {
   // TODO fix const-correctness of bufferlist
   return const_cast<bufferlist &>(arg).contents_equal(
@@ -54,6 +60,8 @@ public:
 
   void expect_test_features(librbd::MockImageCtx &mock_image_ctx);
 
+  librados::MockTestMemCluster& get_mock_cluster();
+
 private:
   static TestClusterRef s_test_cluster;
 };
