@@ -24,8 +24,7 @@
 using namespace std;
 
 
-std::unique_ptr<BlockCrypt> AES_256_CBC_create(CephContext* cct, const uint8_t* key, size_t len);
-
+std::unique_ptr<BlockCrypt> AES_256_CBC_create(const DoutPrefixProvider *dpp, CephContext* cct, const uint8_t* key, size_t len);
 
 class ut_get_sink : public RGWGetObj_Filter {
   std::stringstream sink;
@@ -44,7 +43,7 @@ public:
   }
 };
 
-class ut_put_sink: public rgw::putobj::DataProcessor
+class ut_put_sink: public rgw::sal::DataProcessor
 {
   std::stringstream sink;
 public:
@@ -92,9 +91,9 @@ public:
   }
 };
 
-
 TEST(TestRGWCrypto, verify_AES_256_CBC_identity)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   //create some input for encryption
   const off_t test_range = 1024*1024;
   buffer::ptr buf(test_range);
@@ -112,7 +111,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_identity)
     for(size_t i=0;i<sizeof(key);i++)
       key[i]=i*step;
 
-    auto aes(AES_256_CBC_create(g_ceph_context, &key[0], 32));
+    auto aes(AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32));
     ASSERT_NE(aes.get(), nullptr);
 
     size_t block_size = aes->get_block_size();
@@ -147,6 +146,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_identity)
 
 TEST(TestRGWCrypto, verify_AES_256_CBC_identity_2)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   //create some input for encryption
   const off_t test_range = 1024*1024;
   buffer::ptr buf(test_range);
@@ -164,7 +164,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_identity_2)
     for(size_t i=0;i<sizeof(key);i++)
       key[i]=i*step;
 
-    auto aes(AES_256_CBC_create(g_ceph_context, &key[0], 32));
+    auto aes(AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32));
     ASSERT_NE(aes.get(), nullptr);
 
     size_t block_size = aes->get_block_size();
@@ -195,6 +195,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_identity_2)
 
 TEST(TestRGWCrypto, verify_AES_256_CBC_identity_3)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   //create some input for encryption
   const off_t test_range = 1024*1024;
   buffer::ptr buf(test_range);
@@ -212,7 +213,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_identity_3)
     for(size_t i=0;i<sizeof(key);i++)
       key[i]=i*step;
 
-    auto aes(AES_256_CBC_create(g_ceph_context, &key[0], 32));
+    auto aes(AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32));
     ASSERT_NE(aes.get(), nullptr);
 
     size_t block_size = aes->get_block_size();
@@ -272,6 +273,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_identity_3)
 
 TEST(TestRGWCrypto, verify_AES_256_CBC_size_0_15)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   //create some input for encryption
   const off_t test_range = 1024*1024;
   buffer::ptr buf(test_range);
@@ -289,7 +291,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_size_0_15)
     for(size_t i=0;i<sizeof(key);i++)
       key[i]=i*step;
 
-    auto aes(AES_256_CBC_create(g_ceph_context, &key[0], 32));
+    auto aes(AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32));
     ASSERT_NE(aes.get(), nullptr);
 
     size_t block_size = aes->get_block_size();
@@ -321,6 +323,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_size_0_15)
 
 TEST(TestRGWCrypto, verify_AES_256_CBC_identity_last_block)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   //create some input for encryption
   const off_t test_range = 1024*1024;
   buffer::ptr buf(test_range);
@@ -338,7 +341,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_identity_last_block)
     for(size_t i=0;i<sizeof(key);i++)
       key[i]=i*step;
 
-    auto aes(AES_256_CBC_create(g_ceph_context, &key[0], 32));
+    auto aes(AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32));
     ASSERT_NE(aes.get(), nullptr);
 
     size_t block_size = aes->get_block_size();
@@ -397,6 +400,7 @@ TEST(TestRGWCrypto, verify_AES_256_CBC_identity_last_block)
 
 TEST(TestRGWCrypto, verify_RGWGetObj_BlockDecrypt_ranges)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   //create some input for encryption
   const off_t test_range = 1024*1024;
   bufferptr buf(test_range);
@@ -411,7 +415,7 @@ TEST(TestRGWCrypto, verify_RGWGetObj_BlockDecrypt_ranges)
   for(size_t i=0;i<sizeof(key);i++)
     key[i] = i;
 
-  auto cbc = AES_256_CBC_create(g_ceph_context, &key[0], 32);
+  auto cbc = AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32);
   ASSERT_NE(cbc.get(), nullptr);
   bufferlist encrypted;
   ASSERT_TRUE(cbc->encrypt(input, 0, test_range, encrypted, 0));
@@ -420,9 +424,9 @@ TEST(TestRGWCrypto, verify_RGWGetObj_BlockDecrypt_ranges)
   for (off_t r = 93; r < 150; r++ )
   {
     ut_get_sink get_sink;
-    auto cbc = AES_256_CBC_create(g_ceph_context, &key[0], 32);
+    auto cbc = AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32);
     ASSERT_NE(cbc.get(), nullptr);
-    RGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink, std::move(cbc) );
+    RGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink, std::move(cbc));
 
     //random ranges
     off_t begin = (r/3)*r*(r+13)*(r+23)*(r+53)*(r+71) % test_range;
@@ -443,6 +447,7 @@ TEST(TestRGWCrypto, verify_RGWGetObj_BlockDecrypt_ranges)
 
 TEST(TestRGWCrypto, verify_RGWGetObj_BlockDecrypt_chunks)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   //create some input for encryption
   const off_t test_range = 1024*1024;
   bufferptr buf(test_range);
@@ -457,7 +462,7 @@ TEST(TestRGWCrypto, verify_RGWGetObj_BlockDecrypt_chunks)
   for(size_t i=0;i<sizeof(key);i++)
     key[i] = i;
 
-  auto cbc = AES_256_CBC_create(g_ceph_context, &key[0], 32);
+  auto cbc = AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32);
   ASSERT_NE(cbc.get(), nullptr);
   bufferlist encrypted;
   ASSERT_TRUE(cbc->encrypt(input, 0, test_range, encrypted, 0));
@@ -465,9 +470,9 @@ TEST(TestRGWCrypto, verify_RGWGetObj_BlockDecrypt_chunks)
   for (off_t r = 93; r < 150; r++ )
   {
     ut_get_sink get_sink;
-    auto cbc = AES_256_CBC_create(g_ceph_context, &key[0], 32);
+    auto cbc = AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32);
     ASSERT_NE(cbc.get(), nullptr);
-    RGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink, std::move(cbc) );
+    RGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink, std::move(cbc));
 
     //random
     off_t begin = (r/3)*r*(r+13)*(r+23)*(r+53)*(r+71) % test_range;
@@ -509,9 +514,10 @@ range_t fixup_range(RGWGetObj_BlockDecrypt *decrypt, off_t ofs, off_t end)
 
 TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   ut_get_sink get_sink;
   auto nonecrypt = std::unique_ptr<BlockCrypt>(new BlockCryptNone);
-  RGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink,
+  RGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink,
                                  std::move(nonecrypt));
   ASSERT_EQ(fixup_range(&decrypt,0,0),     range_t(0,255));
   ASSERT_EQ(fixup_range(&decrypt,1,256),   range_t(0,511));
@@ -549,10 +555,11 @@ const size_t obj_size = 30*1024*1024;
 
 TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_simple)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
 
   ut_get_sink get_sink;
   auto nonecrypt = std::make_unique<BlockCryptNone>(4096);
-  TestRGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink,
+  TestRGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink,
 				     std::move(nonecrypt));
   decrypt.set_parts_len(create_mp_parts(obj_size, part_size));
   ASSERT_EQ(fixup_range(&decrypt,0,0),     range_t(0,4095));
@@ -579,10 +586,11 @@ TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_simple)
 
 TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_non_aligned_obj_size)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
 
   ut_get_sink get_sink;
   auto nonecrypt = std::make_unique<BlockCryptNone>(4096);
-  TestRGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink,
+  TestRGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink,
 				     std::move(nonecrypt));
   auto na_obj_size = obj_size + 1;
   decrypt.set_parts_len(create_mp_parts(na_obj_size, part_size));
@@ -606,10 +614,11 @@ TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_non_aligned_obj_size)
 
 TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_non_aligned_part_size)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
 
   ut_get_sink get_sink;
   auto nonecrypt = std::make_unique<BlockCryptNone>(4096);
-  TestRGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink,
+  TestRGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink,
 				     std::move(nonecrypt));
   auto na_part_size = part_size + 1;
   decrypt.set_parts_len(create_mp_parts(obj_size, na_part_size));
@@ -639,10 +648,11 @@ TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_non_aligned_part_size)
 
 TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_non_aligned)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
 
   ut_get_sink get_sink;
   auto nonecrypt = std::make_unique<BlockCryptNone>(4096);
-  TestRGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink,
+  TestRGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink,
 				     std::move(nonecrypt));
   auto na_part_size = part_size + 1;
   auto na_obj_size = obj_size + 7; // (6*(5MiB + 1) + 1) for the last 1B overflow
@@ -668,10 +678,11 @@ TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_non_aligned)
 
 TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_invalid_ranges)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
 
   ut_get_sink get_sink;
   auto nonecrypt = std::make_unique<BlockCryptNone>(4096);
-  TestRGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink,
+  TestRGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink,
 				     std::move(nonecrypt));
 
   decrypt.set_parts_len(create_mp_parts(obj_size, part_size));
@@ -690,6 +701,7 @@ TEST(TestRGWCrypto, check_RGWGetObj_BlockDecrypt_fixup_invalid_ranges)
 
 TEST(TestRGWCrypto, verify_RGWPutObj_BlockEncrypt_chunks)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   //create some input for encryption
   const off_t test_range = 1024*1024;
   bufferptr buf(test_range);
@@ -707,10 +719,10 @@ TEST(TestRGWCrypto, verify_RGWPutObj_BlockEncrypt_chunks)
   for (off_t r = 93; r < 150; r++ )
   {
     ut_put_sink put_sink;
-    auto cbc = AES_256_CBC_create(g_ceph_context, &key[0], 32);
+    auto cbc = AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32);
     ASSERT_NE(cbc.get(), nullptr);
-    RGWPutObj_BlockEncrypt encrypt(g_ceph_context, &put_sink,
-                                   std::move(cbc) );
+    RGWPutObj_BlockEncrypt encrypt(&no_dpp, g_ceph_context, &put_sink,
+                                   std::move(cbc));
 
     off_t test_size = (r/5)*(r+7)*(r+13)*(r+101)*(r*103) % (test_range - 1) + 1;
     off_t pos = 0;
@@ -731,7 +743,7 @@ TEST(TestRGWCrypto, verify_RGWPutObj_BlockEncrypt_chunks)
 
     ASSERT_EQ(put_sink.get_sink().length(), static_cast<size_t>(test_size));
 
-    cbc = AES_256_CBC_create(g_ceph_context, &key[0], 32);
+    cbc = AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32);
     ASSERT_NE(cbc.get(), nullptr);
 
     bufferlist encrypted;
@@ -748,6 +760,7 @@ TEST(TestRGWCrypto, verify_RGWPutObj_BlockEncrypt_chunks)
 
 TEST(TestRGWCrypto, verify_Encrypt_Decrypt)
 {
+  const NoDoutPrefix no_dpp(g_ceph_context, dout_subsys);
   uint8_t key[32];
   for(size_t i=0;i<sizeof(key);i++)
     key[i]=i;
@@ -769,8 +782,8 @@ TEST(TestRGWCrypto, verify_Encrypt_Decrypt)
     memset(test_in, test_size & 0xff, test_size);
 
     ut_put_sink put_sink;
-    RGWPutObj_BlockEncrypt encrypt(g_ceph_context, &put_sink,
-				   AES_256_CBC_create(g_ceph_context, &key[0], 32) );
+    RGWPutObj_BlockEncrypt encrypt(&no_dpp, g_ceph_context, &put_sink,
+				   AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32));
     bufferlist bl;
     bl.append((char*)test_in, test_size);
     encrypt.process(std::move(bl), 0);
@@ -781,8 +794,8 @@ TEST(TestRGWCrypto, verify_Encrypt_Decrypt)
     ASSERT_EQ(bl.length(), test_size);
 
     ut_get_sink get_sink;
-    RGWGetObj_BlockDecrypt decrypt(g_ceph_context, &get_sink,
-                                   AES_256_CBC_create(g_ceph_context, &key[0], 32) );
+    RGWGetObj_BlockDecrypt decrypt(&no_dpp, g_ceph_context, &get_sink,
+                                   AES_256_CBC_create(&no_dpp, g_ceph_context, &key[0], 32));
 
     off_t bl_ofs = 0;
     off_t bl_end = test_size - 1;
@@ -797,9 +810,7 @@ TEST(TestRGWCrypto, verify_Encrypt_Decrypt)
 
 
 int main(int argc, char **argv) {
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **)argv, args);
-
+  auto args = argv_to_vec(argc, argv);
   auto cct = global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT,
 			 CODE_ENVIRONMENT_UTILITY,
 			 CINIT_FLAG_NO_DEFAULT_CONFIG_FILE);
