@@ -23,11 +23,12 @@
 #include <iostream>
 #include <sstream>
 #include <boost/program_options.hpp>
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 
 namespace rbd {
 namespace action {
 namespace trash {
+using namespace boost::placeholders;
 
 namespace at = argument_types;
 namespace po = boost::program_options;
@@ -100,6 +101,9 @@ int execute_move(const po::variables_map &vm,
               << std::endl;
   }
 
+  if (expires_at != "now") {
+    std::cout << "rbd: image " << image_name << " will expire at " << exp_time << std::endl;
+  }
   return r;
 }
 
@@ -353,7 +357,7 @@ int execute_list(const po::variables_map &vm,
   std::string pool_name;
   std::string namespace_name;
   size_t arg_index = 0;
-  int r = utils::get_pool_and_namespace_names(vm, false, &pool_name,
+  int r = utils::get_pool_and_namespace_names(vm, true, false, &pool_name,
                                               &namespace_name, &arg_index);
   if (r < 0) {
     return r;
@@ -404,7 +408,7 @@ int execute_purge(const po::variables_map &vm,
   std::string pool_name;
   std::string namespace_name;
   size_t arg_index = 0;
-  int r = utils::get_pool_and_namespace_names(vm, false, &pool_name,
+  int r = utils::get_pool_and_namespace_names(vm, true, false, &pool_name,
                                               &namespace_name, &arg_index);
   if (r < 0) {
     return r;
