@@ -4,9 +4,9 @@
 #include "common/debug.h"
 #include "common/errno.h"
 #include "common/Timer.h"
-#include "common/WorkQueue.h"
 
 #include "librbd/Utils.h"
+#include "librbd/asio/ContextWQ.h"
 #include "tools/rbd_mirror/Threads.h"
 
 #include "ImageMap.h"
@@ -20,6 +20,8 @@
 #define dout_prefix *_dout << "rbd::mirror::ImageMap: " << this << " " \
                            << __func__ << ": "
 
+using namespace std;
+
 namespace rbd {
 namespace mirror {
 
